function [err,dtotalerr, ddtotalerr,derr,dderr,y] = ...
          adaline_eval(w,x,ytarget,caseweights,errfun);
%
% [err,dtotalerr,ddtotalerr,derr,dderr,yest] = adaline_eval(w,x,y,caseweights,errfun);
%
% Returns the error from an adaline model w, which given x, tries to
% predict y. ie. this model tries to map xw = y.
%
% Error is equal to (yest-y) squared. This function also returns
% derivatives of the error. Using this information, an
% external routine can call this to get the derivative, and then
% minimize the error.
%
% If caseweights are included, then the model gives the error and gradient
% for a Weighted Least Squares model having these weights on each case.
%
% EXAMPLES
%
% % Quantify Error
% w = [1 2 3];yt = [1 1]';casew = [1 1]';x = [1 2 3;4 5 6];
% w1=gradient_optimize('adaline_eval',w,[],[],0.000001,10,inf,inf,0,0,0,x,yt,casew);
% w2=newton_optimize('adaline_eval',w,[],[],10,inf,inf,x,yt,casew);
% w3=fmins('adaline_eval',w,foptions,[],x,yt,casew);
% wdirect = (pinv(x)*yt)';
% fprintf(['matlab pinv %f\ngradient descent %f\n',...
%          'newton %f\nnelder-mead %f\n' ], ...
%          adaline_eval(wdirect,x,yt,casew), ...
%          adaline_eval(w1,x,yt,casew), ...
%          adaline_eval(w2,x,yt,casew), adaline_eval(w3,x,yt,casew));
%
% % Fit a quadratic
% x=[rand(100,3) ones(100,1)];yt=x(:,1).^2+5.*x(:,2);casew=ones(size(x,1),1);w=rand(1,4);
% w1=gradient_optimize('adaline_eval',w,[],[],0.0001,10,inf,inf,0,0,1,x,yt,casew);
% w2=newton_optimize(  'adaline_eval',w,[],[],1,inf,inf,x,yt,casew);
% w3=fmins('adaline_eval',w,foptions,[],x,yt,casew);wdirect=(pinv(x)*yt)';
% figure(1);clf;
% subplot(2,2,1);[a,b,c,d,e,f]=adaline_eval(wdirect,x,yt,casew);plot(yt,f,'.');title('pinv');
% subplot(2,2,2);[a,b,c,d,e,f]=adaline_eval(w1,x,yt,casew);plot(yt,f,'.');title('grad');
% subplot(2,2,3);[a,b,c,d,e,f]=adaline_eval(w2,x,yt,casew);plot(yt,f,'.');title('newt');
% subplot(2,2,4);[a,b,c,d,e,f]=adaline_eval(w3,x,yt,casew);plot(yt,f,'.');title('nelder-mead');
% 
% % Plot the error surface for two dimension problem
% x=rand(100,2);yt=x(:,1).^2+5.*x(:,2);casew=ones(size(x,1),1);w=rand(1,2);
% inc=5;err=zeros(2.*round(1./inc),2.*round(1./inc));
% for i1=-100:inc:100,for i2=-100:inc:100,
%   err(round(i1./inc)+1./inc+1,round(i2./inc)+1./inc+1)=adaline_eval([i1 i2],x,yt,casew);
% end;end;
% figure(1);clf;contourf(err);colorbar;
% SEE ALSO
%
% gradient_optimize, newton_optimize

% Parameters
% ----------

w = w';
% w is a vertical vector

if nargin<5, errfun=0; end;
wls=1;
if nargin<4,
  wls=0;
  caseweights=1;
end;
if isempty(caseweights),
  wls=0;
end;
if wls==0,
  caseweights=1;
end;
if size(ytarget,2)>1 & size(ytarget,1)>1,
  fprintf('there are %d dimensions in the y variable\n',size(ytarget,2));
  fprintf('multidimensional target variables should be treated \n');
  fprintf('as a series of one dimensional targets, using the same\n');
  fprintf('x variable set for each. The gradients, etc, are all \n');
  fprintf('separate also\n');
  return
end;
if (size(w,2)==size(x,2) & size(w,1)==size(ytarget,2)),
  %disp 'here';
  %w=w';
end; 
if (size(ytarget,2)>1 & size(ytarget,1)==1) | ...
   ...  (size(w,2)>1 & size(w,1)==1) | ...
   (size(caseweights,2)>1 & size(caseweights,1)==1),
  fprintf('ytarget is size %d %d\n',size(ytarget,1),size(ytarget,2));
  fprintf('w is size %d %d\n',size(w,1),size(w,2));
  fprintf('caseweights are size %d %d\n',size(caseweights,1),size(caseweights,2));
  fprintf('x is %d %d\n',size(x,1),size(x,2));
  fprintf('ytarget should be a vertical vector\n');
  fprintf('w should be a vertical vector\n');
  fprintf('x should have observations down, dimensions across\n');
  fprintf('caseweights should be a vertical vector for each x observation\n');
  return
end;

  y = x * w; % evaluate model (linear model)

  if nargin<3, % no target, don't calculate
  else

    actual_minus_target = ((y - ytarget) .* caseweights)*ones(1,size(x,2));

    % y is the model estimate (actual), ytarget is the desired
    % actual-desired is the direction that the result has to move in
    % (eg. pos or negative)


    % THIS IS TEMPORARY. WANT TO INVESTIGATE USING INVERSE SQUARE TO AVOID OUTLIERS    
    if errfun==1, err = sum(abs(actual_minus_target(:,1))); return; end; 
    
    err = mean(actual_minus_target(:,1).^2);   if nargout==1, return; end;
    % THIS USED TO BE sum(actual_minus_target(:,1).^2);
    
    derr = (x.* (ones(size(x,1),1)*w') ) .*  ... % w1x1 term
       2.*actual_minus_target; % actual-target term

    dtotalerr = sum(derr);

    dderr = 2.*((x.^2.*(ones(size(x,1),1)*w'.^2)) + ...
        actual_minus_target.*x);
    ddtotalerr = diag(sum(dderr));
    % creates a diagonal matrix with this on the diagonal

    offdiag = ones(size(x,2),1)*sum(x.*(ones(size(x,1),1)*w'));
    offdiag = offdiag - w*ones(1,size(x,2));
    offdiag = offdiag - diag(diag(offdiag));

    ddtotalerr = ddtotalerr + offdiag;
%    ddtotalerr = ddtotalerr';
    derr = derr';
  end;
  return


    % DERIVATIONS
    %
    % err=(w1x1 + w2x2 + w3x3 - ytarget)^2 + (the rest)^2
    %    = sum(actual_minus_target^2)
    %
    % err = (w1x1 + w2x2 + w3x3 - ytarget)*(w1x1 + w2x2 + w3x3 - ytarget)
    %       + (the other cases)
    % derr/dw1 = u*dv/dw + v*du/dw
    %          = (w1x1 + w2x2 + w3x3 - ytarget) * (w1x1) + the same
    %          = 2.*w1x1 * (w1x1 + w2x2 + w3x3 - ytarget)
    %          = 2 w1x1 * actual_minus_target (+ same for the other cases)
    % dtotalerr/dw1 = sum(derr)
    %
    % derr      = 2 (XW-Y)*(X diag(W))
    % dtotalerr = sum(derr)
    %
    % (the diagonal term of the second deriv)
    % dderr    = d/dw (w1x1 * actual_minus_target)*(w1x1*actual_minus_target)
    %          = u*dv/dw + v*du/dw
    %          =
    %          = (w1x1 + w2x2 + w3x3 - ytarget) * 2x1 + 2w1x1 * w1x1
    %          = (w1x1 + w2x2 + w3x3 - ytarget) * 2x1 + 2*w1^2*x1^2
    %
    % ==================================================================
