function [w,sumerr,yest,alpha]= adaline_grad_train(x,y,w,caseweights,alpha,iterations,percenterr,constraints)
% 
% [w,err,yest,alpha]= adaline_grad_train(x,y,w,caseweights,alpha,iterations,percenterr,constraints)
%
% Uses gradient descent to find weights for w 
% Is able to pass in w from a previous run
% w is a dimx1 vector
% Uses geometric breaking on alpha - will report if alpha needed to decrease
%
% OPTIONS
% alpha - pass in the learning constant. Usually 0.001
% percenterr - eg. 0.001 what the err / firsterr should be for the routine to terminate
% constraints - if 1 then constraint to 0..1
% 
% EXAMPLE
% 
% % Example 1: recover the formula
% x = rand(10000,10);y=x(:,1).*2 + x(:,2).*10 - x(:,3).*4;w=rand(10,1);tic,w = adaline_grad_train(x,y,w,ones(size(x,1),1),0.001),toc,
% w,
%
% % Example 2: yest vs y
% x = rand(10000,2);y=x(:,1).^2;w=rand(2,1);tic,[w,err,yest,alpha] = adaline_grad_train(x,y,w,ones(size(x,1),1),0.001);toc,
% yest = w(1).*x(:,1) + w(2).*x(:,2);figure(1);clf;plot(yest,y,'.')
% w,err,alpha,
%
% % Example 3: colinear dimensions
% x = rand(10000,4);y=x(:,1).*4+x(:,2).*2;x(:,3)=x(:,1)+(rand(size(x,1),1)).*0.7;x(:,4)=x(:,1)+(rand(size(x,1),1)).*0.1;w=rand(4,1);tic,[w,err,yest,alpha] = adaline_grad_train(x,y,w,ones(size(x,1),1),0.001);toc,
% w,
%
% % Example 4: 0-1 constraints
% x = rand(10000,10);y=x(:,1).*0.1 + x(:,2).*0.2 - x(:,3).*0.2;w=rand(10,1);tic,[w,sumerr,yest,alpha] = adaline_grad_train(x,y,w,ones(size(x,1),1),.0001,10000,0.00000001,1);toc,w,




if nargin<3, w = rand(size(x,2),1);           end;
if nargin<4, caseweights = ones(size(x,1),1); end;
if nargin<5, alpha = 0.001;                   end;
if nargin<6, iterations = 1000;               end;
if nargin<7, percenterr = 0.0001;             end;
if nargin<8, constraints=0; end;

method=1;

initial_alpha = alpha;
first_sumerr = inf;
sumerr = inf;
last_sumerr = inf;
last_w = w;

it = 1,

while 1,

  % calc the new err
  yest=x*w;
  err = yest-y;
  sumerr = sum(err.^2);
  if it==1,
    first_sumerr = sumerr;
  end;

  if sumerr > last_sumerr,
    % error increasing - geometrically break
    fprintf('b'); 
    alpha = alpha .* 0.5;
    w = last_w;
  else
    % err is lower - keep the w progress
    alpha = alpha .* 1.01;
    % 1% faster
  end;
  
  if sumerr < percenterr * first_sumerr,
    fprintf('\nTermination due to: sumerr < percenterr * first_sumerr\n');
    break;
  end;

  % remmber prior to updating the weights
  last_sumerr = sum(err.^2);
  last_w = w; % in case we have to go back

  if method==1,
    for dim=1:size(x,2),
      w(dim) = w(dim) + alpha .* -1.* sum((x(:,dim) .* err .* caseweights));
      if constraints,
        if w(dim)<0, w(dim)=0; end;
        if w(dim)>1, w(dim)=1; end;
      end;
    end;
  end;

  if method==2,
    w = w + alpha .* -1 .* sum((x .* (caseweights * ones(1,size(x,2))) .* (err * ones(1,size(x,2)))))';
    % This is actually slower - creation of the ones seems to be the problem..
  end;

  it = it+1;
  if it>iterations, 
    fprintf('\nTermination due to: it > iterations\n');
    break; 
  end;
  
  %fprintf('.');
end;

%fprintf('\n');
fprintf('sumerr %f\n',sumerr);
fprintf('first_sumerr %f\n',first_sumerr);
fprintf('err reduction (xlower) %f\n',1./(sumerr ./ first_sumerr));
fprintf('it %f\n',it);
fprintf('initial alpha %f\n',initial_alpha);
fprintf('alpha %f\n',alpha);
fprintf('alpha reduction (xlower) %f\n',1./(alpha./initial_alpha));
%sumerr < percenterr * first_sumerr,

return;
 

% Error function
% --------------
% 
% E = (yest-y)^2
% E = ( (m1x1 + m2x2 + ... + c) - y) ^2
% E = ( -y * m1x1 - y * m2x2 + ... + -yc 
% 
% yest^2 + -2yest*y + y^2
% 
% (m1x1 + m2x2 + ... + c) ^ 2 + -2 (m1x1 + m2x2 + ... + c) + N*y^2
% 
% = (m1^2x1^2 + m2x2*m1x1 + c*m1x1 + m1x1*m2x2 + m2^2x2^2 + c*m2x2 + m1x1*c+m2x2*c+c^2)
% + -2*(m1x1+m2x2+c) * y + y^2
% 
% = m1^2x1^2 + (2 * m2x2*m1x1) + m2^2x2^2 + (2*c*m1x1) +  2*c*m2x2 + c^2
% + -2*yest*y + y^2
% 
% Derivatives
% ------------
% 
% dE/dm1 = (2m1*x1^2) + 2*m2x2*x1 + 2*c*x1      -2*x1*y
%        = 2 (m1*x1^2 + m2*x2*x1 + c*x1)        -2*x1*y
%        = 2x1 (m1*x1 + m2*x2 + c)              -2*x1*y
%        = 2x1 (yest)                           -2*x1*y
%        = 2 * x1 yest - 2 * x1 * y
%        = 2 * (x1 (yest - y))
%        = 2 * x1 * err
% 
% Update rule
% -----------
% 
% m1 = m1 + alpha * - dE/dm1
% m1 = m1 + alpha * -1 * (x1 * err)
% 
% Note that instead of m1 we could have written mi.... The formula is the same...., eg. 
% mi = mi + alpha * -1 * (xi * err)

