function [node,edge,similarity,head]=agglomerative_cluster(dist,method,no_merges);
%
% [node,edge,similarity,head]=agglomerative_cluster(dist,method,no_merges);
% 
% EXAMPLE: % Agglomerative cluster on random distance matrix
% sz = 5;method=5;merges=[];
% dist=round(rand(sz,sz).*sz);dist = triu(dist)+triu(dist)';
% dist=dist-diag(diag(dist));dist,
% [node,edge,similarity] = agglomerative_cluster(dist,method,merges);
% graph_unordered_btree_wrapper(head(1),node,[edge similarity],1,1,0.1,4);
%
% OUTPUTS: node,edge  list of nodes and edges
%  similarity similarity rating for each edge
%  head       the nodeid in node which is the top of the tree. There could
%   be several heads if the agglomeration procedure was not
%   allowed to merge all the centroids (eg. due to the time it
%   takes)
%
% INPUTS: dist is a similarity matrix. It should have 0s on the diagonal.
%  The higher the value, the more the chance that a merge will occur.
%  no_merges are the number of merges - or the number of new centroids 
%  which will be created. If this is specified as [] then the procedure
%  will merge as many as possible.
%  method is AVERAGE_LINKAGE_BETWEEN = 1; AVERAGE_LINKAGE_WITHIN  = 2;
%  SINGLE_LINKAGE = 3; COMPLETE_LINKAGE = 4; CENTROID   = 5; 
%  MEDIAN = 6; AVERAGE = 7; Methods 1 and 2 are slower
%
% [node,edge,similarity,head]=agglomerative_cluster(dist,method,no_merges);

% REFERENCE
% Anderberg, M. (1973), Cluster analysis for applications, Academic Press, New York.

% % Implement W matrix
% compmat = 0.5.*(w+w');
% compmat = compmat-diag(diag(compmat)); % the on-diagonal term isn't counted
% size(find(compmat>0)), % number of merges which will occur
% [node,edge]=agglomerative_cluster(compmat)
% Will stop when all the positive elements have been exhausted
%

% Parameters
% ----------

% method not supplied
if nargin<2, method=7; end;
if isempty(method) | method<1 | method>7, method=7; end;

% no_merges not supplied
if nargin<3, no_merges = size(dist,1)-1;end;
if isempty(no_merges) | no_merges<0,no_merges = size(dist,1)-1;end;

AVERAGE_LINKAGE_BETWEEN = 1;
AVERAGE_LINKAGE_WITHIN  = 2;
SINGLE_LINKAGE          = 3;
COMPLETE_LINKAGE        = 4;
CENTROID                = 5;
MEDIAN                  = 6;
AVERAGE                 = 7;

if method==AVERAGE_LINKAGE_BETWEEN,str='Average linkage between';end;
if method==AVERAGE_LINKAGE_WITHIN,str='Average linkage within';end;
if method==SINGLE_LINKAGE,str='Single linkage (max method)';end;
if method==COMPLETE_LINKAGE,str='Complete linkage (min method';end;
if method==CENTROID,str='Centroid';end;
if method==MEDIAN,str='Median';end;
if method==AVERAGE,str='Average';end;

fprintf('Agglomerative_cluster\n');
fprintf('Method: %s\n',str);
fprintf('%d mergers will occur\n',no_merges);
fprintf('--------------------------------------\n\n');

% Initialize
% ----------

clusterid = [1:size(dist,1)]'; 
  % cluster ids are 1..n
  % this is the id of the cluster which is at position i down in the distance matrix
  % When we start adding new clusters, we re-use the old position. Therefore, we could
  % be at position 4 down, and have a clusterid of 8.
node = clusterid; 
  % this is a complete list of nodes which have been created
  % we could ignore this and just use a maxcluster counter
edge = []; 
  % captures the parent,lchild, parent, rchild edges as they are created
remaining = size(dist,1); 
  % this decreases by 1 each turn. Its the same as a
  % counter from 1 to n. There will be exactly n-1 merges
dist=sparse(dist); 
  % needed so that when we start replacing numbers with 0
  % we get a space advantage
n = ones(size(dist,1),1); 
distsum = zeros(size(dist,1),1);
  % n starts off as 1. Each n is for a different cluster
  
onesmat = ones(size(dist,1),1);

head = zeros(size(dist,1),1);
 % head is a vector of 1s or 0s, a 1 means this is a head of the
 % the tree, and a 0 means it is not a head
 % This is a bit strange, we're saying none of the current centroids
 % are heads. When we add a new centroid, we will say that it becomes
 % a legitimate head, then. This prevents a large number of centroids
 % from being in the head list which never actually get incorporated
 % into the agglomerative hierarchy (eg. they had distances <=0)


% Main
% ----

for i=1:no_merges,
  % while remaining>1,
   
   
  % Find largest element
  % --------------------  
  
  if method~=AVERAGE_LINKAGE_WITHIN & method~=AVERAGE_LINKAGE_BETWEEN,
    [m,down,across] = maxval(dist);
  end;
  if method==AVERAGE_LINKAGE_BETWEEN,
     ncolmat = n*onesmat';
     nrowmat = onesmat*n';
     [m,down,across] = maxval(dist./(ncolmat.*nrowmat)); 
  end;
  if method==AVERAGE_LINKAGE_WITHIN,
     ncolmat = n*onesmat';
     nrowmat = onesmat*n';
     [m,down,across] = maxval( (distsum*onesmat' + onesmat*distsum' + dist) ./ ...
     ( (ncolmat + nrowmat) .* (ncolmat +nrowmat-1) .*0.5));
  end;   
  if m==0, disp('0 dist encountered');break; end;
  
  
  % Data about how many cases or distances are assigned to each centroid
  % ---------------------------------------------------------------------
  
  n(across)       = n(across)+n(down);
  distsum(across) = distsum(across)+distsum(down)+dist(down,across);
  
  
  % Merge distances for the new centroid
  % -------------------------------------
  
  if method==AVERAGE_LINKAGE_BETWEEN,newdist = dist(down,:)+dist(across,:);end;
  if method==AVERAGE_LINKAGE_WITHIN,newdist = dist(down,:)+dist(across,:);end;
  if method==SINGLE_LINKAGE,newdist = max([dist(down,:);dist(across,:)]);end;
  if method==COMPLETE_LINKAGE,newdist = min([dist(down,:);dist(across,:)]);end;
  if method==MEDIAN,newdist = 0.5.*(dist(down,:)+dist(across,:))-(dist(down,across).*0.25);end;
  if method==AVERAGE,newdist = 0.5.*(dist(down,:)+dist(across,:));end;
  if method==CENTROID,
     newdist = n(across)./(n(across)+n(down)) .* dist(across,:) + ...
               n(down)  ./(n(down)+n(across)) .* dist(down,:)   - ...
               ((n(across).*n(down)) ./ ((n(across)+n(down)).^2) ) .* ...
               (dist(down,across) * ones(1,size(dist,2)));end;
  
  
  % Add the new cluster by overwriting across
  % -----------------------------------------
  
  dist(across,:) = newdist; % adds a row to the bottom
  dist(:,across) = newdist'; % adds a column to the right side
  
  % Zero out the diagonal of this new centroid. The reason is, this
  % diagonal will say the distance between distance pp and distance pq
  % ie. this is half way between 0 and distance from p to q. But now
  % p and q are gone and are no longer allowed to bind with any other
  % centroid. Therefore, we ignore this value completely, and put in the
  % distance between newnew which is 0.
  % The other entries, dist(new,x), dist(new,y) are all valid, because those
  % centroids still exist
  
  dist(across,across)=0;
  % Note the strong requirement that 0 is the null value
  
  
  % Remove the down centroid
  % ------------------------
  
  dist(down,:) = zeros(1,size(dist,2));
  dist(:,down) = zeros(size(dist,2),1);


  % Record the parent,child merger
  % ------------------------------
  
  % Create newclust id
  newclust = max(clusterid)+1; % the new cluster's name

  % Update head information
  head(clusterid(across),1)=0; % clusterid(across) is no longer a head 
  head(newclust,1)=1;          % the new centroid (which happens to be at across's pos) is a head
  head(clusterid(down),1)=0;   % the centroid which was at the down pos is wiped out completely

  % Record new parent newclust with edges to clusterid(across) clusterid(down)
  edge(size(edge,1)+1,1:2) = [newclust clusterid(across)];
  edge(size(edge,1)+1,1:2) = [newclust clusterid(down)];
  similarity(size(edge,1),1)   = m;
  similarity(size(edge,1)-1,1) = m;
  node(size(node,1)+1,1) = newclust; % add it to the nodelist

  % Assign the across position of the matrix to the new cluster
  clusterid(across) = newclust; % the name of across is now clusterid

  fprintf('.');
end;
fprintf('\n');

dist1=dist;
node1=node;
edge1=edge;

head = find(head~=0);
% these numbers are the centroid ids which are heads

% Now, of these, find heads which interface with the 
% node list, ie. heads which were actually incorporated into the
% tree. There could be a lot of heads which 


% --------------------------------------------------------------------------
if 0,
% This method grows the distance matrix
% -------------------------------------

% This method is much more straightforward, however, grows the matrix
% with each new centroid. Since there are N mergers, and the matrix grows
% a matrix starting with N square elements would eventually have 2N elements
% down, or 4N^2 total

while remaining>1,

  % Find largest element
  [m,down] = max(dist); % gives the down values of the maximum in each column
  [m,across] = max(m); % gives the across value of the maximum of those column maximums
  down=down(across); % the down value to use is the one at across across
  % largest is down,across

  if m==0, disp('0 distance is max');break; end;
 
  newdist = 0.5.*(dist(down,:)+dist(across,:));
  newclust = max(clusterid)+1;
  dist(newclust,1:newclust-1) = newdist; % adds a row to the bottom
  dist(1:newclust-1,newclust) = newdist'; % adds a column to the right side
  dist(newclust,newclust) = 0; % on the diagonal its zero

  % Remove the two centroids
  dist(down,:) = zeros(1,newclust);
  dist(:,down) = zeros(newclust,1);
  dist(across,:) = zeros(1,newclust);
  dist(:,across) = zeros(newclust,1);
  
  remaining=remaining-1;

  edge(size(edge,1)+1,1:2) = [newclust clusterid(across)];
  edge(size(edge,1)+1,1:2) = [newclust clusterid(down)];
  node(size(node,1)+1,1) = newclust;
  similarity(size(edge,1),1) = m;
  similarity(size(edge,1)-1,1) = m;
  %clusterid(across) = newclust;
  clusterid(newclust) = newclust;
end;
end;


% ======================================================================
% NOTES
%

% Another way - 
% Remove items with only their diagonal. They don't interact with anyone! (hence no comp)
% Remove items with only negative interactions. These items don't compete with anyone
% Remove all improperly formed models
% The remaining items have a negative cross term hurting someone.
% [a,b] = find((w-diag(diag(w)))>0); % find els with a positive crossterm somewhere
% unique(b) % these are models who eat another item if it increases its price (positive term)
% 

% Implement single linkage


