function [xpattern,ydist,support,estdist,yest] = bayesian_train(x,y,missing_val,valtree,cattree);
% [xpattern,ydist,support,yestdist,yest] = bayesian_train(x,y,missing_val,valtree,cattree);
%
% Calculates Pr( yval | xpattern ) where xpattern is a list of conjuncts 
% x(i,1)=val ^ x(i,2)=val ^ ... and yval is one element of the y distribution observed
% when x is equal to xpattern.
%
% This exactly implements a bayesian reasoning system, which looks at all possible
% interactions and combinations of x values, to predict the maximum likihood y value
% (the y value for which its probability is the highest). For each xpattern, a complete
% y distribution is calculated.
%
% This is in contrast to naive bayes which assumes independence between variables,
% and then observed probabilities of Pr(y|x1) ^ Pr(y|x2) to multiply together a likelihood
% for each y value.
% 
% WEAKNESSES
% This method may be unsuitable for predicting y for the following reasons.
% 1) Susceptibility to noise. If one value, eg. x(i,1)=1 is only present once,
% it receives its own special entry for the complete y distribution. This case has the
% same "weight" as another case which is built up from thousands of observations.
% You'd expect error on the prediction made from one case. 
% (we can implement confidence metric on predictions)
% 2) No generalization - zeros on unobserved new cases: If some xpattern 
% (combination of x values) is not observed in the training data, it receives 
% a zero probability distribution. 
% 3) Poor generalization continued: Seeing every possibility is unlikely 
% because the space of xpatterns grows exponentially in the number of values. 
% no_patterns = values^no_dims. Because of above, if an xpattern is ommitted from training,
% there is no value for it when executing. This is in contrast to nearest neighbour
% methods.
% 
% TRAINING SPEED
% Interestingly the method is fast to train < Number of training examples
%
% LIMITATIONS
% missing_val not supported
%
% Similar to nearest neighbour, or a case-based method. Calculates the average distribution
% when the exact same group of input features are shown. No distance metric is applied
% to x values, so exact matches only are supported.
%
% SEE ALSO
% nb_train(x,y)
% nb_eval(x,y)
% entropic_significance(x,y)
% [xpattern,ydist,support,yestdist,yest] = bayesian_train(x,y,missing_val,valtree,cattree);

if nargin<3,
   missing_val=nan;
end;
if nargin<5,
  %fprintf('constructing cattree\n');
  [cattree,no_catvals] = construct_valtree(y,missing_val);
else
   no_catvals = size(cattree,1)-1;
end;
if nargin<4,
  %fprintf('constructing valtree\n');
  [valtree,no_vals,valtree2,xrecoded] = construct_valtree(x,missing_val);
  for d=1:size(valtree,2),
     if isnan(missing_val),
        no_vals(d,1)=min(find(isnan(valtree(:,d))))-1;
     else
        no_vals(d,1)=min(find(valtree(:,d)==missing_val))-1;
     end;
  end;
end;


   % Get y given everything
   remaining=[1:size(x,1)]';
   
   ydist = zeros(size(x,1),no_catvals); 
   % this is the worst case size down, a different pattern for each x row
   
   xpattern = zeros(size(x));
   % this is the worst case size, each xpattern is different, so one for each row
   
   support = zeros(size(x,1),1);
   % worst case again
   
   no_patterns=0; 
   % we currently have no patterns. This will increment as we 
   % look at each new pattern
   
   estdist = zeros(size(x,1),no_catvals);
   yest    = zeros(size(x,1),1);
   
   while ~isempty(remaining),
      no_patterns = no_patterns+1;
      % take head
      % find all cases identical to the head
      xpattern(no_patterns,1:size(x,2))=x(remaining(1),:);
      i=strmatch(xpattern(no_patterns,:),x(remaining,:));
      
      % calc prob of each class, conditional upon seeing this particular set of feature vals
      for yval=1:no_catvals,
         ydist(no_patterns,yval) = size(find( ...
            y(remaining(i),1)==cattree(yval,1)),1) ./ size(i,1);
      end;
      support(no_patterns,1)=size(i,1);
      
      % Set this distribution as the prediction for all xpattern records which 
      % are in the set
      
      %keyboard
      %for ii=i,
      %   estdist(ii,1:no_catvals) = ydist(no_patterns,1:no_catvals);
      %end;
      estdist(remaining(i),1:no_catvals) = ones(size(i,1),1)*ydist(no_patterns,1:no_catvals);
      [a,est] = max(ydist(no_patterns,1:no_catvals)');
      yest(remaining(i),1) = ones(size(i,1),1)*cattree(est,1);
      
      % remove all those cases we just looked at, and continue to find the next xpattern
      remaining=remove_ids(remaining,i);
      fprintf('%d/%d ',size(remaining,1),size(x,1));
   end;
   
   fprintf('\n%d (of %d possible) unique xpatterns found\n',no_patterns,size(x,1));
   
   xpattern = xpattern(1:no_patterns,:);
   ydist    = ydist(   1:no_patterns,:);
   