function [c,centroidlist,cover,cy,level,member] = ...
      binary_cluster(x, number_of_clusters, lc, iterations, y);
%
% [c,centroidlist,cover,cy,level,member] = binary_cluster(x,number_of_clusters,lc,iterations,y);
%
% Performs binary clustering on x. Binary clustering is a top-down clustering
% method which begins with the mean of the data. Two children are then split 
% off. These children compete to cluster the data. After they have settled, 
% they are frozen and themselves become parents. The children are restricted to
% only compete for input examples which their parents already cover. Thus,
% the complexity of the algorithm doesn't grow too much as you go down.
%
% INPUTS
%
% number_of_clusters is the number of clusters, total, in the tree. The
% actual number of clusters which are at the lowest levels (terminals) 
% and used for classification can be up to half this amount
%
% other parameters same as used in kmeans_batch
%
% OUTPUTS 
% 
% c is a full tree matrix containing all clusters and agglomerative
% tree nodes. The children of row is equal to row.*2 and row.*2+1
% c is sparse, so that even though row appears logically in 
% position row, the zeros caused by a deep tree, for example, don't
% consume any memory
%
% centroidlist is a list of rowIDs for the legal centroids. 
% c(centroidlist,:) lists all centroids
%
% cover is a list which gives the number of training cases captured
% by each centroid. cover(centroidlist,:) lists only the centroids
% which are active
%
% cy the y centroid, also sparse
%
% level is the centroid leaves at each level in the tree
% for example, level(1,:) lists the terminals at level 1 (centroid1)
% level(4,:) lists the terminals at level 4. level(size(level,1),:)
% lists the leaves at the lowest level on the tree (probably used by 
% default for classification tasks)
%
% EXAMPLE
%
% x=rand(20,2);y=x.^2;number_of_clusters=12;lc=0.3;iterations=10;
% missing_val=-9;terminal_layer=5;
% 
% 
% 
% SEE ALSO
%
% yest = binary_eval(x,c,centroidlist,cy);
%
% [c,centroidlist,cover,cy,level,member] = ...
% binary_cluster(x,number_of_clusters,lc,iterations,y);

% TODO:
% check this works
% fix cluster graph to give automatic separation scaling
% labels on cluster graph based on variance of clusters dims
% change name
%
% dtree

% Initialize c
% ------------

c(1,1:size(x,2))  = mean(x); cy(1,1:size(y,2)) = mean(y);
% centroid list

member = ones(size(x,1),1);
% initialize member to 1
% member states which parent each case in x belongs to
% This prevents some centroids from looking at those cases

cover(1,1) = [size(x,1)]; 
% the number of cases which centroid 1 covers (all of them)

terminal(1,1) = [1]; 
% list of centroid ids which are terminals

centroidlist(1,1) = [1];
% list of valid centroidIDs

% Convert to sparse
% -----------------

c=sparse(c);
cy=sparse(cy);
cover=sparse(cover);

% The problem. Since it is a binary split, if I go deep,
% I can get very far out (doubles each time). Thus 1024, millions for
% only a 10 level deep tree (with very few clusters).
% Thus, I will have to create a lookup which
% realpos = translates a down location into a physical centroid location
% This is solved if all are sparse arrays!!!!!!!
% No memory loss, no logic hassles!!!!


while size(centroidlist,1)<=number_of_clusters,   
   
  % Determine which parent to split
  % --------------------------------
  
  % a pregnant parent is a terminal centroid
  % read through the parents, find out which
  
  % find the terminal with the largest number of cases owned, and split it
  [a,id]=max(cover(terminal)); % terminal is a set of centroid ids
  % id means that at id down there was a max. This was a terminal list, though,
  % so terminal(id,1) is 
  parent = terminal(id,1); % parent is the terminal listed for 
  lchild = parent.*2;
  rchild = parent.*2+1;
  fprintf('splitting %d\n',parent);  
  
  % Split off two new nodes (set them equal to parent), call kmeans
  % ----------------------------------------------------------------
  
  changeable = find(member==parent); 
  % find x rows which are under control of this parent

  % the new cinit should be the mean of the centroids which 
  % are covered by that membership function
 
  % find mean of the covered centroids
  cluster_mean = mean(x(changeable,:));
  %cinit = full([cluster_mean-eps;cluster_mean+eps]); 
  % this should have worked
  
  if 0,
    cinit = full([cluster_mean+(rand(1,size(x,2))-0.5).*0.000001; ...
                  cluster_mean+(rand(1,size(x,2))-0.5).*0.000001]);
                  % this worked pretty well
  else
    % generate covariance matrix (could take a long time!)
    cov = corrcoef(x(changeable,:));
    iii=find(isnan(cov) | isinf(cov));cov(iii) = zeros(size(iii,1),1);
    % find the first principal component through the remaining data cloud
    [u,s,v] = svds(cov,1); % only want the first principal component
    cinit = full([cluster_mean;cluster_mean] + 0.0000001 .* [v(:,1) -1.*v(:,1)]'); % puts v across and -v across underneath it
  end;
  % use pca to find the principal axis through the remaining data cluster,
  % then assign cluster1 to positive on axis, and cluster2 to negative on 
  % axis. This should result in a good split. svds is quite slow, however,
  % since we are only taking one principal component, hopefully it will be reasonably
  % fast
  
  
  % another attempt
  %[temp,startid] = k_nearest_neighbour(cluster_mean,x(changeable,:),y(changeable,:),2);
  %cinit = [x(startid(1),:);x(startid(2),:)];
  % problem with this one is that if one of the kmeans runs returns and 
  % the child centroid hasn't moved further out than the training
  % case of the parent, then no change will occur. That child will then
  % re-initialize, and the alg gets stuck. By using the actual position
  % the centroid ended up in last time (the mean), errors will occur which
  % can eventually move that centroid out so that it breaks up more
  % cases.
  
  % cinit = full([c(parent,:);c(parent,:)+eps]); 
  % initialize child centroids equal to the parent

  %[cnew,cx,cynew,mem] = kmeans_batch2(x(changeable,:),2,lc,iterations, ... 
  % y(changeable,:),cinit);
  % used to be this (kmeans_batch2
  [cnew,cx,cynew,mem] = kmeans_mean(x(changeable,:),2,iterations,y(changeable,:),cinit);
  
  %[cnew,cx,cynew,mem] = macqueen(x(changeable,:),y(changeable,:));
  
  % retrieve centroids back into c
  c(lchild,:)  = cnew(1,:); 
  c(rchild,:)  = cnew(2,:); 
  cy(lchild,:) = cynew(1,:);
  cy(rchild,:) = cynew(2,:);
  
  
  % Update membership, terminal status, no_of_inputs covered
  % ---------------------------------------------------------
  
  % change mem to lchild, rchild (mem will default to 1 or 2)
  i=find(mem==1);j=find(mem==2);
  mem(i,1)=ones(size(i,1),1).*lchild;
  mem(j,1)=ones(size(j,1),1).*rchild;
  member(changeable) = mem;
  
  % record the number of input examples belonging to each centroid
  % (they are now terminals, or parents)
  terminal(id,1) = lchild; % this was the id of the parent
  % replace it with the child. Thus, parent is off the terminal list,
  % and one of its children is on the terminal list
  terminal(size(terminal,1)+1,1) = rchild; % put the other child on the terminal list
  cover(lchild) = size(i,1); 
  cover(rchild) = size(j,1);
  centroidlist = [centroidlist;lchild;rchild;];

 end;


% Extract levels
% ---------------

% levels are constructed by going bottom-up
% level_in_tree = floor(log2(nodeID));
% terminal is our starting point
% we have a terminal list
% find the largest terminal. Get its level.
% Now, if up the level of all other nodes equal to this max level.
% but if another node is already at a higher level, don't change it
% after recording level, 
% if node is equal to lowest level
% find the parents of all the terminal node 

max_terminal = max(terminal);
lev          = floor(log2(max_terminal));

while lev>0,

  % Record all terminals
  % --------------------

  level(lev+1,1:size(terminal,1)) = terminal';
  
  % Move up lowest level terminals
  % ----------------------------------

  % find all nodes which are on the lowest level. These
  % must move up the tree for next iteration
  lowest = find(floor(log2(terminal))==lev);
  other  = find(floor(log2(terminal))~=lev);
  
  % find the lowest level terminals' parents
  parent = floor(terminal(lowest,:)./2);
  
  % initialize terminals list with others
  terminal = terminal(other,:);

  % add the parents to the terminals list
  terminal = [terminal;unique(parent)];
  
  % lev should be equal to lev-1
  max_terminal    = max(terminal);
  lev             = floor(log2(max_terminal)),
end;

level(1,1) = 1;

