function [ysignif,olssignif,logsignif] = categorical_significance(x,y,missing_val,valtree,cattree,nodestr,dimvalstr);
%
% y is continuous, all xs must be discrete
%
% if y is multi-dimensional, then we assume that we will be inducing a 
% demand curve with y1,y2,...,yn-1 on the x-axis and yn on the vertical 
% axis. The significance tests are then for the curve. This is peculiar, but 
% the application is for finding segmentation variables that significantly 
% change a demand curve. constant changes in the demand curve would not have 
% any impact on profit, and therefore slope changes are the changes that are 
% viewed as significant. 

if nargin<7, dimvalstr = {}; end;

% Parameters
% -----------

y_is_binomial = 1;
no_to_show    = 30;      % number of significant variable-values to show
conf          = 0.75;    % upper and lower confidence bounds
minobs        = 10;      % if a segment has fewer than this number of observations, it is ignored
increment     = 50;      % number of points to draw in graphs - fewer = faster graphs
draw          = 0;       % 1==draw curves
fit_measure   = 2;       % the index of the accuracy measure taken from maccuracy. See help 
                         % maccuracy for a complete list. If maccuracy hasn't changed, 1 might be biserial, 
                         % 2 correlation, 3 percent correct, and so on
use_sample_correction=1; % for extremely large datasets, forces the most number of 
                         % samples from a dataset to be about 800, which is the 
                         % most samples that the distribution can take, and still 
                         % return a number above zero (0) for the case of all-heads.
                         % The rationale here, is that the binomial test is primarily
                         % used for sorting the results, to tell me which is the 
                         % most unusually good, based on the number of trials. 
                         % For sample sizes greater than 800, the binomial test will 
                         % start returning 0s for everything, and when sorted, there 
                         % will be no order. For very large datasets, this ends up 
                         % giving random resuts. Application of this parameter is the 
                         % same as taking a sample to get it down to about 800 samples, 
                         % although its more accurate - because the real mean computed 
                         % for 100,000 samples (say) is being propagated to the lower 
                         % 800 sample for purposes of computing the binomial probability
                         
max_samples = 800; % This is fixed, on some computers it should be dropped 
 
warning off;

% Input arguments
% ---------------

if size(y,1)<2, fprintf('\ny should be two dimensional\n\n');end;

keyboard

if nargin>3, % valtree provided
  for i=1:size(valtree,2),
    no_vals(i) = min(find(valtree(:,i)==missing_val))-1;
  end;
end;

if nargin<3,
  [valtree,no_vals,valtree2,xrecoded]=construct_valtree(x,missing_val);
end;
if nargin<4,
  [cattree,no_cats,cattree2,yrecoded]=construct_valtree(y,missing_val);
end;

ymultdim  = [1:size(y,2)-1];
ydim      = size(y,2);

if use_sample_correction, 
  if size(y,1) > max_samples,
    % only need to do this if we're above the numerical precision 
    % of the machine
    sample_correction = max_samples ./ size(y,1)
    % this is the scale-down factor that will force us to be using 
    % a precision in which we actually get a non-zero probability
  else 
    sample_correction = 1;
  end;
else 
  sample_correction = 1;
end;


% Create overall expectation
% --------------------------

yymean            = mean(y(:,ydim));
yymedian          = median(y(:,ydim));
[yylower,yyupper] = errorbars(y(:,ydim),conf);

ymin = min(y(:,ydim));
ymax = max(y(:,ydim));

if size(y,2)>1,

  % Fit global models
  [yyw,yest_yyw] = polyfitf(y(:,ymultdim),y(:,ydim),1);
  b = glmfit(y(:,ymultdim),[y(:,ydim) ones(size(y,1),1)],'binomial');      
  yyb(1) = b(1);yyb(2,1) = b(2);
  yest_yyb = glmval(yyb,y(:,ymultdim),'logit');

  % Goodness of fit at the global level  
  [yy_accuracy_ols,fit_string] = maccuracy(y(:,ydim),yest_yyb,1);
  yy_accuracy_logistic = maccuracy(y(:,ydim),yest_yyw,1);

  % Graphical plotting ranges  
  minr = min(y(:,ymultdim(1)));
  maxr = max(y(:,ymultdim(1)));
  maxy = max(y(:,ydim));
  miny = min(y(:,ydim));
  r = [minr:(maxr-minr)./increment:maxr]';
  br = glmval(b,r,'logit');
  wr = polyff(yyw,1,r);
end;



% Initialize a full matrix for speed
% ----------------------------------

ymean                 = zeros(size(x,2),100);
ybinomialp            = zeros(size(x,2),100);
ychip                 = zeros(size(x,2),100);
ymedian               = zeros(size(x,2),100);
ylower                = zeros(size(x,2),100);
yupper                = zeros(size(x,2),100);
yw                    = zeros(size(x,2),100,2);
yb                    = zeros(size(x,2),100,2);
wdiff                 = zeros(size(x,2),100);
bdiff                 = zeros(size(x,2),100);
obs                   = zeros(size(x,2),100);
ymeandiff             = zeros(size(x,2),100);
ymediandiff           = zeros(size(x,2),100);
accuracy_ols          = zeros(size(x,2),100);
accuracy_logistic     = zeros(size(x,2),100);
wfitdiff              = zeros(size(x,2),100);
bfitdiff              = zeros(size(x,2),100);
wfitchip              = zeros(size(x,2),100); % one-sided chi test, useful because factors in no of observations
bfitchip              = zeros(size(x,2),100); % one-sided chi test, useful because factors in no of observations

for dim=1:size(x,2),
  remaining = [1:size(x,1)]';
  for xval = 1:no_vals(dim),
    i = find(x(remaining,dim)==valtree(xval,dim));
    
    obs(dim,xval) = length(i);
    if obs<minobs, continue; end;

    % Summary statistics
    ymean(dim,xval)                     = mean(y(remaining(i),ydim));
    ymedian(dim,xval)                   = median(y(remaining(i),ydim));
    [ylower(dim,xval),yupper(dim,xval)] = errorbars(y(remaining(i),ydim),conf);    
    [a,ychip(dim,xval)]                 = chisquare_test(ymean(dim,xval),yymean,2);
    ychip(dim,xval)=1-ychip(dim,xval);

    if y_is_binomial,
      ybinomialp(dim,xval)                = binopdf(ceil(sum(y(remaining(i),ydim) & 1).*sample_correction),ceil(length(i).*sample_correction),yymean);
      % the ceil ensures that if it accidentally scales a 1 down to a 0, then 
      % we still get a valid test result
    end;
    % need to sort from lowest to highest, damnit!
    
    ymeandiff(dim,xval)  = abs(log(ymean(dim,xval)./yymean));
    ymediandiff(dim,xval) = ymedian(dim,xval) ./ yymedian;

    % Induce demand curve
    if size(y,2)>1,
      if length(i)<2, 
        b = [0;0];
        w = [0;0];
      else 
        xx = y(remaining(i),ymultdim);
        yy = y(remaining(i),ydim);
        w = polyfitf(xx,yy,1);if w(2)>0,w(2)=0;end;
        b = glmfit(xx,[yy ones(size(yy))],'binomial');if b(2)>0, b(2)=0;end;
        
        % Goodness of fit
        yest = polyff(w,1,xx);
        m = maccuracy(yy,yest);        
        accuracy_ols(dim,xval) = m(fit_measure);
        wfitchip = chi2cdf(accuracy_ols(dim,xval),yy_accuracy_ols); % 1-tailed, observed is higher than expected
        
        yest = glmval(b,xx,'logit');
        m = maccuracy(yy,yest,1);
        accuracy_logistic(dim,xval) = m(fit_measure);
        bfitchip = chisquare_test(accuracy_logistic(dim,xval),yy_accuracy_logistic,1); % 1-tail, observed is higher than expected
          
        wfitdiff(dim,xval) = log(accuracy_ols(dim,xval)./yy_accuracy_ols(fit_measure));
        bfitdiff(dim,xval) = log(accuracy_logistic(dim,xval)./yy_accuracy_logistic(fit_measure));
        % we will be sorting these from highest to lowest - we want to find 
        % segments where the fit is much higher than expected performance
        % in other words, the fit is extraordinarily good
                
          wwr = polyff(w,1,r);
          bbr = glmval(b,r,'logit');

       if draw,      
          figure(1);clf;
          plot(r,bbr,'r-');title('Logistic fit');hold on;
          plot(r,br,'b-');
          figure(2);clf;
          plot(r,wr,'b-');title('OLS fit');hold on;
          plot(r,wwr,'r-');
        end;
              
        yw(dim,xval,1:2) = w(1:2);
        yb(dim,xval,1:2) = b(1:2);
        wdiff(dim,xval) = [abs(log(abs(w(2)./yyw(2))))]; %   chisquare_test(w(2),yyw(2))      ];
        bdiff(dim,xval) = [-correlation(bbr,br)]; % [distance(b',yyb') chisquare_test(b,yyb) ];
      
        if draw,
          figure(3);clf;bar([wdiff(dim,xval,1);bdiff(dim,xval,1)]);
          v=axis;axis([v(1) v(2) -1 1]);
          drawnow;
        end;
      end;
    end;
 
    if draw,
      figure(4);clf;
      bar([ymean(dim,xval)-yymean]);hold on;title('Observed difference');
      v=axis;axis([v(1) v(2) -(ymax-ymin) (ymax-ymin)]);
    end;
%    bar([yupper(dim,xval) ymean(dim,xval) ymedian(dim,xval) ylower(dim,xval); ...
%        yyupper yymean yymedian yylower]);hold on;

    remaining = remove_ids(remaining,i);
  end;
  fprintf('.');
end;


% Look for significantly different values
% --------------------------------------------

% we can rate by overall significance (some dims may be more signif than others, 
% so may dominate the list of significant values), 
% or we can rate by significance per variable.
 
% Zero out anything below minobs
if 0,
i = find(obs<minobs);
if ~isempty(i),
ybinomialp(i)    = zeros(size(i));
ymean(i)         = zeros(size(i));  
ymedian(i)       = zeros(size(i));
ylower(i)        = zeros(size(i));
yupper(i)        = zeros(size(i));
yw(i)            = zeros(size(i));
yb(i)            = zeros(size(i));
wdiff(i)         = zeros(size(i));
bdiff(i)         = zeros(size(i));
ymeandiff(i)     = zeros(size(i));
ymediandiff(i)   = zeros(size(i));
end;
end;

ybinomialp = sparse(ybinomialp);
ymean = sparse(ymean);
ymedian = sparse(ymedian);
ylower = sparse(ylower);
yupper = sparse(yupper);
ymeandiff = sparse(ymeandiff);
ymediandiff = sparse(ymediandiff);
wdiff = sparse(wdiff);
bdiff = sparse(bdiff);
%yw = sparse(yw);
%yb = sparse(yb);


  % Top variables that change ymean
  % -------------------------------
    
  sortvariable = ybinomialp; % -1.*ymeandiff;
  present = find(ymeandiff~=0 & obs>=minobs & ~isnan(sortvariable) & ~isinf(sortvariable)); 
  [a,i] = sort(sortvariable(present));  

  fprintf('\n');
  fprintf('Top variable-values, such that when conditioned, significantly change the mean of y\n');
  fprintf('-----------------------------------------------------------------------------------\n');
  fprintf('Observed value of y compared to expected value by means of chi-square test\n');
  fprintf('\n');
  fprintf('Expected value for y:\n');
  fprintf('E[y] = %f, obs = %d\n',yymean,size(y,1));
  fprintf('\n');
  %fprintf('Most significant segments\n');
  %fprintf('E[y|xdim = value]\tchisquare\tp\tobs\n');

 % %%%% Select the metric  
  for ii=1:no_to_show,
    [dim,xval] = IND2SUB(size(ymean),present(i(ii)));
    [chi,p]   = chisquare_test(ymean(present(i(ii))),yymean,2);
  %  if y_is_binomial,
  %    binomialp = binocdf(ymean(present(i(ii))).*obs(present(i(ii))), ... % number of successes
  %                      obs(present(i(ii))), ... % number of trials
  %                      yymean); % default fairness of the coinflip (42% chance of a success)                      

  % THIS IS GOOD
  if 0,
  fprintf('E[y | %s = %d ] = %f, binomialp = %f, chisquare=%f, p=%f, obs=%d\n',...
      deblank(nodestr(dim,:)),full(valtree(xval,dim)), ...
      full(ymean(present(i(ii)))),full(ybinomialp(present(i(ii)))), full(chi),full(p),full(obs(present(i(ii)))));
  else
    % When we do this next time, after discretizing, load 
    % dimvalstr with those encodings. Then we can recover them directly
%    if dim==1, 
%      valuestr = num2str(full(valtree(xval,dim))); 
%    else
      numericval = (full(valtree(xval,dim)));
      if numericval<=0 | floor(numericval)~=numericval,
        valuestr = num2str(numericval);
      else
        valuestr = dimvalstr{numericval,dim};
      end;
%    end;
    fprintf('%s\t%s\t%f\t%f\t%f\t%f\t%d\n',...
        deblank(nodestr(dim,:)),valuestr, ...
        full(ymean(present(i(ii)))),full(ybinomialp(present(i(ii)))), full(chi),full(p),full(obs(present(i(ii)))));
  end;
end;

 
  if size(y,2)>1, 


  % Top variables for OLS fit
  % -------------------------

    fprintf('\n');
    fprintf('Top variable-values, such that when conditioned, result in an excellent OLS fit to the demand curve\n');
    fprintf('---------------------------------------------------------------------------------------------------\n');
    fprintf('Observed fit compared to expected fit by means of chi-square test on \n');
    fprintf('goodness-of-fit measure = %d (%s)\n',fit_measure,deblank(fit_string(fit_measure,:)));
    fprintf('\n');
%    fprintf('Expected OLS gradient:\n');
%    fprintf('E[w(2)] = %f, obs = %d, (intercept was %f)\n',yyw(2),size(y,1),yyw(1));
%    fprintf('\n');
%    fprintf('Most significant segments (OLS discrepancies)\n');
%    %fprintf('E[w(2)|xdim = value]\tchisquare\tp\tobs\n');
    
    present = find(ymeandiff~=0 & obs>=minobs & ~isnan(accuracy_ols) & ~isinf(accuracy_ols)); 
    [a,i] = sort(-1.*accuracy_ols(present)); % sorts from highest to lowest
    for ii=1:no_to_show,
      [dim,xval] = IND2SUB(size(ymean),present(i(ii)));
      [chi,p] = chisquare_test(accuracy_ols(dim,xval),yy_accuracy_ols(fit_measure),1);
      fprintf('E[fit | %s = %d ] = %f, y=%f, w2=%f, chisquare=%f, p=%f, obs=%d\n',...
        deblank(nodestr(dim,:)),full(valtree(xval,dim)), ...
        full(accuracy_ols(dim,xval)),ymean(dim,xval),full(yw(dim,xval,2)),full(chi),full(p),full(obs(present(i(ii)))));
    end;


    fprintf('\n');
    fprintf('Top variable-values, such that when conditioned, significantly change the gradient of the demand curve\n');
    fprintf('------------------------------------------------------------------------------------------------------\n');
    fprintf('Observed curve compared to expected curve by means of chi-square test\n');
    fprintf('OLS is tested by comparing gradient of observed curve to gradient of expected\n');
    fprintf('Logistic is compared to expected by quantifying parameter\n');
    fprintf('distance between the two curves - this is not an ideal method of assessing\n');
    fprintf('curve similarity because the two parameters may vary by different amounts\n');
    fprintf('\n');
    fprintf('Expected OLS gradient:\n');
    fprintf('E[w(2)] = %f, obs = %d, (intercept was %f)\n',yyw(2),size(y,1),yyw(1));
    fprintf('\n');
    fprintf('Most significant segments (OLS discrepancies)\n');
    %fprintf('E[w(2)|xdim = value]\tchisquare\tp\tobs\n');
    
    present = find(wdiff~=0); 
    [a,i] = sort(-1.*wdiff(present)); % sorts from highest to lowest
    for ii=1:no_to_show,
      [dim,xval] = IND2SUB(size(ymean),present(i(ii)));
      [chi,p] = chisquare_test(yw(dim,xval,2),yyw);
      fprintf('E[w(2) | %s = %d ] = %f, chisquare=%f, p=%f, obs=%d\n',...
        deblank(nodestr(dim,:)),full(valtree(xval,dim)), ...
        full(yw(dim,xval,2)),full(chi),full(p),full(obs(present(i(ii)))));
    end;

    fprintf('\n');    
    fprintf('Expected logistic parameters\n');
    fprintf('E[b] = [%f , %f], obs = %d\n',yyb(1),yyb(2),size(y,1));
    fprintf('\n');
    fprintf('Most significant segments (logistic discrepancies)\n');

    present = find(bdiff~=0); 
    [a,i] = sort(-1.*bdiff(present)); % sorts from highest to lowest
    for ii=1:no_to_show,
      [dim,xval] = IND2SUB(size(ymean),present(i(ii)));
      [chi,p] = chisquare_test(squeeze(yb(dim,xval,1:2)),yyb);
      fprintf('E[b | %s = %d ] = [%f, %f], chisquare=%f, p=%f, obs=%d\n',...
        deblank(nodestr(dim,:)),full(valtree(xval,dim)), ...
        full(yb(dim,xval,1)),full(yb(dim,xval,2)),full(chi),full(p),...
        full(obs(present(i(ii)))));
    end;

  end;

% Show top variables from each dimension
% --------------------------------------

    
    warning on;


