function [chi,p] = chisquare_test(observed,expected,typeoftest);
% 
% [chi,p] = chisquare_test(observed,expected,typeoftest)
% 
% Runs a chi-square test against all cells of the observed matrix, versus 
% the expected matrix cells. Reports the probability of the observed 
% matrix values having been generated by random, assuming a chi-square 
% distribution. Distributional pre-requisites for the chi-square test are that 
% the deviation of observed and expected values is normal
% 
% typeoftest= -1 means observed is less than expected
% typeoftest=  1 means observed is greater than expected
% typeoftest=  2 means two-tailed
% typeoftest=  0 means probability of observing value
% 
% EXAMPLE
% 
% rr=rand(100,1);
% chisquare_test(rr,rr+1),         % systematic difference between the two 
% chisquare_test(rr,rr+randn-0.5), % random noise added, significantly similar
% 
% figure(1);clf;plot(binopdf([1:1000],1000,0.5));hold on;plot(1-binocdf([1:1000],1000,0.5));
%
% SEE ALSO
%
% ttest_paired, ttest_independent
% 

if nargin<3,
  typeoftest=0; % the probability of observing the chi value
end;

chi = sum( (observed - expected).^2./ expected );

r = size(observed,1);
c = size(observed,2);

df = r.*c; %(r-1).*(c-1);

if typeoftest==0,
  p=chi2pdf(chi,df); 
end;
if typeoftest==-1,
  p=1-chi2cdf(chi,df);
end;
if typeoftest==1, 
  p=chi2cdf(chi,df);
end;
if typeoftest==2,
  p = chi2cdf(chi,df);
  p = min(p,1-p);
end;



% probability of it being due to random 
% is equal to the probability of getting a chisquare value higher than or equal 
% to the current value by chance. This is equal to the probability of getting 
% critval, critval+1, critval+2, etc.
% Because its a distribution that sums to 1, we an take
% 1-chi2cdf(1..10) to get the higher values

% df = size(observed,1) - 1(r - 1)*(c - 1), where r is the number of rows and 
% c is the number of columns

% From http://www2.chass.ncsu.edu/garson/pa765/chisq.htm
% Assumptions
% Random sample data are assumed. As with all significance tests, if you have 
% population data, then any table differences are real and therefore significant. 
% If you have non-random sample data, significance cannot be established, 
% though significance tests are nonetheless sometimes utilized as crude 
% "rules of thumb" anyway. 
% 
% A sufficiently large sample size is assumed, as in all significance tests. 
% Applying chi-square to small samples exposes the researcher to an unacceptable 
% rate of Type II errors. There is no accepted cutoff. Some set the minimum 
% sample size at 50, while others would allow as few as 20. Note chi-square 
% must be calculated on actual count data, not substituting percentages, which 
% would have the effect of pretending the sample size is 100. 
% 
% Adequate cell sizes are also assumed. Some require 5 or more, some require more 
% than 5, and others require 10 or more. A common rule is 5 or more in all cells 
% of a 2-by-2 table, and 5 or more in 80% of cells in larger tables, but no cells 
% with zero count. When this assumption is not met, Yates' correction is applied. 
% 
% Independence. Observations must be independent. 
% 
% Similar distribution. Observations must have the same underlying distribution. 
% 
% Known distribution. The hypothesized distribution is specified in advance, so 
% that the number of observations that are expected to appear each cell in the 
% table can be calculated without reference to the observed values. Normally 
% this expected value is the crossproduct of the row and column marginals divided 
% by the sample si 
% 
% Non-directional hypotheses are assumed. Chi-square tests the hypothesis that 
% two variables are related only by chance. If a significant relationship is 
% found, this is not equivalent to establishing the researcher's hypothesis that 
% A causes B, or that B causes A. 
% 
% Finite values. Observations must be grouped in categories. 
% 
% Normal distribution of deviations (observed minus expected values) is assumed. 
% Note chi-square is a nonparametric test in the sense that is does not assume 
% the parameter of normal distribution for the data -- only for the deviations. 
% 
% Data level. No assumption is made about level of data. Nominal, ordinal, or 
% interval data may be used with chi-square tests. 







