function testresult = compare_two_groups(dat,a,b,topN,nodestr);
%
% testresult = compare_two_groups(dat,a,b,topN,nodestr);
%
% DESCRIPTION
% 
% Runs a number of non-parametric and parametric, and comparison tests simultaneously
% Assumes that  you have a single data set, eg. x, and 
% this is comprised of many variables. You have two groups or sets of observations 
% that you want to compare as being different. Do this by passing in 
% x, a, b where a and b are vectors with the observation numbers that belong to 
% group a and group b respectively
%
% Think of a as the experimental group, and b as the control group
% Interpret the stats as "100 var1 means that a is +100 over b
%
% REFERENCES
%
% Blalock, H. (1972), Social Statistics, McGraw-Hill (for information on t-test)
% 
% SEE ALSO
%
% testresult = compare_two_groups2(x,y,topN,nodestr);
% testresult = compare_two_groups(dat,a,b,topN,nodestr);
%


% TODO: change this to top bottom analysis
% Scatterplot compliant versus non-compliant (MDS)
% 

% Parameters
% ----------

if nargin<3, 
  % group b will be the population excluding a
  b = remove_ids([1:size(dat,1)]',a);
end;
if nargin<4, 
  % no top specified
  topN = min(5,size(dat,2));
end;
if nargin<5,
  % no nodestr
  nodestr = num2str([1:size(dat,1)]');
end;
if length(a)==length(b), 
  % if length is equal, we can run some paired tests that 
  % we wouldn't be able to run otherwise
  length_equal=1;
else 
  length_equal=0;
end;


% Get means and stds
% ------------------

chunksize=2000;
[datmean,datstd] = mean_nomemory(dat,      chunksize);
[amean,  astd  ] = mean_nomemory(dat(a,:), chunksize);
[bmean,  bstd  ] = mean_nomemory(dat(b,:), chunksize);



% create samples
% --------------

samplesize = 50; % drop this number of points to speed up this routine
% the more points the better the test, but the slower it goes. Quadratic slowdown
% ranksum can do no more than this many points before taking a long time
% for 1000 its 0.5 secs, for 2000 its 2 secs. Quadratic scaling obviously

% a sample
r=rand(size(a,1),1);
[m,i]=sort(r);
asample = i(1:min([samplesize size(a,1)]));

% b sample
r=rand(size(b,1),1);
[m,i]=sort(r);
bsample = i(1:min([samplesize,size(b,1)]));



% Initilialize
% ------------

testname = ['ttest           '; ...
            'zscoreA-vrs-pop '; ...
            'zscoreA-vrs-B   '; ...
            'ranksum p value '; ...
            'lift            '; ...
            'correlation pos '; ...
            'rankcorr pos    '; ...
            'chisquare       '; ...
            'signtest        '; ...
            'signrank        '];

fprintf('comparing');
warning off;
alpha = 0.01;
testresult = zeros(8,size(dat,2));



% Score each dimension
% --------------------

for dim=1:size(dat,2),
  %[h,p1]  = ttest2(dat(a,dim),dat(b,dim),alpha);
  %p1=-1.*p1;
  t = ttest_indep(dat(a,dim),dat(b,dim));  
  z1     = (amean(dim) - datmean(dim))./ datstd(dim);if isnan(z1) | isinf(z1), z1=0; end;
  %z2     = (bmean(dim) - datmean(dim))./ datstd(dim);if isnan(z2) | isinf(z2), z2=0; end;
  zdiff  = (amean(dim) - bmean(dim))  ./ datstd(dim);if isnan(zdiff) | isinf(zdiff), zdiff=0; end;
  %chisq1 = (amean(dim)-bmean(dim)).^2 ./ bmean(dim);
  %if isnan(chisq1) | isinf(chisq1), chisq1 = 1; end;
  
  pranksum = ranksum(dat(a(asample),dim),dat(b(bsample),dim),alpha);
  
  lift   = amean(dim) ./ bmean(dim);
  if lift<1, lift = -1./lift; end;
  if isnan(lift) | isinf(lift), lift=1; end;
  
  p2 = 0;p3=0;chisq=0;corr = 0;rankcorr = 0;
  if length_equal==1,
    corr = correlation(dat(a,dim),dat(b,dim));
    rankcorr = correlation(ranks(dat(a,dim)),ranks(dat(b,dim)));
    chisq  = sum( ((dat(a,dim)-dat(b,dim)).^2)./ (dat(a,dim)) );if isnan(chisq) | isinf(chisq), chisq=0; end;
    p2 = signtest(dat(a,dim),dat(b,dim),alpha);p2=-1.*p2;
    p3 = signrank(dat(a,dim),dat(b,dim),alpha);p3=-1.*p3;
  end;
  testresult(1:10,dim) = [t;z1;zdiff;pranksum;lift;corr;rankcorr;chisq;p2;p3];
  fprintf('.');
end;
fprintf('\n');


% Automatically find the most significant variables that they differ on
% ---------------------------------------------------------------------

clear carray;
for i=1:size(nodestr,1),
  carray{i} = deblank(nodestr(i,:));
end;

keyboard;
fig=0;
for dir=[-1 1],

  if dir==-1,
    fprintf('\n');
    fprintf('Top %d Variables Over Control/Baseline Group\n',topN);
    fprintf('--------------------------------------------\n');
    fprintf('Score\tMean\tControlMean\tPopulationMean\tVariableName\n');
  else
    fprintf('\n');
    fprintf('Top %d Variables Below Control/Baseline Group\n',topN);
    fprintf('---------------------------------------------\n');
    fprintf('Score\tMean\tControlMean\tPopulationMean\tVariableName\n');
  end;
  
for test=1:size(testresult,1),
  [m,i] = sort(dir.*testresult(test,:));
  fprintf('%s\n',deblank(char(testname(test,:))));
  fig=fig+1;
  if dir==-1,
    titlestr = sprintf('%s Top %d Variables over control',deblank(testname(test,:)),topN);
    col='b';
  else
    titlestr = sprintf('%s Top %d Variables under control',deblank(testname(test,:)),topN);
    col='r';
  end;
  figure(fig);clf;barh(testresult(test,i(1:topN)),col);hold on;
  for r=1:topN,
    ns = deblank(fliplr(deblank(fliplr(num2str(testresult(test,i(r)))))));
    s = sprintf('%s(%s)',deblank(nodestr(i(r),:)),ns);
    text(testresult(test,i(r)),r,s); %,'fontweight','bold');%,'fontcolor','w');
  end;
  title(titlestr);  
  ylabel('Variable rank');
  xlabel(testname(test,:));
  set(gca,'ytick',[1:topN]);
  %set(gca,'yticklabel',nodestr(i(1:topN),:)); %carray{i(1:topN)});
  for r=1:topN,
    fprintf('%f\t%f\t%f\t%f\t',testresult(test,i(r)),amean(i(r)),bmean(i(r)),datmean(i(r)) ); disp(nodestr(i(r),:));  
  end;
  if length_equal~=1 & test==5, break; end;
end;

end; % next dir

warning on;



% Wilcoxon rank sum test (independent samples)
% The Wilcoxon Rank-Sum (Mann-Whitney) Test for Two Independent Samples
% This test is used in place of a two sample t test when the populations being compared are not normal. It requires independent random samples of sizes  and  . The test is very simple and consists of combining the two samples into one sample of size  , sorting the result, assigning ranks to the sorted values (giving the average rank to any `tied' observations), and then letting T be the sum of the ranks for the observations in the first sample. If the two populations have the same distribution then the sum of the ranks of the first sample and those in the second sample should be close to the same value. Stataquest returns a p value for the null hypothesis that the two distributions are the same. 
% Are the data distributed symmetrically? The Wilcoxon test does not assume that the data are sampled from a Gaussian distribution.  However it does assume that the data are distributed symmetrically around their median. 


