function [mat,no_actual,no_pred,vals,pcorrect,number_correct] = confusion_matrix(actual,pred,fig,cattree);
% [mat,no_actual,no_pred,vals,pcorrect,number_correct] = confusion_matrix(actual,pred,fig,cattree);
%
% Creates a confusion matrix which is the number of times a prediction,actual pair 
% is observed in model forecasts. This reveals any interesting ways in which 
% forecasts are systematically biased. eg. perhaps a model has more difficulty
% on larger values of actuals. The confusion matrix is only applicable if the
% actual and predicted values are *discrete* numbers (or strings). Otherwise there will be 
% far too many categories.
% This should work on strings, too, although I haven't tested it.
% cattree:  is a list of unique values which are found in the actuals, which you
%           might already know ahead of time. If cattree is not supplid,  
%           the algorithm will find unique values in the actuals automatically.
% fig:      is the figure number to start at when displaying results. If fig==[] then
%           no graphs will be plotted
% no_actual, no_pred: number of time each value appears in actuals
% pcorrect:           the diagonal of mat, divided by no_actual, to give percent correct
% number_correct:     sum(diag(mat)). This / sum(no_actual) gives overallpcorrect
%
% EXAMPLE
%
% % confusion matrix from dtree learning addition 
% x=round(rand(20,2).*5);dtree = dtree_train(x,x(:,1)+x(:,2),-9,1,5);
% yest = dtree_eval(x,dtree,-9);
% [mat,no_actual,no_pred,vals] = confusion_matrix(x(:,1)+x(:,2),yest,1);
%
% % confusion matrix from predictions which are normally distributed around actuals
% actual=round(rand(10000,1).*10);pred=round(rand(10000,1).*11);
% [mat,no_actual,no_pred,vals] = confusion_matrix(actual,pred,1);
%
% SEE ALSO
% accuracy_of_model (accuracy_diagnostics): runs comprehensive tests on predictions versus actuals
%

if nargin<3,
   fig=1;
end;
if max(size(fig))>1, 
   fprintf('probably a parameter mixup. fig is a vector, it should be a number\n');
   return
end;
if nargin==4,
   % cattree supplied
   fprintf('using supplied values\n');
   vals = cattree;
else
   fprintf('finding unique actual values\n');
   vals = unique([actual],'rows');
end;

% Determine type of vals. We will recode numerics into a string representation
%s=whos('vals');
%if isempty(strmatch(s.class,'char')),
%   vals = num2str(vals);
%end;
% we could have converted vals to string here, and then used it as such 
% all through

% find any pred values which are not equal to vals, and code them as vals+1 (other)
noact = [1:size(pred,1)]';
for i=1:size(vals,1),
   ii=strmatch(vals(i,:),pred(noact));% finds all pred who equalled that value
   noact = remove_ids(noact,ii); % get rid of them, (they're not homeless)
end;
% noact are now the remaining preds with no value in actual

if ~isempty(noact),
   pred(noact,:) = ones(max(size(noact)),1) * (max(vals)+1);
   vals = [vals;(max(vals)+1)];
end;

mat = zeros(size(vals,1),size(vals,1));
no_actual = zeros(size(vals,1),1);
no_pred   = zeros(size(vals,1),1);

% After we find, remove them from an active list.
% This will speed this up on large confusion matrix

tic,
active = [1:size(actual,1)]';
for i1=1:size(vals,1), % actual
   actual_is_val   = strmatch(vals(i1,:),actual(active,:));
   no_actual(i1,1) = size(actual_is_val,1);
   no_pred(i1,1)   = size(strmatch(vals(i1,:),pred(active,:)),1);
   if ~isempty(actual_is_val),
     % number of times that the actual equalled value vals(i1,:)
     % out of this many cases, how many times did pred equal actual
     predactive = [1:max(size(actual_is_val))]';
     for i2=1:size(vals,1), 
        % look at all possible prediction values
        predisthisval = strmatch(vals(i2,:),pred(active(actual_is_val(predactive)),:));
        mat(i2,i1) = size(predisthisval,1);
        % number of times that pred is i2
        % (where it should have been i1) 
        %predactive = setdiff(predactive,predactive(predisthisval));
        predactive = remove_ids(predactive,predisthisval);
     end; % i2
     active=remove_ids(active,actual_is_val);
     %active=setdiff(active,active(actual_is_val));
  end; % if
  fprintf('.');
end; % i1
fprintf('%d seconds\n',round(toc));
fprintf('\n');

% Rearrange the matrix if there are prediction values for which no actuals occurred
% ----------------------------------------------------------------------------------

if ~isempty(noact),
   vals = vals(1:size(vals,1)-1,:);
   no_actual = no_actual(1:size(vals,1));
   mat = mat(1:size(vals,1)+1,1:size(vals,1));
   % decrease the size of no_actual by one (it was zero anyway)
   %mat(size(vals,1)+1,1:size(vals,1)) = mat(1:size(vals,1),1)';
   % take the scores from the right hand vertical, and put them on
   % the bottom row horizontal
   %mat = mat(1:size(vals,1)+1,1:size(vals,1));
   % resize mat so it no longer has the right hand side vertical
end;

% Display the matrix
% ------------------

if size(vals,1)>100, 
   fprintf('confusion matrix not shown\n');
else
   
  % Values
  places = num2str(max([floor(log10(max(no_actual)))+1 5]));
  for i=1:size(vals,1),
    fprintf(['%' places 'd\t'],vals(i,:));
  end;
  % Number of occurrences
  fprintf('\n');
  fprintf(['%' places 'd\t'],no_actual);
  fprintf('\n');
  % Percentage accuracy
  for i=1:size(vals,1),
    fprintf('%5.1f\t',100.*mat(i,:)./(no_actual'+eps));
    fprintf('\n');
   end;
   fprintf('\n');
   % Absolute number of predictions in each category
   for i=1:size(vals,1),
     fprintf(['%' places 'd\t'],mat(i,:));
     fprintf('\n');
   end;
   if ~isempty(noact),
     fprintf(['%' places 'd\t'],mat(size(vals,1)+1,:));
     fprintf('\n');
   end;
   fprintf('\n');
   if ~isempty(noact),
     fprintf('Note some values were predicted which there were no actuals for.\n');
     fprintf('These are in the last row of the matrix.\n');
  end;
end;


pcorrect = diag(mat)./(no_actual+eps);
number_correct = sum(diag(mat));


% Display figures
% ---------------

vals_str = num2str(vals);

if ~isempty(fig),
no_bins = size(vals,1);
   
% Plot percentage correct for each decile
figure(fig);clf;bar([1:no_bins],pcorrect.*100);colormap(cool);
v=axis;axis([v(1) v(2) 0 100]);set(gca,'xticklabel',vals_str);hold on;
plot([v(1) v(2)],100.*[1./no_bins 1./no_bins],':'); % this is random performance
legend('random','model',0);
xlabel('actual');ylabel('% correct');
title('Performance in predicting each value');

% 3D confusion matrix
figure(fig+1);clf;
bar3(100.*mat./(ones(size(mat,1),1)*no_actual'+eps));
%bar3(mat);
title('confusion matrix');xlabel('actual');
ylabel('predicted');
%zlabel('no predictions');
zlabel('%');view(30,50);set(gca,'xtick',[1:no_bins]);v=axis;
axis([0 no_bins+1 0 no_bins+1 v(5) v(6)]);
set(gca,'xticklabel',vals_str);set(gca,'yticklabel',vals_str);
view(30,30);

% Confusion matrix intensity plot
figure(fig+2);
display_intensitymat(mat,50,vals_str);
title('confusion matrix');
xlabel('actual');ylabel('predicted');
zlabel('no predictions');

%figure(4);clf;zi=interp2(mat,3,'nearest');contourf(zi);colorbar;shading faceted;colormap(pink);lighting phong;material shiny;
% we can plot an interpolated, shaded versoin of the confusion matrix

end;

% We should think of a faster way to do this. Something like correlation, where
% we've coded only pred and act with a 1 if they match, ie.
% for vals = 1:size(vals,2)

      %mat(i1,i2)=size(strmatch(actual(i1,:),vals(i2,:)),1);
      % number of times that prediction (i2) equalled 
      
      
   