function [mat,no_actual,no_pred,vals,pcorrect,number_correct] = confusion_matrix2(actual,pred,fig,cattree);
% [mat,no_actual,no_pred,vals,pcorrect,number_correct] = confusion_matrix2(actual,pred,fig,cattree);
%
% This version of confusion_matrix allows any values for actual and predicted. 
% This is best when you are dealing with a prediction that has been deciled into 
% 10 values, versus an actual which is 1 or 0.
% 
% EXAMPLE
% 
% %%%%%%%%% Various methods for comparing real-valued forecasts to 0,1 actuals
% 
% % Method1: Most preferred, bin based on distribution (shows performance on percentages of the data)
% [p1,p2,p3,member] = decile1(-1.*yest,[1],10,{'mean'});confusion_matrix2(ytest,member,1);
% 
% % Method2: Bin fixed values between lowest and highest value (dangerous)
% [p1,p2,p3,member] = bin(        yest,[1],10,{'mean'});confusion_matrix2(ytest,member,1);
% 
% % Method3: Thresholding based on 50% (least preferred, where do you cut the threshold?)
% confusion_matrix(ytest,yest > 0.5,1);
% 
% 
% SEE ALSO
%
% confusion_matrix(actual,pred,fig);
% confusion_matrix2(actual,pred,fig);
% accuracy_of_model (accuracy_diagnostics): runs comprehensive tests on predictions versus actuals
%

if nargin<3,
   fig=1;
end;
if max(size(fig))>1, 
   fprintf('probably a parameter mixup. fig is a vector, it should be a number\n');
   return
end;
if nargin>=4,
   % cattree supplied
   fprintf('Can no longer supply values\n');
   return;
else
   fprintf('finding unique actual values\n');
   vals = unique(actual,'rows');
   predvals = unique(pred,'rows');
end;

% Determine type of vals. We will recode numerics into a string representation
%s=whos('vals');
%if isempty(strmatch(s.class,'char')),
%   vals = num2str(vals);
%end;
% we could have converted vals to string here, and then used it as such 
% all through

% find any pred values which are not equal to vals, and code them as vals+1 (other)
if 0, % THis code was quite functional - coded pred vals not equalling actual with noact
noact = [1:size(pred,1)]';
for i=1:size(vals,1),
   ii=strmatch(vals(i,:),pred(noact));% finds all pred who equalled that value
   noact = remove_ids(noact,ii); % get rid of them, (they're not homeless)
end;
% noact are now the remaining preds with no value in actual

if ~isempty(noact),
   pred(noact,:) = ones(max(size(noact)),1) * (max(vals)+1);
   vals = [vals;(max(vals)+1)];
end;
end;

mat = zeros(size(predvals,1),size(vals,1));
no_actual = zeros(size(vals,1),1);
no_pred   = zeros(size(predvals,1),1);

% After we find, remove them from an active list.
% This will speed this up on large confusion matrix

tic,
active = [1:size(actual,1)]';
for i1=1:size(vals,1), % actual
   actual_is_val   = strmatch(vals(i1,:),actual(active,:));
   no_actual(i1,1) = size(actual_is_val,1);
   no_pred(i1,1)   = size(strmatch(vals(i1,:),pred(active,:)),1);
   if ~isempty(actual_is_val),
     % number of times that the actual equalled value vals(i1,:)
     % out of this many cases, how many times did pred equal actual
     predactive = [1:max(size(actual_is_val))]';
     for i2=1:size(predvals,1), 
        % look at all possible prediction values
        predisthisval = strmatch(predvals(i2,:),pred(active(actual_is_val(predactive)),:));
        mat(i2,i1) = size(predisthisval,1);
        % number of times that pred is i2
        % (where it should have been i1) 
        %predactive = setdiff(predactive,predactive(predisthisval));
        predactive = remove_ids(predactive,predisthisval);
     end; % i2
     active=remove_ids(active,actual_is_val);
     %active=setdiff(active,active(actual_is_val));
  end; % if
  fprintf('.');
end; % i1
fprintf('%d seconds\n',round(toc));
fprintf('\n');

% Rearrange the matrix if there are prediction values for which no actuals occurred
% ----------------------------------------------------------------------------------

if 0,
if ~isempty(noact),
   vals = vals(1:size(vals,1)-1,:);
   no_actual = no_actual(1:size(vals,1));
   mat = mat(1:size(vals,1)+1,1:size(vals,1));
   % decrease the size of no_actual by one (it was zero anyway)
   %mat(size(vals,1)+1,1:size(vals,1)) = mat(1:size(vals,1),1)';
   % take the scores from the right hand vertical, and put them on
   % the bottom row horizontal
   %mat = mat(1:size(vals,1)+1,1:size(vals,1));
   % resize mat so it no longer has the right hand side vertical
end;
end;

% Display the matrix
% ------------------

if size(vals,1)>100 | size(predvals,1)>100, 
   fprintf('confusion matrix not shown due to matrix size (%d actuals and %d predictions)\n',size(vals,1),size(predvals,1));
else
   
  % Values
  places = num2str(max([floor(log10(max(no_pred)))+1 5]));
  for i=1:size(vals,1),
    fprintf(['%' places 'd\t'],vals(i,:));
  end;
  % Number of occurrences
  fprintf('\n');
  fprintf(['%' places 'd\t'],no_actual);
  fprintf('\n');
  % Percentage accuracy
  for i=1:size(predvals,1),
    fprintf('%5.1f\t',100.*mat(i,:)./(no_actual'+eps));
    fprintf('\n');
   end;
   fprintf('\n');
   % Absolute number of predictions in each category
   for i=1:size(predvals,1),
     fprintf(['%' places 'd\t'],mat(i,:));
     fprintf('\n');
   end;
%   if ~isempty(noact),
%     fprintf(['%' places 'd\t'],mat(size(vals,1)+1,:));
%     fprintf('\n');
%   end;
   fprintf('\n');
%   if ~isempty(noact),
%     fprintf('Note some values were predicted which there were no actuals for.\n');
%     fprintf('These are in the last row of the matrix.\n');
%  end;
end;


pcorrect = diag(mat)./(no_actual+eps);
number_correct = sum(diag(mat));


% Display figures
% ---------------

vals_str = num2str(vals);
predvals_str = num2str(predvals);

if ~isempty(fig),
no_bins = size(vals,1);
no_predbins = size(predvals,1);
  
% Plot percentage correct for each decile
figure(fig);clf;bar([1:no_bins],pcorrect.*100);colormap(cool);
v=axis;axis([v(1) v(2) 0 100]);set(gca,'xticklabel',vals_str);hold on;
plot([v(1) v(2)],100.*[1./no_bins 1./no_bins],':'); % this is random performance
% Random performance is, I pick a forecast bin at random. The actuals may then 
% be biased, for instance, no actual1s in any of the foreacast bins except for one 
% of them. Therefore, random performance is, instead, 
legend('random','model',0);
xlabel('actual');ylabel('% correct');
title('Performance in predicting each value');

% 3D confusion matrix
figure(fig+1);clf;
bar3(100.*mat./ones(size(mat))+eps);
title('confusion matrix');xlabel('actual');
ylabel('predicted');
%zlabel('no predictions');
zlabel('%');view(30,50);
set(gca,'xtick',[1:no_bins]);set(gca,'ytick',[1:no_predbins]);
%v=axis;
%axis([0 no_bins+1 0 no_bins+1 v(5) v(6)]);
set(gca,'xticklabel',vals_str);set(gca,'yticklabel',predvals_str);
view(30,30);

% Confusion matrix intensity plot
figure(fig+2);
display_intensitymat(mat,50,vals_str);
title('confusion matrix');
xlabel('actual');ylabel('predicted');
zlabel('no predictions');
set(gca,'xtick',[1:no_bins]);set(gca,'ytick',[1:no_predbins]);
set(gca,'xticklabel',vals_str);set(gca,'yticklabel',predvals_str);

%figure(4);clf;zi=interp2(mat,3,'nearest');contourf(zi);colorbar;shading faceted;colormap(pink);lighting phong;material shiny;
% we can plot an interpolated, shaded versoin of the confusion matrix

end;

% We should think of a faster way to do this. Something like correlation, where
% we've coded only pred and act with a 1 if they match, ie.
% for vals = 1:size(vals,2)

      %mat(i1,i2)=size(strmatch(actual(i1,:),vals(i2,:)),1);
      % number of times that prediction (i2) equalled 
      
      
   