function ent=cross_entropy(dist1,dist2,missing_val,absolutediff,protectzero);
% ent=cross_entropy(dist1,dist2,missing_val,absolutediff);
%
% Cross-entropy is also known as the Kullback-Leibler information criterion, or 
% KL-distance. This measures the distance between one probability distribution 
% and another. 
% 
% DEFINITION
% KL(a,b) = sum(sum(a.*log(a./b))) 
%
% EXAMPLE
% [cross_entropy(0.1,0.8) cross_entropy(0.1,0.0125)],
% dist1=[0.1 0.2 0.5 0.3];dist2=[0.1 0.1 0.6 0.3];
% [cross_entropy(dist1,dist2), cross_entropy(dist1,dist2,nan,1,1)],
%
% PARAMETERS
% dist1,dist2          two probability distributions of the same size
% missing_val          Missing vals are skipped. If not specified, this function assumes that ther
%                      are *no* missing values in the distributions.
% absolutediff==0or1   For an absolute discrepancy in terms of KL-distance
% protectzero ==0or1   0 probs result in infinite KL-distance. protectzero==1 prevents 
% 0 from being reached. This allows more "resolution" in the cross entropy metric of 
% distance between two distributions, rather than just reporting infinite distance all 
% the time.
%
% KL-distance can be understood in several ways:
% (1) KL(a,b) = -1.*entropy(dist1) - sum(sum(dist1.*log(dist2)));
% Therefore, KL is (roughly) the distance in entropy between dist1 and dist2 -
% entropy of dist1 - entropy counted as importance of prob in dist1 multiplied
% by encodingbis required for dist2. If more encoding bits in dist2, then
% KLdist negative. If fewer then KL dist positive
% (2) A way of understanding KL is that it measures the distance from one dist
% to another in "ratio". For instance KL(0.1,0.8)==KL(0.1,0.0125) because the 
% ratio of each is 1/8. Way of understanding this in encoding bits
% This can be seen to hold in the way the log function operates: log(1/1)=0,
% log(2/1) == log(1/2), log(100/1)(slow growth)==log(1/100) (slow decay)
% figure(1);clf;x=[1:100];plot(x,log(x./1));hold on;plot(x,log(1./x));
% % so a probdist which is x./1 multiples off on the positive side, carries the same
% % weight as a probdist which is off 1./x multiples off on the negative side
%
% PROBLEMS WITH KL-DISTANCE
% Each log discrepancy on each distribution element, can be positive and negative. 
% This is different from Euclidean distance which is always >=0 (and its element
% discrepancies are always positive also - they are squared in fact). The different 
% distribution element diffs can therefore be positive and negative, and sum to
% near zero, even though the distribution elements are all highly discrepant 
% (analogous to mean error versus mean absolute error). Therefore, to use this,
% as a measure of difference between distributions, consider setting 
% absolutediff==1. 
%
% Kullback, S. and Leibler, R. (1951), On Information and Sufficiency,
% The Annals of Mathematical Statistics, Vol. 22, pp. 79-86
% Hobson, A. (1971), Concepts in Statistical Mechanics, Gordon and Breach
% http://finance.wiwi.uni-karlsruhe.de/Lehre/SemOpt/Entropie/2224.html
%
% SEE ALSO
% entropic_significance(x,y,missing_val);
% entropy(x);
% ent=cross_entropy(dist1,dist2,missing_val,absolutediff);

missing_present=1;
if nargin<3,
   missing_val=[];
   % assume that when missing_val is not specified, it means there are no
   % missing_vals in the distributions.
   % This is being done for speed. (because cross_entropy gets called a lot).
   % Normally, no missing_val being specified results it being assigned a 
   % default value of nan, and then missing_val filtering goes on as normal
end;
if isempty(missing_val), 
   missing_present=0; 
end;
if nargin<4,
   absolutediff=0;
end;
if nargin<5,
   protectzero=0;
end;

% Find all mutually present elements in dist1 and dist2
% -----------------------------------------------------

if missing_present,
if isnan(missing_val),
   present=find(~isnan(dist1) & ~isnan(dist2));
else
   present=find(dist1~=missing_val & dist2~=missing_val);
end;
end;

% 0 or 1 probs result in infinite cross-entropies
% -----------------------------------------------

if protectzero,
i=find(dist1==0);dist1(i)=eps.*ones(size(i,1),1);
i=find(dist2==0);dist2(i)=eps.*ones(size(i,1),1);
end;

% Calculate cross-entropy
% -----------------------

if absolutediff,
  ent = sum(dist1(:) .* abs(log(dist1(:)./dist2(:))));
  % This is absolute diff cross-entropy, which keeps tab of all
  % discrepancies, forces them all to become positive deviations
else
  ent = sum(dist1(:) .* log(dist1(:)./dist2(:)));
  % This is the official cross-entropy, in all its glory
end;

