function [total_error,yest,cy] = discretization_quality(c,x,y,errorFunction);
% 
% err = discretization_quality(x,c,y,errorFunction);
%
% Measures the quality of a partiular quantization of x in predicting y. 
% The user supplies an errorFunction or impurityFunction, which measures the 
% impurity or variance of the y value, given the quantization. 
% 
% 
% % EXAMPLE 1
%
% r = randn(1000,1);   % 
% ry = r.^2;           % the thing we are trying to predict
% err_metric = 2;      % variance
% nobins = 6;          % 
% figure(1);figure(2); % because figure 1 is used by decile1 and bin
% 
% % %%%% Measure the quality of a decile1 discretizaton
% proto = decile1(r,[1],nobins,1);
% [err,yest,cy] = discretization_quality(proto,r,ry,err_metric);
% figure(3);clf;plot(proto,cy,'o');hold on;plot(r,ry(:,size(ry,2)),'.');title(['Decile1 of x (no knowledge of y), Error ' num2str(err)]);
% 
% % %%%% Measure the quality of a bin discretization
% proto = bin(r,[1],nobins,{'mean'});
% [err,yest,cy] = discretization_quality(proto,r,ry,err_metric);
% figure(4);clf;plot(proto,cy,'o');hold on;plot(r,ry(:,size(ry,2)),'.');title(['Bin of x (no knowledge of y), Error ' num2str(err)]);
% 
% % %%%% Optimize the discretization using variance as an impurity measure
% proto = r(1:nobins); % 5 random cases
% proto = fmins('discretization_quality',proto,foptions,[],r,ry,err_metric);
% [err,yest,cy] = discretization_quality(proto,r,ry,err_metric);
% figure(5);clf;plot(proto,cy,'o');hold on;plot(r,ry(:,size(ry,2)),'.');title(['Optimized variance quantization, Error ' num2str(err)]);
%
% % %%%% Use a decision tree's discretization
% dtree = dtree_train(r,ry,-9999,err_metric,400); % A problem is, predicting the no of leaves is hard
% [boundingbox,proto1] = dtree_leaves(dtree);
% [err,yest,cy] = discretization_quality(proto1,r,ry,err_metric);
% figure(6);clf;plot(proto1,cy,'o');hold on;plot(r,ry(:,size(ry,2)),'.');title(['Decision tree, Error ' num2str(err) '(nobins used=' num2str(length(proto1)) ')']);
% 
% 
% % EXAMPLE 2
% 
% % Change the y function - y is now discrete 1 or 0
% ry = r<0.1 | r>0.9;
% err_metric=1;
% 
% % %%%% Optimize the discretization using Entropy as an impurity measure
% proto = r(1:5); % 5 random cases
% proto = fminsearch('discretization_quality',proto,[],r,ry,err_metric);
% [err,yest,cy] = discretization_quality(proto,r,ry,err_metric);
% figure(1);clf;plot(proto,cy,'o');hold on;plot(r,ry,'.');title(['Error ' num2str(err)]);
% 
%
% % EXAMPLE 3
%
% % Measure curve-fit quality
% ry = r.^2;
% err_metric = 5; % logistic
% proto = r(1:5);
% proto = fminsearch('discretization_quality',proto,[],r,ry,err_metric);
% [err,yest,cy] = discretization_quality(proto,r,ry,err_metric);
% figure(1);clf;plot(proto,cy,'o');hold on;plot(r,ry,'.');title(['Error ' num2str(err)]);
 


% % %%%% Compare to a decision tree
% dtree = dtree_train(r,ry,-9999,err_metric,1); % 
% proto = unique(dtree(:,3)); % ascending order
% proto1 = proto - diff([min(ry);full(proto)]) ./2;
% [err,yest,cy] = discretization_quality(proto1,r,ry,err_metric);
% figure(1);clf;plot(proto1,cy,'o');hold on;plot(r,ry,'.');title(['Error ' num2str(err)]);
% 
  
  if errorFunction==4,
    % Preparation for binomial test
    max_samples = 800;
    if size(y,1) > max_samples,
      % only need to do this if we're above the numerical precision 
      % of the machine
      sample_correction = max_samples ./ size(y,1);
      % this is the scale-down factor that will force us to be using 
      % a precision in which we actually get a non-zero probability
    else 
      sample_correction = 1;
    end;
    ymean = mean(y);
  end;

  if errorFunction==5 | errorFunction==6,
    fitid=4;
  end;
  
  
  
  for dim=1:size(x,2),


  
    % Assign raw x values to the quantized prototypes
    % ------------------------------------------------
    
    if size(c,1)==1, 
      [nuh,member,centdist]=k_nearest_neighbour(x,c,[1:size(c,1)]',1,-9999);
    else
      diff = abs(x(:,1)*ones(1,size(c,1))-ones(size(x,1),1)*c');
      % this should be placed into the k_nearest_neihgbour routine
      % we now have for each row, that row minus each of the c values    
      [a,i] = sort(diff');
      member = i(1,:)';
    end;

    if nargout>1,
      yest = c(member);
    end;

    % we now have a member structure which lists what prototype that 
    % each case belongs to
    
    
    
    % Measure Impurity of each bin
    % ----------------------------
    
    remainingels = [1:size(y,1)]';
    total_error=0;
    
    for bin=1:size(c,1),

      currentbin = find(member(remainingels)==bin);
      
      if nargout>2,
        cy(bin) = mean(y( remainingels(currentbin),size(y,2) ));
      end;
     
      if length(errorFunction)>1,
        % function call
        fprintf('function call');
        errorFunction = char(errorFunction);
        bin_error = feval(errorFunction,y(remainingels(currentbin),:));
      end;
    
      if errorFunction==0 | errorFunction==2,
        % variance
        bin_error = sum( (y(remainingels(currentbin))-mean(y(remainingels(currentbin)))).^2); %  var(y(remainingels(currentbin),:));
      end;
    
      if errorFunction==1, 
        % entropy
        vals = unique(y(remainingels(currentbin),:),'rows');
        probval = zeros(size(vals,1),1);
        remaining = [1:length(currentbin)]';
        for v =1:size(vals,1),      
          thisvalue = find(y(remainingels(currentbin(remaining)))==vals(v));
          probval(v,1) = length(thisvalue) ./ length(currentbin);
          remaining = remove_ids(remaining,thisvalue);
        end;
        if length(remaining)>0, fprintf('error 1\n');keyboard;end;
        bin_error = length(currentbin).*entropy(probval);
      end;
  
      if errorFunction==4,
        % binomial
        p = binopdf(ceil(mean(y(remainingels(currentbin),:)).*sample_correction),ceil(length(currentbin).*sample_correction),ymean);
        bin_error = p.*length(currentbin)./length(member);
      end;
      
      if errorFunction==5,
        % goodness of fit of a linear demand curve      
        [w,yest] = polyfitf(y(remainingels(currentbin),1),y(remainingels(currentbin),2),1);
        if w(2)>0, w(2)=0; yest = mean(y(remainingels(currentbin),2)); end;        
        acc = sum( (yest-y(remainingels(currentbin),2)).^2); % accuracy = maccuracy(y(remainingels(currentbin),2),yest,1);
        bin_error = acc .* length(currentbin);
        % bin_error = -1.*accuracy(fitid).*length(currentbin);
      end;
          
      if errorFunction==6,
        % goodness of fit of a logistic demand curve      
        b = glmfit(y(remainingels(currentbin),1),[y(remainingels(currentbin),2) ones(size(remainingels(currentbin),1),1)],'binomial');
        yest = glmval(b,y(remainingels(currentbin),1),'logit');
        accuracy = maccuracy(y(remainingels(currentbin),2),yest,1);
        bin_error = -1.*accuracy(fitid).*length(currentbin);
      end;

      if errorFunction==7,
        % bits required to transmit the model and its residual
        resid = 1 + y(remainingels(currentbin))-mean(y(remainingels(currentbin)));
        bin_error = sum(log2(resid)); 
        % plus a constant for the bin that should be applied
        % bits required to transmit the model will equal to a constant 
        % of sum of bits to represent each residual + sum(log2(proto)) + 
        % constant bin number for each case
      end;
      
      % Model fit
      % error
      %
      %       avg -> model complexity (bits required to transmit model) -> proto for each case
      % 

      %if errorFunction==4,
        % regerror
      %end;
 
      total_error = bin_error + total_error;
      
      %remainingels = remove_ids(remainingels,currentbin);
    end; % next bin
  end; % next dim
    
    
      