function [data2,valtree] = discretize_continuous_vars(data, ...
                             no_of_bins, preserve_density, missing_val); 
%
% [data2,valtree] = discretize_continuous_vars(data, ...
%                     no_of_bins,preserve_density,missing_val);
% 
% Converts continuous data into a discretized version. 
% Actually, preserve density means the opposite of what it does
%
% preserve_density 0 means it puts the data into equally spaced bins
% Therefore, there may be outlier customers, and these guys get lumped into
% bins. Or there may be a single bin which covers 99% of the populations, 
% with the other 9 bins then picking up the other custs (because a few outliers
% cause the bins to be spread way apart
% 
% preserve_density 1 means put an equal number of customers into each bin
% Therefore, the bin thresholds are calculated by population. This intuitively
% should be better, since we are giving the maximum power to discriminate 
% the values which the customers have. There won't be any wasted values.
%
% preserve_density 2 (not implemented) could work by running a kmeans, which
% would model the exact density distribution of customers. Actually, the above
% should do this just fine.
%
% it attempts to seed the density of the
%   distribution. This means that there may be more
%   bins in high density areas
%
% Use this procedure before dtree_train, if using a large number of 
% continuous values.
%
% EXAMPLE
%
% r=round(rand(1000,3).*10);r(:,2)=r(:,1).^2;
% [data2,valtree]=discretize_continuous_vars(r,10,1,-9);
% valtree,
%
% % the valtree in dim 2 has more bins crowded into the lower
% % values. This is because ^2 sends more values into lower vals, 
% % and spreads out large values 
%
% FUNCTION
%
% [data2,valtree] = discretize_continuous_vars(data, ...
%   no_of_bins,preserve_density,missing_val);

% Clustering parameters
% -------------------------

    lc              = 0.2;
    number_of_loops = 100; 

% Input parameters
% ----------------

  disp('discretize_continuous_vars');
  if preserve_density==1,
    fprintf(' preserving density\n');
    fprintf(' number of kmeans_batch iterations %f\n',number_of_loops);
    fprintf(' learning rate %f\n',lc);
  else
    fprintf(' sending data to equally sized discrete bins\n');
  end;
  fprintf(' missing_val = %f\n',missing_val);
  fprintf('--------------------------------------------------\n\n');


% Perform conversion
% ------------------
disp('converting');

data2 = ones(size(data,1),size(data,2)).*-9;

for col=1:size(data,2),

  pres = find(data(:,col)~=missing_val);
  % ignore missing values


  if preserve_density==0,

    % create a set of equally sized partitions of the data in data(:,col)
    % -------------------------------------------------------------------

    [n,c] = hist(data(pres,col),no_of_bins);
    % instead of doing the hist here, we should run kmeans_batch
    % and then again set to the cluster centers.

  end;
  if preserve_density==1,
     % [a,i] = sort(data(pres,col));
     % now cut into no_of_bins groups and return the mean value
     % this is exactly the same as decile1
     
     c = decile1(data(pres,col),[1],no_of_bins,[1])'; drawnow;
     % sort by col, into no_of_bins bins, and return the mean for each group
     
  end;
  if preserve_density==2,
     
     % Find density preserving bins
    % ----------------------------

    dat = [data(:,col) data(:,col)]; 
    % my cluster routine can't handle single column vectors

    y               = dat;

    c = memlookup_train(dat,dat,no_of_bins);

    % cluster dat
    c = kmeans_batch2(dat,no_of_bins,lc,number_of_loops,y,c);

    % returns us with c, the clusters hopefully representing most
    % dense areas. Maybe we should use some sort of conscience 
    % clustering to try to really preserve distance

    c = c(:,1); % Remove that extra column
    c = c'; % flip it sideways to be consistent

 end;
  if preserve_density==3,
     c = decile1(data(pres,col),[1],no_of_bins,{'max'})'; drawnow;
  end;

  % Prepare c
  % ---------

  c = sort(c);
  % make sure its in order, as we'll use the delta = c(1,i+1)-c(1,i) 
  % to give the envelope of this bin

  valtree(1:no_of_bins,col) = c';
  % valtree records c

  c = [-inf c inf];
  % when we take the elements in the envelope surrounding c,
  % we take -inf and +inf on the left and right sides respectively


  % For each bin, convert all data falling in this bin to the bin center
  % --------------------------------------------------------------------

  for bin=2:no_of_bins+1,

    leftdelta = (c(1,bin)-c(1,bin-1))./2;
    rightdelta = (c(1,bin+1)-c(1,bin))./2;
    % the envelope that we have to gather up data in
    % and set it to the bin center

    thisbin = find(data(pres,col)>=c(1,bin)-leftdelta & ...
                   data(pres,col)<=c(1,bin)+rightdelta );
                 % equals are used here just in case
    % find all elements in this column which are inside this bin

    data2(pres(thisbin),col) = ones(size(thisbin,1),1).*c(1,bin);
    % set all of these elements equal to the bin center
  end;

  fprintf('.');
end; % next column
fprintf('\n');


% Fill end of valtree with missing_vals
% --------------------------------------

valtree=full(valtree);
valtree(size(valtree,1)+1,1) = missing_val;
% needs to be done, as below we could possibly have a missing val at
% size+1 down. So create this now.
% Below we will uniquify valtree, and insert a missing_val as the last element
% for each column

%valtree(size(valtree,1)+1,1:size(valtree,2)) = ...
%  ones(1,size(valtree,2)) .* missing_val;
% fill the end of valtree with -9s. This is a standard 
% I'm following so that my decision tree can interpret it
% properly. Be aware of this, though

% Post-process valtree: remove duplicates
% ---------------------------------------

% For dtree, there should be as few values as possible. 
% It is common to have this discretization result in multiple duplicate
% values. for example, if one dim has many custs with 0 purchases, there
% will be 0 replicated nine times. To get rid of this redundancy, we can
% now modify the valtree, so that it 

for dim=1:size(valtree,2),
   u=unique(valtree(1:size(valtree,1)-1,dim));
   % find all unique values in valtree before the last missing_val row
   valtree(1:size(valtree,1),dim) = zeros(size(valtree,1),1);
   valtree(1:size(u,1),dim) = u;
   valtree(size(u,1)+1,dim) = missing_val;
end;




if 0,
% Final check (omit for speed)
% ----------------------------

if isempty(find(data==missing_val)) & ~isempty(find(data2==missing_val)),
  fprintf('\n');
  fprintf('missing values (%fs) detected in output data',missing_val);
  fprintf('when no missing_vals were detected in the input data.\n');
  fprintf('This routine seems to have failed!');
  fprintf('\n');
else
  fprintf('discretization completed\n');
  mean(data), mean(data2), std(data), std(data2),
end;
  
end;
