function [y,prob,nodeID]=dtree_eval(x,dtree,missing_val);
%
% [yest,prob,nodeID]=dtree_eval(x,dtree,missing_val);
%
% DESCRIPTION
% "Fast dtree eval". Given a set of data and a previously trained dtree, 
% predicts a value y for each x case. The focus of this routine is on 
% speed, although it does include some rudimentary (not-super accurate) 
% missing value handling. 
%
% EXAMPLE
% r=rand(10,3);
% dtree = dtree_train(r,r(:,1).^2+r(:,2));
% rout  = dtree_eval(r,dtree); % note no missing_val specified
% [r(:,1).^2+r(:,2) rout],
% 
% MISSING VALUE HANDLING
% missing_val needs to be specified to enable missing_val handling.
%
% If a missing_value is encountered on an x(:,xdim) < splitval comparison, 
% it terminates and reports the y value at that level in the tree. y values 
% are provided at each level of the tree, so this report is fast.
% If one were focused on accuracy, the routine could continue down the tree,
% following all possible branches, and then combine the resulting predictions.
% This would require some sophisticated logic or possibly recursion, however,
% and would slow this down. As it is, this eval routine is as fast as possible.
%
% SEE ALSO
%
% nb_train (handles missing values)
% entropic_significance
% discretize_continuous_vars
% [yest,prob,nodeID]=dtree_eval(x,dtree,missing_val);

% write a full missing_val version of dtree_eval
% dtree_prune
% costmatrix2caseweights
% caseweights in split_node and dtree_train


if isempty(dtree), 
  fprintf('dtree is empty, cannot evaluate\n');
  return;
end;


% No splits in tree
% ------------------

if size(dtree,1)==1, % there were no splits
  y = ones(size(x,1),1) * dtree(1,4);
  prob = ones(size(x,1),1) * dtree(1,5);
  nodeID = ones(size(x,1),1);
  return;
end;


% Missing value handling
% ----------------------

missing_enabled=0; % changed this from 1 to 0
if nargin<3,
   missing_val=nan;
   missing_enabled=0;
end;

% Adjust dtree for ease of finding terminal
% ------------------------------------------
% Look for a terminal node, by finding the nodeID dtree(:,1) entry as being 0.
% A 0 means this wasn't filled in during training. To prevent the checking for
% dtree(i,1)==0 from extending past the end of the size(dtree,1), we will 
% put in an artificial missing_val at the max possible check depth on the dtree
% This will force all values in between to assume 0 (implicitly because dtree
% is a sparse matrix)

dtree_size = size(dtree,1);
dtree(dtree_size.*2+2,2)=missing_val; 
% this should be further than the lowest possible dtree existence test
% note that dtree(furthestdown,1) is 0

% Initialize starting nodeID
% ---------------------------

  nodeID = ones(size(x,1),1).*2;
  newcase=[1:size(x,1)]';
  % these training cases have yet to be assigned
  % newcase gives the x case row which we are going to modify
  
% Main loop
% ----------
  
  while ~isempty(newcase),
     
    % check that we are not at the end of the tree
    terminal    = find(dtree(nodeID(newcase),1)==0);
    nonterminal = remove_ids([1:size(newcase,1)]',terminal);
    
    if ~isempty(nonterminal),
       
       % Determine the appropriate xvalue to test against splitval for each nonterminal
       % ------------------------------------------------------------------------------
       
%       xid = sub2ind(size(x),full(newcase(nonterminal)),full(dtree(nodeID(newcase(nonterminal))),2));
       xid = sub2ind(size(x),full(newcase(nonterminal)),full(dtree(nodeID(newcase(nonterminal)),2)));
       % this is the location in the x matrix of each xval<splitval test
       % these tests proceed in a vertical column corresponding to the nonterminals
       % remaining in the cases
       
      % Missing_value handling
      % ----------------------
      
      % There's a big in this block somewhere
      if missing_enabled,
        miss = find(x(xid)==missing_val | isnan(x(xid)));
        if ~isempty(miss),
           y(newcase(nonterminal(miss)),1)      = full(dtree(floor(nodeID(newcase(nonterminal(miss)))./2),4));
           prob(newcase(nonterminal(miss)),1)   = full(dtree(floor(nodeID(newcase(nonterminal(miss)))./2),7));
           nodeID(newcase(nonterminal(miss)),1) = full(dtree(floor(nodeID(newcase(nonterminal(miss)))./2),1)); 
           newcase = remove_ids(newcase,miss);
        end;
        pres = remove_ids([1:size(nonterminal,1)]',miss);
        nonterminal = nonterminal(pres);
        xid=xid(pres);
      end; % if missing_enabled
   
      % All non-terminals move to either the next left or right subbranch
      % ------------------------------------------------------------------
      
      down = x(xid) < dtree(nodeID(newcase(nonterminal)),3);
      i=find(down==1);
%     i  =  find(x(newcase(nonterminal(d)),dtree(nodeID(newcase(nonterminal(d))),2)) ...
%                                   < dtree(nodeID(newcase(nonterminal(d))),3));
      i2 = find(down==0);
      % i2 = remove_ids([1:size(nonterminal,1)]',i);%i2=i2';
      % find which of these nonterminals are right branches      
      if ~isempty(i),  nodeID(newcase(nonterminal(i)))  = nodeID(newcase(nonterminal(i))).*2; end;
      % skip to next left node
      if ~isempty(i2), nodeID(newcase(nonterminal(i2))) = (nodeID(newcase(nonterminal(i2)))+1).*2; end;
      % skip to next right node
   end;
   
   % Read out y values for cases which are at a terminal
   % ---------------------------------------------------
   
   if ~isempty(terminal),
       y(newcase(terminal),1)      = full(dtree(floor(nodeID(newcase(terminal))./2),4));
       prob(newcase(terminal),1)   = full(dtree(floor(nodeID(newcase(terminal))./2),7));
       nodeID(newcase(terminal),1) = nodeID(newcase(terminal),1)./2; % hopefully this pushes us back to the real parent. We were on a zero, now we're at the bonafide parent
%       nodeID(newcase(terminal),1) = full(dtree(floor(nodeID(newcase(terminal))./2),1)); 
% we used to have this code here
% instead of reporting the node that was the parent of the current node, 
% I'll report the current node
       newcase = remove_ids(newcase,terminal);
    end;
    
    fprintf('.');
    % ready for next loop
 end; % while
 
 % report these y values
 
 
