function [leafval,c,cy,prob,nocases] = dtree_leaves(dtree,missing_val);
%
% [boundingbox,c,cy,prob,nocases] = dtree_leaves(dtree,missing_val);
%
% returns a set of leaf bounding boxes created by 
% the decision tree in trying to predict y. The leaf bounding boxes 
% represent a discretization that is more effective than random 
% in predicting y. dtree_leaves also returns the centroids of the boxes c, the 
% predicted y cy, the probability or variance of these predicted values, 
% and number of cases
% 
% LIMITATION
% 
% This can only be used on decision trees built using one dimension
%
% EXAMPLE
%
% %%%% Return a decision-tree-created-discretization that will predict y
% 
% r = rand(100,1);
% dtree = dtree_train(r,r.^2,-9999,2,10);
% [leaf,c,cy,prob,cases] = dtree_leaves(dtree,-9999),
% 
% SEE ALSO
%
% dtree_train, dtree_eval, dtree_calc_significance_wrapper
% 

nodims = length(unique([full(dtree(:,2));0]')) - 1; 
if nodims>1, 
  fprintf('dtree_leaves should only be used for decision tree constructed on\n');
  fprintf('only one dimension. dtree apparently split on %d dimensions. Sorry\n',nodims);
  return;
end;

if nargin<2,
  missing_val = -9999;
end;


% Adjust dtree
dtree_size = size(dtree,1);
dtree(dtree_size.*2+2,2)=missing_val; 


leafval = zeros(size(dtree,1),2);
cy      = zeros(size(dtree,1),1);
prob    = zeros(size(dtree,1),1);
nocases = zeros(size(dtree,1),1);

l = 0;
present = sort(find(dtree(:,1)~=0));
present = remove_ids(present,1); % remove dtree's top-level node

for row=1:length(present),
  if dtree(present(row).*2,1)~=0, % this is a nonterminal
    % ignore this row
  else 
    if rem(present(row),2)==0, 

      % We're on a < if of a terminal
      % -----------------------------
      
      l=l+1;
      
      leafval(l,2) = full(dtree(present(row),3)); % this is a < constraint, 
      cy(l,1)      = dtree(present(row),4);
      prob(l,1)    = dtree(present(row),5);
      nocases(l,1) = dtree(present(row),6);

      % we need to now find the >= constraint that is immediately above in the tree
      % the >= constraint will be on an odd numbered level. Therefore, skip back 
      % through the tree levels until we arrive at an odd numbered level. This level 
      % will be the >= split, and we can read off the value that was in effect
      
      f = present(row);
      while f>1,
        f = f./2;
        if rem(f,2)==1, % found the >= split
          break;
        end;
      end;

      % if we did not locate an odd numbered level, meaning we travelled all the 
      % way back to level 1, then there was no lower bounding >= case in effect
      % In this case, all the values from -inf to current value are in effect

      if f==1, 
        leafval(l,1) = -inf; 
      else
        leafval(l,1) = dtree(f,3);
      end;
        
    else 
    
      % We're on a >= branch of a terminal
      % ----------------------------------
    
      l=l+1;
      leafval(l,1) = full(dtree(present(row),3)); % this is a >= constraint, 
      cy(l,1)      = dtree(present(row),4);
      prob(l,1)    = dtree(present(row),5);
      nocases(l,1) = dtree(present(row),6);

      f = present(row);
      while f>1,
        f = (f-1)./2;
        if rem(f,2)==0, % found the >= split
          break;
        end;
      end;    
    
      if f==1, 
        leafval(l,2) = inf; 
      else
        leafval(l,2) = dtree(f,3);
      end;

    end; % end which side of the terminal we're on
  end; % if on a terminal 
end; % foreach row that is present
     
leafval = leafval(1:l,:); % remove all of the zeros at the end
cy      = cy(1:l,:);
prob    = prob(1:l,:);
nocases = nocases(1:l,:);

[leafval,i] = sortrows(leafval,[1 2]); % sort by rows 1,2
cy = cy(i,:);
prob = prob(i,:);
nocases = nocases(i,:);

% Convert bounding boxes into centroids
% --------------------------------------
% 
% The centroid for inner boxes will be equal to the 
% center of the box. 
% The lower and upper boundaries are special because they 
% span all the way to +inf and -inf. We need to set a 
% centroid such that it is d distance away from the lower 
% penultimate boundary. This is so that the centroid for the 
% lower..lower+1 is still able to capture its boundary
% 
% -inf ----|----x----|----x---|------------
%      <--> <-->
%   eps+d     d
% 

c = leafval(:,1) + (leafval(:,2)-leafval(:,1))./2; % the center of each box

% note that c(1) and c(length(c)) will both be -inf and inf respectively)
% Special modification for end boxes now follows:

dist_to_boundary = c(2) - leafval(1,2);
c(1) = leafval(1,2)-dist_to_boundary-eps;

dist_to_boundary = leafval(size(leafval,1),1) - c(length(c)-1);
c(length(c)) = leafval(size(leafval,1),1) + dist_to_boundary + eps;

% this makes the thresholds for the discrete boundaries come out in order