function [dtree,valtree,cattree,var_contrib] = dtree_train(x,y,...
          missing_val,split_metric,min_cases, ...
          valtree, cattree, nodestr1,tabulist1);  
% 
% [dtree,valtree,cattree,variable_significance] = dtree_train(x,y, ...
%  missing_val,split_metric,min_cases, valtree,cattree,nodestr);  
% 
% Performs a complete decision tree expansion on the data, in order to 
% predict y. The method expands all nodes to their maximum limit,
% irrespective of information gain. All nodes are expanded based on best
% new entropy (or variance) result. For example, to predict y=x(:,1).^2
% the tree will return a series of rules such as "if 1>=2 then y=4", etc
% 
% INPUTS 
%
% min_cases the smallest number of cases allowed to be covered by a node 
% If there are less than this amount, then the tree returns 
% method==1 means entropy metric, ==2 means variance metric. Entropy 
% should be used for categorical y values, variance should be 
% used if the numbers of the y values form a continuum 
% 
% OUTPUTS 
%
% each row of dtree is [nodeID "if" dim "<" val "then y=" y "with var/prob" var/prob no_cases];
% 
% EXAMPLE 
%
% % figure out that the right answer is the square of the first dimension (variance metric)
% x=(round(rand(20,2).*7);y=x(:,1).^2;
% dtree = dtree_train(x,y,-9,1,2); % metric==entropy, min_cases==2
%
% % train a dtree to do addition (entropy metric)
% x=round(rand(20,2).*2);dtree = dtree_train(x,x(:,1)+x(:,2),-9,1,1);
% dtree_eval([1 2; 2 1;0 1; 0 0],dtree,-9)
%
% SEE ALSO
% 
% [dtree,valtree,cattree] = dtree_train(x,y, ...
%  missing_val,split_metric,min_cases, valtree,cattree,nodestr);  

% TODO:
% Implement legitimate gain
% Implement gini, twoing, and so on
% something's wrong with entropy

% Input Parameters
% ----------------

global nodestr;
global tabulist;
global experiment; 

experiment = 0; % 0 means no experiment % was 1

if experiment~=0,
  fprintf('\n **** Warning: running with experiment==%d, this should be set to 0 for ordinary dtree operation\n',experiment);
end;
if nargin>7, % nodestr1 was entered
   nodestr=nodestr1;
else 
   nodestr=num2str([1:size(x,2)]');
end;
if nargin>8, 
  tabulist = tabulist1;
else 
  tabulist = [];
end;

if nargin<3,
   fprintf('missing_val needs to be specified. Setting missing_val==-9999\n');
   missing_val=-9999;
   %return;
end;
if isnan(missing_val),
   fprintf('missing_val==nan not supported, sorry: setting missing_val=-999999\n');
   missing_val=-99999999;
   fprintf('recoding all nans to %d\n',missing_val);
   i=find(isnan(x));
   x(i)=ones(size(i,1),1).*missing_val;
   fprintf('completed\n');
   % not sure about execution. Nans will exist in the execution, check what happens
end;
if isempty(missing_val),
   missing_val=-9999;
end;
if nargin<4,
   fprintf('split_metric (1 or 2) needs to be specified. Setting split_metric==2\n');
   split_metric=2;
end;
if size(split_metric,2)>1, 
  % assume it is a function
  func = char(split_metric);
  split_metric = 9999;
else 
  func = split_metric;
end;
if (split_metric<1 | split_metric>4) & split_metric~=9999,
   fprintf('invalid split_metric (%d), possible values are 1,2,3,4 or a function name. Setting split_metric=2\n',split_metric);
   split_metric=2;
end;
if nargin<5,
   fprintf('min_cases needs to be specified. Setting min_cases==1\n');
   min_cases=1;
end;
if isempty(min_cases),min_cases=1;end;
if size(x,1)~=size(y,1),
  disp('x and y don''t have the same number of rows');
  return
end;

% Display Input Parameters
% ------------------------

disp(' ');
disp('dtree_train');
fprintf(' missing_val %f\n',missing_val);
fprintf(' split_metric==%d ',split_metric);
if split_metric==1,
  fprintf('[entropy]');
end;
if split_metric==2,
  fprintf('[variance]');
end;
if split_metric==3,
  fprintf('[entropy gain ratio == entropy gain normalized by entropy of xdim]');
end;
if split_metric==4,
  fprintf('[chi-square]');
end;
if split_metric==9999,
  fprintf('[%s]',func);
end;
fprintf('\n');
fprintf(' stop if fewer than %d cases\n',min_cases);
fprintf(' ---------------------------------------\n\n');

% Create valtree
% --------------
% list of values in each dimension

if nargin>=6,
  % valtree was supplied
  disp('using supplied valtree');
else
   valtree = construct_valtree(x,missing_val);
%disp('constructing valtree');
%valtree = sparse([]); 
%for d=1:size(x,2),
%  fprintf('.');
%  col = unique(full(x(:,d)));
%  col = setdiff(col,missing_val)';
  % find unique values
%  valtree(1:size(col,1),d) = col; 
  % put onto valtree
%  valtree(size(col,1)+1,d) = missing_val; 
  % add a missing value terminal after the last val
%end;
%fprintf('\n');
end;

% Create cattree
% --------------

if nargin>=7,
  disp('using supplied cattree');
else
   cattree = construct_valtree(y,missing_val);
%disp('constructing cattree');
%cattree = sparse([]); 
%for d=1:size(y,2),
%  col = unique(full(y(:,d)));
%  col = setdiff(col,missing_val)';
  % find unique values
%  cattree(1:size(col,1),d) = col; 
  % put onto valtree
%  cattree(size(col,1)+1,d) = missing_val; 
  % add a missing value terminal after the last val
%end;
end;

% calc entropy for top level
% --------------------------

dtree = sparse([]);
dtree(1,1) = 1;            % nodeID 1
dtree(1,2) = size(x,2);    % dims
dtree(1,3) = split_metric; % split_metric
dtree(1,6) = size(x,1);    % cases

if split_metric==1 | split_metric==3 | split_metric==4, % we may have to fix cases 3 and 4
  % Entropy top-level
  occurrences = zeros(size(cattree,1)-1,2);
  for i=1:size(cattree,1)-1,
    occurrences(i,1) = size(find(y==cattree(i,1)),1);
  end;
  [a,id] = max(occurrences(:,1)); 
  dtree(1,4) = cattree(id,1); % most common class
  dtree(1,5) = entropy(occurrences(:,1)'./size(y,1)); % entropy at top level
  dtree(1,7) = a./size(y,1); % probability
end;
if split_metric==2,
   % Variance top-level
   dtree(1,4) = mean(y);
   dtree(1,5) = var(y);
   dtree(1,7) = dtree(1,5);
end;
if split_metric==9999,
   % Custom func at top-level
   dtree(1,4) = mean(y);
   if (experiment==1), 
     dtree(1,5) = feval(func,x,y);
   else
     dtree(1,5) = feval(func,y);
   end;
   dtree(1,7) = dtree(1,5); % I could collapse 'var' as a func function!
end; 



% Splitinfo / entx
% ----------------

entx = zeros(1,size(x,2));
for v=1:size(valtree,2),
  novals = find(valtree(:,v)==missing_val);
  occurrences = zeros(novals,1);
  for i=1:novals,
    occurrences(i,1) = size(find(x(:,v)==valtree(i,v)),1);
  end;
  entx(1,v) = entropy(occurrences./size(x,1));
end;


% Induce dtree
% ------------

disp('inducing tree');
disp(' ');

dtree = split_node(1,x,y, ...
                   valtree,cattree(:,1),missing_val,func, ...
                   min_cases, ...
                   dtree,entx);

                
% send in entx, entropy of each xdim, in terms of prob(val) - ok, gotit
% when splitting on entropy,
% set (entorig-ent)./entx for that dim
%
% splitinfo = 

fprintf('\n%d number of cases total\n',size(x,1));
fprintf('induction complete\n\n');


% Calculate variable significance
% -------------------------------

% dtree_calc_significance_wrapper(dtree);