function [newloc_mean,   newloc_std, ...
          newloc_stderr, newloc_num] = estimate_mean_newregion( ...
                                              data,         ...
                                              newloc,       ...
                                              kernel_size,  ...
                                              missing_dims, ...
                                              missing_val);
%
% function [newloc_mean, newloc_std, ...
%           newloc_stderr, newloc_num] = estimate_mean_newregion(data,
%                                               newloc,
%                                               kernel_size,
%                                               dims_to_estimate, 
%                                               missing_val);
%
% Estimates mean of a subpopulation centered at newloc,
% with variance equal to kernel_size * stddevs of main population
%
% newloc       = vector (row) which is the new mean (val of interest
% should be specified, but won't be used)
% kernel_size  = width of subpopulation in standard deviations. 
% dims         = dims to be estimated in the new subpopulation
%                (the one centered at newloc)
%
% Given a large population of datapoints, assumes that this greater
% population covers a number of subpopulations, corresponding to 
% different regions. This being the case, we can estimate the mean
% for each of these subpopulations by examining the kernel around
% these areas.
%
% This provides a way of performing "what-if" analysis, to see if
% the population moved and become composed of a population around
% these local regions, what the mean of some value would be
%
% Example
%
% [nmean,nstd,nstderr,nnum] = estimate_mean_newregion(s2(tr,[37:43 236]),...
% [-9 -9 8 -9 -9 -9 -9 -9],1,[8],-9);
% nmean(1,8),nstd(1,8),nstderr(1,8),nnum(1,8)
%
% function mean_new = estimate_mean_newregion(data,newloc,kernel_size);
%

% standardize data and eliminate missing_dims
% -------------------------------------------
% first order of the day is to separate the dimensions which 
% need to be estimated

present_dims = setdiff([1:size(data,2)],missing_dims);
% missing_dims means that the dims are what we want to estimate
% in the new subpopulation. They aren't really missing, we just
% don't know them in the new pop. Therefore, we can think of them
% as being missing. The user provides the mean location for the other
% dims, and we have to estimate the missing dims.

missing_data = data(:,missing_dims);
present_rows = find(missing_data~=missing_val);
% save missing_data
% missing_data isn't standardized, we only use it at the end to impute a
% mean

data   = data(:,present_dims);   % data only consists of present_dims
newloc = newloc(:,present_dims); % newloc is also in terms of present dims

% standardize dimensions so that distance calc isn't biased by
% any dim having a larger scale

dataz = replace_nulls_with_mean(data,missing_val);
[dataz,meantrue,stdtrue]  = data2zscore(dataz);

% force_mean_std_missing procedure 
% force newloc to the same scale as the data above

for i=1:size(newloc,2),
  
  % replace missing values with the zscore mean (should be zero)
  jj=find(newloc(:,i)==missing_val);
  ii=setdiff(1:size(newloc,1),jj);

  if ~isempty(jj),
    % zscore for -9 is the mean which for zscore is 0
    newlocz(jj,i) = zeros(size(jj,1),1);
    %newlocz(jj,i) = ones(size(jj,1),1) * meanz(:,i); 
  end;

  if ~isempty(ii),
    % perform scaling for vectors that are there
    newlocz(ii,i) = ( newloc(ii,i) - ...
                      ones(size(ii,1),1) * meantrue(1,i) ) ./ ...
                    ( ones(size(ii,1),1) * stdtrue(ii,i)  );
  end;

end;
%keyboard
  %newlocz = -1.*newlocz;

% calc stddev (avg distance) of the general population
% ----------------------------------------------------

mean_dataz     = mean(dataz); % should be 0!
dist_mean      = distance(ones(size(dataz,1),1)*mean_dataz, dataz);
avg_dist_mean  = mean(dist_mean);

% we now have the stddev of the population. This stdev will be a
% basic kernel size. Find all points within avg_dist_mean*kernel_size from
% newloc

% calc every point's distance from newloc

% expand distance until we have enough points
within_and_present = [];
while size(within_and_present,1)<20,
  dist_newlocz   = distance(ones(size(dataz,1),1)*newlocz, dataz);
  within_kernel  = find(dist_newlocz < kernel_size .* avg_dist_mean);
  within_and_present = intersect(within_kernel,present_rows);
  kernel_size = kernel_size.^1.1;
end;

% we've now found all points that are within a kernel away from
% the newloc. We can simply take these points missing_dim values,
% and get the mean for them. This mean will be the mean for this
% subpop


newloc_mean = newloc;
[ newloc_mean(:,missing_dims),   newloc_std(:,missing_dims), ...
  newloc_stderr(:,missing_dims), newloc_num(:,missing_dims) ] = ...
  mean_missing(missing_data(within_and_present,:),missing_val);

% calcs mean, dropping missing_vals out of the mean

%keyboard
