function [err,yest] = exp_error(a,x,y,w);
% [err,yest] = exp_error(a,x,y,w);
%
% a is a 2x1 vector of parameters, x is a column vector, and y is a column vector
% w is a vector of weights (optional)
% Computes the root sum of squared error (ie. the Euclidean distance) 
% between an exponential fit of a demand curve, g(x) and 
% the known demand y. This error can be used to optimize the exponential model.
% The exponential model can also be optimized using analytic derivatives
%
% EXAMPLE
% 
% a = fminsearch('exp_error',[0.5;0.4],[],x(:,7),y);
% yest = exp(a(2).*x).*a(1);
% 
% SEE ALSO
%
% exp_error, pow_error, gutenberg_error, adaline_error, polyfitf, 
% 

if length(a)>2,
  % Use an additive term
  yest = exp(a(2).*x).*a(1) + a(3);
else
  % No additive term
  yest = exp(a(2).*x).*a(1);
end;

if nargin>2,
  % Do we have a y supplied. If so, calculate the 
  % error between our estimate and actual
  if nargin<4,
    % No weight term
    err = norm(yest-y);
  else
    % User did provide a weight term
    err = sum(  (w .* (yest-y)).^2);
  end;
else
  % No y was supplied. We cannot calculate the error
  % However, above we calculated a yest, and this 
  % may be useful for the user
  err = [];
end;  

%if a(2)>=0, err = inf; end;

