function [characteristic,demoprofile, cols, z, val, ... 
          baseline_val]=find_sig_cluster_attributes(profile, ...
                              centroidlist, ...
                              vec_mean, vec_std, ...
                              no_characteristics);
%
% [characteristic, demoprofile, ...
%  cols, z, val, baseline_val] = find_sig_cluster_attributes(profile, ...
%          centroidlist, vec_mean, vec_std, no_characteristics);
%
% Given a list of clusters, returns the most significant columns
% in each cluster. This is useful for characterising each cluster.
% 
% INPUTS
% profile      is a zscore centroids of the original data
% centroidlist is a list of the profile ids to record. Use 
%  [1:size(profile,1)]' if you want all of them
% vec_mean vec_std is the original data mean and std
%
% OUTPUTS
% characteristic is the list of ids which are significant
% cols         is the ids of the n columns which are significant
% v            is the zscores (stdevs) for those columns
% val          is the true value for those columns
% baseline_val is the mean value for those columns
% demoprofile  is a relational list version of the significant columns
% [profile_number, colID, stdevs_above_below_mean, val, baselineval]
%
% SEE ALSO
% [characteristic, demoprofile, ...
%  cols, z, val, baseline_val] = find_sig_cluster_attributes(profile, ...
%          centroidlist, vec_mean, vec_std, no_characteristics);


% Initialize
% ----------

characteristic = zeros(size(profile,1),no_characteristics.*2);
demoprofile    = sparse(size(characteristic,2).*size(centroidlist,1),6);
cols           = sparse(size(profile,1),no_characteristics.*2);
z              = sparse(size(profile,1),no_characteristics.*2);
val            = sparse(size(profile,1),no_characteristics.*2);
baseline_val   = sparse(size(profile,1), no_characteristics.*2);
fprintf('%d profiles to convert to relational list\n',size(centroidlist,1));

crange = [1:no_characteristics.*2];

% Collect the significant terms for each cluster (profile)
% --------------------------------------------------------

for i=1:size(centroidlist,1),
  fprintf('.');

  % Find the significant columns
  % -----------------------------

  [a,colid] = sort(profile(centroidlist(i,1),:));

  % top stdevs
  characteristic(centroidlist(i,1),1:no_characteristics) = ...
      flipud(colid(1,1:no_characteristics));

  % bottom stdevs
  characteristic(centroidlist(i,1), ...
      no_characteristics+1:no_characteristics.*2) = ...
         colid(1,size(colid,2)-no_characteristics+1:size(colid,2));

  % Select attributes from this profile which are unusually high or low
  % colid is the id of the column which has the high or low zscore
  % this flipping is done so that the ids are entered in the following
  % way: highest_to_lowest negative, highest_to_lowest_positive std
  % we can then do rem(id,no_characteristics)+1 to get the rank 1..no

  % Record these columns & their stdevs, etc
  % ----------------------------------------

  cols(centroidlist(i,1),crange) = characteristic(centroidlist(i,1),crange);
  z(centroidlist(i,1),crange)    = profile(centroidlist(i,1), characteristic(centroidlist(i,1),crange));
  val(centroidlist(i,1),crange)  = zscore2data(profile(centroidlist(i,1), ...
     characteristic(centroidlist(i,1),crange)), ...
     vec_mean(1,characteristic(centroidlist(i,1),crange)), ... 
     vec_std(1,characteristic(centroidlist(i,1),crange)));
  baseline_val(centroidlist(i,1),crange) = vec_mean(1,characteristic(centroidlist(i,1),crange));

  % Create the relational format list
  % ---------------------------------

  for ii=1:size(characteristic,2),
    demoprofile((i-1).*(size(characteristic,2))+ii,1:6) = ...
              [centroidlist(i,1)                  ...
               cols(centroidlist(i,1),ii)         ...
               z(centroidlist(i,1),ii)            ...
               val(centroidlist(i,1),ii)          ...
               baseline_val(centroidlist(i,1),ii) ...
               rem(ii-1,no_characteristics)+1];
  end;
end;
fprintf('\n');

% compact the relational list since may vals will be zero
% i=find(demoprofile(:,3)~=0);
% demoprofile = demoprofile(i,:);

