function [yest,rows_with_pattern,number_variables_present,       ...
          corr,meanabserr,meanerr] =                             ...
         ind_missing_predict(traininp,traindep, testinp,testdep, ...
                             missing_val,                        ...
                             degree,                             ...
                             min_training_pool);
%
% [yest, rows_with_pattern, number_variables_present,              ...
%          corr, meanabserr, meanerr] =                            ... 
%          ind_missing_predict(traininp,traindep, testinp,testdep, ...
%                             missing_val,                         ...
%                             degree,                              ...
%                             min_training_pool);
%
% train and test on same input case, and induce model for each input case
% using all input examples with the same pattern of missing values. 
%
% This runs a polyfit from input case + other cases which have same
% pattern of present values, to targets, and then applies this 
% induced model to case. 
% Therefore, this creates a model for each case, but does so by
% using the mappings from all other cases with the same pattern of
% present values in the population. This allows us to induce a model
% for each case, skirting missing values completely (uses the 
% maximum number of fields which are present in the row we are on
% to derive a model). 
%
% Notes: missing value pattern in train and testinput must be identical.
% (otherwise the model we induce on traininput won't be able to work on
% testinput due to new missing values in that vector)
%
% train is completely independent of test!!! 
% traininput and dep are just there as a pool of training cases, which
% can be used to induce a mapping for this missing val pattern.
% test can be a completely new set of points. Thus, we can put in
% test vectors which are customers with no previous history, and
% let it induce a mapping from the training set's demographics to rev.
% train and test can be different sizes. (we just use train as a pool
% of cases to pick out training examples from)
% 
% min_training_pool = smallest size of a training pool. Any smaller,
% and the program automatically drops the most constrained variable
% in succession, until the min_training_pool size is reached.
% min_training_pool must be >=1
%
% EXAMPLES
%
% % train on year1+demographics -> revyear2, 
% % test on  year2+demographics -> revyear3
%
% r = 1:1000; % records to use
%
% yest = ind_missing_predict( ...
%        [vec(r,5:7) vec(r,13:208)],vec(r,12), ...
%        [vec(r, 9:11) vec(r, 13:208)],vec(r,16), ...
%        -9, 8);
%
% % note1 that vec(train,13:208) is a demographic vector, the missing
% % value content won't change from one set to the other
% % note2 that the training and test records are identical. We are
% % training and testing on the same people. Training set is one
% % vect, test set is the second.
%
% % train on mapping from demo->revyear1, test on demo->revyear2
%
% r = 1:1000;
% 
% [yest,rows_with_pattern,vars_present] = ...
%        ind_missing_predict( ...
%        vec(r,13:208),vec(r,8), ... % trainingset, target
%        vec(r,13:208),vec(r,12), ... % testset, target
%        -9, 8);
%
% % note1 that training and test inputs are the same. This isn't always
% % the case, above we changed the revenue in the test set.
%
% [yest, rows_with_pattern, number_variables_present,              ...
%          corr, meanabserr, meanerr] =                            ... 
%          ind_missing_predict(traininp,traindep, testinp,testdep, ...
%                             missing_val,                         ...
%                             degree,                              ...
%                             min_training_pool);



rows_with_pattern        = zeros(size(testinp,1),1);
% number of rows with the missing value pattern of testinp(row,:)

number_variables_present = zeros(size(testinp,1),1);
% number of nonmissing values on this particular missing val pattern

% Dynamic programming:
% if programming efficiently, we would define a table, with each
% pattern combination a key (eg. a multidim array key), and holding a
% pointer to the weights for this model (or a -1 if not yet built)
% Then if we've already derived the model, no need to retrain.
%
% Dropping variables to increase training population:
% Is there a correlation between smallness of candidates, and 
% error of fit. If so, then we have to make sure we have at least n
% candidates, by dropping variables from the testinput.
%
% Effect of min_training_pool (dropping variables)
% smaller the min_training_pool, more chance of sporadic vals from
% small sample. larger min_training_pool, more reliable regression, 
% but eventually becomes the mean.
% 

ALLMISSING = 133; 
% this is the number of nonmissing_dims when all the vars are missing. If
% unsure, set this to -1 and watch the screen to see the allmissing case
% (pretty common, and slow)
% Below, when nonmissing_vals==ALLMISSING, we know all values are 
% missing, so use a pre-generated w_allmissing. In C code, we'd have
% stored all possible cases, and would perform lookup. Here, we'll
% just hardcode the most common case (allmissing, which accounts for 
% 1/3rd of the rows).


w_allmissing=[];

row=1;
while row<=size(traininp,1),
  %row=row+1;

  nonmissing_dims = find(testinp(row,:)~=missing_val);
  % find nonmissing_dims in testinp
  % testinp is the input we ultimately want the value for. 
  % Find out which dims are present.

  % if all missing, use w_allmissing
  if size(nonmissing_dims,2)==ALLMISSING & ~isempty(w_allmissing),
    % Its common for all the values to be missing, so 
    % use previously generated w_allmissing
    yest(row,:) = polyff(w_allmissing,degree,testinp(row,nonmissing_dims));
    disp([num2str(round(yest(row,:))) ' ' num2str(testdep(row,:))]);
    row=row+1;   
 else

  % Now find rows in the training pool with the same pattern of 
  % missing values. (we'll use these to induce a correct mapping)
  
  candidate_rows = [1:size(traininp,1)]';
  % initialize candidates in the training set
  
  %candidate_rows = [1:row-1 row+1:size(traininp,1)]';
  % this one skips the current test example (assuming train and test 
  % are identical

  i=0;
  redo=0; 
  % redo occurs if we can't find vars in the training pool
  % with the same pattern of missing_vars

  for dim=nonmissing_dims,
  % loop over each nonmissing column. Check the candidate set to
  % make sure its present

      cindex = find(traininp(candidate_rows,dim) ~= missing_val);
      candidate_rows = candidate_rows(cindex);

      % in the current candidates, keep all without a missing val
      % in the nonmissing_dims(dim) position
      % If we end up with no vars, we have to relax the inp and retry.

      % check the number of candidates (in case we're at 0)
      i=i+1;
      num_candidates(row,i) = size(cindex,1);
 
      if num_candidates(row,i)<min_training_pool,
      % oh no, NO cases in the training set match the test pattern
      % of missing values! We have to relax the exact match
      % constraint. We'll do this by finding the variable which caused the
      % most dramatic decrease in exemplars. Then remove that var
      % from the test variable. And try again.

        % find most dramatic decrease
        decrease = ...
        (num_candidates(row,1:i-1)-num_candidates(row,2:i)) ./ ...
        num_candidates(row,1:i-1);
        [m,dropdim] = max(decrease);dropdim=dropdim+1;
 
        % found the max decrease var, now drop it by mangling it
        % from the original input vector!
        testinp(row,nonmissing_dims(dropdim)) = missing_val;

        disp(['dropped input ' num2str(nonmissing_dims(dropdim))]);

        %row = row-1;
        redo = 1;
        break; % wish this could be used continue;
        % breaks out of the for loop, the redo flag forces skip of
        % calculation
      end;
  end;         

  if ~redo,

  % calc a regression on this selected "population" 
  % (have the same pattern of missing values as the individual 
  % we're trying to predict, so we have the maximum number of
  % present values for this individual's population)

  w = polyfitf(traininp(candidate_rows,nonmissing_dims), ...
               traindep(candidate_rows,:), ...
               degree);

  yest(row,:) = polyff(w,degree, ...
                       testinp(row,nonmissing_dims));

  if size(nonmissing_dims,2)==ALLMISSING, w_allmissing = w; end;

  % print info
  % -----------

  rows_with_pattern(row,1)        = size(candidate_rows,1);
  number_variables_present(row,1) = size(nonmissing_dims,2);
  disp(' ');
  disp(['row ' num2str(row)]);
  disp(['vars present ' num2str(number_variables_present(row,1)) ' rows with this missing value pattern ' num2str(rows_with_pattern(row,1)) ]);
  disp([num2str(round(yest(row,:))) ' ' num2str(testdep(row,:))]);
  
  row=row+1;
  end % ~redo
  end % calc w as normal

end;

