function [yest,id,centdist] = k_nearest_neighbour(x,c,cy,k,metric,missing_val);
%
% function [yest,id,centdist]=k_nearest_neighbour(x,c,cy,k,missing_val,metric);
%
% finds the k nearest centroid neighbours to each datapoint x.
% The k ids are listed across in order from nearest to farthest
% x data, c centroids, k number of neighbours to extract
%
% EXAMPLE
% % the following finds low nearest neighbours of high customers
% % neighbours are matched on demographic vars, eg. age,
% % no_stays, etc. Therefore, it finds customers who have the
% % same demographic profile as high spenders, but who are not
% % spending a lot. A validation of this choice is in the next year
% % if those customers spend higher than the mean.
% 
% %%%% find custs in low population who have same demo profile as high
% h=find(vec(:,24)>5000); % try to predict low by finding high neighbours
% l=find(vec(:,24)<=5000);
% [vecn,vecmean,vecstd] = data2zscore(vec); % each dim equal "weight"
% % in the distance calculation
% [yest,id] = ...                                                
% k_nearest_neighbour(vecn(h,[3:19]),vecn(l,[3:19]),vec(l,22),1);
% list=remove_duplicates(id(:));
% mean(vec(list,25)) % avg revenue for the low neighbours (next year)
% mean(vec(l,25))    % avg revenue for the population
% mean(vec(:,25))
%
% %%%% mean of custs selected using k nearest neighbour for values 1..k
% k=100; % number of nearest neighbours to retrieve as a match
% [yest,id,dist] = ...
% k_nearest_neighbour(vec2(h,neighblist),vec2(l,neighblist),vec2(l,12),k);
% for i=1:k,
%  a=id(:,1:i); list=remove_duplicates(a(:));
%  rev(i,1) = mean(vec(list,12));
% end;
% figure(1);clf;plot(1:k,rev);hold on;
% plot(1:k,ones(k,1)*mean(vec(:,12)));
% 
% %%%% show the neighbours of the points
% neighblist = [208 158]; 
% [yest,id] = ...
% k_nearest_neighbour(vecn(h,neighblist),vecn(l,neighblist),vecn(l,12),4);
% figure(1);clf;
% plot(vecn(h,neighblist(:,1)),vecn(h,neighblist(:,2)),'rx');hold on;
% plot(vecn(l,neighblist(:,1)),vecn(l,neighblist(:,2)),'bo');
% plot(vecn(list,neighblist(:,1)),vecn(list,neighblist(:,2)),'ro');
%
% %%%% show effectiveness of just sorting by present year revenue
% %%%% (exclude high)
% prop_of_set = 0.5;els = (prop_of_set.* (size(vec2,1)-1)) +1;
% [a,i] = sort(vec2(:,8)); % [a,i] = sort(vec2(:,12)); % theoretical up
% i=flipud(i);
% for cc=1:els, rev(cc,1) = mean( vec2(i(1:cc,1),12) ); end;
% figure(1);clf;
% plot(1:els,rev); hold on;
% plot(1:els,ones(els,1) * mean(vec2(l,12)));
% plot(1:els,ones(els,1) * mean(vec2(:,12)));
%
% %%%% ("theoretical") limit to total upsell possible in the year in
% %%%% question
%
% function [yest,id,centdist]=k_nearest_neighbour(x,c,cy,k,metric,missing_val);

% Parameters
% ----------

if nargin<5, % metric not supplied
   metric=0;
end;
if metric==-9,
   fprintf('Probable old function call: metric is -9. Usually this is a missing value\n');
   fprintf('Update the program which calls k_nearest_neighbour so that \n');
   fprintf('it uses the k,metric,missing_val form, rather than k,missing_val\n');
   keyboard
end;

if isempty(c),
  fprintf('k_nearest_neighbour c matrix of prototypes is empty, unable to find closest prototypes');
  yest = [];
  return
end;

if size(c,1)==1,
  yest=ones(size(x,1),1)*cy;
  id = ones(size(x,1),1);
  centdist = abs(x - c);
  return;
end;

if size(x,2)==1,
  if metric==-1,
    % Centroid for which x is greater than or equal to it, and for which 
    % the centroid is the closest. Ie. closest centroid less than or equal to x.
    % Example, Have a set of auction prices. If you bid x=$5.00, what centroid will 
    % we beat?
    diff = (x(:,1)*ones(1,size(c,1))-ones(size(x,1),1)*c');
    %keyboard;
    i = find(diff(:)<0);if ~isempty(i),diff(i)=nan.*ones(size(i));end;
  else
    diff = abs(x(:,1)*ones(1,size(c,1))-ones(size(x,1),1)*c');
    % this should be placed into the k_nearest_neihgbour routine
  end;
  [a,i] = sort(diff');
  id = i(1,:)';
  % if we really wanted to get rid of the nans we could use this
  % find(isnan(a(1,:)));
  yest = cy(id,:);
  centdist = diff;
  return
end;


% Non-missing_val k_nearest_neighbour
% -----------------------------------

if nargin<6, 
   if metric==0,
      %fprintf('distance metric\n');
      [mini,dist] = find_closest2(c,x);
   end;
   if metric==1, % means use correlation
      fprintf('correlation metric\n');
      dist=corrmat2(c',x')'; % should be a matrix with
      % correlation of c down, x across
      dist=1-dist; % if corr is 1, dist is 0, if corr is -1, dist is 2
   end; 
   if metric==2, % mean abs diff
     fprintf('metric is sum abs diffs\n');
     dist = zeros(size(x,1),size(c,1));
     for i=1:size(c,1),
       dist(1:size(x,1),i) = abs(x - ones(size(x,1),1)*c(i,:)) * ones(size(x,2),1);
       fprintf('.');
     end;
  end;
   if metric==3, % means use sum diff
     fprintf('metric is sum dimensional diffs\n');
     dist = zeros(size(x,1),size(c,1));
     for i=1:size(c,1),
       dist(1:size(x,1),i) = (x - ones(size(x,1),1)*c(i,:)) * ones(size(x,2),1);
       fprintf('.');
     end;
  end;
  if metric==4, % use dist between two earth surface lat,long positions
    fprintf('metric is earth surface latlong\n');
    if size(x,2)~=2, 
      fprintf('x does not appear to be a (lat,long) vector (it has %d dimensions).',size(x,2));
      fprintf('This method shouldn''t be used\n');
      return;
    end;
    dist = zeros(size(x,1),size(c,1));
    for i=1:size(c,1),
      dist(1:size(x,1),i) = latlong2dist(x(:,1:2),ones(size(x,1),1)*c(i,:));
    end;
  end;
end;


% Missing_val k_nearest_neighbour
% -------------------------------

if nargin>=6,
  disp('using missing val version');
  [mini,dist] = find_closest_missing(c,x,missing_val);
end;


% Find the centroid which is closest to each x point
% --------------------------------------------------

%disp('sorting distances to find min k matches for each row');
[cent,centroid_match] = sort(dist');
centroid_match=centroid_match';
cent=cent';
% cent contains the actual distance, and is not used
% (probably should use it)
% dist is c1x1 c1x2 c1x3 etc
% sort for each row. then flip it so that we have row again

% select the top k of the centroid_matches. Do this by simply
% extracting the first k of the list.
% we put them into an id list

if size(c,1)==1,
  yest = ones(size(x,1),1)*cy;
  id   = ones(size(x,1),1);
  return;
end;


id       = centroid_match(:,1:k);
centdist = cent(:,1:k);
% id contains the ids of the closest centroids to each datapoint row

yest = cy(id(:,1),:);
% use the closest neighbour's y value
% another scheme is to average all the k nearest neighbours

if metric==1,
   % using correlation metric
   % we converted corr into a dist measure so that the sort (above)
   % would work, and we'd get the strongest corrs. Now we convert 
   % that distance=esque measure back to a correlation, so that 
   % the output results are the real correlations
   centdist = -1.*(centdist-1);
end;
