function [c,cx,cy,member] = ...
         kmeans2(x,number_of_clusters, lc,number_of_loops,y,c);
%
% function [c,cx,cy,member] = kmeans(x,number_of_clusters,lc,iterations,y);
%
% This is a batch version of kmeans. The algorithm calculates distance
% from all points to each centroid in turn, and then adjusts each winning
% centroid towards all the input rows.
%
% EXAMPLES
% 
% clustering (or input compression)
% 
% x=rand(100,2);
% clusters=10;iterations=5;c = kmeans2(x,clusters,0.1,iterations);
% figure(1);plot(x(:,1),x(:,2),'ro');hold on;plot(c(:,1),c(:,2),'x');
%
% nearest neighbour function mapping
% 
% x = rand(100,2);y=rand(100,1);
% [c,cx,cy] = kmeans(x,clusters,0.1,iterations,y);
% yest = knn(x,c,cy,1); % lookup scheme=1, nearest cluster category
%
% To view cluster membership
% 
% x=rand(30,2);y=rand(30,1);
% [c,cx,cy,member] = kmeans(x,2,0.1,5,y); % 2 clusters
% a=find(member==1);b=find(member==2); 
% figure(1);clf;plot(x(a,1),x(a,2),'ro');hold on;plot(c(1,1),c(1,2),'rx');
% plot(x(b,1),x(b,2),'bo');plot(c(2,1),c(2,2),'bx');
%
% To view clusters being formed:
%
% if ~isempty(c), c=rand(size(c)); end;
% x=rand(30,2); y=rand(30,1);
% for it=1:20,
%   [c,cx,cy,member] = kmeans2(x,4,0.2,1,y,c); % 4 clusters
%   a=find(member==1);b=find(member==2);
%   cc=find(member==3);d=find(member==4);
%   figure(1);clf;plot(x(a,1),x(a,2),'ro');hold on;
% if ~isempty(a),plot(c(1,1),c(1,2),'rx');end;
% if ~isempty(b),plot(x(b,1),x(b,2),'bo');plot(c(2,1),c(2,2),'bx');end;
% if ~isempty(cc),plot(x(cc,1),x(cc,2),'go');plot(c(3,1),c(3,2),'gx');end;
% if ~isempty(d),plot(x(d,1),x(d,2),'mo');plot(c(4,1),c(4,2),'mx');end;
% end;
%  
% To view correctness of cluster category
%
% To test view which input is being used for the category vector cy
% (should be the closest input to the centroid, but this algorithm is
% on-line, so it can cause problems)
% 
% [c,cx,cy,member] = kmeans(x,2,0.1,2,y); % 2 clusters
% figure(1);clf;plot(x(:,1),x(:,2),'ro');hold on;plot(c(:,1),c(:,2),'x');
% plot(cx(:,1),cx(:,2),'go'); % closest inputs to each cluster.
%
% function [c,cx,cy,member] = kmeans(x,number_of_clusters,lc,iterations,y);



  % initialize clusters to the k random vectors in the input set
  % ------------------------------------------------------------

  if nargin==6, 
    % if cs supplied, don't initialize
  else
    % otherwise select random inputs for vectors
    i = ceil(rand(number_of_clusters,1).*(size(x,1)-1));
    c = x(i,:);
  end;

  xones         = ones(size(x,1),1);
  member        = xones .* -1;
  mini          = member;
  closest_input = ones(number_of_clusters,1).*-1;
  cy = [];
  cx = []; % these are only returned if y is supplied


  % kmeans loop
  % -----------

  for count=1:number_of_loops,
    disp(['iteration ' num2str(count)]);
    mini = find_closest(x,c);
    % find closest centroids (mini is a vect of these ids)

    c(mini,:) = c(mini,:) + lc .* (x-c(mini,:));
    % adjust the mini centroids
  end; % number_of_loops

  member(:,1) = mini; % input i belongs to centroid mini

  if nargin>=5,
    % create the cy vector
    % find the closest input to each centroid. Use this input's y
    % value for the cy value

    closest_inputs = find_closest(c,x);

    % now just get the y value for those inputs

    cx = x(closest_inputs,:);
    cy = y(closest_inputs,:);
    % (the category for the cluster is the same as the category for the
    % closest input to the category, ie. nearest neighbour)
  end;

return % return those centroids

