function g = loess_adaline(x,y,newx,alpha);
% 
% Curve fit using local non-parametric regression
% 
% PARAMETERS
% 
%  x,y			data points
%  newx,g		fitted points
%  alpha		smoothing typically 0.25 to 1.0
% 
% REFERENCE
% 
% Cleveland, W. S. (1979) Robust locally weighted regression and smoothing scatterplots. 
%  J. Amer. Statist. Assoc. 74, 829-836. 
% Cleveland, W. S. (1981) LOWESS: A program for smoothing scatterplots by robust locally weighted regression. 
%  The American Statistician, 35, 54. 
% 
% EXAMPLE
% 
% %%%% Look at loess for different versions of alpha
% x=randn(100,1);y=x(:,1).*2+randn(100,1);newx=[min(x):0.01:max(x)]';
% for alpha=0.1:0.1:1.0,g=loess(x(:,1),y(:,1),newx,alpha,1);figure(2);clf;plot(x(:,1),y(:,1),'.');
% hold on;plot(newx,g,'-');title(['alpha ' num2str(alpha)]);drawnow;end;
% 
% %%%% Compare my WLS routine against the loess least2
% figure(1);clf;plot(x,y,'.');g=loess(x,y,newx,0.6,1);hold on;plot(newx,g,'-');
% figure(2);clf;plot(x,y,'.');g=loess_adaline(x,y,newx,0.6);hold on;plot(newx,g,'-');
% 
% SEE ALSO
%
% [err,dtotalerr, ddtotalerr,derr,dderr,y] = adaline_eval(w,x,ytarget,caseweights);
% g = loess_adaline(x,y,newx,alpha);
% g = loess(x,y,newx,alpha,degree,robust); % This is in the datavis toolbox!
% 

lambda=1;
robust = 0;

n = length(x);      %  number of data points
q = floor(alpha*n);
q = max(q,1);
q = min(q,n);       %  used for weight function width
tol = 0.003;        %  tolerance for robust approach
maxiterations = 100;

%  perform a fit for each desired x point
for ii = 1:length(newx)
   deltax = abs(newx(ii)-x);     %  distances from this new point to data
   deltaxsort = sort(deltax);     %  sorted small to large
   qthdeltax = deltaxsort(q);     % width of weight function
   arg = min(deltax/(qthdeltax*max(alpha,1)),1);
   tricube = (1-abs(arg).^3).^3;  %  weight function for x distance
   index = tricube>0;  %  select points with nonzero weights

p=rand(1,2);
p=fmins('adaline_eval',p,foptions,[],[ones(size(x(index),1),1) x(index)],y(index),tricube(index));
%p=gradient_optimize('adaline_eval',p,[],[],0.000001,10,inf,inf,0.2,0,0,[ones(size(x(index),1),1) x(index)],y(index),tricube(index));

[err11,dtotalerr11, ddtotalerr11,derr11,dderr11,newg] =adaline_eval(p,[[1;1] [newx(ii);1]],[0;0],[0;0]);
newg = newg(1);
%disp ' past it!';
%   p = least2(x(index),y(index),lambda,tricube(index));  %  weighted fit parameters
%   newg = polyval(p,newx(ii));  %  evaluate fit at this new point


   if robust
      %   for robust fitting, use bisquare
      test = 10*tol;
      niteration = 1;
      while test>tol
         oldg = newg;
         residual = y(index)-polyval(p,x(index));  %fit errors at points of interest
         weight = bisquare(residual);  %  robust weights based on residuals
         newWeight = tricube(index).*weight;  %  new overall weights


p=fmins('adaline_eval',p,foptions,[],[ones(size(x(index),1),1) x(index)],y(index),newWeight);
%p=gradient_optimize('adaline_eval',p,[],[],0.000001,10,inf,inf,0.2,0,0,[ones(size(x(index),1),1) x(index)],y(index),newWeight);
% seconds, iterations, maxno, adaptive, l
[err11,dtotalerr11, ddtotalerr11,derr11,dderr11,newg] =adaline_eval(p,[[1;1] [newx(ii);1]],[0;0],[0;0]);
newg=newg(1);


%         p = least2(x(index),y(index),lambda,newWeight);  
%         newg = polyval(p,newx(ii));  %  revised fit


         niteration = niteration+1;
         if niteration>maxiterations
            disp('Too many iterations')
            break
         end
         test = max(0.5*abs(newg-oldg)./(newg+oldg));
      end
   end
   g(ii) = newg;
end
