
function [meanx,stdx,varx,countx] = mean_nomemory(x,chunksize,missingVal);
%
% [meanx,stdx,varx,countx] = mean_nomemory(x,chunksize,missingVal);
% 
% Calculates mean, std, and var, using a procedure that doesn't consume 
% a large amount of memory. If Matlab is asked to take a mean, it seems to duplicate 
% the dataset. It might even convert a sparse matrix into a full. This routine 
% avoids this problem. 
% 
% chunksize is the number of rows that can fit into memory at a time
%
% EXAMPLE 
%
% %%%% Take the mean of a matrix that would normally be too big to fit into memory
% x = rand(3000,50);
% [meanx,stdx,varx] = mean_nomemory(x,2000);
%
% 

if nargin<3, 
  missingVal = -9999;
end;

N = size(x,1);
meanx  = zeros(1,size(x,2));
sumx   = zeros(1,size(x,2));
sumsq  = zeros(1,size(x,2));
countx = zeros(1,size(x,2));

it=1;
no_iterations = ceil(N./chunksize);
while 1,

  % Figure out lower and upper range of chunk
  % ------------------------------------------
  
  lower  = (it-1).*chunksize+1;
  upper  = it.*chunksize;
  if upper>=N, upper=N; end;
  range1 = [lower:upper]';
  
  % Auto-extend mean vector length 
  % -------------------------------
  
  len = size(x(range1,2));
  if len > size(meanx,2), 
    meanx(1,len) = 0;
    sumx(1,len)=0;
    sumsq(1,len)=0; 
  end;
  
  % Calculate sum of this chunk
  % ----------------------------
  
  if nargin>=2,
    % missingVal version
    [s,count_notmissing] = sum_missing(x(range1,:),missingVal);
    countx = countx + count_notmissing;
  else 
    % normal version
    if length(range1)==1, 
      s = x(range1,:);
    else
      s = sum(x(range1,:));
    end;
  end;
  
  % Add sum to running total
  % ------------------------
    
  sumx  = sumx  + s;
  sumsq = sumsq + s.^2;
  %sumx  = sumx + sum(x(range1,:));
  %sumsq = sumsq + sum(x(range1,:).^2);
  if rem(it,5)==0, fprintf('%d/%d ',it,no_iterations); end;
  it=it+1;
  if upper>=N, break; end;
end;

if nargin<3, % no missing values
  countx = ones(1,size(x,2)) .* N;
end;



meanx = sumx./countx;
varx  = (sumsq - 2.*meanx.*sumx + countx.*meanx.^2)./countx;
stdx  = sqrt(varx);


% var(x) = 1/N sumof (x-meanx)^2
%        = 1/N sumof (x^2-2meanx*x + meanx^2)
%        = 1/N sumsq  -2*meanx*sumx + N.*meanx^2
