function [yest,prob,probdist,prob_xval_given_yval,yprior,valtree,cattree] = ...
   nb_train(x,y,missing_val,missing_handling,caseweights,valtree,cattree);
%
% [yest,prob,probdist,prob_xval_given_yval,yprior,valtree,cattree] = ...
%   nb_train(x,y,missing_val,missing_handling,caseweights,valtree,cattree);
%
% DESCRIPTION
%
% Performs a naive bayes induction to predict y given the values in x.
% naive bayes operates by (roughly) summing the weight of each x dimension onto y, and
% then choosing the yval which has the largest ultimte activation. Naive bayes is identical
% (with a product instead of a sum) to a linear regression or adaline, where all xvals and
% yvals are represented as 1,0 nodes. Thus we should get the same result if we ran a linear
% regression with these class codings
% missing_handling 1==ignore_missing, 2==average_present, 3==model_missing, 4==bayesian_probability
%
% EXAMPLES
%
% %%%%%%%%%%%%%%%% Performance on complete data
% no_cases=100;x=round(rand(no_cases,4).*5);y = x(:,1)+x(:,2); 
% tr=[1:round(size(x,1)./2)]';te=[round(size(x,1)./2):size(x,1)]';missing_val=nan;
% [yest,prob,probdist,prob_xval_given_yval,yprior,valtree,cattree] = nb_train(x(tr,:),y(tr,:),missing_val);
% yest2 = nb_eval(x(te,:),valtree,cattree,missing_val,prob_xval_given_yval,yprior);
% result(1,:)=[rank_correlation(yest,y(tr,:)) rank_correlation(yest2,y(te,:))];
% 
% %%%%%%%%%%%%%%%% Accuracy of different missing_val schemes
% xx=x;percent_missing=0.5;
% i=rand(size(tr,1).*size(x,2),1);[a,i]=sort(i);i=i(1:round(size(i,1).*percent_missing));
% x(i)=missing_val.*ones(size(i,1),1);
% yest  = nb_eval(x(tr,:),valtree,cattree,missing_val,prob_xval_given_yval,yprior);
% yest2 = nb_eval(x(te,:),valtree,cattree,missing_val,prob_xval_given_yval,yprior);
% result(2,:)=[rank_correlation(yest,y(tr,:)) rank_correlation(yest2,y(te,:))];
% for missing_handling=1:3,
%  [yest,prob,probdist,prob_xval_given_yval,yprior,valtree,cattree]=nb_train(x(tr,:),y(tr,:),missing_val,missing_handling);
%  yest2 = nb_eval(x(te,:),valtree,cattree,missing_val,prob_xval_given_yval,yprior);
%  result(2+missing_handling,:)=[rank_correlation(yest,y(tr,:)) rank_correlation(yest2,y(te,:))];
% end;
% fprintf('Trained and tested on Full data         \t%f\t%f\n',result(1,1),result(1,2));
% fprintf('Trained on full data, tested on missing \t%f\t%f\n',result(2,1),result(2,2));
% fprintf('Trained on missing data,  ignore_missing\t%f\t%f\n',result(3,1),result(3,2));
% fprintf('Trained on missing data, average_present\t%f\t%f\n',result(4,1),result(4,2));
% fprintf('Trained on missing data, model_missing  \t%f\t%f\n',result(5,1),result(5,2));
% 
% %%%%%%%%%%%%%%%% Weighted Bayes
% caseweights=ones(size(tr,1),1);
% for t=1:3,
%  [yest,prob,probdist,prob_xval_given_yval,yprior,valtree,cattree] = nb_train(x(tr,:),y(tr,:),missing_val,1,caseweights);
%  yest2 = nb_eval(x(te,:),valtree,cattree,missing_val,prob_xval_given_yval,yprior);
%  result(t,:)=[rank_correlation(yest,y(tr,:)) rank_correlation(yest2,y(te,:))];
%  i=find(yest~=y(tr,1));caseweights(i,1)=ones(size(i)).*10;
%  fprintf('Incorrect cases %d\n',i');
% end;
%
% SEE ALSO
% 
% [yest,prob,probdist,prob_xval_given_yval,yprior,valtree,cattree] = ...
%   nb_train(x,y,missing_val,missing_handling,caseweights,valtree,cattree);

% The Adding Problem
% Is this problem linearly separable? (or what are the characteristics)
% 0 0 0 symmetric so only do one side
% 1 0 1
% 1 1 2
% 2 0 2
% 2 1 3
% 2 2 4
% 3 0 3
% 3 1 4
% 3 2 5
% 3 3 6

% we're making a prediction from each dimension separately.
% The predictions (probs) are then multiplied (rather than added for an adaline)
% Therefore, let's look at the prediction each dim will make
% 
% If normally distributed, and x1 for first number, then expected(x2) = mean(x2), 
% Therefore, argmax_yval_prob(yval|x1)=expected(y|x1) = x1+max/2
% Other values of y trail off at normal distribution
%
% Now if we have two dims, then we will make this prediction twice and multiply the
% two. So we have:
% Pred(y|x1) = Normal(x1+max/2,std)
% Pred(y|x2) = Normal(x2+max/2,std)
% 
% If uniform, then given x1, all other values in x2 are equally likely, and we have:
% Pred(y|x1) = 1/max where x=x1+1..x1+max, and x=0 elsewhere
% Pred(y|x2) = Uniform(x2+1..x2+max) and 0 elsewhere
% Pred(y|x1,x2) = (x1+1..x1+max) .* (x2+1..x2+max)
% since x1,x2 both lay on the same range (1..max), we must have overlap at some point,
% meaning x1+i=x2+j. Let x2>x1, this overlap will occur when i=(x2-x1) and j=1, and
% continue until i=max and j=max-(x2-x1). Therefore we have:
% Pred(y|x1,x2) = Pred(y|x1).*Pred(y|x2)
%               = wherever 0 above, will be 0
%               = iff xval appears in all of the ranges above, it is constant prob
%    (regarding constant, in reality because of differences in samples, the probs will actually
%                vary (rather than having a true (1/max)^no_dims prob for each feasible
%                value)
%                 (x1+1 .* 0)...(x1+i .* x2+j) (x1+i+1 .* x2+j+1)
% Pred(y|x1,x2) = (x1+1 .* 0)... 2(x1+i) 2(x1+i+1) ... (x2+max(x2-x1))
% Pred(y|x1,x2) = basically an AND function on every (xi+1..xi+max). Find the remaining
% values which all reside within. 
% Pred(y|x1,x2) = x1+i 2(x1+i+1) x2+j
% 
% We could calculate difficulty of problem as a function of dims and number of vals:
% 
% for each dim, we have uniform(x1+1..x1+max). In calculating the correct addition,
% the final result will be 


% Need to extend this so it works with real valued inputs
% naive bayes and nearest neighbour?
% cluster with label, where label is adjusted using update rule
%  . . .  label (output cy) is migrates according to input winner, a VAM
% . . . . input is clustered

 
% Input parameters
% ----------------

if nargin<3,
   fprintf('missing_val must be specified. Defaulting to missing_val=Nan\n');
   missing_val=nan;
end;

% Missing_val handling
% ---------------------

ignore_missing       = 1;
average_present      = 2;
model_missing        = 3;
bayesian_probability = 4;

if nargin<4,
   fprintf('missing_handling defaulting to ignore_missing(%d)\n',ignore_missing);
   missing_handling=ignore_missing;
end;
if missing_handling==model_missing,
   % check that this is appropriate
   nogood=0;
   for i=1:size(x,2),
      if isnan(missing_val),
         ii=find(isnan(x(:,i)));
      else
         ii=find(x(:,i)==missing_val);
      end;
      if isempty(ii), 
        nogood=1;i=size(x,2)+1;
      end;
   end;
   if nogood,
      fprintf('missing_handling==model_missing(%d) is unlikely to work well because\n',model_missing);
      fprintf('some dimensions have no missing values in this training data.\n');
      fprintf('Resetting missing_handling==ignore_missing (%d)\n',ignore_missing);
      missing_handling=ignore_missing;
   end;      
end;
if missing_handling==bayesian_probability,
   fprintf('missing_handling==bayesian_probability(%d) not supported\n',bayesian_probability);
   fprintf('Resetting missing_handling==ignore_missing(%d)\n',ignore_missing);
   missing_handling=ignore_missing;
end;

% caseweights
% -----------

cwsupplied=1;
if nargin<5,
   caseweights=ones(size(x,1),1);
   cwsupplied=0;
end;
if isempty(caseweights),
   caseweights=ones(size(x,1),1);
   cwsupplied=0;
end;

% Display parameters
% ------------------

fprintf('\n');
fprintf('nb_train\n');
fprintf(' missing_val %f\n',missing_val);
if missing_handling==1,st='ignore_missing';end;
if missing_handling==2,st='average_present';end;
if missing_handling==3,st='model_missing';end;
if missing_handling==4,st='bayesian_probability';end;

fprintf(' missing_handling %d %s\n',missing_handling,st);
if cwsupplied, 
   fprintf(' caseweights supplied by user\n');
end;
fprintf('--------------------------------\n');



% Construct cattree and valtree
% -----------------------------

if nargin<7,
  %fprintf('constructing cattree\n');
  [cattree,no_catvals] = construct_valtree(y,missing_val);
else
   no_catvals = size(cattree,1)-1;
end;
if nargin<6,
  %fprintf('constructing valtree\n');
  [valtree,no_vals,valtree2,xrecoded] = construct_valtree(x,missing_val);
end;

% Create no_vals(dim)
% -------------------

  for d=1:size(valtree,2),
     if isnan(missing_val),
        no_vals(d,1)=min(find(isnan(valtree(:,d))))-1;
     else
        no_vals(d,1)=min(find(valtree(:,d)==missing_val))-1;
     end;
  end;


% Perform training
% ================


% Get priors for each y
% ---------------------

for yval=1:size(cattree,1)-1,
  yprior(yval,1) = sum(caseweights(find(y==cattree(yval,1)),1));
end;
%keyboard;
% Calculate probability of seeing every combination of xval, and yval
% --------------------------------------------------------------------
% we now have to furnish a table with prob(x=xval|y=yval) = count(x==xval and y==yval)/count(y=yval)
% prob_xval_given_yval has xdim down by xval down and yval across table has 

fprintf('attempting to allocate prob table (%d dimsx%d valsx%d yvals=%d values)...',size(x,2),size(valtree,1),no_catvals,no_catvals.*size(valtree,1).*size(x,2));
prob_xval_given_yval = zeros(size(x,2),size(valtree,1)-1,size(cattree,1));
fprintf(' allocated\ntraining\n');
for xdim=1:size(x,2),
   for xval=1:no_vals(xdim)+1, % we will go through missing case as well
      for yval=1:no_catvals,
         
         if xval==no_vals(xdim)+1, 
            
            % Missing value handling
            % ----------------------
            
            if missing_handling==ignore_missing,
              % in ignore_missing mode, the missing dimension simply isn't counted
              % this is equivalent to multiplying by 1
              prob_xval_given_yval(xdim,xval,yval)=1;
           end;
           if missing_handling==average_present,
              prob_xval_given_yval(xdim,xval,yval) = mean(prob_xval_given_yval(xdim,1:no_vals(xdim),yval));
           end;
           if missing_handling==model_missing,
              prob_xval_given_yval(xdim,xval,yval) = sum(caseweights(find(x(:,xdim)==valtree(xval,xdim) & y==cattree(yval)),1)) ./ yprior(yval);
           end;            
           if missing_handling==bayesian_probability,
              fprintf('bayesian probability handling not supported');return;
           end;
        else
           
           % Normal value handling
           % ----------------------
           
           prob_xval_given_yval(xdim,xval,yval) = sum(caseweights(find(x(:,xdim)==valtree(xval,xdim) & y==cattree(yval)),1)) ./ yprior(yval);
        end; % if xval==missing_val
        
      end; % yval
   end; % xval
   fprintf('.');
end; % xdim

% Eval
% ----

bayesian_prior = 0;
no_prior       = 1;
estimate_prior = 2;

priormode=bayesian_prior;%estimate_prior;%no_prior;%estimate_prior;%bayesian_prior;

% recode all xvals to indexes into valtree
[x2,count,no_vals] = data2valtree_index(x,valtree,missing_val);

prob_class=ones(size(x,1),no_catvals);
for xdim=1:size(x,2), % foreach dim
   prob_class = prob_class .* squeeze(prob_xval_given_yval(xdim,x2(:,xdim),1:no_catvals));
end;

%keyboard
if priormode==bayesian_prior, 
   prob_class = prob_class .* (ones(size(x,1),1)*yprior'); 
end;
if priormode==estimate_prior,
   for yval=1:size(cattree,1)-1,
     targ=zeros(size(y,1),1);
     i=find(y==cattree(yval,1));
     targ(i,1)=ones(size(i,1),1);
     yprior(yval,1) = prob_class(:,yval)\targ;
  end;
  prob_class=prob_class.*(ones(size(x,1),1)*yprior');
end;  
if priormode==no_prior,
   % prob_class stays the same, not multiplied by any prior
end;

probdist(:,1:no_catvals) = prob_class ./ (sum(prob_class')' * ones(1,size(prob_class,2))+eps);
% choose the highest probability
[prob,yval] = max(probdist');
prob=prob';yval=yval';
yest = cattree(yval,1);




% =============================================================================
% Following is old code (which still works - uncomment to check the above
% 

if 0,
yyest=yest;pprob=prob;pprobdist=probdist;
tic,
prob_class=[];
for i=1:size(x,1),
   for yval=1:no_catvals,
      prob_class(yval,1) = 1;
      for xdim=1:size(x,2),
         % find out what xval down x(i,xdim) corresponds to
         xval = find(valtree(:,xdim)==x(i,xdim));if isempty(xval),fprintf('error');return;end;
         % need to replace this, recode all x to values in valtree (1,2,3)
         % idvals = [1:size(x,2)]
         % then prob_class(yval,1)=prob_class(yval,1).*prob_xval_given_yval(xdim,x(i,xdim),yval)
         if isnan(missing_val),
            if ~isnan(x(i,xdim)),
               prob_class(yval,1)=prob_class(yval,1).* ...
                  prob_xval_given_yval(xdim,xval,yval);
            end;
            % missing value handling:
            % ignore, average, or prob
            % ignore means skip this,
            % average means lookup prob_xval_given_yval(xdim,missingdown,yval) and
            % missing case must have been calculated above in prob_xval_given_yval
            % prob means split off size(x,2) separate threads, multiply out, and then average
         else
            if x(i,xdim)~=missing_val,
               prob_class(yval,1)=prob_class(yval,1).* ...
                  prob_xval_given_yval(xdim,xval,yval);
            end;
         end; % isnan missing_val
      end; % foreach xdim
   end; % foreach yval
   
   %keyboard
   % multiply these probs of x1 and x2 and x3 given y ... 
   % with prob y
   % ie. prob of y occurrence, and then prob of the observed x values
   if priormode==0, prob_class = prob_class .* yprior; end;
   probdist(i,1:no_catvals) = (prob_class') ./ sum(prob_class);
   % choose the highest probability
   [prob(i,1),yval] = max(probdist(i,:));
   yest(i,1) = cattree(yval,1);
   fprintf('.');
 end;% foreach case
 toc,
 keyboard
 
 end;