function [xnew,v,s,covmat] = pca(x,dims_to_keep,method);
%
% [xnew,eigenvect,eigenval,covmat] = pca(x,dims_to_keep,method);
%
% Finds the principal components of the matrix x. 
% The principal components are typically the eigenvectors of the 
% covariance matrix of the data, which tells you how each dimension 
% varies with every other dimension. (Hence if two dimensions
% covaried with each other, they would in effect be collapsed 
% into one dimension). Principal compoments may be deceptive, 
% since each principal component is a projection through the data 
% made so as to minimize the resulting variance (error) from the 
% projection of the data onto the vector. Because of this, 
% you may inadvertantly collapse low variance dimensions 
% which never-the-less are important.
% 
% dims_to_keep is the number of principal components to return
% if method==0 or not specified, use the covariance matrix
% if method==1, then use a correlation matrix
% if method is a string, then execute a function by this name on the x data
% to generate a matrix for singular value decomposition
%
% EXAMPLES
% 
% %%%%%%%%%%%%% Verify that it works (compared to Matlab's stats toolkit method)
% 
% xx = rand(100,3);
% [xnew,eigenvect,eigenval,covmat] = pca(xx,3);
% [pc,score,latent,tsquare ] = princomp(xx);
% eigenvect, pc,
% 
% %%%%%%%%%%%%% PCA using a distance matrix
% 
% pca(xx,size(xx,2),'dist2');
%
% %%%%%%%%%%%%% Show principal axes using scatterplot graphically
% 
%  x = [];
%  width=0.2;no_centroids=1;t=[1:1000]';
%  colorid = ['b';'r';'c';'k';'m';'g'];
%  while 1,
%    c=[0.5 0.5];points=[];id=[];figure(1);clf;hold on;figure(2);clf;hold on;
%    for cid=1:no_centroids,
%      x(t,1:2)=ones(size(t,1),1)*c(cid,:)+(randn(size(t,1),2)).*(ones(size(t,1),1)*[rand+0.1 rand+0.1]).*width;
%      points=[points;x];id=[id;ones(size(t,1),1)*cid];figure(1);
%      plot(x(t,1),x(t,2),[colorid(rem(cid,7)+1) '.'],'markersize',4);
%    end;  
%    hold on;
%    axis([0 1 0 1]);
%    [xnew,v,s] = pca(x);
%    m = mean(x);
%    varx = var(x);
%    sumvar = 1./sum(varx);
%    for component=1:2,
%      plot(m(1) + [0 v(component,1).*s(component).*sumvar],m(2) + [0 v(component,2).*s(component).*sumvar],'-');
%      text(m(1) + v(component,1).*s(component).*sumvar,m(2) + v(component,2).*s(component).*sumvar, ...
%      char(['Component ' num2str(component) ' ']));end;drawnow;pause;end;
%
% %%%%%%%%%% Reduce dimensionality
% 
%  x = [];
%  width=0.2;no_centroids=1;t=[1:200]';
%  colorid = ['b';'r';'c';'k';'m';'g'];
%  while 1,
%    c=[0.5 0.5 0.5];points=[];id=[];
%    figure(1);clf;hold on;
%    for cid=1:no_centroids,
%      x(t,1:3)=ones(size(t,1),1)*c(cid,:)+(randn(size(t,1),3)).*(ones(size(t,1),1)*[rand+0.1 rand+0.1 rand+0.1]).*width;
%      points=[points;x];id=[id;ones(size(t,1),1)*cid];
%      plot3(x(t,1),x(t,2),x(t,3),'.'); %[colorid(rem(cid,7)+1) '.'],'markersize',4);
%    end;  
%    axis([0 1 0 1 0 1]);
%    view(30,30);
%    drawnow;
%    [xnew,v,s] = pca(x,2); % only want 2 dimensions please
%    figure(2);clf;plot(xnew(:,1),xnew(:,2),'.');
%    axis square;
%    drawnow;pause;
%  end;
%    
% 
% SEE ALSO
%
% [xnew,eigenvect,eigenval,covmat] = pca(x,dims_to_keep,method_or_covfun);

if nargin<2,
  dims_to_keep = size(x,2);
end;

if nargin<3,
  method=0;
end;

if ischar(method),
  method = char(method);
  covmat = feval(method,x);
end;
  
if method==0,
  covmat = cov(x); % Is this outside of statsitics toolkit? ./ size(x,1);
end;

if method==1,
  covmat = corrcoef(x);
end;

%if method==2,
%  covmat = distane
[u,s,v] = svd(covmat);

s = diag(s);
for i=1:length(s),
  fprintf('Principal vector %d accounts for %5.2f%% of variance\n',i,100.*(s(i)./sum(sum(s))));
end;

% Sort the eigenvalues


v = v(:,1:dims_to_keep);
s = s(1:dims_to_keep);

% Project the data onto this matrix

if dims_to_keep < size(x,2),
  xnew = x * [v';zeros(size(x,2)-dims_to_keep,size(x,2))];
else
  xnew = x * v';
end;
