function r = point_biserial(actual,pred)
% 
% r = point_biserial(1or0,continuous);
%
% Reports the relationship between a continuous and dichotomous variable.
% Will probe for whether the continuous values are low when the dichotomous value 
% is 0, and whether the continuous values are high when the dichotomous value is 
% 1. Does this by looking at the difference of the means between the continuous variables 
% conditional upon actual=1 versus actual=0. If the difference of the means is large, 
% it (should) mean that the values of the continuous variable are lower on 0 and higher on 
% 1 (although it is susceptible to outliers because of the use of the mean). Therefore, 
% it has to assume a distribution that is well behaved such as the normal distribution.
%
% Guilford, J. P., & Fruchter, B. (1973). Fundamental statistics in psychology 
%   and education. (5th ed.). New York: McGraw-Hill.
% Brown, J. D. (1988). Understanding research in second language learning: 
%   A teacher's guide to statistics and research design. Cambridge: 
%   Cambridge University Press. 
% 
% EXAMPLE
% 
% r = rand(20,1);
% actual = r < 0.4; 
% pred   = r+randn(size(r));
% point_biserial(actual,pred),
% 
% SEE ALSO
%
% r = point_biserial(1or0,continuous), 
% correlation, tstat, tstat_indep, tstat_paired, fstatf


% actual forecast

act0=find(actual==0);
act1=find(actual==1);

predmean0 = mean(pred(act0));
predmean1 = mean(pred(act1));
prop0     = length(act0)./length(actual);
prop1     = length(act1)./length(actual);
predstd   = std(pred);

r = ((predmean1 - predmean0) ./ predstd) .* sqrt(prop0.*prop1);

if isinf(r), r = 0; end;
if isnan(r), r = 0; end;

% Type of Correlation Coefficient Types of Scales 
% Pearson product-moment Both scales interval (or ratio) 
% Spearman rank-order Both scales ordinal 
% Phi Both scales are naturally dichotomous (nominal) 
% Tetrachoric Both scales are artificially dichotomous (nominal) 
% Point-biserial One scale naturally dichotomous (nominal), one scale interval (or ratio) 
% Biserial One scale artificially dichotomous (nominal), one scale interval (or ratio) 
% Gamma One scale nominal, one scale ordinal 

