function corr = rank_correlation(x,y,method);
% corr = rank_correlation(x,y,method);
% 
% method == 1 (very fast, least accurate) compare sort indices of the two vectors
% method == 2 (slower, more accurate) recoding method. This replaces each duplicate
%             number with a unique sequential order number. If there are a different
%             set of unique numbers in x and y, then these order ids will be different, 
%             which could give rise to error.
% method == 3 (very slow, very accurate) use full-blown rank method. This replaces each
%             set of duplicates with the average of their ranks
% method == 4 use Jon Garibaldi's rank routine to perform rank correlation. This is the
%             fastest!
%
% The accuracy of methods 1 and 2 will be impaired if there are any duplicate numbers
% in either x or y. If x and y are real values, then it should be safe to use method 1.
%
% EXAMPLE
% 
% for i=1:10,
% r1=round(rand(10,1).*(i+5));r2=round(rand(10,1).*(i+5));c1=correlation(r1,r2);c2=rank_correlation(r1,r2,1);
% c3=rank_correlation(r1,r2,2);c4=rank_correlation(r1,r2,3);fprintf('%f\t%f\t%f\t%f\n',c1,c2,c3,c4);end;
%
% See also
% convert_to_ranks


% Input Parameters
% -----------------

if nargin<2,
  fprintf('Not enough arguments supplied\n\n');
  help rank_correlation;
end;
if nargin<3,
  method=4;
end;

% Fastest method
% --------------

if method==1,
  [a,i] = sort(x);[a,rank_x] = sort(i);
  [a,i] = sort(y);[a,rank_y] = sort(i);
  corr = correlation(rank_x,rank_y);
  return;
end;

% Encoding method
% ---------------

if method==2,
  r1 = recode(x);
  r2 = recode(y);
  corr = correlation(r1,r2);
  return
end;

% Meticulous Ranking method
% --------------------------

if method==3,
  r = convert_to_ranks([x y]);
  corr=correlation(r(:,1),r(:,2));
  return
end;

% Jon Garibaldi method for creating ranks
% ----------------------------------------

if method==4,
   r=ranks([x y]);
   corr=correlation(r(:,1),r(:,2));
   return
end;