function [c,w,yest] = rbf_train(x,y,number_of_centroids,iterations,lc, ...
                                norml,spline_method,width);
%
% function [c,w,yest] = rbf_train(x,y,number_of_centroids,iterations, lc, ... 
%                                 norml, kernel, width);
%
% Trains spline-based models
% 
%   norml  == 1 normalize all activations to percentages of 1
%   kernal == 0..4
%
%   0: GAUSSIAN:
%      return (exp( -1* ((r*r)/(width)) ));
%   1: MULTI_QUADRATIC:
%      return ( sqrt(width + r*r) );
%   2: LINEAR:
%      return (1+r);
%   3: CUBIC:
%      return (1+r*r*r);
%   4: THIN_PLATE:
%      return (r*r*log(r));
%
%   width is a parameter used in gaussian and multiquadratic splines
%
% EXAMPLE
%
% x = [1:20]';y = sin(x);clf;plot(x,y,'r-');
% [c,w,yest] = rbf_train(x,y,7,10,0.2,1,4,0);
% % 5 centroids, 3 kmeans iterations, 0.1=lc, 1=normalized, 4=thinplate
% hold on;plot(x,yest,'b-');
%
% NOTES
%
% If you stop kmeans prematurely, centroids will still be migrating, and
% may be passing through regions not occupied by the function. The problem
% gets worse as you increase the number of centroids (since there will
% be lower movement for any particular centroid each iteration).
% If your spline returns 0 for everything, increase the number of
% iterations to kmeans and the problem should go away.
%
% Thin_plate splines often get log(0) errors. These need to be 
% dealt with if we use this in C. If r==0, return 0. (0*0*log(0)=0).
% 
% A Radial Basis Function is a spline with a gaussian kernel. 
%
% These splines are a little bit different from regular splines in 
% that their centroids are estimated using kmeans. In graphics, splines
% are always used on a uniform mesh. The switch is necessary because
% the functions we are approximating are not uniformly distributed over
% Rn. (unlike light rays). Kmeans is how rbfs estimate centroid
% spacing. Unfortunately its slow to run.
% function [c,w,yest] = rbf_train(x,y,number_of_centroids,iterations, lc, ... 
%                                 norml, kernel, width);


if spline_method==0,
  disp('gaussian');
end;
if spline_method==1,
  disp('multiquadratic');
end;
if spline_method==2,
  disp('linear');
end;
if spline_method==3,
  disp('cubic');
end;
if spline_method==4,
  disp('thin_plate');
  if norml==1,
    disp('warning: normalization may not work for thin plate splines');
  end;
end;

  c = zeros(number_of_centroids,size(x,2));
  c = kmeans(x,number_of_centroids,lc,iterations);
  
  % now calc dist to each centroid and put through nonlinear function
  % -----------------------------------------------------------------

  activ = zeros(size(x,1),number_of_centroids);
  for i=1:size(x,1),

    total_activ = 0;

    for cid=1:number_of_centroids,
      d = norm(x(i,:)-c(cid,:));
      
      if spline_method==0, % gaussian
         activ(i,cid) = exp( -1 .* (d*d) / width );
      end;
      if spline_method==1, % multi_quadratic
         spline_method==2;
      end;
      if spline_method==2, % linear
         activ(i,cid) = 1+d;
      end;
      if spline_method==3, % cubic
         activ(i,cid) = 1+d^3;
      end;
      if spline_method==4, % thin_plate
         activ(i,cid) = d*d*log(d); % add 1 to this to prevent neg vals
      end;

      total_activ = total_activ + abs(activ(i,cid));
    end;

    % normalize activations
    if norml==1,
      activ(i,1:number_of_centroids) = activ(i,1:number_of_centroids) ./ ...
        (ones(1,number_of_centroids) * (total_activ+eps));
    end;
              
  end;
  
  % now solve the linear weights which map x in cluster coords to y
  
  w = activ \ y;
  
  % perform the mapping for x, to give a training result
  
  yest = activ * w;

return
