function [roc_area,best_thresholdid,threshold,truepos,falsepos,trueneg,falseneg,totalwins,totalloss] = roc_curve(y,yest,roundfactor,showgraph,constraints,costmatrix);
%
% [roc_area,best_thresholdid,threshold,truepos,falsepos,trueneg,falseneg,totalwins,totalloss] = roc_curve(y,yest,roundfactor,showgraph,constraints,costmatrix); 
% 
% y           == actuals, must be column vector of 1 or 0
% yest        == predictions, any continuous number, assume that lower means 0 and higher means 1
% roundfactor == number such as 1000 that indicates how many thousandths to round to. Lower means faster. eg. 10 is very fast.
% showgraph   == 1 means show graphs
% constraints == 1x4 vector: The values in order are as follows:
%
%   mintruepos  == a truepos probability that must be met. eg. 0.8 means predictor must be able to 
%                  identify 80% of positives in the data. Allow the misclassifications to fall where 
%                  they might. Function will report on the threshold that will enable this criteria to 
%                  be achieved.
%   mintrueneg  == a trueneg probability that must be met. eg. 0.2 means that predictor must be able to 
%                  identify 20% of all negatives in the data. Function will report on the minimum threshold 
%                  that enables this classification goal to be achieved. Misclassifications, eg. 
%                  false negatives, and the like, can fall where they might.
%   maxfalsepos == a falsepos probability that the predictor must not exceed
%   maxfalseneg == a falseneg probability that the predictor must not exceed
% 
% costmatrix  == 1x4 vector. The values are [trueposvalue,truenegvalue,falseposcost,falsenegcost]
%                The cost matrix can be used after identifying a feasible region based on constraints, 
%                in order to return the best threshold
% 
% threshold_for_detailed_testing == a continuous number in the range of the predictions. 
%   This threshold will be used to threshold the predictions (turning them into 1s and 0s based on >= threshold), 
%   and the results of this will be analyzed in detail. For instance, you could use this to say: 
% Assume that y is a binary categorical variable, and yest is a continuous variable 
% that will try to estimate y
% 
% If in doubt with regard to roundfactor pass in []
%
% EXAMPLE
%
% %%%% Show some pretty graphs
% yest = rand(1000,1);y = round(yest-0.5);
% roc_curve(y,yest,[],1);
% 
% SEE ALSO
%
% 

% We could also run the various chi-square tests 


fast = 1; % if 1, use a fast method. If 0, use the slow but sure method

if fast==1,
%  fprintf('fast method\n');
else 
%  fprintf('slow method\n');
end;
if nargin>=3 & ~isempty(roundfactor),
  % Apply the rounding factor
  yest = round(yest.*roundfactor)./roundfactor;
  % This will histogram it
end;
if nargin<4,
  showgraph=0;
end;
if showgraph==1,
  %close all;
end;
if nargin>=5, 
  if length(constraints)>=1, mintruepos  = constraints(1);else mintruepos = -inf; constraints = -inf; end;
  if length(constraints)>=2, mintrueneg  = constraints(2);else mintrueneg = -inf; end;
  if length(constraints)>=3, maxfalsepos = constraints(3);else maxfalsepos = inf; end;
  if length(constraints)>=4, maxfalseneg = constraints(4);else maxfalseneg = inf; end; 
else
  constraints=[];
end;
if nargin>=6, 
else
  costmatrix=[1 1 1 1];
end;



% Begin work
% ----------

threshold = unique(yest); 

% find all unique values, this will become the 
% set of possible threshold points to use in testing for a discriminant threshold between
% positive and negative 
% For each of these cases, we now want to calculate the true positives and true negatives



% Initialization for speed
% -------------------------

%count = zeros(length(threshold),1);
%pos   = zeros(length(threshold),1);
%neg   = zeros(length(threshold),1);
%n     = zeros(length(threshold),1);

if fast, % This was the fast way of doing it. But fucking complicated

  r = [1:size(y,1)]'; % remaining yest cases that have yet to be analyzed 
  for thresholdid=1:length(threshold),
    % For reasons below, if we just went up incrementally, the vector 
    % when we perform a cumsum, would be the sum of 1..i, which would therefore 
    % calculate the sum of all things below the cutpoint!
    % By going from top to bottom, the cumsum will always go from n..i, which will 
    % always take points above the cutpoint
    i = find(yest(r)==threshold(thresholdid));
    pos(thresholdid) = sum(y(r(i)));
    neg(thresholdid) = length(i)-pos(thresholdid);
    n(thresholdid)   = length(i); % this is the number of positive forecasts
    r = remove_ids(r,i);
  end;

  % We now know, for each thresholdid, the number pos and number neg

  warning off;
  % above we have counted the number of wins, losses, etc, for items <= value
  % now we're going to count them for > value
  % on the last bin, we'll have none. This will cause a div by zero warning

  %if sum(n)~=size(y,1), fprintf('problem1\n');keyboard;end;
  %tot = sum(n); % shouldn't this be the size of y? Check it
  tot = size(y,1);
  
  % forecasted wins is tot-cumsum(n), cumsum(n) at a point is the number of 
  % cases that were below the cutpoint!
  % assume the cutpoint is <=!

  totalwins = sum(y);
  totalloss = length(y)-totalwins;

  forecastedwins = tot-cumsum(n);                      % We forecasted this number of wins below the given threshold
  matches        = totalwins-cumsum(pos);                    % The number of positives below the threshold
  winrate        = matches ./ forecastedwins;

  mismatch = totalloss-cumsum(neg);                        % the number of losses below the threhsold

  truepos  = matches ./ totalwins;               % number of win - wins below threshold / wins available
  falsepos = mismatch ./ totalloss;               % number of mismatches in predicting loss below threshold / losses available
  trueneg  = (-1.*mismatch + totalloss) ./ totalloss; % This number of wins are forecasted below the threhsold. Above the threshold we've forecasted 0s
  %trueneg  = (cumsum(neg)) ./ totalloss; % This number of wins are forecasted below the threhsold. Above the threshold we've forecasted 0s

%  totalneg = cumsum(n);
%  falseneg = (totalneg-cumsum(neg)) ./ totalwins;

   totalneg = -1.*forecastedwins +tot;
  falseneg = (totalneg-(-1.*mismatch+totalloss)) ./ totalwins;
 

else

  totalwins = sum(y);
  totalloss = length(y)-totalwins;
  
  for thresholdid=1:length(threshold),
    yy = yest >= threshold(thresholdid);
    correctpos(thresholdid)   = length(find(yy==y & yy==1));
    correctneg(thresholdid)   = length(find(yy==y & yy==0));
    incorrectpos(thresholdid) = length(find(yy~=y & yy==1));
    incorrectneg(thresholdid) = length(find(yy~=y & yy==0));
  end;
  
  totalcorrect   = correctpos   + correctneg;
  totalincorrect = incorrectpos + incorrectneg;  
  totalpos       = correctpos   + incorrectpos;
  totalneg       = correctneg   + incorrectneg;

  truepos  = correctpos   ./ totalwins; % % of true cases that were classified
  trueneg  = correctneg   ./ totalloss; % true 0 cases per 0 cases - % of 0 cases that were classified
  falsepos = incorrectpos ./ totalloss; % falsepositives per occurrence of a 0 case
  falseneg = incorrectneg ./ totalwins; % falsenegatives per occurrence of a 1 case - there could be more misdiagnosed negatives than 

  % This is the thing that I don't understand, 
  % false positives could be > 1!
  
end;



% Try to meet constraints
% -----------------------

best_threshold = nan;
if ~isempty(constraints),
  i = find(truepos>=mintruepos & trueneg>=mintrueneg & falsepos<=maxfalsepos & falseneg<=maxfalseneg);
  % Of these, select the remaining cases with the max truepos+trueneg
  % Here we're making an assumption that we care about them equally
  % Could introduce a cost matrix here

  feasible_region = i;  
  if isempty(i), 
    fprintf('Constraints were infeasible. Using cost matrix\n');
    i = [1:length(threshold)]';
  end;
  
  [a,ii] = max(   + truepos(i)  .* costmatrix(1) .* totalwins ...
                  + trueneg(i)  .* costmatrix(2) .* totalloss ...
                  - falsepos(i) .* costmatrix(3) .* totalwins ...
                  - falseneg(i) .* costmatrix(4) .* totalloss ...
                 ); 
  i = i(ii);

  best_threshold = threshold(i);
  best_thresholdid = i;

%keyboard;
  if showgraph,
    figure;clf;hold on;
    axis([0 1 0 1]);
    
    % Draw the yellow-filled feasible region  
    % --------------------------------------
    
    if length(feasible_region)>1,
      v=axis;
      max_threshold = max(threshold(feasible_region));
      min_threshold = min(threshold(feasible_region));
      h=fill([min_threshold min_threshold max_threshold max_threshold], ...
           [v(3) v(4) v(4) v(3)],'y'); % This has to go on first, so that all other lines are drawn over it
      set(h,'EdgeColor',[1 1 0]); % yellow
    end;
    
    % Basic plot of threshold versus truepos and falsepos!  
    % ----------------------------------------------------

    plot(threshold,truepos,'b-','linewidth',3);
    plot(threshold,trueneg,'c:','linewidth',3);
    plot(threshold,falsepos,'ro-','linewidth',3);
    plot(threshold,falseneg,'g+-','linewidth',3);
    legend('truepos','trueneg','falsepos','falseneg');
    xlabel('predictor variable thresholded >= this value');ylabel('Probability');
    title('Prediction performance, yellow region contains feasible solutions given constraints');
    v = axis; % just in case it moved
    
    % Draw a line for the best threshold
    % -----------------------------------
    
    if ~isnan(best_threshold),
      plot([best_threshold best_threshold],[v(3) v(4)],'--','linewidth',2); % plots the optimum
    end;

    % Add in constraint intercepts
    % -----------------------------

    if length(feasible_region)>1,
      if mintruepos >-inf, plot([v(1) v(2)],[truepos(best_thresholdid)  truepos(best_thresholdid)],'b-','linewidth',3);end;
      if mintrueneg >-inf, plot([v(1) v(2)],[trueneg(best_thresholdid)  trueneg(best_thresholdid)],'c-','linewidth',3);end;
      if maxfalsepos< inf, plot([v(1) v(2)],[falsepos(best_thresholdid) falsepos(best_thresholdid)],'r-','linewidth',3);end;
      if maxfalseneg< inf, plot([v(1) v(2)],[falseneg(best_thresholdid) falseneg(best_thresholdid)],'g-','linewidth',3);end;      
    end;    

  end;
%  keyboard;
  
end;

fprintf('best_threshold was %f\n',best_threshold);
%keyboard;

% Generate the ROC curve
% -----------------------
%keyboard;
[group,pgroup,count] = group_by_numerical([1;falsepos';0],[ [1;truepos';0] [min(threshold);threshold;max(threshold)]] ); % will do mean by default
% WE NEED TO STOP TAKING MEANS OF MEANS

if showgraph,
  figure;clf;scatter(group,pgroup(:,1),count.*40,count,'^','filled');
  title('ROC curve');
  xlabel('falsepos');ylabel('truepos');
  colorbar;

  figure;clf;plot(group,pgroup(:,1),'-');hold on;
  plot([group(1) group(length(group))],[0 1],'r-');
  legend('ROC Curve','random',4);
%  set(gca,'XColor','r','YColor','r');
  tic = get(gca,'xtick')';
  val = k_nearest_neighbour(tic,group,pgroup,1);
  val = round(100.*val)./100;
  %set(gca,'xticklabel',[num2str(val(:,1)) ones(length(tic),1) * '/' num2str(val(:,2))]);
  set(gca,'fontsize',8);
  xlabel('false positive');
  ylabel('true positive');
  ax1 = gca;
  ax2 = axes('Position',get(ax1,'Position'),...
           'XAxisLocation','top',...
           'YAxisLocation','right',...
           'Color','none',...
           'XColor','k','YColor','k');
  set(ax2,'xtick',tic);
  set(ax2,'xticklabel',num2str(val(:,2)));
  set(ax2,'fontsize',8);
  xlabel('predictor variable threshold');
  ylabel('true positive');
  grid on;

  figure;clf;hold on;
  plot(threshold,truepos.*totalwins,'b-');
  plot(threshold,falsepos.*totalloss,'r-');
  plot(threshold,trueneg.*totalloss,'c:');
  legend('truepos','falsepos','trueneg');
  xlabel('predictor variable thresholded >= this value');ylabel('Number of cases');

  if ~isnan(best_threshold), 
    threshtest=0.5;
    %confusion_matrix2(y,yest>=best_threshold,5);
    if 1,
      [nuh,id] = k_nearest_neighbour(threshtest,threshold,[1:length(threshold)]',1);
      figure(5);clf;hold on;
      bar3([truepos(id) trueneg(id) ; falsepos(id) falseneg(id)]);
      xlabel('Outcome');
      ylabel('Forecast');
      set(gca,'xtick',[1 2]);
      set(gca,'xticklabel',['True ';'False']);
      set(gca,'ytick',[1 2]);
      set(gca,'yticklabel',['Pos';'Neg']);
      title(['Confusion matrix at threshold = ' num2str(threshtest(1))]);
      view(30,30);
    end;
  end;
  
  % What about a distribution, Distribution of yest values, and the predicted win/loss prob curve for each
  %roundfactor2=1000;
  %[group,pgroup,count] = group_by_numerical(round(threshold.*roundfactor2)./roundfactor2,[truepos' falsepos']); 
  %% WE HAVE TO FIX THIS BLOODY THING truepos and falsepos are means. Don't use mean of mean!
  %figure(4);clf;hold on;plot(group,pgroup(:,1),'-');plot(group,pgroup(:,2),'r-');
  %legend('truepos','falsepos');

end;


% Calculate area under the ROC curve
% ----------------------------------

%  width  = diff(group);
%  midpoint = diff(pgroup(:,1))./2; % difference between 2 and 1, 
%  height   = pgroup(1:length(pgroup)-1,1) + midpoint;

  width  = diff(group);
  height = 0.5 .* (pgroup(2:size(pgroup,1),1) + pgroup(1:size(pgroup,1)-1,1));

  roc_area = sum(height.*width);
  
warning on;