function [c,w,yest] = spline_train(x,y,number_of_centroids, ...
                                   iterations,lc, ...
                                   norml,spline_method,width,caseweights);
%
% function [c,w,yest] = spline_train(x,y,number_of_centroids,
%                                    iterations, lc, ... 
%                                    norml, kernel, width);
%
% Trains spline-based models
% 
%   norml  == 1 normalize all activations to percentages of 1
%   kernal == 0..4
%
%   0: GAUSSIAN:
%      return (exp( -1* ((r*r)/(width)) ));
%   1: MULTI_QUADRATIC:
%      return ( sqrt(width + r*r) );
%   2: LINEAR:
%      return (1+r);
%   3: CUBIC:
%      return (1+r*r*r);
%   4: THIN_PLATE:
%      return (r*r*log(r));
%
%   width is a parameter used in gaussian and multiquadratic splines
%
% EXAMPLE
%
% x = [1:20]';y = sin(x);clf;plot(x,y,'r-');
% [c,w,yest] = rbf_train(x,y,7,10,0.2,1,4,0);
% % 5 centroids, 3 kmeans iterations, 0.1=lc, 1=normalized, 4=thinplate
% hold on;plot(x,yest,'b-');
%
% NOTES
%
% If you stop kmeans prematurely, centroids will still be migrating, and
% may be passing through regions not occupied by the function. The problem
% gets worse as you increase the number of centroids (since there will
% be lower movement for any particular centroid each iteration).
% If your spline returns 0 for everything, increase the number of
% iterations to kmeans and the problem should go away.
%
% Thin_plate splines often get log(0) errors. These need to be 
% dealt with if we use this in C. If r==0, return 0. (0*0*log(0)=0).
% 
% A Radial Basis Function is a spline with a gaussian kernel. 
%
% These splines are a little bit different from regular splines in 
% that their centroids are estimated using kmeans. In graphics, splines
% are always used on a uniform mesh. The switch is necessary because
% the functions we are approximating are not uniformly distributed over
% Rn. (unlike light rays). Kmeans is how rbfs estimate centroid
% spacing. Unfortunately its slow to run.
%
% function [c,w,yest] = spline_train(x,y,number_of_centroids, ...
%                                    iterations, lc, ... 
%                                    norml, kernel, width);

   show_splines(spline_method,norml);

   c = zeros(number_of_centroids,size(x,2));
   tmp = zeros(size(x,1),1); % tmp used for cy

  % find knot points

   %c = kmeans_batch2(x,number_of_centroids,lc,iterations);
   c = memlookup_train(x,tmp,number_of_centroids);
  
  % calc dist to each centroid and 
  % put through nonlinear function

  activ = spline_activ(x,c,norml,spline_method,width);

  % solve the linear weights which map x in cluster coords to y
  disp('solving mapping from coords to y');  
  if nargin<9, % no caseweights
     w = activ \ y;
  else
     w=rand(1,size(activ,2));
     w=fmins('adaline_eval',w,foptions,[],activ,y,caseweights);
     w=w';
  end;
  
  % perform the mapping for x, to give a training result
  
  yest = activ * w;

return
