function [varlist_out,degree_out,F_series,cumvar,totalcontrib] = ...
   stepwize(data,dep,criterion,Fconf,degree,write_to_file,display_output); %,
            %resample_its, traintest_percent);
%
% function [v,deg,F,cumF] = ...
%   stepwize(data,dep,criterion,min_improvement,maxdegree,savedisk);
%
% criterion=1 == F, criterion=2 == R2, criterion=3 == meanabs
%
% Performs stepwize forward regression. Begins with an empty model,
% tests the effect of adding each variable, selects the variable which
% results in the maximum decrease in criterion.
%
% criterion can be used to change between Rsq, F and meanabs error
% each can result in different variabels being added to the model.
% min_improvement is the minimum criterion change required to allow a
% variable to be added (or to prevent an old variable from being
% dropped).
% 
% Example
%
% % plant a relationship into the variable, y using a handful of
% % variables (the others are all random). The problem is to correctly
% % add only the variables which have a true relationship.
% % We'll conveniently select variables 10,20,30,40 to be the hidden
% % variables.
%
% x=rand(1000,40);y=x(:,20).^2 - x(:,10).^4 - x(:,30).^3 - x(:,40).*2;
% pause off;
% [v,deg] = stepwize(x,y,2,0.001,3);
% % v,deg should equal v   [20 10 30 40]
% %                    deg [ 2  4  3  2]
%
% % here's a faster one
% pause on;
% x = rand(100,4); y=x(:,1).^2 + x(:,2).^3 - 5.*x(:,3);
% [v,deg,vals] = stepwize(x,y,2,0.001,3);
% % v,deg should equal v   [1 2 3] 
% %                    deg [2 3 1]
%
% In the example the form of the model is important. If we define
% a 1st degree model, stepwize won't pick up on variables which
% have 2nd and 4th degree relationships with y, and doesn't add
% them. A nice demonstration that model form matters.
%
% Running large regressions
%
% Before running type "pause off". This bypasses the prompting.
% out_stepwize and out_val in the current directory contain the 
% last iteration's model and the F_series associated with those 
% vars. These files can be viewed while the function is running.
% 
% Example
% 
% [v,deg] = stepwize(x,y,2,0.001,3,1);
% !cat out*
%
% function [v,deg,criterion_val,cumF] = ...
%   stepwize(data,dep,criterion,min_improvement,maxdegree,savedisk);

% todo:
% convert stepwize so that when we add a variable, we train on 
% training set, get ws, run ws on test, get effect. continue.
% then resample (randomize training set) and continue

%maxtrain = round(traintest_percent .* (size(data,1)-1))+1;


% Parameters
% ----------

if nargin<6, % write_to_file not included
  write_to_file=0;
end;
if isempty(write_to_file),
  write_to_file=0;
end;

if nargin<7,
   display_output=1;
end;
if isempty(display_output) | display_output<0 | display_output>1,
   fprintf('display_output should be 0 (no output) or 1 (output)\n');
   display_output=1;
end;

warning off;

if criterion==1, criterion_str = 'F '; end;
if criterion==2, criterion_str = 'R^2 '; end;
if criterion==3, criterion_str = 'Mean Absolute Error '; end;
if criterion<1 | criterion>3, 
  disp(' ');
  disp('criterion should be 1..3, defaulting to F');
  criterion_str = 'F ';
  criterion=1;
end;

%if display_output==1,
disp(' ');
disp([criterion_str ' Max degree ' num2str(degree)]);
%end;

% if we get in degree as a vector, or if degree is 2 or more, then
% we have to create a multiple set of variables for these higher
% degree terms. We then perform the stepwize, and choose to keep
% or remove these higher or lower degree terms.

% modify the data matrix so that it contains new "variables",
% these variables are just higher degree terms

  if size(degree,2)==1, 
    % turn degree into vector
    degree = ones(1,size(data,2)) .* degree;
  end;

  disp('converting data to vander');

  original_data_size = size(data,2);

  % create a vander
  datat = data;
  [data,id,iddeg] = create_vanderf(data,degree);
  data(:,1)=[]; % omit the first col (all 1s)

  % now we just have to remember to report that these "vars" 
  % are just higher degree versions of the original variables

F_series = [];
currentF = -Inf;
varlist = [];

while size(varlist,2)<size(data,2),

  candidatelist = setdiff(1:size(data,2),varlist);

  best_partialF = -1.*Inf;
  best_candidate = nan;

  for candidate_var=candidatelist,
    % add every outstanding variable to the varlist
    % calc F from adding each
    
    %for res=1:resample_its,
       % randomize data and choose a sample
       
    % calc the F resulting from adding candidate_var
    partialF(candidate_var,1) = ...
      f_statf(data(:,[varlist candidate_var]),dep,1,criterion);

    [v,d,st] = vanderid_to_varid(candidate_var+1,id,iddeg);
    if partialF(candidate_var,1)>best_partialF,
      best_candidate = candidate_var;
      best_partialF  = partialF(candidate_var,1);
      if display_output==1, 
      disp([st ' ' num2str(partialF(candidate_var,1)) ' *']);
      end;

%      disp([num2str([candidate_var]) ' ' ...
%            num2str(partialF(candidate_var,1)) ' *']);
    else
       if display_output==1,
       disp([st ' ' num2str(partialF(candidate_var,1))]);
       end;
%      disp([num2str([candidate_var]) ' ' ...
%            num2str(partialF(candidate_var,1))]);
    end;
  end;

  % pick the best candidate_var (best partialF)

  if (best_partialF-currentF < Fconf),
    if display_output==1,
    disp('no improvement through best addition. stopping');
    end;
    break;
  end;

  % add best
  % --------

  currentF = best_partialF;
  varlist = [varlist best_candidate]; % add it to the varlist
  F_series = [F_series currentF]; % update F progression

 %  disp(['adding ' num2str(best_candidate) ' (' num2str(currentF) ')']);
    [v,d,st] = vanderid_to_varid(best_candidate+1,id,iddeg);
    if display_output==1,
      disp(['adding ' st ' (' num2str(currentF) ')']);
    end;

  % remove vars which are now lower than Fconf
  removelist = [];
  for var=varlist,
     without_var = setdiff(varlist,var);
     if ~isempty(without_var),
       F_without_var = f_statf(data(:,without_var),dep,1,criterion);
       if currentF-F_without_var < Fconf,
         % add it to the removelist

  % disp(['removing ' num2str(var) ' (' num2str(F_without_var) ')']);
    [v,d,st] = vanderid_to_varid(var+1,id,iddeg);
    if display_output==1,
      disp(['removing ' st ' (' num2str(F_without_var) ')']);    
    end;
         removelist = [removelist var];
       end;
     end;
  end;

  [varlist,i] = setdiff(varlist,removelist);
  F_series    = F_series(1,i); 
  % update the F_series (remove the removed vars from F_series)


  % write running results to file (enables us to view progress in auto)
  [varlist_out,degree_out,st] = vanderid_to_varid(varlist+1,id,iddeg);
  if display_output==1,
     disp(['varlist ' st]);
  end;
  if write_to_file,
    fid=fopen('out_stepwize.txt','w');
    fprintf(fid,'%s\n',st'); fclose(fid);
    save out_val.txt     F_series    -ascii
    save out_degree.txt  degree_out  -ascii
    save out_varlist.txt varlist_out -ascii
  end;

  if isempty(varlist),
    disp('varlist empty, check this');
    keyboard
  end;
 
  % display the F and R2
  
  if display_output==1,
    disp(' ');
    disp([criterion_str ' ' num2str(currentF)]);
    disp('ready to continue');
  end;
  pause

end; % continue to add vars
  
disp('completed');
warning on;


% now we have to translate the vander data matrix columns
% which have been selected, to find out what the original
% variable was which gave rise to them (and what power that
% column is)
% col 200, etc
% vander is
% dim1 powers, dim2 powers, etc.
% for each col,
% count up dim1 powers, dim2 powers, etc,
% until we reach the col.
% or count up, check 

if 0,

varlist_out = []; 
degree_out  = [];

% these are lists of vars (original ids), and underneath it
% the degree of that id

col_id = 1; 
% we start at 1 instead of 2 because no 1s in this vander

for i=1:original_data_size,
  for deg=1:degree(1,i),
    if ismember(col_id,varlist),
      % got one, report this varid (i) and the degree (deg)
      varlist_out = [varlist_out   i];
      degree_out  = [degree_out  deg];
    end;
    col_id = col_id + 1;
  end;
end;

% now let's look at id and iddeg
end;

%keyboard


  % Report form of model
  [varlist_out,degree_out,st] = vanderid_to_varid(varlist+1,id,iddeg);
  fprintf('----------------------------------------------\n\nForm of model:\n');
  disp(['varlist ' st]);
  
  if display_output==1,    
  % WE SHOULD ADD A RANKCORR METRIC
  
  
  % following is new, report the cummulative F or R accounted for
  
  fprintf('\n');
  
  if criterion==3, 
     baseline = (-1.*mean(mean(abs(dep-ones(size(dep,1),1)*mean(dep)))));
     % in F and R, our baseline is 0, ie. that is the prediction
     % accuracy with no terms in the model. (R and F are both like correlation
     % so a flat line is 0). But meanabserror has a baseline prediction accuracy
     % of the meanabserror around the mean, expressed as a negative. All
     % variance (meanabs) contributions must be subtracted from that baseline
     % constant level
     varlevel = F_series - baseline;
     baseline=-1.*baseline;
  else
     varlevel = F_series;
  end;
  
  v = varlist_out;
  deg = degree_out;
  
  % varlevel will be overwritten so it is invalid in the course of this loop
  [a,i] = sort(varlevel); % sort Flevelss in order from low to high
  for ii=1:size(i,2),
     cumvar(i(ii)) = varlevel(i(ii));
     varlevel=varlevel-cumvar(i(ii)); % remove that variance from all others 
     % (since its accounted for by this var)
  end;
%  fprintf('Stepwize variable contributions (metric == ');
%  if criterion==1, fprintf('F)\n');end;
%  if criterion==2, fprintf('R)\n');end;
%  if criterion==3, fprintf('MeanAbsDiff)\n');end;
     
  [a,i] = sort(v);
  fprintf('Dim^Pow\t');
  if criterion==2 | criterion==3, fprintf('Contrib\t');end;
  if criterion==1, fprintf('F');end;
  if criterion==2, fprintf('R');end;
  if criterion==3, fprintf('MeanAbsDiff');end;
  fprintf('\n');
  
  for ii=1:size(i,2),
     fprintf('%d^%d\t',v(i(ii)),deg(i(ii)));
     if criterion==3, fprintf('%3.0f%%\t',100.*cumvar(i(ii))./baseline);end;
     if criterion==2, fprintf('%3.0f%%\t',100.*cumvar(i(ii)));end;
     fprintf('%f\n',cumvar(i(ii)));
  end;
  
  fprintf('total\t');
  if criterion==2, fprintf('%3.0f%%\t',100.*sum(cumvar));end;
  if criterion==3, fprintf('%3.0f%%\t',100.*sum(cumvar)./baseline); end;
  fprintf('%f ',sum(cumvar));
  % value
  if criterion==2,fprintf('(max %f)',1.0);end; % baseline extra
  if criterion==3,fprintf('(max %f)',baseline);end; % baseline extra
  fprintf('\n\n');
  
  if criterion==3, cumvar = [cumvar;cumvar./baseline];end;
  
  % Contribution by dimension
  % -------------------------
  % I now have a v and cumvar, which is the variance contribution
  % for each var with the others subtracted. Now summarize this again, 
  % by grouping together the same variable, and adding their contribution.
  
  fprintf('Dim\t');
  if criterion==2 | criterion==3, fprintf('Contrib\t');end;
  if criterion==1, fprintf('F');end;
  if criterion==2, fprintf('R');end;
  if criterion==3, fprintf('MeanAbsDiff');end;
  fprintf('\n');
  
  totalcontrib = zeros(1,original_data_size);
  for i=1:original_data_size, % foreach dimension
     ii=find(v==i);
     if ~isempty(ii),
       % add up the contribution from that var
       totalcontrib(1,i)=sum(cumvar(size(cumvar,1),ii));
     else
       totalcontrib(1,i)=0;
     end;
     if criterion==2 | criterion==3, 
        fprintf('%d\t%3.0f%%\t%f\n',i,100.*totalcontrib(1,i),totalcontrib(1,i));
     else
        fprintf('%d\t%f\n',i,totalcontrib(1,i));
     end;
  end;
  if criterion==2 | criterion==3,
     fprintf('total\t%3.0f%%\t%f',100.*sum(totalcontrib),sum(totalcontrib));
     if criterion==2, fprintf(' (max %f)',1.0);end;
     if criterion==3, fprintf(' (max %f)',baseline);end;
     fprintf('\n');
  else
     fprintf('total\t%f\n',sum(totalcontrib));
  end;
     
%varlist_out = id(1,varlist+1);
%degree_out  = iddeg(1,varlist+1);

end; % if display_output