function [dat,modelform,corr,meanabs,meanerr] = timeseries_imputation(timeaxis,dat,criterion,min_improvement,maxdegree,missing_val,last_resort,showplot);
% [dat,modelform,corr,meanabs,meanerr]=timeseries_imputation(timedate,dat,criterion,min_improvement,maxdegree,missing_val,last_resort,showplot);
%
% EXAMPLE
% 
% dat = [1 2 -9 4 -9 -9 7 8 9 -9]'; % 1..10 but with missing vals
% 
% [ddat,modelform]=timeseries_imputation([1:size(dat,1)]',dat,2,0.02,3,-9);
% ddat, 
% [new1,new2,new3,new4] = impute_missing_vals(dat,dat,-9,2,-1,1);
% [new1 new2 new3 new4],
%
% last_resort is used if a dimension has no present values. If last_resort
% is not provided, that dimension is not touched (will continue to have its all
% or all except one missing)
%

% assume all points to estimate have been coded as missing vals

if nargin<7,
   last_resort = missing_val;
end;
if nargin<8,
  showplot=1;
end;

original_dat = dat;

pause off;
modelform = [];
modelform = ones(size(dat,2),1).*nan;
corr      = ones(size(dat,2),1).*nan;
meanabs   = ones(size(dat,2),1).*nan;
meanerr   = ones(size(dat,2),1).*nan;

%timeaxis = [1:size(dat,1)]';

for dim=1:size(dat,2),
   
   % find all present dimensions
   % ---------------------------
   
   if isnan(missing_val), 
      present = find(~isnan(dat(:,dim)));
   else
      present = find(dat(:,dim)~=missing_val);
   end;
   
   if length(present)==size(dat,1), % max(size(present)) 
      % no imputation necessary, set values equal to the known values
   else
     missing = setdiff([1:size(dat,1)]',present);
     if length(present)<2, % 1 or 0 are special cases, no imputation can be performed
        fprintf('warning: dimension %d has %d rows are present. %d\n',dim,size(present,1));
        if length(present)==1,
           % impute the constant to everything
           dat(missing,dim) = ones(size(missing,1),1).*dat(present,dim);
           fprintf('constant imputed to everything\n');
         end;
         if length(present)==0,
            dat(missing,dim) = ones(size(missing,1),1).*last_resort;
            fprintf('imputation of last_resort (%d) imputed to everything\n',last_resort);
         end;
     else
      
      % Construct a model based on present datapoints in the series
      % ------------------------------------------------------------
      
      % Note, we could use the values of all the other dims as series
      % to predict this dimension. But many of the other dims will have
      % missing values on them.
      % Try running the hotdeck imputation code.
      % To avoid this problem, use the ultra simple procedure of just
      % estimating the dimension based on its own series of values
      
      if std(dat(present,dim))<10.*eps,
         % stepwise will crash on this - all zeros gives an r of div by zero
         % impute 0 to everything
         dat(missing,dim) = ones(size(missing,1),1).*dat(present(1),dim);
      else
     % run stepwize to determine best model form
     [v,deg,critval] = ...
        stepwize(timeaxis(present,1),dat(present,dim),criterion,min_improvement,maxdegree,0);
     
     % train this model on the present datapoints to get the weights
     w=polyfitf(timeaxis(present,1),dat(present,dim),deg);
     
     % execute the model on the missing datapoints to get values for them     
     imputed_vals = polyff(w,deg,timeaxis(missing,1));
          
     dat(missing,dim) = imputed_vals;
     modelform(dim,1:size(deg,2))=deg;
     
     % Try to estimate the accuracy of the imputation (by looking at prediction versus actual on 
     % training points)
     trainingset_estimates = polyff(w,deg,timeaxis(present,1));
     corr(dim,1) = rank_correlation(trainingset_estimates,dat(present,dim));
     meanabs(dim,1) = mean(abs(trainingset_estimates-dat(present,dim))) ./ mean(abs(dat(present,dim)-mean(present,dim)));
     meanerr(dim,1) = mean(trainingset_estimates-dat(present,dim));
     
     % note that next loop, we could use this now full series as another
     % dimension to predict the next dim with missing vals
  end;   
  end;
  end;
end; % for


%pause on;

if showplot, % Display imputation
   for i=1:size(dat,2),
      if isnan(missing_val),
         missing = find(isnan(original_dat(:,i)));
         present = find(~isnan(original_dat(:,i)));
      else
         missing = find(original_dat(:,i)==missing_val);
         present = find(original_dat(:,i)~=missing_val);
      end;
      
      if length(present)~=size(dat,1),
      figure(1);clf;
      plot(timeaxis(present,1),original_dat(present,i),'x');hold on;
      plot(timeaxis(missing,1),dat(missing,i),'o');
      plot(timeaxis,dat(:,i),'-');
      str = ['Imputation for dim ',num2str(i),...
         ' degree [',num2str(modelform(i,1:size(modelform,2))), ']', ...
         ' corr ',num2str(corr(i,1)), ...
         ' meanabsper ',num2str(meanabs(i,1))]; %, ...
 %        ' meanerr ',num2str(meanerr(i,1)));
      title(str);
      pause;
   end;
   end;
 end;
 
return


   
      
   

