function [A,id,iddeg] = create_vanderf(x,degree)
%
% V = create_vanderf(x,degree);
%
% creates an overdetermined vandermonde matrix. This is a matrix
% of the form  [1  x1^1  x1^2  x1^3  and so on for x2 
%               1  x1^1  x1^2  x1^3
%               1  x1^1  x1^2  x1^3];
%
% x is a data matrix (observations down, dims across)
% degree is either a scalar (eg. 1,2), or a vector for each x dimension.
% if degree is a vector, then each degree is applied to each dim
% in the data matrix.
%
% Example
% 
% x = rand(100,4); y=rand(100,1);
% create_vander(x,[1 2 1 4]);
% 
% % creates a model of the form 
% % [x1^1    x2^1 x2^2    x3^1     x4^1 x4^2 x4^3 x4^4]
% 

if size(degree,2)>1,
  % we have a degree array
  % calc the total number of cols which will be in the vander
  % matrix. This used to be dim*degree+1, now we have
  % variable degrees on each dim, so we have to explicitly
  % calc sum of each degree col (ie. just sum each of the 
  % degree els, 1, 4, 2, 1, etc
  vander_size = sum( degree' ) + 1;
else
  % we have a scalar
  % turn the scalar into a degree array
  % ie. every dim has this as its max power

  vander_size = size(x,2) * degree + 1; 
  % vander size is dim*degree+1

  degree = ones(1,size(x,2)) .* degree;
  % degree is now a vector of [degree degree degree etc]
end;    

id    = zeros(1,vander_size); 
iddeg = zeros(1,vander_size);
% these are a vector which map the dim of the current vander
% back to the id of the original variable, and the degree
% which has been applied to that var

A = ones(size(x,1),vander_size);

col_id = 2; % start on col2
% col_id keeps track of what column we are on. Previously
% we calculated (dim-1)*degree+i+1. Now we just increment
% col_id every time we cover a dim.

for dim=1:size(x,2),
  for i=1:degree(1,dim); % go up to the degree on this dim
     %A=[A x(:,i).^i];
     %A(:,(dim-1)*degree+i+1) = x(:,dim).^i;

     A(:,col_id) = x(:,dim).^i;
 
     id(1,col_id)    = dim;
     iddeg(1,col_id) = i;
     % remember the dim and degree of this new column
     % we've created
 
     col_id = col_id+1;
  end;
end;

%disp 'form of matrix  1 x1^1 x1^2 x1^3 x2^1 ...'

