function [meanabs,r,absaroundmean,rankr] = crossval(x,y,degree,                     ...
                               number_of_clusters, iterations, ...
                               lc, norml,                       ...
                               kernel, width,                   ...
                               model,k);
%
% [meanabs,r] = crossval(x,y,degree, number_of_clusters, iterations, ...
%                        lc, norml, kernel, width, model,k);
%
% k is the number of crossval groups
% Other parameters are specified in general_model
%
% Performs k-fold cross-validation using model model. Reports the mean abs
% error and correlation for this model, after dividing up its data into k
% training sets, training on k-1 sets, and evaluating on the remaining set
%
% SEE ALSO
% 
% general_model_train, general_model_eval


  % randomize x
  % -----------

  [a,i] = sort(rand(size(x,1),1));
  xtrain = x(i,:);
  ytrain = y(i,:);
  
  foldsize = round(size(x,1)./k); % number of elements in each group
  foldsize = foldsize - 1; 
  % to be safe, make the foldsize 
  % one less than its maximum. This ensures that we will not outstrip
  % our training data


  % divide x into k folds
  % ---------------------

  fold = [];
  for i=1:k,
    fold(1:foldsize,1:size(x,2),i) = ...
      xtrain((i-1).*foldsize+1:i.*foldsize,:);
    foldy(1:foldsize,:,i) = ytrain((i-1).*foldsize+1:i.*foldsize,:);
  end;


if number_of_clusters>foldsize, 
  % eg. number_of_clusters was all of the
  % datapoints, and our fold is only 1/5th the number
  % If we have more clusters than points in our fold, reset 
  % number_of_clusters for the crossval testing only
  fprintf('crossval error estimation note: number_of_clusters in model %f, ',number_of_clusters);
  fprintf('is less than the number of points in a k-fold (%f).',foldsize);
  fprintf('Resetting number_of_clusters = %f.');
  fprintf('This change is local to crossval error estimation only');
  number_of_clusters = foldsize; 
end;

  % perform a train and eval on all folds minus i
  % ---------------------------------------------

  for i = 1:k,

    % build a training set of every fold except i    
    xtrain = [];
    ytrain = [];
    for ii=1:k,
      if ii~=i, 
        xtrain = [xtrain;squeeze(fold(:,:,ii))]; 
        ytrain = [ytrain;squeeze(foldy(:,:,ii))];
      end;      
    end;
      
    % train on this set (every fold except i)
    [c,w,yest] = general_model_train(xtrain,ytrain,degree, ...
       number_of_clusters, iterations, lc, norml, kernel, width, ...
         model);

    % test on the single holdout fold
    yest = general_model_eval(fold(:,:,i),degree, ...
                                c, w, norml, kernel, width, model);

    % calc error between yest and true foldy(:,:,i)
    meanabserr(i,1) = mean(abs(yest - squeeze(foldy(:,:,i))));
    r(i,1) = correlation(yest,squeeze(foldy(:,:,i)));
    rankr(i,1) = rank_correlation(yest,squeeze(foldy(:,:,i)));
    fprintf('fold %d meanabs %f\n',i,meanabserr(i,1));
  end;
  
  meanabserr,
  r,
  rankr,
  
  meanabs = mean(meanabserr);
  absaroundmean = mean(abs(y-ones(size(y,1),1)*mean(y)));
  r       = mean(r);
  rankr   = mean(rankr);
  fprintf('overall meanabs %f, absaroundmean %f, overallr %f, overallrankr %f\n',meanabs,absaroundmean,r,rankr);
  
  return

