function [xp,xptorg, xprand, xpstress, torgstress, randstress] = ...
   direct_optimize_kruskal(x,targetdims,start_with_torgerson,xaredist,xp,iterations);
% 
% [xp,xptorg, xprand, xpstress, torgstress, randstress] = ...
% direct_optimize_kruskal(x,targetdimensions,start_with_torgerson,xaredist,xp);
% 
% Performs direct optimization using matlab's quadratic program function.
% Optimizer operates on torgerson solution to improve it.
% Also shown is stress from random selection of points.
%
% x                    - highD datapoints, or distance matrix
% targetdims           - number of dimensions in the target space
% start_with_torgerson - 1=torgerson, 0=torgerson+random
% xaredist             - 1 means x is really a matrix of distances
%                        0 is the default, highD coordinates
% xp                   - set of initial 2D positons. Use this to 
%                        incrementally call it
% 
% EXAMPLE
% 
% % ------------------ Basic projection ----------------
% currentdimensions=2;targetdimensions = 1;datapoints=8;
% x = rand(datapoints,currentdimensions);
% [xp,xptorg,xpstress,torgstress] = direct_optimize_kruskal(x,targetdimensions);
% figure(1);plot(x(:,1),x(:,2),'.');text(x(:,1),x(:,2),num2str([1:datapoints]'));
% [xp xptorg]
% 
% % ------------ Project high dimensional simplex to 2D  -----------
% d = 12;dist = ones(d,d)-diag(ones(1,d),0);
% targetdims = 2;
% xp = direct_optimize_kruskal(dist,targetdims,0,1);
%
% % --------- Optimize and pause until it looks right
% xp = direct_optimize_kruskal(dist,targetdims,0,1,xp); 
%                              % start_with_torgerson==1,xaredist==1,xp
% xp3 = [xp zeros(size(xp,1),1)];
% plot_fully_connected(xp3);view(90,90);axis('equal');
% % keep doing the above until the figure looks good
%
% % ------- 3D projection (assuming you've initialized above) -------
% targetdims=3;xp = direct_optimize_kruskal(dist,targetdims,1,1);
% 
% % -------- optimize until looks good
% xp = direct_optimize_kruskal(dist,targetdims,1,1,xp); 
%                              % start_with_torgerson==1,xaredist==1,xp
% plot_fully_connected(xp);
% 
% SEE ALSO
%
% [xp,xptorg, xprand, xpstress, torgstress, randstress] = ...
% direct_optimize_kruskal(x,targetdimensions,start_with_torgerson,xaredist,xp);

  if nargin<6, % iterations not supplied
    iterations = 1000;
  end;

  v = version('-release');
  if v<14,
  
  % Minimizer parameters
  % --------------------

  options=foptions;
  options(1,14) = iterations;
  options(1,3)  = 1e-4;
  options(1,4)  = 1e-4;
  else
    options=zeros(1,14);
  end;

  % Display parameters
  % ------------------

  fprintf('\n');
  fprintf('direct_optimize_kruskal\n');
  fprintf('-----------------------\n');

  if nargin<3, % no startwithtorgerson
    start_with_torgerson=1;
  end;
  if nargin<4, % no xaredist
    xaredist=0;
  end;
  if nargin<5,
    initialize=1;
  else
    initialize=0; % xp is present
  end; 

  if xaredist==1,
    disp(' x is a distance matrix');
  else
    disp(' x is a highd coordinate matrix');
  end;

  if initialize==1,
    if start_with_torgerson,
      disp(' initial xp coordinates are torgerson''s projection');
    else
      disp(' initial xp coordinates are random'); 
    end;
    % otherwise we skip the initialization and use the supplied xp
    % coordinates
  else
    disp(' using supplied xp coordinates');
  end;

  if v<14,
    % fprintf('Minimizer parameters\n');
    fprintf(' use %d iterations\n',options(1,14));
  end;
  fprintf(' stop if stress change less than %f \n',options(1,3));
  
  fprintf('\n\n');

  


  % dist is original highD distance
  % -------------------------------

  if xaredist==0,
    [mini,dist] = find_closest2(x,x);
  else
    dist = x;
  end;
  n = size(x,1);




  % Random points
  % -------------
 
  xprand = rand(n,size(x,2));
  randdist = dist2(xprand,xprand);
  randstress = stress(dist,randdist);




  % Torgerson's projection
  % ----------------------
  % Calculate Torgerson's projection (used as starting position
  % & also another solution)

  [xptorg,torgdist,torgstress] = torgerson(dist,targetdims); 
  


  % Initialize starting configuration
  % ---------------------------------

  if initialize==1,
    if start_with_torgerson==1, 
      xp = xptorg; % starting xp
      fprintf('initializing to torgerson points\n');
    else 
      fprintf('1000 iterations of random point initialization\n');
      % xp = rand(n,targetdims); % random starting positions 
      fprintf('xptorg stress %f\n',torgstress);
      xp = random_optimize_kruskal(dist,targetdims,1000,1,xptorg);
    end;
  end;




  % Call minimizer
  % --------------
  
  xp = xp(:);  % have to split it out for minimizer
  
  if v<14,
    xp = fmins('stress_wrapper',xp,options,[],dist,targetdims);
  else
    xp = fminsearch('stress_wrapper',xp,[],dist,targetdims);
    % Untested
  end;
  
  % determine finishing stress and re-assemble xp vector
  [xpstress,xp] = stress_wrapper(xp,dist,targetdims);
  
  fprintf('\nstress direct %f\nstress torgerson %f\nstress random %f\n\n', ...
          xpstress, torgstress, randstress);

