function [RMSE,stdy,RMpercent,meanabs,meanabsdev,meanabs_percent] = ...
general_model_plot(x,y,degree, ...
                                   c, w, norml, kernel, width, ...
                                   model, pad_percent, trailpoint, ...
                                   lab, labdensity, gridsize, ...
                                   modelstr);
%
% [RMSE,stdy,RMpercent,meanabs,meanabsdev,meanabs_percent] = ...
%    general_model_plot(x,y,degree, c, w, norml, kernel, width, model, ...
%                       pad_percent, trail, lab, labdensity, ...
%                       gridsize, modelstr);
%
% Assumes a general_model_train has already been generated. Now
% evaluate that model using x (including the other points around
% x to form a square grid)
%
% y is the true value of y at point x. If y==[], then plotting true
% values is ommitted
%
% trail = [downvector downvector] == set of ids to be connected into line
% lab   = char vector == label for each point. [] if no labels
% labdensity == number of squares before a label is automatically placed
% gridsize == number of squares in the interpolation of the surface
%
% EXAMPLE
%
% trainset = rand(100,2);traindep = x(:,1).
% testset  = rand(100,2);testdep  = 
% model=2;degree=2;
% number_of_centroids=50;iterations=20;lc=0.3;norml=1;width=0;
%
% [c,w,yest,modelstr]=general_model_train(trainset,traindep,degree, ...
%  number_of_centroids, iterations, lc, norml, kernel, width, model);
% 
% general_model_plot(testset,testdep,degree, ...
% c,w,norml,kernel,width,model,0.1,[],[],100,20,modelstr);
%


% Input parameters
% ----------------

MAXPOINTS=1000;

% Check x is only 2 dims
% ----------------------

if size(x,2)>2, 
  disp('x should be two-dimensional. Deleting dimensions 3 and on');
  disp('If you want, use direct_optimize_kruskal to get a 2D version');
  disp('of high dimensional data');
  x = x(:,1:2);
end;
if size(x,2)==1,
  disp('x should be two-dimensional - setting dimension 2 to a constant');
  x(:,2) = mean(x(:,1)).*ones(size(x,1),1);
end;
if nargin<13, 
  disp('Looks like you missed a parameter');
  disp('type help general_model_plot to see the call');
  return
end;

% Check lab and trailpoints
% -------------------------

if isempty(lab),
  lab = char(ones(size(x,1),1).*32);
end;
if isempty(trailpoint),
  show_trajectory=0;
else
  show_trajectory=1;
end;

% Check y
% -------

if isempty(y),
  show_true = 0;
else
  if size(y,1)==size(x,1),
    show_true=1;
  else
    disp('y supplied is not the same size as x');
    disp('not displaying y points');
    show_true=0;
  end;
end;
if size(y,2)>1,
  disp('y has greater than one dimension. This can''t be plotted as ');
  disp('a 3d plot. Removing extra y dimensions');
  y=y(:,1);
end;


  % Build landscape
  % ---------------

  fprintf('inducing fitness (surf) at each map gridpoint');

  % initial size of landscape
  maxx     = max(x); 
  minx     = min(x);
  xrange   = maxx-minx;

  % determine padding and add it
  padding  = pad_percent.*xrange;
  maxx     = maxx + padding;
  minx     = minx - padding;
  xrange   = maxx-minx; % new xrange

  % everything else is as normal
  delta    = xrange ./ gridsize; % size of each square on the grid

  % create a meshgrid to call polyfit over
  gridx1 = ones(gridsize,1)* ([delta(1):delta(1):xrange(1)]+minx(1));
  gridx2 = ([delta(2):delta(2):xrange(2)]'+minx(2))*ones(1,gridsize);
  
  z = zeros(gridsize,gridsize);

  both = [gridx1(:) gridx2(:)];
  yest = general_model_eval(both,degree, ...
                            c, w, norml, kernel, width, model);

  for i1=1:gridsize,
    z(:,i1) = yest((i1-1).*gridsize+1:i1.*gridsize,1);
  end;


  % Graphics defaults
  % -----------------

  over_landscape_eps = 0.05.*std(yest);
  textcolor          = 'b';
  textheight         = 0.6.*std(yest);
  textwidth          = 2;
  linewidth          = 2;
  fontsize           = 6;
  show_lines         = 1;

  zz = general_model_eval(x, degree, ...
                          c, w, norml, kernel, width, ...
                          model);

  % Draw landscape
  % --------------

  clf;
  cmap = hsv;
 
  % USER: User choose what kind of surface plot to generate

  % Top-view contour plot
  % [cc,hh] = contour(z); clabel(cc,hh); 

  % faceted surface over a contour
  % surfc(z);colorbar;shading faceted;

  % Simple mesh
  mesh(z);

  % Ray-trace
  % cmap=pink;surfl(z);shading interp; colorbar;

  colormap(cmap);
  hold on;

  % Show the x points
  if show_true,
    % display the actual value as a point over or under the surface
    gridxy = (x-ones(size(x,1),1)*minx) ./ (ones(size(x,1),1)*delta);
      r = rand(size(x,1),1);
      [a,r] = sort(r);
      if size(x,1)>MAXPOINTS,
        maxpoints = MAXPOINTS;
      else
        maxpoints = size(x,1);
      end;
    plot3(gridxy(r(1:maxpoints,1),1),gridxy(r(1:maxpoints,1),2),y(r(1:maxpoints,1)),'x');
    xlabel('x1');
    ylabel('x2');

    % draw a line from the surface to each point to represent
    % error
    if show_lines,
      for ii=1:maxpoints,
        line([gridxy(r(ii),1) gridxy(r(ii),1)], ...
             [gridxy(r(ii),2) gridxy(r(ii),2)],[y(r(ii),1) zz(r(ii),1)]);
      end;
    end;

  end;

  fprintf('plotting trajectory\n');

  % Display trajectory
  % ------------------

  traj=[];
  if show_trajectory,
    for t=1:size(trailpoint,2),
    % one trail for each 

    traj_temp = [];
    % this will record the trajectory for this t trail
    % traj will add this in after traj_temp is finished being built

    % place points between each x trailpoint

  for i=2:size(trailpoint,1), % for each point

    % plot line from x(i-1) to x(i)
    % -----------------------------

    % create a discrete set of points connecting x(i-1) to x(i)
    xymovement = x(trailpoint(i,t),:)-x(trailpoint(i-1,t),:);
    disttravelled = distance(x(trailpoint(i-1,t),:),x(trailpoint(i,t),:));
    reduction_factor = disttravelled ./ (min(delta)./1); %mean(delta);
% USED TO BE 10
    % factor we have to reduce x and y components by to equal
    % a single delta distance step each discrete step

    xymovement = xymovement ./ reduction_factor;
    a = xymovement(1);
    b = xymovement(2);
    % angle or ratio is invariant under scaling of the distance 
    % (hypotenuse)

    %the other way to do it was
    %theta = atan(xymovement(2)./xymovement(1));
    %a = min(delta) .* cos(theta);
    %b = min(delta) .* sin(theta);

    xtrail = x(trailpoint(i-1,t),1):a:x(trailpoint(i,t),1);
    ytrail = x(trailpoint(i-1,t),2):b:x(trailpoint(i,t),2);
    trail = [xtrail' ytrail'];

    % at each of these points, now convert back to grid coordinates
    % and plot a point on the grid

    gridxy = (trail-ones(size(trail,1),1)*minx) ./ ...
                    (ones(size(trail,1),1)*delta);
    % grid display position
                 
    traj_temp = [traj_temp;gridxy];

    for i2=2:size(gridxy,1),
      line([gridxy(i2-1,1) gridxy(i2,1)], ...    % x1..x2
           [gridxy(i2-1,2) gridxy(i2,2)], ...    % y1..y2
           [general_model_eval(trail(i2-1,:),degree, ...
                               c, w, norml, kernel, width, model) ...
            general_model_eval(trail(i2,:),degree, ...
                               c, w, norml, kernel, width, model)] + ...
                           over_landscape_eps, ...
           'color',textcolor,'linewidth',linewidth);
    end; 
    % we've now plotted the trail between x(i-1) and x(i)

  end; % for each x(i-1) x(i) pair
  end; % for each t trail in trailpoint
  %traj(1:size(traj_temp,1),t) = traj_temp;
end; % show_trajectory


  gridxy = (x-ones(size(x,1),1)*minx) ./ ...
           (ones(size(x,1),1)*delta);


  % Place Label every labdensity squares
  % -------------------------------------

  % USER: Uncomment this to show all labels on the map

   % we can write code to automatically reveal points every 
   % gridsquare position. This would ensure that we'd always get
   % a spread of points. Well, if It could reveal too many in
   % high density. If large grid squares, will skip over the 
   % density areas.
   % Since we are only revealing one per square this should be ok
   %

   if labdensity<=gridsize, 
     for x1=0 : labdensity : gridsize,
       for x2=0 : labdensity : gridsize,
         
         %dat = [315;65;         ... % poor
         %       232;144; ... %]; % ... %56;336; ... % rich
         %       4;48;261;167;320;256;64]; 
         %       % 140;153]; % mid];

         % if we are within labdensity squares of a 
         % trajectory (trail) point, then do not print this square
    
         outside_range = 1;
         for t=1:size(trailpoint,2),
           disttotrail = dist2([x1 x2],gridxy(trailpoint(:,t),:));
           % dmatrix is x1y1 
           %            x2y1
           %            x3y1

           if find(disttotrail'<labdensity), 
             % if any of the disttotrail (a horizontal vector since we
             % are testing from a single point to all points on the trail)
             % is closer then labdensity squares, then to avoid crowding
             % don't print this point
             outside_range=0;
             break;
           end;
         end;
         if outside_range,
           mini = find_closest2([x1 x2],gridxy);
         % display_map_label( x(mini,:), minx, delta, lab(mini,:), ...
       %                    degree,c,w,norml,kernel,width,model, ...
     %                      over_landscape_eps, textwidth, linewidth, ...   
   %                        textcolor, textheight, fontsize);

    % plot line
%    line([gridxy(mini,1) gridxy(mini,1)], ...
%         [gridxy(mini,2) gridxy(mini,2)], ...
%         [zz(mini,1) y(mini,1)], ...
%         'color',textcolor,'linewidth',linewidth./2);

    % plot label
    text(gridxy(mini,1),gridxy(mini,2), ...
         y(mini,:), ...
         char(lab(mini,:)),'color',textcolor, ...
         'fontsize',fontsize);

         end; % if outside_range
       end; % for x2
     end; % for x1
   end; % if 1


  % Label the trail
  % ---------------

  if show_trajectory,
    for t=1:size(trailpoint,2),
    % plot line
 %   line([gridxy(trailpoint(:,t),1) gridxy(trailpoint(:,t),1)], ...
 %        [gridxy(trailpoint(:,t),2) gridxy(trailpoint(:,t),2)], ...
  %       [y(trailpoint(:,t),1)+over_landscape_eps  ...
   %       y(trailpoint(:,t),1)+textheight], ...
    %     'color',textcolor,'linewidth',linewidth./2);

    % plot label
    text(gridxy(trailpoint(:,t),1),gridxy(trailpoint(:,t),2), ...
         y(trailpoint(:,t),1)+textheight, ... % y(i-1,:)+textheight
         char(lab(trailpoint(:,t),:)),'color',textcolor, ...
         'fontsize',fontsize);

    end;
  end;

  % Relabel the x1 and x2 axes with the appropriate actual x values
  % ----------------------------------------------------------------

  % log correlation
  mn = min([y;zz]);
  ly = log((y-mn)+1);
  lzz = log((zz-mn)+1);
  logcorr    = round(correlation(ly,lzz).*100)./100;

  % corr
  corr       = round(correlation(y,zz).*100)./100;
  
  % meanabs
  meanabs    = mean(abs(y-zz));
  meanabsdev = mean(abs(y-mean(y)));
  meanabs_percent = meanabs ./ meanabsdev;

  % medabs
  medabs = median(abs(y-zz));
  medabsdev = median(abs(y-median(y)));
  medabs_percent = medabs ./ medabsdev;

  % root means squared (same as stddev)
  RMSE       = sqrt( mean((y-zz).^2) );
  stdy       = std(y);
  RMpercent  = RMSE ./ stdy;
  
  % Change to out of 100
  RMpercent = round(RMpercent.*100)./100;
  meanabs_percent = round(meanabs_percent.*100)./100;

  % Rounding for RMSE, etc
  % pow = floor(log10(stdy));
  % RMSE = round(RMSE.*10^pow)./(10^pow);
  % stdy = round(stdy.*10^pow)./(10^pow);
  % pow=floor(log10(meanabsdev));
  % meanabsdev = round(meanabsdev.*10^pow)./(10^pow);
  % meanabs    = round(meanabs.*10^pow)./(10^pow);
  
  titlestr = [modelstr ' corr ' num2str(corr) ...
              ' logcorr ' num2str(logcorr) ...
              ' MedAbsErr ' num2str(medabs) ...
              ' MedAbsDev ' num2str(medabsdev) ...
              ' MedAbsPer ' num2str(medabs_percent)];

  title(titlestr);
