function [x,err] = gradient_optimize_stress( dist,        ...
                                             target_dims, ...
                                             alpha,       ...
                                             seconds,     ...
                                             iterations,  ... 
                                             max_no_improvement, ...
                                             adaptive,    ...
                                             line_search, ...
                                             conjugate,   ...
                                             x);
%
% [x,err] = gradient_optimize_stress( dist,         ...
%                               target_dims,        ...
%                               alpha,              ...
%                               max_seconds,        ...
%                               max_iterations,     ... 
%                               max_no_improvement, ...
%                               adaptive,           ...
%                               line_search,        ...
%                               conjugate_gradient, ...
%                               x_initial);
%
% optimize stress function using gradient descent on the stress
% function. Allows user to perform conjugate gradient, line search,
% adaptive step optimization
%
% EXAMPLES
%
% % ==================== Accuracy of each method after 6 seconds
% d=6;dist = ones(d,d)-diag(ones(1,d),0);targetdims = 2;
% a = 0.001;mi=10000;mni=30;s=6;
% r = rand(d,targetdims); % starting configuration
% % ----------------------------- conjugate with adaptive step
% [x,err]=gradient_optimize_stress(dist,2,a,s,mi,mni,1,1,1,r);
% figure(1);clf;plot(err,'r-');figure(2);clf;plot(x(:,1),x(:,2),'ro');
% figure(3);clf;bar(1,err(size(err,1)),'r');
% % ------------------------------ conjugate
% [x,err]=gradient_optimize_stress(dist,2,a,s,mi,mni,0,1,1,r);
% figure(1);hold on;plot(err,'m-');figure(2);hold on;plot(x(:,1),x(:,2),'mo');
% figure(3);hold on;bar(2,err(size(err,1)),'m');
% % ------------------------------- line search
% [x,err]=gradient_optimize_stress(dist,2,a,s,mi,mni,0,1,0,r);
% figure(1);plot(err,'b-');figure(2);plot(x(:,1),x(:,2),'bo');
% figure(3);bar(3,err(size(err,1)),'b');
% % ------------------------------- gradient descent with adaptive step
% [x,err]=gradient_optimize_stress(dist,2,a,s,mi,mni,1,0,0,r);
% figure(1);plot(err,'g-');figure(2);plot(x(:,1),x(:,2),'go');
% figure(3);bar(4,err(size(err,1)),'g');
% % -------------------------------- gradient descent
% [x,err]=gradient_optimize_stress(dist,2,a,s,mi,mni,1,0,0,r);
% figure(1);plot(err,'c-');figure(2);plot(x(:,1),x(:,2),'co');
% figure(3);bar(5,err(size(err,1)),'c');
% figure(1);legend('cj+ad','cj','lnsrch','grad+ad', 'grad');
% ylabel('stress');xlabel('iterations');
% figure(3);legend('cj+ad','cj','lnsrch','grad+ad', 'grad');
% ylabel('stress');xlabel('method');
 

% Begin timer
% -----------

t0 = clock;


% Standardize dist
% ----------------

dist = standardize_distmat(dist);
d    = mat2triulist(dist);


% Initialization parameters
% -------------------------

disp(' ');
disp('gradient optimize stress');
disp([' learning rate   ' num2str(alpha)]);
disp([' terminate after ' num2str(seconds) ' seconds']);
disp([' terminate after ' num2str(iterations) ' iterations']);
disp([' or ' num2str(max_no_improvement) ' failures to improve']);
if adaptive,
  disp(' adaptive stepsize');
end;
if line_search & conjugate~=1,
  disp(' method is line search');
end;
if conjugate,
  disp(' method is conjugate gradient');
end;
if ~conjugate & ~line_search,
  disp(' method is gradient descent');
end;
if nargin<9, % x was not supplied
  % initialize with torgerson
  disp(' initialize x as torgerson');
  x = torgerson(dist,target_dims); 
end;
disp(' -----------------------------');


% Standardize distp
% -----------------

distp_new = dist2(x,x);                     % distance estimate for x_initial
distp_new = standardize_distmat(distp_new); % standardize
s         = stress(dist,distp_new);         % calc initial stress


% Initialization gradient
   distp_old  = distp_new;
   distp_new = monotonic_average(dist,distp_old,d);
   deltax = calc_stress_gradient(distp_old, distp_new, dist, x);
   dir    = -1.*deltax;


% Optimization loop
% -----------------

err       = [s+eps; s];
iteration = 2;
no_improvement = 0;

while (iteration < iterations) & (etime(clock,t0)<seconds), 

   % Monotonic averaging
   % -------------------

   distp_old  = distp_new;
   distp_new  = monotonic_average(dist,distp_old,d);
   % Experiment try: monotonic_average(dist,distp_old,d,0.01);
   %                                                    ^^^^ tolerance
   % See if it still moves down the gradient, even if the distances
   % aren't being averaged properly


   % Calculate gradient
   % ------------------

   dir_old   = dir;
   delta_old = deltax;
   % save old direction and gradient

   deltax    = calc_stress_gradient(distp_old, distp_new, dist, x);
   % new gradient

   if conjugate,
   % set conjugate gradient direction
     b = ((deltax - delta_old) .* deltax) ./ (delta_old .^2);
     dir = -1 .* deltax + b.* dir_old;
     % dnew = -1 * grad + b*d_old
     % b = ((gradnew - gradold) * gradnew) / (gradold.^2)
   else 
     dir = -1.*deltax;
   end;


   % Line search
   % -----------

   enter = 1;
   while err(iteration)<err(iteration-1) | enter==1,
       enter=0;

       if adaptive>0,
       % Adaptive stepsize
         if err(iteration)<err(iteration-1),
           % speed up
           alpha = alpha + 0.1 .* alpha;
         else
           % brake
           alpha = alpha ./ 10;
         end;
       end;

       % update x
       x = x + alpha .* dir;

       % Restandardize distp
       % -------------------

       distp_new = dist2(x,x);
       distp_new = standardize_distmat(distp_new);
       iteration = iteration + 1;
       err(iteration,1) = stress(dist,distp_new);
if err(iteration)<2.9, return;end;
       
       if line_search==0 & conjugate~=1, break; end;
       if (etime(clock,t0)>=seconds), break; end;
       fprintf('.');

   end; % endwhile linesearch

   % check for no_improvement
   % ------------------------

   if err(iteration)>err(iteration-1),
     no_improvement = no_improvement+1;
     if no_improvement>max_no_improvement, 
       fprintf('\nno improvement after %d consecutive attempts\n', ...
               max_no_improvement);
       break;
     end;
   else
     no_improvement=0;
   end;
   fprintf('%f ',err(iteration));
end;
fprintf('\n');
