function ynew = grnn(x,y,xnew,method);
%
% yest = grnn(x,y,xnew);
%
% generalized regression neural net. Given a set of training cases
% (x,y), estimates the y value at a new point xnew by simply 
% linearly interpolating between the y values of the various x points.
% This is done as follows: 
%
% xnew = f[ idist(x,x1)/sum(idist(x,xi)) ] * y1 + ...
%        f[ idist(x,x2)/sum(idist(x,xi)) ] * y2 + ...
%
% So each point is equal to the percent distance to each neighbouring
% training case put through some function. GRNNs typically implement a
% distance function / kernel f to weight cases even more strongly to their
% nearest cases. In this case I've simply used a distance inverse
% function. (which does indeed tend to perform that weighting - the result
% is very non-smooth  landscape, see fig)
%
% method == 1, inverse distance
% method == 2, max-distance
%
% Inverse versus max-dist
% max-distance incurs a lot of error for the training
% cases. They will be pulled strongly by every other point in 
% the grid, and so the training case y values will be incorrect.
% Invert gets around this by exerting a strong gravitational pull
% towards the nearest neighbour, the closer you get (inverse
% square law without the square). However, inverse also will result in 
% some low-radius bumps or holes in the sheet corresponding to the local
% region around a training case, separated by expanses of mean-like areas
% because influence from each case is about the same
%
% EXAMPLE
%
% x = rand(5,2);
% y = rand(5,1);
% mapdata(x,y, ...)
%
% SEE ALSO
% 
% spline_train  % spline interpolation
% polyfitf      % polynomial surface estimation
% 
% yest = grnn(x,y,xnew);
%
% % mds toolbox (c) Brendan Kitts 1996


if nargin==3,
  method=1;
end;

loweps = 0.000000000000000000001;


dist = dist2(xnew,x); % calc dist

if method==1, 
  % invert
  dist = 1./(dist+loweps);  
else
  % max - dist
  % note that this method incurs a lot of error for the training
  % cases. They will be pulled strongly by every other point in 
  % the grid, and so the training case y values will be incorrect
  % Invert gets around this by exerting a strong gravitational pull
  % towards the nearest neighbour, the closer you get (inverse
  % square law analogue). However, inverse also will result in 
  % some small holes in the sheet, separated by expanses which are
  % pretty similar
  dist = max(dist')' * ones(1,size(dist,2)) - dist;
end;
  
dist = dist ./ (sum(dist')' * ones(1,size(x,1)));  % normalize


for i=1:size(xnew,1),
  ynew(i,:) = dist(i,:) * y;
end;

ynew = real(ynew);


% =========================================================
% if we want to look at the training set estimates
%disttrain = dist2(x,x);
%disttrain = 1./(disttrain+loweps);
%disttrain = disttrain ./ (sum(disttrain')' * ones(1,size(x,1)));
%for i=1:size(xnew,1),
%  ytrain(i,1) = sum( (ones(size(y,2),1)*disttrain(i,:)) * y);
%end;

% dist is 
% [xnew(1) and x(1) | xnew(1) and x(2) ...
%  xnew(2) and x(1) | 
%   ...
% across is the id of the x which it is closest to

% note that ynew carries heavy error. It is being pulled by
% every other training case. 
% landscape. select point. calc ynew at this point
% calc y-y discrepancy at centroid position
% get this average discrepancy. Take away from ynew


