function [z,traj,x] = mapdata2( xoriginal1,         ...
                               convert_to_zscores, ...
                               y,                  ...
                               trailpoint,         ...
                               lab,                ...
                               gridsize,           ...
                               labdensity,         ...
                               method,             ...
                               x,                  ... 
                               model, missing_val);
%
% z = mapdata(xoriginal,convert_to_zscores,y,trail,lab,gridsize,labdensity,method,x,model,missing_val);
% % MDSKit multi-dimensional map formation
%
% Takes multidimensional x and single dimensional y, calculates a model to
% fit the surface, and plots it as a 3D surface plot. If trail is supplied
% (this is a vertical list of x ids), then the xs with these ids are 
% connected together to form a trail, which is plotted on the landscape.
%
% xoriginal              high dimensional data
% convert_to_zscores     ==1 means standardize before scaling
% y                      y value which will be plotted on the height axis
% trailpoint             vector of indices which form a connected trail
%                        If there is more than one col, this is a second 
%                        trail to plot
% lab                    string label for every datapoint
% gridsize               number of squares on the 2D surface
% labdensity             one label every this many squares. To switch off
%                        general labels, set this > gridsize
% mds_method             projection method 
%                        ==1 torgerson
%                        ==2 direct_optimize_kruskal
%                        ==3 gradient_descent
%                        problem is each of these needs parameters
% x [optional]           if a projection has been performed before, then
%                        x can be supplied to mapdata and it will use
% surface_model          ==1 polynomial, ==2 spline, ==3 grnn
%
% If x is higher than 2 dimensions, then it performs a multi-dimensional
% scaling to reduce the data to two dimensions. It also displays 
% interpolation error in n and 2 dimensions, and so allows you to
% estimate the error in the z value. Stress is also reported.
% The surface in a reduced 2dimensions is inferred from the 2D data,
% so may not truly represent the higher dimensional shape
%
% mapdata performs the same operation that griddata does in 
% matlab and grid does in SAS. ie. it takes unevenly spaced data, 
% and then imposes it onto a grid so that it can be plotted in 3D. 
% If there are say, 10 datapoints, there will be a large number of 
% grid squares which have no data. In these unseen points, the surface
% is inferred using the generalization bias of the model used, 
% eg. spline, mlp, or polynomial surface. 
%
% EXAMPLES
%
% % 1) Compare the fitting behaviour of backprop, splines, or polynomials
% x = rand(100,2).*4-2; % random numbers on the -2,2 interval
% y = x(:,1).*exp(-x(:,1).^2 - x(:,2).^2);
% % Go into mapdata.m and set model==2,1,3 % (thinplate,poly,grnn)
% [z,traj] = mapdata2(x,1,y,[],[],20); 
% % (no projection will be performed because x is two dimensional)
% % (no trail, labtrail, lab)
% % grnn (inverse distance) overgeneralizes, 
% % polyfit doesn't fit the cross terms,
% % thinplate nails the surface exactly
%
% % 2) high-dimensional company data as a timeseries across a possibility landscape
% load sk3.txt % SkandiaBanken
% x = sk3(:,[1 3:12]);
% y = sk3(:,2); % total operating income
% z=mapdata2(x,1,y,[1:4]',['94';'95';'96';'97'],20);
% 
% SEE ALSO
% direct_optimize_kruskal, general_model_train, crossval
% z = mapdata(xoriginal,convert_to_zscores,y,trail,lab,gridsize,labdensity,method,x,model);

% SPEEDING UP
%
% mat2triulist
% triulist2mat (for monotonic_averaging)
%
% change dist2(x,x) in stress_wrapper for direct_optimize_kruskal
% to a n^2/2 operation (because of symmetry).
% 
% gradient_optimize_stress = monomds2 should rename it
% % too slow normally due to averaging stage
%
% monotonic_averaging then torgerson
% 
% Give direct_optimize_kruskal a FIXED NUMBER OF ITERATIONS
% 2 sec per distance calc. Therefore, 10 iterations.
% 

  fprintf('\n\nmapdata\n');

  % Convert data to zscores
  % -----------------------

  if convert_to_zscores, 
    fprintf(' data subject to zscore transform\n');
    % Perform zscore transformation
    % xoriginal is a zscore version of the incomming data
    % zmean and zstd can be used to get new estimates back in the
    % original coordinates
    [xoriginal,zmean,zstd] = data2zscore(xoriginal1);
  else 
    % Use data as-is
    % set xoriginal equal to the xoriginal1
    % the inverse zscore transform must be an identity transformation,
    % inverse transform is zmean + zstd.*zscore value

    fprintf(' no zscore transform\n');
    xoriginal = xoriginal1;
    zmean = zeros(1,size(xoriginal,2));
    zstd  =  ones(1,size(xoriginal,2));
  end;

% doesn't seem to be any reason for xoriginal1

  % Input parameters
  % ----------------

  % trailpoints    
  if isempty(trailpoint), show_trajectory=0;
  else show_trajectory=1; end;

  % labdensity
  if nargin<7, labdensity=gridsize./4;end;
  if labdensity<1,
    fprintf(' labdensity must be >=1 (1 label per grid square). labdensity is being readjusted\n');
    labdensity=gridsize./4;
  end;
  fprintf(' labdensity==%d\n',labdensity);
  % once every 4 squares should be enough
  if labdensity>gridsize,
    fprintf('   (no landmark labels will be displayed on the map)\n');
  end;
  
  % MDS method
  if nargin<8 | method<1 | method>3, % no method specified
    method=1;
  end;

  % MDS method
  if nargin>8,  % 2-dim x data already supplied
    fprintf(' using supplied 2-dimensional x data (no MDS will be peformed)\n');
    if size(x,2)>2,
      fprintf(' **** WARNING: supplied x data was > 2 dimensions. Cutting it down\n');
    end;
  else
    fprintf(' Performing MDS using method==%d ',method);
    if method==1, fprintf('torgerson');end;
    if method==2, fprintf('direct_optimize_kruskal (nelder-mead)');end;
    if method==3, fprintf('gradient_descent');end;
    fprintf('\n');
  end;
  
  if nargin<11, % no missing_val
     missing_val=nan;
  end;
  
  % surface estimation model    
  if nargin<10 | model<1 | model>3,
    model = 2;
  end;
  fprintf(' Estimating surface using model==%d ',model);
  if model==1,
    fprintf('polynomial');
  end;
  if model==2,
    fprintf('spline');
  end;
  if model==3,
    fprintf('grnn');
  end;
  fprintf('\n');

  fprintf('-------------------------------------------\n\n');

  xstress = nan; % initialize it to something
  if size(xoriginal,2)>2,

     if nargin>8, 

       % x was supplied
       % --------------

       if size(x,2)~=2 | size(x,1)>size(xoriginal,1),
         fprintf(' x data which was supplied does not match xoriginal\n');
         fprintf(' please check this data. Or call mapdata without x\n');
         fprintf(' and fresh x data (2d) will be computed\n');
         return
       end;
       xstress = stress(dist2(xoriginal,xoriginal),dist2(x,x));
     else 

       % Perform multidimensional scaling
       % --------------------------------

       targetdims = 2;

       % torgerson
       if method==1,
         dist = dist2(xoriginal,xoriginal);
         x = torgerson(dist,targetdims);
         xstress = stress(dist,dist2(x,x));
       end;

       % nelder-mead
       if method==2, 
         start_with_torgerson = 1;
         [x,xtorg, xrand, xstress, torgstress, randstress] = ...
         direct_optimize_kruskal(xoriginal,targetdims,start_with_torgerson);
       end;

       save x.mat x

     end; % nargin<8 (perform the MDS)
  else
    x = xoriginal; 
    % if xoriginal is 2 dimensional, then set x to equal it exactly
    % it could be equalling the zscore version
  end;


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TO DO:
%
%  recheck the log in thinplate splines. Either fix up, or
%  put it back as rigorous log(1+a) + a^2
% 
%  check grnn is a good interpolation method - 
%  need way to validate the inverse mapping. crossval. but
%  so few points
% 
%  monomds see if it reduces stress
%  second figure which displays nd vector nicely
%  graph, as you move your mouse across, each dimension
%  value changes. Can use this to show how you move
%  from 96 to 97. 
%  Could gather a number of points, or even 
%  use trail, and show the underlying variables.
%  (save a trail, or save the entire traj, and then
%  inverse it, and put in 95, 96, for each 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% User parameters
% -------------------------------------------------------------
% 
% The following parameters can be adjusted by the user
% If you don't want to modify any of these parameters, the algorithm
% should still work. (if there are 1000 points, it could be
% quite slow). If its taking too long, try it with fewer points.
% Then when you get brave enough, you can start trying things with
% the parameters
%

  % Display parameters
  % ------------------

  over_landscape_eps = 0.1.*std(y); % height of trail over surface [1]
  textheight         = 0.5.*std(y);  % text height above surface   [1]
  fontsize           = 10;
  textcolor          = 'k';          % blue
  cmap               = hsv;          
  textwidth          = 2;            % how thick the marker is
  linewidth          = 2;            % how thick the trail is
  pad_percent        = 0.05;         % used to be [0.2]
  % what percent border around the max and min point there is.
  % we get the range of the points, that is max-min. This is the
  % total breadth of the x1 and x2 dimensions. This is an extra   
  % padding percent which is used to extend this landscape a little.
  % Therefore, if a point was at 10,2, this would extend the surface
  % say 10% past this threshold

  % Model parameters
  % ----------------

  % model = 2; % surface estimator
             % Change this model to splines to start with
             % 1==polyfit, 2==spline, 3==grnn
  % this is now defined from outside

  k=2; % number of error estimation pools
       % check how much data you have. If you have a lot,
       % you can create a large number of groups. If you
       % only have 5 points, then we can only afford 2
       % groups. size of crossval group
       % if we change the amount of data to larger, and we
       % don't use one centroid for each point, then
       % change crossval

  degree=1;  % Polynomial parameter

                    % Spline parameters
  iterations = 50;  % no of iterations to find centroids
  lc         = 0.3; % movement speed of those centroids
  norml      = 0;   % normalize spline activations [1]
  kernel     = 4;   % 4==thinplate, 3==cubic, 2==linear
  width      = 0;   % width parameter used for gaussian splines

  number_of_centroids = size(x,1); % round(0.9.*size(x,1));  
%  number_of_centroids = size(x,1)./2;
  % Since my data only has a small number of points (four), I have
  % forced the splines to have only as many centroids as there 
  % are datapoints. If you have 100 points, then you might want
  % to have fewer centroids, maybe 10 or so.

%
% --------------------------------------------------------------
% End of user parameters
% ==============================================================




  % Calculate error of the projected surface
  % ----------------------------------------

% USER: If you don't want error measures (it takes some time
% to run the crossval), set if 0 and error will revert to nans
if 0,
    warning off;
    meanabs_original = crossval(xoriginal,y,degree, ...
       number_of_centroids, iterations, lc, norml, kernel, width, ...
       model,k),
    meanabs_projection = crossval(x,y,degree, ...
       number_of_centroids, iterations, lc, norml, kernel, width, ...
       model,k),
    meanabs_mean = mean(abs(y - ones(size(y,1),1)*mean(y)));
    warning on;
else
  meanabs_original = nan;
  meanabs_projection = nan;
  meanabs_mean = nan;
end;

  % Induce model for 2D x to predict y
  % ----------------------------------
      
     fprintf('inducing surface model\n');
     
     if isnan(missing_val),
        dependent_present = find(~isnan(y));
     else
        dependent_present = find(y~=missing_val);
     end;
     [c,w,yest,method] = general_model_train(x(dependent_present,:),y(dependent_present,:),degree, ...
     number_of_centroids, iterations, lc, norml, kernel, width, ...
     model);

  % Build landscape
  % ---------------

  fprintf('inducing fitness (surf) at each map gridpoint\n');

  % initial size of landscape
  maxx     = max(x); 
  minx     = min(x);
  xrange   = maxx-minx;

  % determine padding and add it
  padding  = pad_percent.*xrange;
  maxx     = maxx + padding;
  minx     = minx - padding;
  xrange   = maxx-minx; % new xrange

  % everything else is as normal
  delta    = xrange ./ gridsize; % size of each square on the grid

  % create a meshgrid to call polyfit over
  gridx1 = ones(gridsize,1)* ([delta(1):delta(1):xrange(1)]+minx(1));
  gridx2 = ([delta(2):delta(2):xrange(2)]'+minx(2))*ones(1,gridsize);
  
  z = zeros(gridsize,gridsize);

  both = [gridx1(:) gridx2(:)];
  yest = general_model_eval(both,degree, ...
                            c, w, norml, kernel, width, model);

  for i1=1:gridsize,
    z(:,i1) = yest((i1-1).*gridsize+1:i1.*gridsize,1);
  end;

if 0, 
  % this is the slow way of calculating the fit
  % however, we can go back to this if we get any strange
  % results
  tic,
  for i1=1:gridsize,
    for i2=1:gridsize,
      xcoords = [gridx1(i1,i2) gridx2(i1,i2)];
      z(i1,i2) = polyff(c,degree,xcoords);
    end;
  end;
  toc,
end;


  % Draw landscape
  % --------------

  clf;
 
% USER: User choose what kind of surface plot to generate

  surfc(z);colorbar;shading faceted;colormap(summer);
  %[cc,hh] = contourf(z); clabel(cc,hh);colormap(summer);colorbar;
  %Top-view contour plot
  % mesh(z); colorbar; colormap(pink);
  % faceted surface over a contour
  %surfl(z);shading interp; colorbar;colormap(autumn);
  hold on;

  fprintf('plotting trajectory\n');

  % Display trajectory
  % ------------------

% --------------
% TODO: 
% CHECK and FIX TRAJECTORY
% IS IT PICKING UP THE LAST ELEMENT?
% 
  trail_res = 0.01;
  traj=[];
  if show_trajectory,
    for t=1:size(trailpoint,2),
    % for each trajectory,
    
    %x(trailpoint(:,t),1) is x1
    % skip the nans
    present=find(~isnan(x(trailpoint(:,t),1)) & ~isnan(x(trailpoint(:,t),2)) & ...
                 ~isnan(y(trailpoint(:,t),1)));
    linmethod='cubic';
    a = interp1(1:size(present,1),x(trailpoint(present,t),1),1:trail_res:size(present,1),linmethod);
    b = interp1(1:size(present,1),x(trailpoint(present,t),2),1:trail_res:size(present,1),linmethod);
    % this can be replaced with spline
    trail = [a' b'];
    gridxy = (trail-ones(size(trail,1),1)*minx) ./ ...
                    (ones(size(trail,1),1)*delta);
    % grid display position
                 
    plot3(gridxy(:,1),gridxy(:,2), ...    % y1..y2
           general_model_eval(trail,degree, c, w, norml, kernel, width, model) ... 
           + over_landscape_eps, ...
           'color',textcolor,'linewidth',linewidth);
    end; % for each t trail in trailpoint
  end; % show_trajectory


  % Place Label every labdensity squares
  % -------------------------------------

  % USER: Uncomment this to show all labels on the map

   % we can write code to automatically reveal points every 
   % gridsquare position. This would ensure that we'd always get
   % a spread of points. Well, if It could reveal too many in
   % high density. If large grid squares, will skip over the 
   % density areas.
   % Since we are only revealing one per square this should be ok
   %
   
   fprintf('putting general map labels\n');
   if labdensity<=gridsize & ~isempty(lab), 
     gridxy = (x-ones(size(x,1),1)*minx) ./ (ones(size(x,1),1)*delta);
     for x1=0 : labdensity : gridsize,
       for x2=0 : labdensity : gridsize,
         
         %dat = [315;65;         ... % poor
         %       232;144; ... %]; % ... %56;336; ... % rich
         %       4;48;261;167;320;256;64]; 
         %       % 140;153]; % mid];

         % if we are within labdensity squares of a 
         % trajectory (trail) point, then do not print this square
    
         outside_range = 1;
         for t=1:size(trailpoint,2),
           disttotrail = dist2([x1 x2],gridxy(trailpoint(:,t),:));
           % dmatrix is x1y1 
           %            x2y1
           %            x3y1

           if find(disttotrail'<labdensity), 
             % if any of the disttotrail (a horizontal vector since we
             % are testing from a single point to all points on the trail)
             % is closer then labdensity squares, then to avoid crowding
             % don't print this point
             outside_range=0;
             break;
           end;
         end;
         if outside_range,
            mini = find_closest2([x1 x2],gridxy);
            ok=0;
            if isnan(missing_val),
               if ~isnan(x(mini,1)) & ~isnan(x(mini,2)),ok=1;end;
            else
               if x(mini,1)~=missing_val & x(mini,2)~=missing_val, ok=1;end;
            end;
            if ok==1,
              display_map_label( x(mini,:), minx, delta, lab(mini,:), ...
                              degree,c,w,norml,kernel,width,model, ...
                              over_landscape_eps, textwidth, linewidth, ...   
                              textcolor, textheight, fontsize);
           end;
         end; % if outside_range
       end; % for x2
     end; % for x1
   end; % if 1


  % Label the trail
  % ---------------
  
  fprintf('labelling the trail\n');
  if show_trajectory,
     for t=1:size(trailpoint,2),
        present = find(~isnan(x(trailpoint(:,t),1)) & ~isnan(x(trailpoint(:,t),2)));
        display_map_label( x(trailpoint(present,t),:), minx, delta, ...
                         lab(trailpoint(present,t),:),...
                         degree,c,w,norml,kernel,width,model, ...
                         over_landscape_eps, textwidth, linewidth, ... 
                         textcolor, textheight, fontsize);
    end;
  end;


  % Graph title
  % -----------

  st = sprintf('stddev %f | nd modelerror %f | 2d stress %f | 2d modelerror %f', ...
       round(meanabs_mean.*100)./100, round(meanabs_original.*100)./100, xstress, ...
       round(meanabs_projection.*100)./100);

  fprintf(st);
  title([method ' | ' st]);


  % What-if
  % -------

  fprintf('\n\nType RETURN to perform what-if\n');
  figure(1);
  % keyboard
  %return

  fprintf('\n');

  gridxy = (x-ones(size(x,1),1)*minx) ./ ...
           (ones(size(x,1),1)*delta);
  % translates xcoords (actual projection points) into coords 
  % on the 2D grid we've created for display. When we get a 
  % ginput, its the same coord system as the gridxy (actual
  % projection points)

  fprintf('\n');    
  fprintf('lastpoint 2d:\n');
  fprintf('%s\n',num2str(gridxy(size(x,1),:)));
  fprintf('lastpoint nd:\n');
  %fprintf('%s\n',num2str(xoriginal(size(x,1),:)));
  fprintf('%s\n',num2str(zscore2data(xoriginal(size(x,1),:),zmean,zstd)));
  fprintf('\n');

  xoriginal_std = std(xoriginal);
  % if we're not using zscores, this will be a number different to
  % 1. Otherwise it'll be 1
  % The data reported will be the original values, not the zscores
  % which have been generated for purposes of estimating/mapping
  % Thus, we inverse transform all these zscore values

  view(0,90); % top view
  while 1,
    fprintf('Select a location, or press space to quit');

    % Get input position
    % ------------------

    figure(1);
    [x1,x2,button] = ginput(1);
    if button<1 | button>3, 
      disp('rotate and then RETURN to continue or DBQUIT to exit');
      keyboard
      view(0,90);
    end;

    % show nearest 2d point
    % ---------------------

    mini = find_closest2([x1 x2],gridxy);
    if ~isempty(lab),
    display_map_label( x(mini,:), minx, delta, lab(mini,:),    ...
                     degree,c,w,norml,kernel,width,model,      ...
                     over_landscape_eps, textwidth, linewidth, ...
                     textcolor, textheight, fontsize);
    end;
        
    % find nd vector for [x1,x2]
    % --------------------------

    % now lookup value at x1,x2
    
    xo = grnn(gridxy,xoriginal,[x1 x2]);
    % gridxy is the x value, xoriginal (the full coord) is the y
    % [x1 x2] is the xnew point in 2d, now interpolate to get full
    % coord

    % differences between selected point and xoriginal(n,:)
    diff = xo-xoriginal(size(x,1),:);
    realdiff = zscore2data(xo,zmean,zstd) ...
               - zscore2data(xoriginal(size(x,1),:),zmean,zstd);

    % rank diff by percent of standard deviation
    % this will be superfluous because we've converted to zscore
    % therefore, xoriginal_std will be equal to 1. Thus, the most
    % significantly different dimensions are just the dimensions which
    % vary the most in terms of their zscore values

    percent_std = diff ./ xoriginal_std;
    sign_of_percent = sign(percent_std);

    [a,i] = sortrows([  abs(percent_std)'      ...
                        sign_of_percent'       ...
                        [1:size(xoriginal,2)]'      ]);

    a = flipud(a); % highest differences first

    %if size(a,1)>3, a = a(1:3,:); end;
    % user needs to specify the number of dimensions they want to see

    % Display the differences
    % -----------------------

    fprintf('\n');
    fprintf('newpoint 2d coords:\n');
    fprintf('%s\n',num2str([x1 x2]));
    fprintf('newpoint nd coords:\n');
    %fprintf('%s\n',num2str(xo));
    fprintf('%s\n',num2str(zscore2data(xo,zmean,zstd)));
    %fprintf('diff:\n');
    %fprintf('%s\n',num2str(diff));
    %fprintf('%s\n',num2str(realdiff));
    fprintf('dimension differences in order from highest to lowest:\n');
    fprintf('%d\t%f\t%f\n',[a(:,3)         ...
                            a(:,1).*a(:,2) ...
                            realdiff(1,a(:,3))']');
    fprintf('\n');
  end;



% ================================================================
% ================================================================
%
% Changes to make
%
% kmeans copy back to home directory (uses memlookup so no duplicates)
% spline_train unchanged except for local choice of kmeans or memlookup
% spline_train has to be changed so that the kmeans algorithm is more
% efficient
% 
% weighted mds for points from other companies (deweight points)
% in gradient opt, for those rows which are high confidence, increase
% the stepsize, and decrease the stepsize
%
% in the monotonic phase, don't worry so much if out of order
%
% Whatif
% ------
% 
% We want to interp between points in 2D space to read off their x values,
% (so if we see a ridge, click on it and get its underlying nD vector)
% If we keep metric, linear fit between old and new points, then
% can we linearly interp? linear fit means central dilation plus offset
% if point half way between x1 and x2, in new space, x1 & x2 should
% keep their distance relations in the least squares sense, then
% interped point between x1 and x2 in new space may be correct. 
%
% Inverse map. take point on map, read in coords of x1,x2,...,xn,
% translate into original in coords of x1,x2,...,xn, find y value
% Note that we have a predictive model of the 2D coordinate y values
% We also need a predictive model of the nD coordinate y values
% we can then compare the discrepancy!
%
% Error estimates
% ---------------
%  
% average distance from point to mean - error of mean model b1
%
% interp model on nD data. calc error on hold-out points.
% This is the normal generalization error. g1
%
% nD-2D projection. hold out same points. interp model. calc error on 
% hold-out points. This is the generalization error g2.
% 
% b1..g1 is predictability of domain
% 
% g2/g1 = percent error increase due to surface projection. Should be
% proportional to stress.
%
% stress
%
% 
% Graphs
% ------
%
% torgerson, kruskal direct, kohonen net points and stresses, and linear interp
% kruskal_direct
% mesh, contour, surfc, surfl
% pan out 
% thin plate, cubic, linear, polynomial surfaces
% companies
%
% graphs to do:
% plot of dimensions versus stress. (externally, function which calls
% direct optimize) to get an idea of if the 2d projection is a poor fit
% 


  % Build the landscape
  % -------------------

if 0,
  x = rand(30,2);
  y = x(:,1)+x(:,2).^2;

  [xi,yi]=meshgrid([0:.1:1,0:.1:1]);zi=griddata(x(:,1),x(:,2),y,xi,yi,'linear');
  figure(1);clf;mesh(zi);colormap(hsv);
  %hold on;plot3(x(:,1),x(:,2),y,'b*');
end;

  

