function distp = monotonic_average(dist,distp,d,tol,method);
%
% distp = monotonic_average(dist,distp,d,tolerance,method);
%
% given dist and an estimate matrix distp, removes distance discrepencies
% in distp by averaging (smudging) values of distp which don't have the
% same monotonic < ordering of dist. eg. if dist had [1 2 3] and distp
% had [1 5 4], monotonic_average would average those out-of-order
% values, to give, [1 4.5 4.5]. Thus, after monotonic_average has 
% finished, the distp distances are monotonically sound with respect
% to dist. Note, however, that distp may still represent a matrix which
% is impossible to generate using low-dimensional coordinates. Below
% we've used torgerson to retreive the coordinates. In 
% gradient_optimize_stress we perform monotonic_averaging, and then 
% follow with gradient descent on the coordinates, so that distp
% represents a compromise distance which x tries to aim for.
% On the other hand, the monotonic_average distp matrix should be easier
% to achieve than the original matrix dist, because distp will have had
% several values tied (due to the averaging), and hence, these won't
% exert stress anymore.
%
% dist      - desired distance matrix
% distp     - distance matrix estimate
% tolerance - if the difference between two distances is less
%             than this value, then they will be tied
% method    - averaging method 1==pair averaging, or 2==block averaging
%             Block averaging is more accurate but takes longer
% d         - vector version of dist [optional, if you leave this
%             out it will be created]
%
% NOTE
%
% 1) standardize(distp); and standardize(dist); 
% should be performed before using this routine
%
% 2) To save on speed, d - a vector version of dist
% created as d=mat2triulist(dist); can be supplied to 
% this routine. This saves time when monomds calls this
% routine iteratively. Similarly, standardize are outside
% to help on speed. (standardize(dist) need only be performed
% once, since the desired distances don't change)
%
% 3) tol can be set to zero. Method 1 doesn't seem to work with 
% tol=0.
%
% 4) Try to use method 2 unless the data is too large. In that case,
% monotonic_averaging might not be able to be used anyway
% 
% EXAMPLE
% 
% % Torgerson's projection with monotonic_averaging
% 
% tol=0.01;
% dist = rand(10,10);x = torgerson(dist,2);distp = dist2(x,x); 
% standardize_distmat(dist);standardize_distmat(distp);
% so = stress(dist,distp),
% tic, distp1 = monotonic_average(dist,distp,[],tol,1);toc,
% tic, distp2 = monotonic_average(dist,distp,[],tol,2);toc,
% x = torgerson(distp1,2);distp = dist2(x,x); 
% distp=standardize_distmat(distp);s1 = stress(dist,distp),
% x = torgerson(distp2,2);distp = dist2(x,x); 
% distp=standardize_distmat(distp);s2 = stress(dist,distp),
% so,
% 
% SEE ALSO
%
% distp = monotonic_average(dist,distp,d,tolerance,method);
% 

% Initialize defaults
% --------------------

if nargin<2,
  disp(' ');
  disp('monotonic_average wasn''t called properly.');
  disp('Type help monotonic_average for how to call it'); 
  disp(' ');
end;
if nargin<4,
  % no tolerance
  tol=0;
end;
if nargin<5,
  % no method specified
  method=2;
end;
if nargin<3,
   d = mat2triulist(dist);
end;
if isempty(d),
   d = mat2triulist(dist);
end;



dp = mat2triulist(distp);
% use faster method to read in and out of linear array
% (remove trilower, (:), remove last n^2/2 zeros of sorted list
%

   % sort by d,dp and retain the index of each
   % (so if d is tied then order is on db)

   [both,i1] = sortrows([d dp]);
   % i is the sorted order of d,dp

   d  = both(:,1);
   dp = both(:,2);

   j1=find_reverse_index(i1); 

   % Average distances to retain monotonicity
   % -----------------------------------------

changes=1;
while changes>0,
   changes=0;

   if method==1,

   % Vectorized Pair averaging
   % -------------------------
   % this finds every pair which is in the wrong order, and 
   % averages them. This shouldn't be as accurate as method 2, 
   % since the averaging process could distort some distances,
   % just because of the order that something was averaged.
   % The method 2 on the other hand, averages an entire block
   % until it finds an element which is in correct order. Thus, 
   % the average represents the distances better

     diff = dp(2:size(dp,1),1)-dp(1:size(dp,1)-1,1);
     % difference between m and m-1

     % Look for ties
     m = find(diff<0 & diff>-tol); 
     % if there is a difference but its small, tie those items
     m=m+1;
     dp(m,1) = dp(m-1,1);
     
     % Look for out-of-order
     m = find(diff<=-tol);
     % if val[m] - val[m-1] is negative, then it means that 
     % val[m] is smaller than val[m-1]. Hence, these items both
     % needed to be smugged out

     changes = size(m,1); % record number of changes

     m=m+1; 
     % diff was offset back one because it started at 2. This
     % puts it back to the dp coords

     mstart = m-1; 
     % the elements we are going to average are m and
     % the element before m, (this element was out of position)

     dp(m,1) = (dp(mstart)+dp(m)) ./ 2;
     dp(mstart,1) = dp(m,1);

   else

     % Sequential Block averaging
     % --------------------------

     for m=2:size(dp,1),
     % pass through the list and average ds which are out of order
   
       % if distance difference is less than tolerance, then
       % tie the two distances (no sense propagating averages 
       % up and down since the distance between them is so minor)
       if dp(m-1,1)>dp(m,1) & dp(m-1,1)<dp(m,1)+tol,
         dp(m,1) = dp(m-1,1);
         % this tieing will set m equal to m-1. Thus, in one round
         % we can propagate ties up the chain as we loop over m.
         % Ordering before m-1 won't be affected
       end;

       if dp(m-1,1)>dp(m,1),
         % we've found a set which is out of order
         % read forward and find out whether new element is less than mean
         % (if this is the case, it would mean that after performing mean
         % we'd have to mean that element next turn). Thus, keep looking
         % until we find an element which is in order. Then mean the group
       
         i=m+1;
         while i<=size(dp,1),
           dmean = mean(dp(m-1:i-1));
           if dp(i)<dmean, 
             % if we performed mean, i would be out of order,
             % therefore, keep reading ahead
             i=i+1;
           else
             % i would be greater than dmean, which means that
             % i will be in the correct order. Therefore, we
             % can stop the mean here, and 
             % rest assured that i is a correct element
             break
           end;
           % if its the last element and we still haven't found
           % an element which would be in order, we mean it anyway
         end;
  
         dmean = mean(dp(m-1:i-1));
         dp(m-1:i-1) = ones(i-m+1,1) * dmean;
         m = i;

         % instead of the above we can do
         % dmean = (dp(m-1,1) + dp(m,1)) ./ 2;
         % dp(m-1,1) = dmean;
         % dp(m,1) = dmean;

         changes=changes+1;
       end;
     end; % next distance element

   end; % method 2

   fprintf('%d ',changes);
end;
fprintf('\n');

% read back into matrix
% not quite, these were sorted. dp
% use dp
% and dp were sorted

dp = dp(j1,:); % put back into previous order
% d =  d(j1,:);
% check this is occurring properly
distp = triulist2mat(dp); % read back into triupper
distp = distp + distp'; % adds in the trilower

return
