function [c,cy,member] = ...
         vam_online(x,number_of_clusters, lc, stop_thresh, number_of_loops,y,c,cy);
%
% c is the position of the clusters.
% c is just like the vectors in the data, except these are the centroids,
% so it is in the same coordinate system.
% y is: labels supplied for each x (each data point)
% lc is: learning constant
%
% function [c,cx,cy,member] = kmeans(x,number_of_clusters,lc,iterations,y);
%
% This performs kmeans clustering on the input data, x. Returns a set of
% centroids c, for the input data. Each row of c is a different centroid.
% The algorithm also returns cx, the closest input to each cluster,
% and if a y is supplied, cy, the y value which corresponds to the 
% closest input to the centroid. Therefore, the cy value is the /nearest
% known category to the centroid/. Therefore, if we want to perform 
% a k-nearest neighbour algorithm on new data, we can match the new
% inputs, and then assign their category to the category belonging to
% the cluster.
% 
% EXAMPLES
% 
% clustering (or input compression)
% 
% x=rand(100,2);
% clusters=10;iterations=5;c = kmeans(x,clusters,0.1,iterations);
% figure(1);plot(x(:,1),x(:,2),'ro');hold on;plot(c(:,1),c(:,2),'x');
%
% nearest neighbour function mapping
% 
% x = rand(100,2);y=rand(100,1);
% [c,cx,cy] = kmeans(x,clusters,0.1,iterations,y);
% yest = knn(x,c,cy,1); % lookup scheme=1, nearest cluster category
%
% To view cluster membership
% 
% x=rand(30,2);y=rand(30,1);
% [c,cx,cy,member] = kmeans(x,2,0.1,5,y); % 2 clusters
% a=find(member==1);b=find(member==2); 
% figure(1);clf;plot(x(a,1),x(a,2),'ro');hold on;plot(c(1,1),c(1,2),'rx');
% plot(x(b,1),x(b,2),'bo');plot(c(2,1),c(2,2),'bx');
% 
% To view correctness of cluster category
%
% To test view which input is being used for the category vector cy
% (should be the closest input to the centroid, but this algorithm is
% on-line, so it can cause problems)
% 
% [c,cx,cy,member] = kmeans(x,2,0.1,2,y); % 2 clusters
% figure(1);clf;plot(x(:,1),x(:,2),'ro');hold on;plot(c(:,1),c(:,2),'x');
% plot(cx(:,1),cx(:,2),'go'); % closest inputs to each cluster.
%
% function [c,cx,cy,member] = kmeans(x,number_of_clusters,lc,iterations,y);



  % initialize clusters to the k random vectors in the input set
  % ------------------------------------------------------------

if nargin<8, 
  i = ceil(rand(number_of_clusters,1).*(size(x,1)-1 ));
  c = x(i,:);
  cy = y(i,:);
  myplot(x, y, 'w.',c, cy, 'r*', 1, 2);
end;

  % general initialization
  % ----------------------

  member = ones(size(x,1),1) .* (-1);
  closest_inputd = ones(number_of_clusters,1).*inf;
  closest_inputi = ones(number_of_clusters,1).*(-1);
  cx = []; % these are only returned if y is supplied

  for count=1:number_of_loops,

    % record the current position of each c
	c_old = c;
	
    disp(['iteration ' num2str(count)]);
    for i=1:size(x,1),

      %disp([num2str(i)]);

      % find closest centroid for this datapoint
      % ----------------------------------------

      mindist = norm(c(1,:)-x(i,:));
      mini    = 1;
      for cid=2:number_of_clusters,
        d = norm(c(cid,:)-x(i,:));
        if d<mindist,
          mindist = d;
          mini = cid;
        end;
      end;
  
      % adjust the winning centroid
      % ---------------------------

      c(mini,:) = c(mini,:) + lc .* (x(i,:)-c(mini,:));
      cy(mini,:) = cy(mini,:) + lc .* (y(i,:)-cy(mini,:));
	  member(i,1) = mini; % input i belongs to centroid mini

      % is this the closest input to this centroid
      % ------------------------------------------

      if 0,
      if nargin==5,
        mindist = norm(x(i,:)-c(mini,:));
        % because centroid has moved, recalc the mindist
        if mindist<closest_inputd(mini,1),
          closest_inputd(mini,1) = mindist;
          closest_inputi(mini,1) = i;
        end;
      end;
	  end;
      % the problem with this is that when the centroid moves, it may
      % undo its mindist to another centroid which was previously the 
      % winner. Therefore, if there is a string of movements before
      % leaving the loop, it could be poor. More iterations should reduce
      % this problem. (also, an annealing so that movement slows and stop)

    end; % loop over all datapoints
	
	% if we haven't moved very far, terminate the loop
	if sum(mean(abs(c-c_old)))<stop_thresh,
	  disp(['stopping criteria reached ' num2str(count) ' iterations']);
	  break;
	end;
	 
  end; % number_of_loops

if 0,
  if nargin==5,
    % create the cy vector
    % closest_inputd,
    cx = x(closest_inputi,:); % closest input unit to this cluster
    cy = y(closest_inputi,:); % y vector for this closest input unit
    % (the category for the cluster is the same as the category for the
    % closest input to the category, ie. nearest neighbour)
  end;
end;

return % return those centroids


