function [wmatcorrlist, itemcorrlist, pricecorrlist, tree_graph, itemdescript_additions] = ...
          analyse_group(pricegroup,itemgroup,itemdescript,wmatgroup,num);
% 
% [wmatcorrlist, itemcorrlist, pricecorrlist, treegraph, itemadditions] = ...
%   analyse_group(pricegroup,itemgroup,itemdescript,wmatgroup,num);
%
% Full-matrix analyses on a very small group
% of items. (usually this is an aggregated group of items, eg. turn the
% 16,000 items into 300 item categories and continue.
% The maximum number of items (groups) being analysed in the itemgroup/
% pricegroup series is around 900. 
% 
% Group-level analysis of items
% - group similarities
% - similarity hierarchies
% - MDS of items
% - graph of cross-elastic effects (price cannibalization)
% - graph of series correlation similarity
%
% EXAMPLES
% analyse_group(pricegroup,itemgroup,group,wmat,5);
% % with a timeseries, the item-item results will be similarities, the
% % price competitor results will be price cross-effects
%
% analyse_group(pricebasket,itembasket,basket,wmat,5); 
% % if you send in a basket-level series, the item-item results will be substitutions
% % and affinities
%
% to analyse a subgroup of items,
% % catall is a set of items to analyse together as a group 
% wmat_sub = wmat(catall,:);wmat_sub = wmat_sub(:,catall); % restricts to items in catall
% analyse_group(filled_price(catall,1:365),item(catall,2:366),itemdescript(catall,:),wmat_sub,306);
%
% wmatcorrlist = analyse_group ...
% generate_graph_format(tree_graph,itemdescript_additions(:,:),1,'tomsawyercomplex');
% generate_graph_format(wmatcorrlist,itemdescript(catall,:),1,'tomsawyercomplex');
% 
% SEE ALSO
% analyse_store, analyse_group, analyse_item


% Parameters
% ----------

diary off
if exist('out_analyse_group')==2,
   delete('out_analyse_group');
end;
diary('out_analyse_group');
diary on

fprintf('\n');
fprintf('analyse_group\n');
fprintf(' size of list: %d\n',num);
fprintf(' results written to file: out_analyse_group\n');
fprintf('\n\n');
%fprintf('-------------\n\n');


% List strongest wmat cross-elasticites
% --------------------------------------

s = abs(sign(wmatgroup)); 
% creates a support matrix which is 1 if a wmat value is present
% and 0 everywhere else. We then pass in a sthresh of 0, so that any
% elements which have 0 are ommitted.

fprintf('Price cannibalization (wmat)\n');
fprintf('----------------------------\n\n');
display_correlations(wmatgroup,num,s,0, ... 
                     itemdescript,itemdescript, ...
                     ['Driver item\tAffected item\t' ...
                      '+1price = +x affected item demand']);
fprintf('(*) wmat is probably a small version of the relations which are probably going on.\n');
fprintf('see Price cannibalization (corr) for more complete cannibalizations\n');

% Graphdraw wmat
% --------------

  % wmat is a good candidate to graph draw because
  % - if using group, should have few nodes and they should call back
  %   to each other
   
  nn = max(size(find(wmatgroup~=0)));
  % decrease this if graphing is trouble

  wmatcorrlist = find_sig_corrmat_els(wmatgroup,nn,s,0);
  % extracts out all non-zero cross-elasticities. We'll now graph these
  % call graphdraw


% Item correlation matrix
% -----------------------

  fprintf('\ncalculating item-item correlation matrix\n\n');
  corr  = corrmat2(itemgroup',itemgroup');
  %corr2 = corr-diag(diag(corr));
  corr2 = corr-tril(corr); % since its symmetric, don't report the 

% Display item correlations
% -------------------------

% strongest pos & neg correlations between categories
fprintf('Item-item correlations\n');
fprintf('----------------------\n\n');
  display_correlations(corr2,num,ones(size(corr)),zeros(size(corr)), ... 
                       itemdescript,itemdescript, ...
                       ['Item1\tItem2\tSeries correlation']);
                    
              
% Graphdraw item-item correlations
% ---------------------------------

  % only show num edges
  fprintf('\nextracting graphdraw item-item matrix\n'); 
  itemcorrlist = find_sig_corrmat_els(corr2,num); 
    % extracts the top num pos & neg correlations. We'll now graph these
  % call graphdraw
  % graphdraw(itemcorrlist,itemdescript,1,'tomsawyercomplex');
  
  % Graphdraw item-item distance
  % -----------------------------
  
  if 0,
  dist=dist2(itemgroup,itemgroup);
  dist = (dist-mean(mean(dist)))./(std(dist(:)));
  % remove all high distances. The far-away relation doesn't graph
  % well, I get everything far away from missing. Like correlation,
  % this will only show closeness
  dist = dist-tril(dist);
  i=find(dist>0);dist(i)=zeros(size(i,1),1);
  % figure(1);clf;h=log(1+dist);h1=hist(h(:));plot(h1);
  itemdistlist=find_sig_corrmat_els(dist,num);
  generate_graph_format(itemdistlist,group,1,'tomsawyercomplex');
  end;  
  
% MDS item correlations (distances)
% ---------------------------------

fprintf('calculating item-item distances\n');
dist=dist2(itemgroup,itemgroup);  % one or the other
%dist = 1-corr;                   % one or the other
fprintf('performing MDS on item-item distances\n');
x=torgerson(dist,2);
r=1:size(itemdescript,1);
figure(2);clf;hold on;
plot(x(r,1),x(r,2),'.');
text(x(r,1),x(r,2),char(itemdescript(r,:)),'fontsize',8);
voronoi(x(r,1),x(r,2));
zoom on;


% Price competitors - item,price correlation matrix
% --------------------------------------------------

  fprintf('\ncalculating price cannibalization matrix\n');
  corr  = corrmat2(pricegroup',itemgroup');
    
% Display price competitors
% --------------------------

fprintf('Price cannibalization (correlation matrix)\n');
fprintf('------------------------------------------\n\n');
display_correlations(corr,num,ones(size(corr)),zeros(size(corr)), ... 
                     itemdescript,itemdescript, ...
                     ['Driver item\tAffected item\t' ...
                      'Cross-correlation']);
fprintf('(*) positive correlation means b eats a, (ainc price equals binc demand)\n');

% Graphdraw price competitors
% ---------------------------

  fprintf('\nextracting graphdraw price cannibalization matrix\n');
  pricecompcorrlist = find_sig_corrmat_els(corr,num);
  % extracts out all non-zero cross-elasticities. We'll now graph these
  % call graphdraw
  generate_graph_format(pricecompcorrlist,group,1,'tomsawyercomplex');

% Friendly items
%  corr2 = (corr2 + corr2')./2; 
  % average of effect of a on b and b on a
%  [node,edge,similarity,head] = agglomerative_cluster(corr2,method,centroids);

% wmat price assist hierarchy
% ---------------------------

%  corr2 = (wmatgroup + wmatgroup')./2; 
    % average of effect of a onto b (wab)
    % and effect of b onto a (wba). Strong negative correlations are the
    % ones we want to isolate. These are items which are hurting each
    % other. problem is, the group we selected may just be one of several.   

%  belowzero = find(corr2<0);
 % corr2(belowzero) = zeros(size(belowzero,1),1);
  %[node,edge,similarity,head] = agglomerative_cluster(corr2,method,centroids);
  
  
  % item-item   hierarchy graph liststrongest mutual_cooperators mutual_competitors
  % price-item
  % wmat (selected price-item)
  % space-item
  % ad-item
  
diary off;
%return 

  
% Item correlation (distance) hierarchy
% -------------------------------------

method    = 5;
centroids = size(pricegroup,1);


fprintf('\nAgglomerating item series to find hierarchies\n');
fprintf('Immediately following this, hierarchies will be extracted into separate graphs\n');
fprintf('for graph drawing. Graph will be recorded in file "out_graph" in the current directory\n');
  [node,edge,similarity,head]=agglomerative_cluster(corr2,method,centroids);
  % Agglomerate the items

%  % Display items
%  for i=1:size(edge,1),
%    if edge(i,2)<size(group,1),
%      fprintf('%s\n',char(group(edge(i,2),:)));
%    else 
%      fprintf('--\n');
%    end;
%  end;

  % also try just selecting largest correlations, one after another
  % also, try graph drawing
  
  % Draw tree
  tree_graph = []; 
  % tree_graph is the saved collection of all these hierarchies
  % we can graph this using the graph formats
  itemdescript_additions = itemdescript;
  original_down = size(itemdescript,1);
  % this is the string version of the head's numeric id
  for i=1:max(size(head)),
    fprintf('\n');
    traversed_edges = graph_unordered_btree_wrapper(head(i),node,[edge similarity],itemdescript,1,1,0.01,20);
    v=axis;axis(v.*1.5);
    tree_graph=[tree_graph;traversed_edges];
    st = num2str(head(i));
    blanks = char(ones(1,size(itemdescript,2)-size(st,2)).*32);
    itemdescript_additions(head(i),:) = [st blanks];
  end
  

% Basis items in store
% --------------------

group = itemdescript;
fprintf('top-down clustering item series\n');
[zscore,datamean,datastd] = data2zscore(itemgroup); % convert series levels to zscores
[c,centroidlist,cover,cy,level,member] = binary_cluster(zscore,10,0.1,100,zeros(size(itemgroup,1),2));

if 0,
% collapse the tree by looking for identical centroids
start=centroidlist(size(centroidlist,1));
pos = size(centroidlist,1);
samethresh = 0.01;
while start~=1,
   % get the parent
   parent = floor(start./2);
   % check l child
   l = parent.*2;
   if distance(c(parent,:),c(l,:))<samethresh,
      centroidlist = setdiff(centroidlist,l);
      
      fprintf('%d (lchild) same as parent %d, cutting child\n',l,parent);
   end;
   r = parent.*2+1;
   if distance(c(parent,:),c(r,:))<samethresh,
      centroidlist = setdiff(centroidlist,r);
      fprintf('%d (rchild) same as parent %d, cutting child\n',r,parent);
   end;
   start = parent;
end;
end;   

figure(1);graph_btree_wrapper(centroidlist,1,1,1);title('preliminary graph');
% show the tree we induced
            
% Build a description for each centroid
% determine which item is closest to the centroid. This is the protoypical
% item for that centroid
fprintf('finding closest descriptions for each centroid\n');
k=3;
[yest,id,dist] = k_nearest_neighbour(c(centroidlist,:),zscore,zeros(size(zscore,1),2),k);
% note, group(id(1),:) is the description for centroid c(centroidlist(1),:) 
c_descript = sparse(size(c,1),size(group,2));
for i=1:k,
   c_descript(centroidlist,size(group,2).*(i-1)+[1:size(group,2)]) = group(id(:,i),:);
   if i<k, c_descript(centroidlist,size(group,2).*(i-1)+size(group,2)-2: ...
         size(group,2).*(i-1)+size(group,2)) = ones(size(centroidlist,1),1)*double([' / ']);
   end;
end;

% convert that centroidlist into a conventional edge graph
fprintf('converting binary_cluster centroidlist into edge graph\n');
edge = [];
for i=1:size(centroidlist,1),
   % look for left child
   l = find(centroidlist(i).*2==centroidlist);
   if ~isempty(l),edge = [edge;centroidlist(i) centroidlist(l) dist(l,1)];end;
   r = find(centroidlist(i).*2+1==centroidlist);
   if ~isempty(r), edge = [edge;centroidlist(i) centroidlist(r) dist(r,1)];end;
end;

% Pass over that edge graph and recode links which are
% below the thresh
fprintf('removing centroids which are the same\n');
same_thresh=0.0001;

i=1;
no_edges = size(edge,1);
while i<=no_edges,
   if distance(c(edge(i,1),:),c(edge(i,2),:))<same_thresh,
      % find all instances of node edge(i,2)
      % and recode it as edge(i,1)
      fprintf('node %d is the same as its parent %d, merging\n',edge(i,2),edge(i,1));
      ii = find(edge(:,1)==edge(i,2));
      if ~isempty(ii),
        edge(ii,1) = ones(size(ii,1),1).*edge(i,1);
        % assign all those nodes to be identical to the parent
      end;  
      edge = [edge(1:i-1,:);edge(i+1:size(edge,1),:)];
      % remove edge(i,1),edge(i,2)
      i=i-1;no_edges = no_edges-1;
   end;
   i=i+1;
end;

figure(1);clf;figure(2);clf;figure(3);clf;
figure(4);graph_btree_wrapper(centroidlist,1,1,1);
figure(5);graph_unordered_btree_wrapper(centroidlist(1),[],edge,char(full(c_descript)),1,1,0.1,5);


pause on;
for centid=1:size(centroidlist,1),
   fprintf('%d\t%d\n',centroidlist(centid),centid);
   figure(1);clf;plot(c(centroidlist(centid),:));title(char(full(c_descript(centroidlist(centid),:))));
   figure(2);clf;if ismember(centroidlist(centid).*2,centroidlist), plot(c(centroidlist(centid).*2,:));title(char(full(c_descript(centroidlist(centid).*2,:))));end;
   figure(3);clf;if ismember(centroidlist(centid).*2+1,centroidlist), plot(c(centroidlist(centid).*2+1,:));title(char(full(c_descript(centroidlist(centid).*2+1,:))));end;
   pause;
end;



% ========================================================================
% Substitute items (note basket form needed to get substitutes)
%corr = corrmat2(itemgroup',itemgroup');
%corr2 = corr-diag(diag(corr));
%[node,edge,similarity,head] = agglomerative_cluster(-corr2,method,centroids);
%for i=1:size(edge,1),if edge(i,2)<size(group,1),fprintf('%s\n',char(group(edge(i,2),:)));else fprintf('--\n');end;end;
