function [x,err] = bisection_optimize(fun, lower, upper, dimension, ...
                                       seconds, iterations, varargin);
% 
% [x,err] = bisection_optimize(fun, lower, upper, dimension, ...
%                              seconds, iterations, varargin);
% 
% Uses the bisection method to optimize a function of a single variable. 
% This method can be used when a single optimum is known to exist between two 
% finite extrema. The method is similar to the Golden ratio search described 
% by Press et. al (1999).
%
% ASSUMPTIONS
%
% Lower and upper bound must be included. Non-gradient method
% Optimization of single variable only.- if you have a multi-dimensional variable, 
% then you must specify the one dimension on which the bisection is to be performed
% Function must NOT have any local optima. Function must be smooth and monotonic. 
% 
% Because this routine is so fast, it can be a useful method for checking what 
% an optimum is likely to be, versus the optimum being generated by gradient_optimize
% 
% EXAMPLE
%
% opt = bisection_optimize('quadratic',[-100 -100],[100 -100],1,inf,inf);
% % lower is [-100 -100], upper is [100 -100], dimension to optimize is 1, 
% % inf number of seconds, inf number of iterations
%
% opt = bisection_optimize('logistic_priceprofit_curve',-10,8,1,inf,100,cost,qty,b);
% 
% SEE ALSO
%
% gradient_optimize, newton_optimize


% 10                  100            50               8 
% lower --------------p1-------------p2--------------upper
% 10                  50             100              8
% 10                  4               50
% between 10,100 and 100,50
% 
% 


if isempty(dimension), dimension=1; end;
  

t0 = clock;
iteration=0;
%keyboard;

y = [feval(fun,[lower;upper],varargin{:});nan;nan]; % point1,point2,lower,upper

while 1,

  boundy = y([1:2],1); % boundy is the lower and upper from last iteration
  point1 = lower;
  point2 = lower;

  point1(dimension) = lower(dimension) + (upper(dimension)-lower(dimension))./3;   % 1/3rd the way from lower to upper 
  point2(dimension) = lower(dimension) + 2.*(upper(dimension)-lower(dimension))./3;

  if 0,
    y = feval(fun,[point1;point2],varargin{:}); 
    y = [y;boundy];
  else
    % used to be   
    y = feval(fun,[point1;point2;lower;upper],varargin{:});
  end;

  % Figure out which point is the new optimum
  % -----------------------------------------

  opt = nan;
  [m,i] = min(y);
  yopt = y(i);
  if i==1, % point1
    opt = point1;
    %lower = lower;
    upper = point2;
  end;
  if i==2,
    opt=point2;
    lower=point1;
    %upper=upper;
  end;
  if i==3,
    opt=lower;
    %fprintf('Unable to continue\n');
    break;
  end;
  if i==4,
    opt=upper;
    %fprintf('Unable to continuen');
    break;
    opt = upper;
  end;


  % Detect local optima
  % -------------------

  if lower==upper | point1==point2 | y(1)==y(3) | y(2)==y(4),
    break;
  end;
  if y(1)>y(3) | y(2)>y(4), 
    fprintf('WARNING: Function has local optima. Golden search is not guaranteed to find global optimum\n');
    %;
  end;


  % Stopping conditions
  % -------------------
  
  iteration=iteration+1;
  err(iteration) = yopt;
  if iteration>iterations, 
    fprintf('iterations reached\n');
    break;
  end;
  if etime(clock,t0)>=seconds, 
    fprintf('%d seconds have elapsed\n',seconds);
    break;
  end;
end; % while

fprintf('%d iterations\n',iteration);
fprintf('%d seconds\n',etime(clock,t0));
fprintf('optimum at %f with y-value %f\n',opt(dimension),yopt);
x = opt;