function [corr,covariance] = corrmat(x,y);
%
% [corr,covariance] = corrmat(x,y);
%
% Calculates the correlation matrix between x and y
% Corr(x,y) = [corr(x1y1) corr(x1y2) corr(x1y3) ...
%              corr(x2y1) corr(x2y2) corr(x2y3) ...
%              corr(x3y1) 
%              ...
%
% x1 is a the row vector for dimension 1, and so on.
% Thus, it calcs the correlation between each dimension in x with 
% every other dimension in y. 
% 
% This is the same as the covariance matrix, with each element
% normalized by the variance between those two vectors.
%              
% x and y must have the same number of dimensions
% and rows
%
% EXAMPLE
% 
% rows = 10; dims = 2;
%
% % (1) Check corrmat(x,y) is equal to the correlation of each dimension of x
% %     with each dimension of y
%
% r1=rand(rows,dims);r2=rand(rows,dims);
% corrmat(r1,r2)
% [correlation(r1(:,1),r2(:,1)), correlation(r1(:,1),r2(:,2)); ...
%  correlation(r1(:,2),r2(:,1)), correlation(r1(:,2),r2(:,2))],
%
% % (2) corrmat(x,x)==corrcoef(x)
%
% tic,corrmat(r1,r1),toc,
% tic,corrcoef(r1),toc,
%
% NOTES
%
% corr is a correlation matrix described above
%
% covariance is the covariance matrix divided by the number of
% elements. This is done as this is the traditional definition of
% covariance matrix (average covariance between each element of the 
% dimension (x,y))
%
% corrmat(x,x) == corrcoef(x) and is symmetric, whereas corrmat(x,y)
% with x~=y results in a non-symmetric correlation matrix
%
% corr(x,y) = 
% xx=x-mean(x);yy=y-mean(y);
% (xx'*yy)/(sum(xx.^2).^0.5*sum(yy.^2).^0.5)

if nargin==1,
   y = x;
end;
if nargin<1,
   disp('no arguments');
   return
end;
%if size(x,2)~=size(y,2),
%   disp('x and y dims are different');
%   return
%end;
%if size(x,1)~=size(y,1),
%   disp('x and y rows are different');
%   return   
%end;

x = x-ones(size(x,1),1)*mean(x);
y = y-ones(size(y,1),1)*mean(y);

covariance = (x' * y);

sumx = sum(x.^2);
sumy = sum(y.^2);

normx = ones(size(x,2),1)*sumx;
normy = ones(size(y,2),1)*sumy;

%normx = ones(size(x,2),1) * sum(  (x.^2)  );
% sum of row
%normy = ones(size(x,2),1) * sum(  (y.^2)  );


corr = covariance ./ ((normx' .* normy).^0.5);
% Correlation. Is the covariance between x and y divided by the 
% variance(x,x) * variance(y,y)

covariance = covariance ./ size(x,1);
% For return. Covariance matrix is traditionally divided by the number of elements
% (so it is the average covariance between each element
% BTW, we might as well return the covariance matrix, as we have 
% computed it