function corr=corrmat2(r1,r2,missingVal);
%
% corr=corrmat2(x,y,missingVal);
% 
% DESCRIPTION
% 
% correlation matrix between x and y
% corr(i,j) is equal to correlation(x(:,i),y(:,j))
% Function now handles missing values. Missing values are handled by zeroing them out 
% They then produce no weight on the covariance matrix, or the variance-variance matrices
% 
% EXAMPLE
% 
% % Correlation between price series and a particular demand series
% % (price and demand are sideways vectors)
%
% % fill_price_array, so no zero gaps
% price2 = [price(:,1) fill_price_array(price(:,2:size(price,2)))];
% 
% % Reduce price and item via change_resolution (makes these smaller series, with binned values)
% price2 = [price(:,1) change_resolution(price(:,2:size(price,2)),16,1)]; % price is a mean
% item2  = [item(:,1)  change_resolution(item(:,2:size(item,2)),16,0)];
%
% corr = corrmat(price(:,2:size(price,2))'  , item(1,2:size(item,2))');
%
% % transpose means both series have rows (days) down, and many dimensions (series) across
% % Note that in computing the correlation, the days operate like dimensions in the correlation -
% % two series are identical if their entries in days (dimensions) match up
%
% % display the best correlations
% % resulting corrmatrix is no_price_series x 1
% 
% [a,i] = sort(corr);
% disp('best correlations: (correlation, seriesID)');
% [ a(1:10,1) i(1:10) ],
% disp('worst correlation: [correlation, seriesID)');
% [ a(size(a,1)-9:size(a,1)) i(size(a,1)-9:size(a,1))],
%
% SEE ALSO 
% corrmat -- another version which only handles matrices of exactly the same size
% 


if size(r1,1)~=size(r2,1),
   disp('rows in r1 and r2 are different');
   disp('routine gives the correlation between each dim of r1 and r2.');
   disp('this is equal to the correlation between a set of vectors across dimensions 1..r');
   return
end;

%keyboard
if nargin>2, 

  % MissingVal handling
  % --------------------

  % Calculate the mean
  r1mean = mean_missing(r1,missingVal);
  r2mean = mean_missing(r2,missingVal);
  
  % Find the missing values
  if isnan(missingVal),
    r1miss = find(isnan(r1));
    r2miss = find(isnan(r2));
  else
    r1miss = find(r1==missingVal);
    r2miss = find(r2==missingVal);
  end;

else

  % Regular processing
  % ------------------
  
  % Calculate the mean
  r1mean = mean(r1);
  r2mean = mean(r2);
end;


% Subtract the mean
r1 = r1-ones(size(r1,1),1)*r1mean;
r2 = r2-ones(size(r2,1),1)*r2mean;

if nargin>2,
  % Zero out the missing values  
  r1(r1miss) = zeros(size(r1miss));
  r2(r2miss) = zeros(size(r2miss));
end;


corr=(r1'*r2) ./ (   ((sum(r1.*r1)'.^0.5)*ones(1,size(r2,2))) .* (ones(size(r1,2),1)*sum(r2.*r2).^0.5));
