function [model,wout,modelspecs, ...
          meanabs,perabs,correl, totpred, totactual, ...
          trainout,yestout,depout,count_mat] = create_demand_models( price, qty,  ...
          train1, test1, ...
          number_of_stepwize_candidates, ...
          number_of_autocorr_terms1, days_in_model, minlag, ...
          max_degree, accuracy_resolution, use_indicator, pricegroup);

% [model,wout,modelspecs,meanabs,perabs,correl,totpred,totact,...
%  trainest,yest,dep,count_mat]=create_demand_models(price,qty,...
%  train, test, number_of_stepwize_candidates,       ...
%  number_of_autocorr_terms1, ...
%  days_in_model, minlag, max_degree, accuracy_resolution, ...
%  use_indicator);
%
% given a price and qty timeseries, perform a variable selection
% and induce a model for those variables. The variables in the model
% include (i) dayofweek, monthofyear indicator variables
% (ii) price series of every other item and itself
% (iii) items own demand at lag
%

% INPUTS
%
% price, qty   price and qty timeseries of the form above
%
%               time1 time2 time3 ... timen
%        price   1.25  0      0         1.00
%        qty     1     0      0         1
%  
% train, test  id ranges for the train series and test series
% number_of_stepwize_candidates number of price variables
% (the top and bottom correlating price series), which will be
% put into the variable selection
% number_of_autocorr_terms 0 == switch demand forecasting off
% days_in_model  number of days in autoregressive demand forecasting
% min_lag        delay between series and the prediction. This is
%              the time it takes to implement an action
% max_degree   highest degree used in the regression
% accuracy_resolution 1==normal, 7==sum all forecasts to a 7 day
%                     running result

% View price, item relationship
% 
% for i=1:size(price,1),
% figure(1);clf;plot(price30(i,:)./sum(price30(i,:))); ...
% hold on;plot(item30(i,:)./sum(item30(i,:)),'r-'); ...
% correlation(item30(i,:)',price30(i,:)'), pause;end


% To check the accuracy of the demand model,
% induce demand model for 10% of data, ie. price(1:4000,:)
% for each day, 
%   get actual rev total ./ 10
%   get model rev 

% Todo:
% (*) auto-cutting of timeseries: Only start timeseries at the
%     first non-zero day (this is only important if using
%     demand autocorrelation or series)
% (*) make stepwize return the weights
% (*) smoothly implement the different variants: arma on timetospike
%     series, with and without demandids, and so on

% Experiments:
% (*) pass it item(:,2:size(item,2)-lag), item(:,2+lag:size(item,2))
%     This sees if there's a correlation with the demand series of other
%     items and the current. If this predicts well (check the R), then 
%     we may have to modify this routine to take both price and demand
%     ids. (so the autocorr isn't the driver. It is the movement of
%     other products)
% (*) don't force i to be part of model
% (*) test auto-retraining at a lag of 7 days [ideally, run stepwize 
%     again using original input. (save it, rerun it). However, 
%     we could just retrain weights. (again would require input to
%     be built). In any case it will slow it down by a factor of 250.
%     equals 250 hours. (no corr though & save input)
%     lag = 10;
%
% (*) longer timeseries, longer timeseries with demand autocorr component
%     or demand variable selection
% (*) Run with MEANABS versus R versus F
% Run (i) whole model, with all lag terms in (uncomment above)
%    (ii) lag sample with stepwize selection (comment out above)
%   (iii) only lag (remove input price) versus only price (remove inp demand)
%   
% From this decide whether full or partial demand lag model is best
% Determine accuracy increase from including demand lag terms
  

% Global model analysis:
% (*) run create_models (save them)
% (*) eval models using each price point on each day of the price/item
%     series. Compare to actual total. Get 99% confident actual is +/-
%     $xx of actual revenue of store
% (*) write optimization code 
% (*) run optimization on models to find best price
% (*) determine price change, projected revenue increase

% Add categories to candidate set
% [group2,pricegroup_median,count2] = group_by(itemcategory,filled_price,'median');
% [group2,pricegroup_mean,count2]   = group_by(itemcategory,filled_price,'mean');
% 


train=train1;
test=test1;

  % Parameters
  % ----------
  
  disp(' ');
  
  series_size = size(price,2); 
  % do we include the code at the start or not? nope

  if nargin<5,
    if size(train,2)>100
      number_of_autocorr_terms = 2;
    else
      number_of_autocorr_terms = 0;
    end;
  end;
    
  if nargin<6, 
    minlag = series_size./2; 
    % don't trust anything beyond this lag
  end;
  
  if size(price,1)<number_of_stepwize_candidates,
     disp('fewer rows (items) than stepwize_candidates!');
     fprintf('There are %d rows, and you have requested %d candidates\n',...
        size(price,1),number_of_stepwize_candidates);
     disp('please decrease the number of candidates');
     return   
  end;
  
  if size(find(price(1,:)==0),1)>0,
     disp('price must be a filled array');
     disp('if you sent filled_price and you got this error, then disable this warning');
     disp('temporarily for your data');
     return
  end;
  
  if nargin<10,
     accuracy_resolution=1;
  end;

  if max(train)>series_size | max(test)>series_size,
    % training / test range bounds checking
    disp('train or test range goes past the end of the series');
    disp(['series_size is ' num2str(series_size) ...
          ' max(test) is ' num2str(max(test))]);
    disp('please reenter the train/test range');
    disp(' ');
    return
  end;
 
  % Set top and bottom stepwize vars
  % --------------------------------
  
  %minlag = min(lagrange);
  maxlag = minlag + days_in_model; %max(lagrange);
  
  number_of_stepwize_candidates = round(number_of_stepwize_candidates);
  number_of_top    = round(number_of_stepwize_candidates./2);
  number_of_bottom = number_of_stepwize_candidates-number_of_top;

  % Display params
  % --------------

  fprintf('\n');
  fprintf('create demand models\n');
  fprintf(' number_of_autocorr_terms %d\n',number_of_autocorr_terms1);
  fprintf(' number_of_stepwize_candidates %d\n',number_of_stepwize_candidates);
  fprintf(' autocorr terms are legal between minlag %d and maxlag %d\n',minlag,maxlag);
  fprintf(' accuracy resolution %d\n',accuracy_resolution);
  fprintf(' ---------------------------------\n');
  fprintf('\n');

  pause off; % for stepwize


  % Create dayofweek indicator variable
  % -----------------------------------
  
  if use_indicator,
    indicator_width  = 1;
    indicator_period = 7; % weekly indicator
    % indicator_period=1; % variable is useless
  else
    indicator_width  = 1;
    indicator_period = 7;
  end;
  dayofweek = create_indicator_variable(indicator_width, indicator_period, series_size);
  % Add three more to make indicator size come to 10 (same as size of 7 day plus 3 mean prices)
  dayofweek = [dayofweek;ones(3,size(dayofweek,2))];
  INDICATOR_SIZE = 10;
  
  %dayofweek = pricegroup;
  %INDICATOR_SIZE = size(pricegroup,1);
  
  dayofweek_train = dayofweek(:,train);
  dayofweek_test  = dayofweek(:,test);
  % If we want to include a monthly indicator variable, create it
  % here and add it to the bottom of dayofweek indicator
    
  if 0,
  disp('building zscore');
  zprice = data2zscore(price')';
  avg_zprice = mean(zprice);
  disp('building percent');
  m = mean(price')';
  percent_price = price ./ (m * ones(1,size(price,2)));
  avg_percent_price = mean(percent_price);
  disp('building avg_price');
  avg_price = mean(price);
  
  dayofweek = [dayofweek(1:7,:);avg_zprice;avg_percent_price;avg_price];
  dayofweek_train = dayofweek(:,train);
  dayofweek_test  = dayofweek(:,test);
  end;
  
  
  % m=mean(filled_price')';
  % percent = filled_price(:,2:size(filled_price,2)) ./ ( m * ones(1,size(filled_price,2)-1) );
  
  
  
  % Initialize model
  % ----------------

  totpred   = 0;
  totactual = 0;  
  yestcomb = sparse(1,size(test,2));
%  yestout = sparse(zeros(1,size(test,2)));
%  depout  = sparse(zeros(1,size(test,2)));
  %correl  = zeros(size(price,1),1);
  %meanabs = zeros(size(price,1),1);
  %perabs  = zeros(size(price,1),1);

  model   =   sparse(size(price,1), ...
                     number_of_stepwize_candidates + ...
                     indicator_period + ...
                     days_in_model);

   modelspecs = [];
%  modelspecs = [number_of_stepwize_candidates,  ...
%                indicator_period, ...
%                days_in_model];

  wout = sparse(size(price,1),size(model,2)+1);
  
  count_dayindicator = 0;
  count_meanprice    = 0;
  count_ownprice      = 0;
  count_crossprice    = 0;
  count_lagdemand     = 0;

  % ==========================================
  % MAIN LOOP
  % ==========================================
  
  % rrr=[1 8 16 24 32 50 67 80 91 105 110 130 140 156 171 184 195 202 209 213 1000];
  for i=1:400, %size(price,1), %size(price,1), % for each item
 % for i=1:200,  

% uncomment if you want to skip to 
% fast movers
%
%  i=1;
%  while 1,
%     fprintf('\n');
%     while 1,
%        i=i+1;
%        if i>size(price,1), break; end;
%        fprintf('.');
%        if size(find(qty(i,:)>0),2)>300,
%           break;
%        end;
%     end;
%     if i>size(price,1), break; end;
%     fprintf('\n');
     

%keyboard

  % autoretraining
  % comment out here and at the end if 
  % you don't want autoretrainingc
  % also uncomment at the end ****
  % these are constants, we won't worry about passing them in
%  time_between_retrain = 50;
%  no_retrains = floor(size(test1,2)./time_between_retrain);
%  for t=1:no_retrains,
%   disp(['retrain ' num2str(t)]);
%   % determine training range
%   if (t-1).*time_between_retrain > 0,
%      train = [train1 test1(1,1:(t-1).*time_between_retrain-1)];
%      test  = [test1(1,(t-1).*time_between_retrain:size(test1,2))];
%   else
%      train = train1;
%      test  = test1;
%   end;
   
   % No autoretraining
   % uncomment this for normal
   test = test1;
   train = train1;
   
     tic,
     number_of_autocorr_terms = number_of_autocorr_terms1;
     
     if number_of_autocorr_terms>0,

     % Determine best demand autocorrelation
     % -------------------------------------
     % if demand is autocorrelated with itself, we can use it as a timeseries
     % in order to predict its next level. Two caveats though, 
     % a) its normal for demand not to be correlated with itself, (cf. walmart)
     % b) demand at lag can't be used beyond a certain minlag of perhaps 4 weeks
     % c) due to the lack of a strong relationship, we'll only use a single
     % day to autopredict the demand at a later point in time (ie. an AR(1)
     % model

    % Following is code for picking terms by using auto-correlation
    % at lag
    %corr = xcorr(full(qty(i,:)),full(qty(i,:)),maxlag,'coeff');
    % corr is symmetrical since it is forward and backward shift, and 
    % series is the same (it is an autocorrelation)
    % corr = corr(1,series_size+1+minlag:size(corr,2));
    % only view half the mirrored corr series, and half greater than minlag
    % the indices of corr are now lags +minlag+1
    %[a,lag] = sort(abs(corr));
    %if size(lag,1)<number_of_autocorr_terms & size(lag,1)>=1, 
    %   number_of_autocorr_terms = size(lag,1);
    %end;
    %lag=1;
    %if ~isempty(lag),
    % lags = lag(1:number_of_autocorr_terms); %+minlag;
      % only use the best n lags
      % choose best index of corr. This is the best lag
      % -1 because corr(series_size) is the center of the shift, and so
      % is exactly 1.0. -minlag means that we can't choose an autocorrelation
      % at a lag less than minlag, ie. we assume that the users cannot 
      % implement decisions within this time window
      % Due to this autocorrelation term, we have to adjust the 
      % training and test ranges (so that we have a series at lag for
      % the prediction


      % Build the demand_at_lag series
      % ------------------------------

      lags = [minlag:1:maxlag]'; % try different step

      max_selected_lag = max(lags);
      % our series must have at least this many lags for a prediction
      % to take place
  
      for id=1:size(lags,1),
        l=lags(id,1);
        series_range = max_selected_lag-l+1:size(price,2)-max_selected_lag;
        demand_at_lag(id,1:size(series_range,2)) = full(qty(i,series_range));
        % normal demand at lag

        % convert to time to spike
        %demand_at_lag(id,:)=real(time_to_next_spike(demand_at_lag(id,:)));
      end;

  else % only call if more than one autocorr terms
    % demand_at_lag = [];
    max_selected_lag = 0; % no lag for this model!
  end;


  % Determine tr and te range
  % -------------------------

  tr = train(max_selected_lag+1:size(train,2));
  te = test(max_selected_lag+1:size(test,2));
  % tr and te and demand equivalents 
  % adjust the training and test series so they have enough space for the
  % lag term. train and test are 

  if ~isempty(find(qty(i,tr)>0)),

    % Select stepwize candidates
    % --------------------------

% retrain_step
% retrain_days
% number_of_retrains = floor(size(test,2) ./ retrain_days)+1;
% for retrain_step=1:number_of_retrains,
% if retrain_step.*retrain_days > size(test,2),
%   break;
% end;
% tr = train(max_selected_lag+1:size(train,2));
% tr = [tr test((retrain_step-1)+1.*retrain_days : 
%                retrain_step.*retrain_days))];
% 
% 
    if number_of_stepwize_candidates>0,

      corr2 = corrmat2(price',qty(i,:)');
      % Determine allitems Price correlation
  
      [a,id] = sort(corr2);
      % pick top and bottom correlations

      % find first isnan (this is the bottom of the low corrs)
      min_isnan = min(find(isnan(a)));
      if isempty(min_isnan), min_isnan=size(a,1);end;

      vars = [id(1:number_of_top); ... % top corrs
              id(min_isnan-number_of_bottom+1:min_isnan)]; % bottom corrs
    end;

    % Construct the prediction series
    % --------------------------------

    if ismember(i,vars),
      vars = setdiff(vars,i)';
    end;

    input = [price([i;vars],tr)' dayofweek(:,tr)'];

    if number_of_autocorr_terms>0,
      input = [input demand_at_lag(1:size(lags,1),1:size(tr,2))'];
      % Comment this out to force the stepwize to ignore 
      % autocorr terms
      % Mandatorily adding later gives model using full lag components
      % rather than just a stepwize selection of the lag days
      % Still don't know if a stepwize on the lag days will be good
      % enough, or if a complete set of days should be included in an
      % autocorr series
    end;

    % Perform stepwize
    % ----------------
   
    input=full(input);
    [v,deg,crit] = stepwize(input,full(qty(i,tr)'),2,0.1,max_degree);
    % R^2 criterion 0.1
 
    % Determine which vars stepwize picked up
    % ----------------------------------------
  
    if ~isempty(v), v=v-1; end; % ignore the first variable
    vars = vars';
    priceids  = find(v<=size(vars,2));
    if ~isempty(priceids),
       priceids  = vars(1,priceids);
       count_crossprice = count_crossprice+size(priceids,2);
       fprintf('crossprice %d\t',size(priceids,2));
      % the original variables referenced at pos priceids
    else fprintf('crossprice 0\t');end;
    if ~ismember(i,priceids), 
      priceids  = [i priceids];
      deg       = [1 deg];
      fprintf('ownprice 0\t');
    else count_ownprice = count_ownprice+1;fprintf('ownprice 1\t');end;
    % adds i to the priceid variables
  
    dayids    = find(v>size(vars,2) & v<=size(vars,2)+INDICATOR_SIZE); % indices into v
    if ~isempty(dayids),
      dayids    = v(1,dayids);
      dayids    = dayids - size(vars,2);
      
      no_dayindicators = size(find(dayids<=7),2);
      count_dayindicator = count_dayindicator + no_dayindicators;
      no_meanprice = size(find(dayids>7),2);
      count_meanprice = count_meanprice + no_meanprice;
      fprintf('day indicators %d\tmeanprice %d\t',no_dayindicators,no_meanprice);
   else fprintf('day indicators 0\tmeanprice 0\t');end;
   % these are indices into the dayofweek indicator series
  
    if number_of_autocorr_terms>0,
      demandids = find(v>size(vars,2)+INDICATOR_SIZE);
      if ~isempty(demandids),
         count_lagdemand = count_lagdemand + size(demandids,2);
         fprintf('lagdemand %d',size(demandids,2));
        demandids = v(1,demandids);
        demandids = demandids-size(vars,2)-INDICATOR_SIZE;
        % gives a single id into the special demand_at_lag series defined above
        outputdemandids = lags(demandids); 
        % when we save the model spec, we need to save the lag id. This is 
        % an id into a hypothetical full demand_at_lag series. We don't
        % create this full lag series because it takes time
      else
        number_of_autocorr_terms=0;
        fprintf('lagdemand 0');   
     end;
    else
      outputdemandids = 0;
      demandids = [];   
    end;
    fprintf('\n');
    
    % Reconstruct the input using the reduced model
    % ---------------------------------------------

    % Do this to put in all of the lag autocorr terms
    %demandids=1:size(lags,1);
    %number_of_autocorr_terms=1;

    input=[];

    % comment out if no price in model  
    if ~isempty(priceids),
       input = [full(price(priceids,tr)')];
    end;
    % if we want to only test i
    % input = [full(price(i,tr)')]; 
  
    if ~isempty(dayids),
      input=[input dayofweek(dayids,tr)'];
    end;
    if number_of_autocorr_terms>0,
      % demand is part of model
      input=[input demand_at_lag(demandids,1:size(tr,2))'];
  
      % time to spike prediction
      %w1=polyfitf(demand_at_lag(demandids,1:size(tr,2))',qty(i,tr)',1);
      %yest1 = polyff(w1,1,demand_at_lag(demandids,1:size(tr,2))');
      %trainmean = sum(sum(qty(i,tr)')') ./ size(tr,2);
      %yest1 = time_to_spike2_series(yest1).* trainmean .* size(tr,2);
    end;     
  
    input=full(input);
    w=polyfitf(input,qty(i,tr)',max_degree);
    % Change model here
    % note that we added price of i after the fact

    yest=polyff(w,max_degree,input); % CHANGED
    % yest = 0.0.*polyff(w,1,input) + 1.0 .* yest1;
    figure(1);clf;plot(full(qty(i,:)'),'b-');hold on;plot(tr,yest,'r-');title(num2str(i));
    
    % THIS MUST BE COMMENTED IN FOR LARGE RUNS
    % if 0,
       trainout(i,1:size(yest,1)) = yest';
    % end;
    
    % Test model
    % ----------
   
    input = []; 
    input = [price(priceids,te)'];
    % if we want to only test i
    % input = [price(i,te)']; 
  
    if ~isempty(dayids),
      input=[input dayofweek(dayids,te)'];
    end;  
    if number_of_autocorr_terms>0,
      % demand is part of model
      input=[input demand_at_lag(demandids,min(test):min(test)+size(te,2)-1)'];
 
     % time_to_spike prediction
%yest1=polyff(w1,1,demand_at_lag(demandids,min(test):min(test)+size(te,2)-1)');
      %yest1=time_to_spike2_series(yest1).* trainmean .* size(yest1,2);
    end;
    input=full(input);
    
    % CHANGE HERE
    yest = polyff(w,max_degree,input); % RESTORE deg
    %yest = 0.5.*polyff(w,1,input) + 0.5 .* yest1;
   
    % Remove <zero forecasts (important post-processing step)
    ii=find(yest<0);
    yest(ii,1) = zeros(size(ii,1),1);

    % Roll up to month level to check accuracy
    % ----------------------------------------

    % Plot before the rollup  
    %if 0, % COMMENTED OUT DUE TO LARGE DATA
    figure(2);clf;plot(full(qty(i,:)'),'b-');hold on;plot(te,yest,'r-');title(num2str(i));
    %end;
 
    dep = full(qty(i,te)');
    if accuracy_resolution>1,
      %yest=change_resolution(yest',accuracy_resolution,0)';
      %dep =change_resolution(dep', accuracy_resolution,0)';
      yest = sumseries(yest',accuracy_resolution,-1,0)';
      dep  = sumseries(dep',accuracy_resolution,-1,0)';      
   end;

    % Record model
    % ------------

    if ~isempty(priceids),
      % record model price var ids
      model(i,1:size(priceids,2)) = priceids;
    end;
 
    wout(i,1:size(w,1)) = w';
    % record weights on these model vars
    % as long as we are not recording
    % any of the other vars, modelspec is implicit
    % Omit recording the others. For now we will simplify the model
    % and its specs by disallowing any autocorrelation series
    % When we check autocorrelation terms on a longer timeseries 
    % (eg. 3 years), we may find that we should include a 
    % lag autocorrelation series, or a stepwize selection of those
    % lags
    % no need to even specify modelspec. The model is entirely the
    % model

  %if ~isempty(dayids),
  % model(i,number_of_stepwize_candidates+1: ...
  %          number_of_stepwize_candidates+size(dayids,2)) = dayids;end;
  %if ~isempty(outputdemandids),
  %model(i,number_of_stepwize_candidates+size(dayofweek,1)+1: ...
  %         number_of_stepwize_candidates+size(dayofweek,1)+...
  %          size(outputdemandids,2)) = outputdemandids;end;
 
    % Record accuracy
    % ---------------
    if 0, % COMMENTED OUT BECAUSE OF LARGE SIZE OF DATA, RECORDING TAKES TIME
    totpred   = totpred + sum(yest); 
    totactual = totactual + sum(dep);
    % note that for auto-retraining, totpred and actual will
    % actually sum up the training and test ranges for each segment
    % (which is correct). stddev, etc, however, will only pick up
    % the final segment
    % mean(totpred) and mean(totactual) is the average daily difference
    % between 30 days of cases and predicted 30 days of cases

    stddev(i,1)   = std(dep); % do we need this
    correl(i,1)   = correlation(yest,dep);
    meanabs(i,1)  = mean(abs(yest-dep));

    % perabs 0 actual days and >0 yest becomes 1
    ii=find(dep==0 & yest>0);
    dep(ii,1)=ones(size(ii,1),1);
    yest(ii,1) = ones(size(ii,1),1).*2; % diff between dep and yest is 1

    % perabs 0 actual days and 0 yest becomes 0
    ii=find(dep==0 & yest==0);
    dep(ii,1) = ones(size(ii,1),1);
    yest(ii,1) = ones(size(ii,1),1); % diff between dep and yest = 0

    perabs(i,1)   = mean(abs(yest-dep)./dep);
    end; % COMMENT OUT RECORDING FOR LARGE DATASETS
  else
     disp('no training data');
     yest = zeros(size(test,2),1);
     stddev(i,1)=nan;
     meanabs(i,1)=nan;
     correl(i,1)=nan;
     perabs(i,1)=nan; 
  end;
  
  if 0, % COMMENTED OUT DUE TO LARGE DATA SIZE
  figure(3);clf;plot(dep,'b-');hold on;plot(yest,'r-');title(num2str(i));
  end;

  %depout(i,1:size(dep,1)) = sparse(dep)';
  %not used because we can access this from outside
  
% if 0, % RECOMMENT FOR LARGE RUNS
yestout(i,1:size(yest,1)) = sparse(yest)';
% end;
  
  drawnow;
  toc,
  pause on;
  %pause;
  pause off;
  %keyboard
  
  fprintf('%d/%d\n',i,size(price,1));
  if rem(i,500)==0,
%%%%%%%%%%%%     save w_model_save.mat wout model
  end;
  
  % COMMENT THIS OUT TO STOP AUTORETRAINING
  % after then end, end;, then put this, picks up yestout
  %loc = test; % t.*time_between_retrain-1;
%  yestcomb(i,test) = yest(:,1)'; % saves it into yestout at this loc
  % (this is the range immediately after the train, for time_between_retrain
  % days)
  %yestout(i,loc);
%  end; % AUTORETRAINING specific

end; % foreach item i


count_mat = [count_crossprice count_ownprice count_meanprice count_lagdemand count_dayindicator],
