function x = create_indicator_variable(width,period,series_size);
% x = create_indicator_variable(width,period,series_size);
% 
% an indicator_variable is a rudimentary way of establishing a 
% dayofweek or other date variable which has a 1 on certain days, and a 0
% on others, without explicitly dealing with dates. The indicator variable
% usually indicates a recurring event, such as first of the month, or
% dayofweek. See below for examples
%
% INPUTS
%
% width means the size of the indicator block, eg. a width of 1 means a
% single 1, a width of 7 means 7 ones in the indicator block.
% period means the number of blocks until the series repeats itself
% series_size means the total number of elements which need to be 
% created, eg. 400 means the indicator_variable should be of size 400
% 
% EXAMPLE
%
% create_indicator_variable(1,7,series_size); % dayofweek indicator
% 
% [1 0 0 0 0 0 0 1 0 0 0 0 0 ...
%  0 1 0 0 0 0 0 0 1 0 0 0 0 ...];
%
% create_indicator_variable(2,4,series_size);
%
% [1 1 0 0 0 0 0 0 0 0 1 1 0 0 0 0 ...
%  0 0 1 1 0 0 0 0 0 0 0 0 1 1 0 0 ...
% 
% create_indicator_variable(30.4,12   , series_size); % monthofyear indicator
% create_indicator_variable(7   ,4.3452,series_size); % weekofmonth indicator
% 
% % note that the above are 365/12 = 30.4 days (width) per month
% % 30.4/7 = 4.34 weeks (period) per month
% % can't do the first of the month
% % note the above with decimals doesn't really work, so 30.4
% % will result in a 30 series, and the actual month will accumulate
% % error over time
% 
% SEE ALSO
%
% x = create_indicator_variable(width,period,series_size);

% create the indicator matrix
% ---------------------------

  mat = zeros(round(period),round(period.*width));
  for d=1:round(period),
    range = round((d-1).*width)+1:round(d.*width);
    mat(d,range)=ones(1,size(range,2));
  end;

% append indicator matrices end to end until we've filed the timeseries
% ---------------------------------------------------------------------

x = [];
while size(x,2)<series_size,
  x = [x mat];
end;

% cut any extra indicator series after the series end
% ---------------------------------------------------

x = x(:,1:series_size);
