function [critval, dcritval, ddcritval, wmat, cons, ...
          qty,   rev,  prof, ...
          dqty,  drev, dprof, ...
          totalqty,   totalrev,   totalprof, ...
          dtotalqty,  dtotalrev,  dtotalprof, ...
          ddtotalqty, ddtotalrev, ddtotalprof] = demand_model_eval(price,  ...
                                                 model,modelspec,w, ...
                                                 critstr,cost,wmat,cons);
% [critval,dcritval,ddcritval,wmat1,cons1,qty,rev,prof,dqty,drev,dprof, ...
%  totalqty,totalrev,totalprof,dtotalqty,dtotalrev,dtotalprof, ...
%  ddtotalqty, ddtotalrev, ddtotalprof] = ...
% demand_model_eval(price',model,modelspec,w,critstr,cost',wmat,cons);
%
% Evals the model obtained from create_demand_models at pricepos
%
% PRICE MUST COME IN AS DAYS DOWN, ITEMS ACROSS
% ie price' and cost' if price is in its standard timeseries form
%
% INPUTS
% critstr = 'totalprof' 'totalrev' 'prof(10,300)'
%
% EXAMPLE
%
% SEE ALSO
%
% [critval,dcritval,ddcritval,wmat1,cons1,qty,rev,prof,dqty,drev,dprof, ...
%  totalqty,totalrev,totalprof,dtotalqty,dtotalrev,dtotalprof, ...
%  ddtotalqty, ddtotalrev, ddtotalprof] = ...
% demand_model_eval(price',model,modelspec,w,critstr,cost',wmat,cons);


  % Precondition
  % ------------

  % Note that price must be days down, items across 
  no_items = size(price,2);
  no_days  = size(price,1);


  % Parameters
  % -----------

  if nargin<6, % no cost field
    cost = sparse(no_days,size(model,1));
  end;

  if isempty(cost),
    % this can happen if they pass in cost, but pass it in as a []
    cost = sparse(no_days,size(model,1));
  end;
  
  if nargin<5, % no critstr field
    critstr = 'totalrev';
  end;

  build_wmat=0;
  if nargin<8, build_wmat=1; % cross wasn't supplied
  else
    % wmat was supplied
    if isempty(wmat), % but it was supplied as a [], so build it
      build_wmat=1; 
    end;
  end;

  % Build wmat
  % ----------

  if build_wmat,

    disp(' ');
    disp('building w and cons sparse matrix');
    disp('please save these matrices and pass them in next time, ');
    disp('as they are expensive to build');
    disp(' ');

    wmat = sparse(no_items,no_items);
    cons = sparse(1,no_items);
    for i=1:size(model,1), % foreach item
      % look at the w / model matrix for this item, and record
      % the weights into the large wmat
      lastnum = max(find(model(i,:)~=0));
      if ~isempty(lastnum),
         if w(i,2)>=0 | w(i,1)<=0, 
            % if own weight >=0 or model intercept <= 0 then improperly formed
         else
            
           wmat(model(i,1:lastnum),i) = w(i,[1:lastnum]+1)';
           
           % i is the model position
           % 
           cons(1,i) = w(i,1);
        end;
      end;
      if rem(i,100)==0, fprintf('%d/%d ',i,size(model,1));end;
    end;
    % wmat(size(model,1)+1,:) = cons;
    % keep cons with the model. Reason is so that 
    % we can pass it in and out easily
    
    % ZERO OUT IMPROPER MODELS
    % ------------------------
    
    
%    wmat = wmat';
    % now wmat is
    % itemodel1 itemmodel2 itemmodel3 ...
    % crosstermfrom1 
    % crosstermfrom2
    % ...

  end; % build wmat

    
  % price = price';
  % cost  = cost';
  % from now on, price is passed in as days down, items across
  %
  % price is (item1..itemn across)
  %          (day1
  %           day2
  %           down)

  % wmat is (item1model item2model, ... across
  %          crossterm_from_it1 
  %          crossterm_from_it2
  %          crossterm_from_it3 
  %          down)

  
% Qty, Rev, Prof
% --------------

  qty  = [price ones(no_days,1)] * [wmat;cons];
  i = find(qty<0);qty(i)=zeros(size(i,1),1);
  rev  = price.*qty;
  marg = price-cost; 
  prof = marg.*qty;

  totalrev  = sum(rev);
  totalqty  = sum(qty);
  totalprof = sum(prof);

  if nargout==1, % no derivatives requested
    eval(['critval = ' critstr ';']);
    critval = -1.*critval;
    % better hope the user didn't request a derivative
    return
  end;


% dqty/dprice
% -----------

  dqty = ones(no_days,1)*diag(wmat)';
  
  % 0.0026
  % this only lists the partial which is on the diagonal. This partial is
  % dqtyi/dpricei. We know, of course, that we have a whole other matrix
  % of partials which we could list.
  % Reason for expanding to days: drev will change each day due to 
  % the different prices, therefore, move this qty out for no_days

  % another way to do that
  % dqty = ones(no_days,no_items)*diag(diag(wmat));
  % 0.0036

  % dqty = diag(wmat); % the deriv of dqtyi/dpricei for each qtyi,pricei
  % again, this is dqtyi/dpricei for each i,
  % but there are also a whole set of additional partials,
  % dqtyj/dpricei.
  % When we take these into account we get
  % dqty = wmat'; % this is the matrix of derivatives of drevj/pricei
  % the derivative is the amount of impact that pricei has on revj, or the
  % weight from pricei onto revj (in the model, revj = w.pricej + w.pricei

% dtotalqty/dprice
% ----------------

  % When we sum these individual contributions, we get the total sum of 
  % contributions for pricei to all models
  % (or to all quantities)
  % This is

  % dtotalqty = ones(no_days,1)*sum(wmat');
  % 0.0263
  % total contribution of each cross-elastic term to all of the models
  % ie. take cross-elastic term i. Flip it up and sum its contributions to
  % all models. This is the change in pricei will have on a change in 
  % (cross-elastic term i onto all others is horizontal across wmat
  
  % another way to do that
  % 0.0055

  dtotalqty = ones(no_days,no_items) * wmat';
  ddtotalqty = sparse(no_days,no_items);

  
% drev/dprice
% -----------
  
  witimespricei = price * diag(diag(wmat)); % equals diag(diag(wmat'))
  drev = witimespricei + qty;

  % 0.0069
  % doesn't add in the other cross-terms, so doesn't pollute this number.
  % this number is exactly the effect of i on its own revenue, no other number
  % is counted in. (which would happen if we used full wmat instead of diag(diag(wmat))
  %
  % the diag component of revi/pricei plus the general benefit in terms
  % of quantity from ?
  % this only gives the derivs of revi/pricei (since on diagonal)
  % but there are also derives of revj/pricei, 
  
  % another way
  % tic, witimespricei = price' .* ([w(:,2);zeros(32,1)]*ones(1,no_days));
  % drev=witimespricei' + qty;toc,
  % 0.0108

% dtotalrev/dprice
% ----------------

  % I believe this is right
  % dtotalrev = (price * wmat') - witimespricei + drev;

  dtotalrev = (price * wmat') + qty;
  ddtotalrev = wmat+wmat';
%i=find(dtotalrev<0);dtotalrev(i)=zeros(size(i,1),1);

  % Second derivative of totalrev 
  % ddtotalrev = d(price * wmat') + ones(no_days,1) * sum(wmat');
  % price * wmat' is identical to qty except that the constant 
  % is not included. So it can be replaced with
  % d( qtyn
  % Interp: price1..n * crosstermi onto 1..n
  % effect of crosstermi on everyone (weighted by price)
  % d (..) = wmat
  % ddtotalrev = wmat + wmat' (for each of the days down)
  % price * wmat' =?= (price*wmat)' for non-symmetric wmat. No
  % 
  
% dprofit/dprice
% --------------
  
  % dprof = ( (price-cost) * wmat') + (qty); % er use difference here?
  % or 
  % dprof = drev - dcost ((cost * wmat') + qty);
  
  % Margin method:
  % same steps as in calculating drev, but with (price-cost) instead
  % marg = price-cost;
  % qqty = [marg ones(no_days,1)]*[wmat;cons];
  % dprof = marg * diag(diag(wmat)) + qqty;
  % dtotalprof = (marg*wmat')+qqty;
  % 0.0415

   dprof = drev - cost*diag(diag(wmat));
   dtotalprof = dtotalrev - cost*wmat';
  ddtotalprof = ddtotalrev;

  % 0.0153
  % check these are the same! they should be
  % prof  = rev - cost*qty
  %       = rev - cost*(p*w with c) (c goes away)
  % dprof/dpricei = drev - all js which i is cross-term for, costj*w
  %               = drev - cost * w'
  %                        costj onto wjdown, cost contr of 

% Check the results!    

ddqty=[];ddrev=[];ddprof=[];
eval(['critval   =   ' critstr ';']);
eval(['dcritval  =  d' critstr ';']);
eval(['ddcritval = dd' critstr ';']);

if 0,
self_excitatory = find(w(:,2)>=0);
% the derivatives & second derivatives for these items will be set to 0.

dcritval(1,self_excitatory)  = zeros(1,size(self_excitatory,2));
end;
%ddcritval(1:no_items,self_excitatory) = ...
%  ones(no_items,1)*zeros(1,size(self_excitatory,2));
% FUCK

critval = -1.*critval;
dcritval = -1.*dcritval;


return




% ==============================================================
% PREVIOUS CODE: THIS WORKS ALSO, DO NOT DELETE

% Note that price is the normal timeseries form,
% price has days across, items down


  % Initialize qty
  % --------------

  qty = sparse(size(model,1),max(testrange));
  qty(:,testrange) = w(:,1)*ones(1,size(testrange,2));
  % put the constant in each day location on every qty
  % note that w(:,1) must be the correct size down, (that is, 
  % size must be equal to size(price,1)

  % rev = qty.*price;
  % use the initial qty value, multiply by price to get the initial
  % rev value. Extras caused by multiplier now add to this initial
  % rev number
  % sparse(size(price,1),max(testrange)); % rev = 0
  % rev is calculated below from the final qty number,
  % so we don't even need this

  % remove that constant, now that its served its purpose
  % (easier referencing of w, since w and model both correspond
  % exactly now)

  % w = w(:,2:size(w,2));

  % Due to sparse matrix problems, the following will all now 
  % be declared as full (and will eat lots of memory)

  price = full(price);
  qty   = full(qty);
  w     = full(w);

  for day=testrange,
 %   fprintf('.');

    for varid=1:size(model,2), 
      % move through each of the terms in the model, 
      % there must be at least one term. (or maybe if stepwize
      % is no good we could put in the mean?)
      % If no terms in model, we default to the 0 value
      % (above, qty = w(:,1), if w(:,1)==0 then qty will be 0
      % and this should be fine for our model)

      i=find(model(:,varid)>0); 
      % find all models which have a term in position varid

      term = model(i,varid);
      qty(i,day) = qty(i,day) + w(i,varid+1) .* price(term,day);
    end; % next term in model
  
    % next day  
  end;

  % Revert to sparse
  % ----------------

  % clear price;
  % clear w;
  qty   = sparse(qty);

  % Calc rev, prof foreach item, from qty
  % -------------------------------------

  rev  = sparse(qty) .* sparse(price);
  prof = sparse(qty) .* (sparse(price)-cost);

  % Calc global rev, prof, qty
  % ---------------------------

  totalrev  = sum(rev);
  totalqty  = sum(qty);
  totalprof = sum(prof);

  % Derivatives
  % -----------

  % qtyi = (wi*pricei + wj*pricej + ... + c)
  % dqtyi/dpricei = wi  % Individual optimum
  % dqty / dpricei = find any occurrence of a pricei cross elastic
  %                  term, retrieve the w
  %                 = sum(w(find(model==i)))
  
  % revi = qtyi * pricei
  %      = (wi*pricei + wj*pricej + ... + c) * pricei
  %      = wi * pricei^2 + wj*pricej*pricei + ... + c*pricei
  % drevi/dpricei = 2*wi*pricei + wj*pricej + ... + c
  %               = wi*pricei + wj*pricej + ... + c + wi*pricei
  %               = qtyi + wi*pricei % Individual optimum
  % drev/dpricei  = 

  % profi = qtyi * (pricei-costi)
  %       = qtyi*pricei - qtyi*costi
  %       = revi - qtyi*costi
  %       = (wi*pricei^2     + wj*pricej*pricei + ... + c*pricei) - ...
  %         (wi*pricei*costi + wj*pricej*costi  + ... + c*costi)
  % dprofi/dpricei = drevi/dpricei - wi*costi
  

  % BIG ASSUMPTION WITH w POSITION
  % w(:,2) (ie. the first variable position) is the weight for itemi !!!!
  % This must be the case

  % dqtyi/dpricei
  % -------------

  dqty = sparse(size(model,1),max(testrange));
  dqty(:,testrange) = w(:,2)*ones(1,size(testrange,2));

  % dqtyi = wi
  % in other words, continue to increase price in the direction of
  % wi (normally negative) will continue to increase qty
  % so moving in direction of wi, we will eventually reach 0

  % drevi/dpricei
  % -------------

  drev = sparse(size(model,1),max(testrange));
  witimespricei = (w(1:size(model,1),2)*ones(1,size(testrange,2))) ...
                      .* price(1:size(model,1),testrange);
  drev(:,testrange) = witimespricei;
  drev = drev + qty;

  % drevi = qtyi
  % so to increase revenue, move pricei in the direction of 
  % qtyi. Higher qtyi at this price point means move pricei higher.
  % 

  % dprofi/dpricei
  % --------------

  dprof = drev;
  dprof(:,testrange) = dprof(:,testrange) - ...
                       ((w(:,2)*ones(1,size(testrange,2))).*cost);

  f=find(w(:,2)==0);
  dprof(f,testrange) = zeros(size(f,1),size(testrange,2));
  % blanks out the profit (set it to zero) for any item 
  % which has a weight equal to 0. (this would result in dprof
  % being high positive, which doesn't make sense

  % dprofi = qtyi - wi*costi
  % be careful about ws which are 0. Must not allow
  % profit to increase
