function [critval,   dcritval, ...
          qty,       rev,       prof, ...
          dqty,      drev,      dprof, ...
          totalqty,  totalrev,  totalprof, ...
          dtotalqty, dtotalrev, dtotalprof, ...
          wmat] =               demand_model_eval2(price,testrange,...
                                                 model,modelspec,w, ...
                                                 critstr,cost,wmat);
% [critval,dcritval,qty,rev,prof,dqty,drev,dprof, ...
%  totalqty,totalrev,totalprof,dtotalqty,dtotalrev,dtotalprof,wmat] = ...
% demand_model_eval(price_day,testrange,model,modelspec,w,critstr,cost,wmat);
%
% Evals the model obtained from create_demand_models at pricepos
%
% INPUTS
% critstr = 'totalprof' 'totalrev' 'prof(10,300)'
%
% EXAMPLE
%
% % train model -- takes 4 days
% [model,modelspec,w] = create_demand_models(price30,item30, ...
%   1:220, 221:382, 20, 0, 0, 0, 1, 1, 0);
%   % (the test range isn't used to obtain weights, so its still clean)
%
% [yest,rev] = demand_model_eval(filled_price,221:412,model,modelspec,w);
% % Evaluate model at each price point
% 
% predicted_rev_series = sum(rev);
% actual_rev_series    = sum(filled_price.*item - filled_price.*0);
% % get organization-wide rev
%
% figure(1);clf;plot(actual_rev_series); hold on;
% plot(predicted_rev_series,'r-');
% mean(abs(actual_rev_series-predicted_rev_series)'),
% correlation(actual_rev_series,predicted_rev_series),
% % determine error bars
% 
% SEE ALSO
%
% [critval,qty,rev,prof,dqty,drev,dprof,totalqty,totalrev,totalprof] = ...
% demand_model_eval(price_day,testrange,model,modelspec,w,critstr,cost,cross);

% Test this using only the 1:200. And check global correlation

  % Parameters
  % -----------

  if nargin<7,
    % no cost field
    cost = sparse(size(model,1),max(testrange));
  end;

  if isempty(cost),
    % this can happen if they pass in cost, but pass it in 
    % as a []
    cost = sparse(size(model,1),max(testrange));
  end;
  
  if nargin<6, 
    % no critstr field
    critstr = 'totalrev';
  end;

  % Determine if wmat needs to be built
  % -----------------------------------
  
  build_wmat=0;
  if nargin<8, 
    build_wmat=1; % cross wasn't supplied, so build it
  else
    % wmat was supplied
    if isempty(wmat), % but it was supplied as a [], so build it
      build_wmat=1; 
    else
      % wmat provided
    end;
  end;

  % Build wmat
  % ----------

  if build_wmat,

    disp('building w sparse matrix');
    disp('please save this, it is expensive to build');

    wmat = sparse(size(model,1),size(model,1));
    cons = sparse(1,size(model,1));
    for i=1:size(model,1), % foreach item
      % look at the w / model matrix for this item, and record
      % the weights into the large wmat
      lastnum = max(find(model(i,:)~=0));
      if ~isempty(lastnum),
        wmat(i,model(i,1:lastnum)) = w(i,[1:lastnum]+1);
        cons(1,i) = w(i,1);
      end;
      if rem(i,100)==0, fprintf('%d/%d ',i,size(model,1));end;
    end;
    % wmat(size(model,1)+1,:) = cons;
    % keep cons with the model. Reason is so that 
    % we can pass it in and out easily

    wmat = wmat';
    % now wmat is
    % itemodel1 itemmodel2 itemmodel3 ...
    % crosstermfrom1 
    % crosstermfrom2
    % ...

  end; % build wmat

    
  % Standardize price for this algorithm
  % ------------------------------------

  price = price';
  % price is (item1..itemn across)
  %          (day1
  %           day2
  %           down)

  % Calc qty
  % --------

  % w is (item1model item2model, ... across
  %       crossterm_from_it1 
  %       crossterm_from_it2
  %       crossterm_from_it3 
  %       down)

  no_days  = size(price,1);
  no_items = size(model,1);
  
% Qty
% ---
  
qty = [price ones(no_days,1)] * [wmat;cons];

dqty = ones(no_days,1)*diag(wmat)'; 
% this only lists the partial which is on the diagonal. This partial is
% dqtyi/dpricei. We know, of course, that we have a whole other matrix of
% partials which we could list.
% Reason for expanding to days: drev will change each day due to 
% the different prices, therefore, move this qty out for no_days

% another way to do that
% dqty = ones(no_days,no_items)*diag(diag(wmat));

%dqty = diag(wmat); % the deriv of dqtyi/dpricei for each qtyi,pricei
% again, this is dqtyi/dpricei for each i,
% but there are also a whole set of additional partials,
% dqtyj/dpricei.
% When we take these into account we get
% dqty = wmat'; % this is the matrix of derivatives of drevj/pricei
% the derivative is the amount of impact that pricei has on revj, or the weight
% from pricei onto revj (in the model, revj = w.pricej + w.pricei

% When we sum these individual contributions, we get the total sum of 
% contributions for pricei to all models
% (or to all quantities)
% This is

  dtotalqty = ones(no_days,1)*sum(wmat');
  % total contribution of each cross-elastic term to all of the models
  % ie. take cross-elastic term i. Flip it up and sum its contributions to all
  % models. This is the change in pricei will have on a change in 
  % (cross-elastic term i onto all others is horizontal across wmat
  
  % another way to do that
  dtotalqty = ones(no_days,no_items) * wmat';
  
  % Rev
  % ---
  
  witimespricei = price * diag(diag(wmat)); % equals diag(diag(wmat'))
  drev = witimespricei + qty;
  % doesn't add in the other cross-terms, so doesn't pollute this number.
  % this number is exactly the effect of i on its own revenue, no other number
  % is counted in. (which would happen if we used full wmat instead of diag(diag(wmat))
  %
  % the diag component of revi/pricei plus the general benefit in terms
  % of quantity from ?
  % this only gives the derivs of revi/pricei (since on diagonal)
  % but there are also derives of revj/pricei, 
  
  % dtotalrev = (price * wmat') - witimespricei + drev;
  dtotalrev = (price * wmat') + qty 
  % I believe this is right
  
  % Profit
  % ------
  
  % dprof = ( (price-cost) * wmat') + (qty); % er use difference here?
  % or 
  % dprof = drev - dcost ((cost * wmat') + qty);
  
  % same steps as in calculating drev, but with (price-cost) instead
  marg = price-cost;
  qqty = [marg ones(no_days,1)]*[wmat;cons];
  dprof = marg * diag(diag(wmat)) + qqty;
  dtotalprof = (marg*wmat')+qqty;
  
  dprof = drev - cost*diag(diag(wmat));
  % I believe
  dtotalprof = totalrev - cost*wmat';
  
  dprof = drev;
  dprof(:,testrange) = dprof(:,testrange) - ...
                       ((w(:,2)*ones(1,size(testrange,2))).*cost);

  f=find(w(:,2)==0);
  dprof(f,testrange) = zeros(size(f,1),size(testrange,2));
  % blanks out the profit (set it to zero) for any item 
  % which has a weight equal to 0. (this would result in dprof
  % being high positive, which doesn't make sense

  % dprofi = qtyi - wi*costi
  % be careful about ws which are 0. Must not allow
  % profit to increase

  
  
% Check the results!    
  

  % Initialize qty
  % --------------

  qty = sparse(size(model,1),max(testrange));
  qty(:,testrange) = w(:,1)*ones(1,size(testrange,2));
  % put the constant in each day location on every qty
  % note that w(:,1) must be the correct size down, (that is, 
  % size must be equal to size(price,1)

  % rev = qty.*price;
  % use the initial qty value, multiply by price to get the initial
  % rev value. Extras caused by multiplier now add to this initial
  % rev number
  % sparse(size(price,1),max(testrange)); % rev = 0
  % rev is calculated below from the final qty number,
  % so we don't even need this

  % remove that constant, now that its served its purpose
  % (easier referencing of w, since w and model both correspond
  % exactly now)

  % w = w(:,2:size(w,2));

  % Due to sparse matrix problems, the following will all now 
  % be declared as full (and will eat lots of memory)

  price = full(price);
  qty   = full(qty);
  w     = full(w);

  for day=testrange,
 %   fprintf('.');

    for varid=1:size(model,2), 
      % move through each of the terms in the model, 
      % there must be at least one term. (or maybe if stepwize
      % is no good we could put in the mean?)
      % If no terms in model, we default to the 0 value
      % (above, qty = w(:,1), if w(:,1)==0 then qty will be 0
      % and this should be fine for our model)

      i=find(model(:,varid)>0); 
      % find all models which have a term in position varid

      term = model(i,varid);
      qty(i,day) = qty(i,day) + w(i,varid+1) .* price(term,day);
    end; % next term in model
  
    % next day  
  end;

  % Revert to sparse
  % ----------------

  % clear price;
  % clear w;
  qty   = sparse(qty);

  % Calc rev, prof foreach item, from qty
  % -------------------------------------

  rev  = sparse(qty) .* sparse(price);
  prof = sparse(qty) .* (sparse(price)-cost);

  % Calc global rev, prof, qty
  % ---------------------------

  totalrev  = sum(rev);
  totalqty  = sum(qty);
  totalprof = sum(prof);

  % ================================================
  % IF YOU HAVE NO NEED FOR DERIVATIVES, RETURN HERE
  % ================================================

  % return
  
  % call demand_model_eval with cross=[1] to suppress the 
  % automatic building of cross, and uncomment the return here
  % This should be much faster at function evaluating.


  % qtyi = (wi*pricei + wj*pricej + ... + c)
  % dqtyi/dpricei = wi  % Individual optimum
  % dqty / dpricei = find any occurrence of a pricei cross elastic
  %                  term, retrieve the w
  %                 = sum(w(find(model==i)))
  
  % revi = qtyi * pricei
  %      = (wi*pricei + wj*pricej + ... + c) * pricei
  %      = wi * pricei^2 + wj*pricej*pricei + ... + c*pricei
  % drevi/dpricei = 2*wi*pricei + wj*pricej + ... + c
  %               = wi*pricei + wj*pricej + ... + c + wi*pricei
  %               = qtyi + wi*pricei % Individual optimum
  % drev/dpricei  = 

  % profi = qtyi * (pricei-costi)
  %       = qtyi*pricei - qtyi*costi
  %       = revi - qtyi*costi
  %       = (wi*pricei^2     + wj*pricej*pricei + ... + c*pricei) - ...
  %         (wi*pricei*costi + wj*pricej*costi  + ... + c*costi)
  % dprofi/dpricei = drevi/dpricei - wi*costi
  

  % Calc derivatives for individual items
  % -------------------------------------

  % this probably won't realy be used, therefore comment out for speed
  
  %
  % BIG ASSUMPTION WITH w POSITION
  % w(:,2) (ie. the first variable position) is the weight for itemi !!!!
  % This must be the case

  dqty = sparse(size(model,1),max(testrange));
  dqty(:,testrange) = w(:,2)*ones(1,size(testrange,2));
  % dqtyi = wi
  % in other words, continue to increase price in the direction of
  % wi (normally negative) will continue to increase qty
  % so moving in direction of wi, we will eventually reach 0

  drev = sparse(size(model,1),max(testrange));
  witimespricei = (w(1:size(model,1),2)*ones(1,size(testrange,2))) ...
                      .* price(1:size(model,1),testrange);
  drev(:,testrange) = witimespricei;
  drev = drev + qty;

  % drevi = qtyi
  % so to increase revenue, move pricei in the direction of 
  % qtyi. Higher qtyi at this price point means move pricei higher.
  % 

  dprof = drev;
  dprof(:,testrange) = dprof(:,testrange) - ...
                       ((w(:,2)*ones(1,size(testrange,2))).*cost);

  f=find(w(:,2)==0);
  dprof(f,testrange) = zeros(size(f,1),size(testrange,2));
  % blanks out the profit (set it to zero) for any item 
  % which has a weight equal to 0. (this would result in dprof
  % being high positive, which doesn't make sense

  % dprofi = qtyi - wi*costi
  % be careful about ws which are 0. Must not allow
  % profit to increase


  % Calculate total derivatives
  % ---------------------------

  wlist = zeros(size(model,1),size(model,1));
  % for each item down, this is a horizontal list of cross-term 
  % weights 

w down is item, across is crossterm weight at position across=item driver
rest is zeros
foreach item, itdemand = price(days(rows),items) * w
price*w
  for i=1:size(model,1),
    ids = cross(i,1:cross(size(cross,1),i));
    wlist(i,1:cross(size(cross,1),i)) = w(ids)';
    % we could save this instead and not have to calc it
 
    wtimesprice(i,1) = (ones(1,size(testrange,2)) *
wlist(i,1:size(ids,2))') .* price(ids,testrange)
% but we would have to 
    % the wids at the cross-term position
    % the id down gives the actual item whose model this is
    %plist(i,1:cross(size(cross,1),i)) = model(ids-size(w,1));
    % this is the list of 
  % dtotalqty = sum up all of the w. terms which have a pricei on 
  % them

  dtotalqty = sparse(size(model,1),max(testseries));
  dtotalqty(:,testseries) = ones(1,size(testseries,2))*sum(wlist')'; 
  % the sum won't take into account the 
  % zeros, so this is fine

  wtimesprice = 
  dtotalrev = 

  % Return critval
  % --------------

  eval(['critval = ' critstr ';']);
  % note that all critstr must be in quotes identical to the
  % variable name, so 'totalrev', 'totalprof', 'prof(10,300)', etc

  return




if 0,
  input=[];

  % collect up the priceids
  % -----------------------

  price_end = min(find(model(i,1:modelspec(1))==0)) - 1;
  if price_end>0,
    priceids = model(i,1:price_end); 
  else
    priceids=[];
  end;

  % collect up the demandids
  % ------------------------

  demand_end = min(find(model(i,modelspec(3):size(model,2))==0))-1;
  if demand_end>0,
    demandids = model(i,modelspec(3):demand_end);
  else
    demandids=[];
  end;

  yest = polyff(w,1,input);

end;

end;

