function [x,err] = gradient_optimize( funfcn, x, lower,upper, ...
                                      alpha,              ...
                                      seconds,            ...
                                      iterations,         ...
                                      max_no_improvement, ...
                                      adaptive,           ...
                                      line_search,        ...
                                      conjugate,          ...
                                      varargin);

% [x,f] = gradient_optimize(funstr, x0, vlb, vub, ...
%     alpha, seconds, iterations, maxno, adaptive, linesearch, conj, ...
%     funparams... );
%                             
% FORM OF THE FUNCTION TO BE OPTIMIZED
% 
% Form funstr must be
% [f(x), df(x)] = funfcn(x,otherparams)
% x and df are horizontal vectors of the same size
%  f(x)  = scalar
%  x     = [x1 x2 x3 ... xn]
%  df(x) = [df/dx1 df/dx2 df/dx3 ... df/dxn]
%
% EXAMPLE
%
% % 0.1 seconds with infinite iterations
% % See how far each can get (admittedly this is a specific function -
% % a convex quadratic with one optimum)
% % note that [f,df] = quadratic(x); is a special function
% % which must return the criterion and first derivative
%
% start = 20;
% x1=gradient_optimize('quadratic',start,[],[],0.1,0.1,inf,inf,0,0,0);
% x2=gradient_optimize('quadratic',start,[],[],0.1,0.1,inf,inf,0,0,1);
% x3=gradient_optimize('quadratic',start,[],[],0.1,0.1,inf,inf,1,0,1);
% x4=gradient_optimize('quadratic',start,[],[],0.1,0.1,inf,inf,1,0,0);
% x5=gradient_optimize('quadratic',start,[],[],0.1,0.1,inf,inf,0,1,0);
% fprintf(['\n\ngraddescent\t%f\nconjugate\t%f\nconj_with_adapt\t%f\n' ...
%          'adaptive\t%f\nlinesearch\t%f\n' ...
%          'start      \t%f\noptimum  \t0\n\n'],x1,x2,x3,x4,x5,start);
%
% [x,f] = gradient_optimize(funstr, x0, ...
%     alpha, seconds, iterations, maxno, adaptive, linesearch, conj, ...
%     funparams ... );

if 0,
if isempty(lower),
  lower=-inf.*ones(size(x));
end;
if isempty(upper),
  upper=inf.*ones(size(x));
end;
end;


% Tolerance
% ----------

using_tolerance=0;

% Print error
% -----------


print = 1;


% Begin timer
% -----------

t0 = clock;

% Convert to inline function
% --------------------------

%funfcn = fcnchk(funfcn,length(varargin));
% what happens if we take this out?


% Input parameters
% ----------------

if print==1,
  disp(' ');
  disp('gradient optimize');
  disp([' learning rate   ' num2str(alpha)]);
  disp([' terminate after ' num2str(seconds) ' seconds']);
  disp([' terminate after ' num2str(iterations) ' iterations']);
  disp([' or ' num2str(max_no_improvement) ' failures to improve']);
  if adaptive,                   disp(' adaptive stepsize');           end;
  if line_search & conjugate~=1, disp(' method is line search');       end;
  if conjugate,                  disp(' method is conjugate gradient');end;
  if ~conjugate & ~line_search,  disp(' method is gradient descent');  end;
                               disp(' -----------------------------');
end;

% Initialize gradient
% -------------------


[f,df] = feval(funfcn,x,varargin{:});
dir    = -1.*df;
% move in the negative direction of the gradient (downhill)

% Optimization loop
% -----------------

err       = [f+eps; f];
iteration = 2;
no_improvement = 0;
starterr = f+eps;
minerr = f+eps;
minx   = x;
mindf  = df;
firstloop = 1;
oldx = x;
oldf = f;

%    OPTIONS(2)-Termination tolerance for X.(Default: 1e-4).
%    OPTIONS(3)-Termination tolerance on F.(Default: 1e-4).
options = foptions;

while (iteration < iterations) & (etime(clock,t0)<seconds), 

   % Calculate gradient
   % ------------------

   dir_old = dir;
   df_old  = df;
   % save old direction and gradient

   oldf = f;
   [f,df] = feval(funfcn,x,varargin{:});
   % get the new gradient


   if conjugate,
   % set conjugate gradient direction
     b = ((df - df_old) .* df) ./ (df_old .^2);
     dir = -1 .* df + b.* dir_old;
     % dnew = -1 * grad + b*d_old
     % b = ((gradnew - gradold) * gradnew) / (gradold.^2)
   else 
     dir = -1.*df;
   end;

   % Line search loop (no gradient readings performed in here)
   % ---------------------------------------------------------

   enter = 1;
   oldx = x;
   while err(iteration)<err(iteration-1) | enter==1,
       enter=0;

       % Update x
       % --------

       x = x + alpha .* dir;

       % Look for upper / lower constraint violations
       % ---------------------------------------------

       if ~isempty(upper),
         violation = find(x>upper);
         x(violation) = upper(violation);
       end;
       if ~isempty(lower),
         violation = find(x<lower);
         x(violation) = lower(violation);
       end;

       % Get function value at new point
       % -------------------------------

       iteration = iteration + 1;
       err(iteration,1) = feval(funfcn,x,varargin{:});
       
       % Remember this point if its the minimum
       % ---------------------------------------
       
       if (err(iteration) < minerr), 
         minerr = err(iteration);
         minx   = x;
         mindf  = df;
       end;
     
       % Accelarate if adaptive
       % ----------------------

       if adaptive>0,
       % Adaptive stepsize
         if err(iteration)<err(iteration-1),
           % speed up
           alpha = alpha + 0.1 .* alpha;
         else
           % brake
           alpha = alpha ./ 10;
         end;
       end;

       % Exit loop if time, error increased, or no linesearch
       % ----------------------------------------------------

       if (print==1), fprintf('%f ',err(iteration)); end;
       
       if etime(clock,t0)>seconds,       break; end;
       if line_search==0 & conjugate~=1, break; end;
       if (err(iteration) > starterr),     
         err(iteration) = minerr;
         df = mindf;
         x  = minx;
         alpha = alpha ./ 10;
         break;
       end;
   end; % endwhile linesearch

   % check for no_improvement
   % ------------------------

   if err(iteration)>err(iteration-1),
     no_improvement = no_improvement+1;
     if no_improvement>max_no_improvement, 
       fprintf('\nno improvement after %d consecutive attempts\n', ...
               max_no_improvement);
       break;
     end;
   else
     no_improvement=0;
   end;


   % Look for x or f lower than tolerance
   % ------------------------------------
   
   if (firstloop==0), 
     if using_tolerance,
       if abs(f-oldf) < options(3), 
         fprintf('delta f below tolerance');
         break;
       end;
       if abs(x-oldx)<options(2), 
         fprintf('delta x below tolerance');
         break;
       end;
     end;
   else
     firstloop=0;
   end;
   

end;
if print==1, fprintf('\n');end;

% Report reason for stopping
% --------------------------

if (iteration >= iterations),
  fprintf('max iterations reached (%d)\n',iterations);
end;
if etime(clock,t0)>=seconds, 
  fprintf('%d seconds have elapsed\n',seconds);
end;

x = minx; % report this x
% err(iteration+1) = minerr; % no need to contaminate the plot
%fprintf('last err %f \n',err);
fprintf(' min err  %f \n',minerr);