function [xout,yout,count,ycellout] = group_by_numerical(x,y,operation,cartesian_product,varargin);
% 
% [xout,yout,count,ycellout] = group_by_numerical(x,y,operation,cartesian_product,varargin);
% 
% Calculates summary statistics on y for every group of x
% if operation==[] then the mean(y) is produced for each group
% if length(operation)==1, eg. {'sum'} then the operation is applied to all columns of y
% if length(operation)==size(y,2), eg. {'sum','max'} then operation{i} is applied to column y(i)
% if cartesian_product==1 then apply every operation to every dimensions of y. The output will be 
%   operation{1}.y(:,1:N) operation{2}.y(:,1:N). By default the cartesian product is not performed.
% params don't know how to handle that yet
% 
% EXAMPLE
% 
% %%%% Calculate means on everything
% [group,pgroup,count]=group_by_numerical([3;1;1;2;3;3;4;5;6],[2;5;5;1;2;0;4;5;6]);
% [group pgroup count],
%
% %%%% Calculate sums on everything
% [group,pgroup,count]=group_by_numerical([3;1;1;2;3;3;4;5;6],[2;5;5;1;2;0;4;5;6],{'sum'});
% [group pgroup count],
% 
% %%%% Calculate mean and median
% [group,pgroup,count]=group_by_numerical([3;1;1;2;3;3;4;5;6],[2;5;5;1;2;0;4;5;6]*ones(1,2),{'sum','median'});
% [group pgroup count],
% 
% %%%% Call a histogram on each group!
% r1=round([1:0.001:3]'.*3);[a,b,c,d]=group_by_numerical(r1,r1+randn(size(r1)),{'hist'},0,[0:1:10]); 
% % last param are the bin centers
% a,
% d{1},
% d{2},
% d{3},
% d{4},
% 
% SEE ALSO
%
% group_by
 

if isempty(x),
  fprintf('WARNING: Empty GroupBy\n');
  dbstack,
  xout=[];yout=[];count=[];ycellout=[];
  return;
end;

  % Parameters
  % ----------

  ycellout = {};
%  if ~isempty(params),
%    fprintf('Don''t know how to handle params as yet\n');  
%    keyboard;
%  end;
  if nargin<4,
    cartesian_product=0;
  end;
  if nargin<2,
    fprintf('need x and y\n');
  end; 
  if nargin<3, 
    operation = {'mean'};
%    operation = [];
  end;  
  if length(operation)==1 & cartesian_product==0,
    for i=2:size(y,2),
      operation{i}=operation{1};
    end;
  end;
  if cartesian_product==0 & length(operation)~=size(y,2), 
    fprintf('cartesian_product is 0, and the length of operation (%d) does not equal the dimensions in y (%d)\n',length(operation),size(y,2));
    fprintf('When cartesian_product mode is 0, you have to specify a specific operation for every dimension.\n');
    fprintf('If you want your listed operations to be applied to every dimension, then run again with cartesian_product=1\n');
    return;
  end;
%  if isempty(params),
%    for i=1:length(operation),
%      params{i}=[];
%    end;
%  end;
  
  
  
  [xsorted,i] = sortrows(x);
  ysorted = y(i,:);
  d = diff(xsorted,1,1) & 1; % 1st order difference, taken in the column vector
  rowdiff = cumsum(d,2) & 1;
  rowdiff = rowdiff(:,size(rowdiff,2));
  rowdiff = [1;rowdiff]; % the first row is of course different
  groupstart = find(rowdiff(1:size(rowdiff,1))==1);
  groupend   = [groupstart(2:size(groupstart,1))-1;size(rowdiff,1)];

  count = groupend-groupstart+1;    
  if cartesian_product==1,
    yout = zeros(size(groupstart,1),size(y,2).*length(operation));
  else
    yout = zeros(size(groupstart,1),size(y,2));
  end;

  for groupid=1:length(groupstart),
    if isempty(operation), 
      yout(groupid,:) = mean(ysorted(groupstart(groupid):groupend(groupid),:),1);
    else
     
      if cartesian_product==0,
      
        % not cartesian product
        % ---------------------
      
        for opid=1:length(operation),
          result = aggregate_group(operation{opid},ysorted(groupstart(groupid):groupend(groupid),opid),varargin{:}); % opid
          if length(result)>1, yout(groupid,opid) = nan;ycellout{groupid,opid} = result; else yout(groupid,opid) = result;end;
        end; % foreach opid
      
      else
        
        % cartesian product mode
        % -----------------------
        
        for opid=1:length(operation),
          for dim=1:size(y,2),
            result = aggregate_group(operation{opid},ysorted(groupstart(groupid):groupend(groupid),dim),varargin{:}); % opid
            across = ((opid-1).*size(y,2))+dim;
            if length(result)>1, 
              yout(groupid,across) = nan;
              ycellout{groupid,across} = result; 
            else 
              yout(groupid,across) = result;
            end;
          end;
        end;
        
      end; % cartesian_product mode    
    end; % empty operation
    
  end; % next groupid
  
  xout = xsorted(groupstart,:);



% ------------------------------------------------------------------------------------------------

function result = aggregate_group(opstr,yy,varargin);  

  %opstr = char(opstr);
  %n = str2num(char(opstr));
  if ~ischar(opstr),
    n = opstr;
    if ~isempty(n), 
      if n>0 & n<1, 
        result = percentile(yy,n);
      else 
        fprintf('Argument was %s.\n',char(opstr));
        fprintf('This was parsed as a request to run percentile on the decile at percentile level %f\n',n);
        fprintf('The percentile level must be >0 and <1.\n');
        fprintf('Changing this to the mean. PLEASE CORRECT THIS PROBLEM.');
        keyboard;
      end;
    end;
  else
    % opstr is a string, evaluate it as a function
    result = feval(opstr,yy,varargin{:});
  end;

return;
