function key1descript = lookup_description(key1,key2,key2descript,missingval);
% newdescript = lookup_description(key_where_data_will_be_imputed,key2,key2descript,missingval);
% 
% Given some data which is keyed by key2 (key2 data), 
% and a new set of keys (which might have additional data), key1,
% impute key2's data onto key1
%
% Given [key2 key2descript] and a new set of keys, [new] 
% attach the key2descripts to new
% 
% To be safe, make sure that key1 and the data attached to key1 is sorted by
% key1 before entering this routine. 
%
% EXAMPLE
%
% key2         = [1;2;3];
% key2descript = ['apples   ';'orange   ';'pineapple'];
% new          = [1;1;1;1;2;2;2;2;3];
% missingval   = ['missing' char(ones(1,size(key2descript,2)-7).*32)];
% lookup_description(new,key2,key2descript,missingval), % should return strings
% 
% key2descript = [1 1 1;2 2 2;3 3 3];
% missingval   = [-9 -9 -9];
% lookup_description(new,key2,key2descript,missingval), % should return a matrix 
%
% % item already exists. We are now given list of it, descript
% % itemdescript = lookup_description(item,item1,descript);
% % will find an itemdescript for each original item

% Parameters
% ----------

fprintf('\nlookup_description\n');
fprintf(' select tab1.key1,tab2.* from tab1,tab2 where tab1.key1=tab2.key2\n');
fprintf(' looking up description in [key2,key2descript] table for each key1 entry\n');
if nargin<4,
   fprintf(' NOTE: Missingval not supplied. Defaulting to ');
   if ischar(key2descript),
      missingval = 32.*ones(1,size(key2descript,2));
      fprintf('missingval==blank spaces\n');
   else
      missingval = -9.*ones(1,size(key2descript,2));
      fprintf('missingval==-9s\n');
   end;
end;
if  ischar(missingval), fprintf(' missingval: %s\n',missingval);end;
if ~ischar(missingval), fprintf(' missingval: %s\n',num2str(missingval));end;
fprintf(' %d elements in new to find a description for\n',size(key1,1));
fprintf(' ---------------------------------------------\n\n');

if size(missingval,2)~=size(key2descript,2),
   fprintf('mismatch between size across of missingval and key2descript\n');
   fprintf('key2descript is size across is %d, missingval size across is %d\n', ...
      size(key2descript,2),size(missingval,2));
   fprintf('Please come up with a missingval which is the same size as the\n');
   fprintf('full data matrix which is being imputed\n');
   return
end;

% Sort both into order
% --------------------

fprintf('sorting...');
[key1,k1id] = sortrows(key1); 
[key2,k2id] = sortrows(key2);
key2descript = key2descript(k2id,:); % puts key2descript into order of key2

[b,reverse_mapping] = sort(k1id); 
% this is the index list which will put key1 
% back into its original order (ie, so that its indices would be 1..n
fprintf('completed\n');


% Initialize key1descript
% -----------------------

% A default filler is 32 for a char array, or 0 for an integer.
% Pre-allocating the key1descript array makes things a little faster
% this shouldn't be necessary now! But will speed things up
% we could switch this to sparse
% key1descript = 32.*ones(size(key1,1),size(key2descript,2)); 
% fill with blanks

if ischar(key2descript),
   key1descript = 32.*ones(size(key1,1),size(key2descript,2));
else
   key1descript = zeros(size(key1,1),size(key2descript,2)); 
end;


% Initialize key1_down
% --------------------

key2_down = 1; % key2 is on the lookup list
no_missing=0;
key1_down = 1; % key1 is the data key which we're trying to find descripts for


% Fast forward key1_down until key1 is greater than key2_down
% -----------------------------------------------------------
% this needs to be done, because we assume throughout that key2_down
% is always below key1, and we need to search forward on key2

 while string_less_than(key1(key1_down,:),key2(1,:)),
    % key1(key1_down,:) is lower than the first value on key2.
    % therefore, key2 can't possibly have an entry for it. So its missing
    no_missing = no_missing+1;
    key1descript(key1_down,1:size(key2descript,2)) = missingval;
    key1_down=key1_down+1;
  end;


% Now run down key1 and key2, and lookup the description
% ------------------------------------------------------

while key1_down<=size(key1,1),
   % lookup the key2 description for this key1
   % try to find it in key2
   
   currentpos = key2_down; 
   % remember where we were on key2 list when the search began
   % this is the currentpos of key2_down
   
   % Look forward from key2_down on, to see if there is a match with key1(key1_down,:)
   % If there is a match, impute it. Otherwise, this key1 is missing
   foundit=0;
   
   % say that key2_down runs off the end of the list. In that case, key2_down 
   % the string_less_than was never triggered, and so all the items on key1 must
   % not have any counterparts in key2
   
   % say that key2_down gets caught by string_less_than. Then we've gone past where key1
   % should have been. But key2 now should be reset to exactly one before this past
   % condition.
   
   while key2_down<=size(key2,1) & string_less_than_equal_to(key2(key2_down,:),key1(key1_down,:)),
      % we're searching for key1 on key2's list. 
      % if as we're going down key2, key2 becomes larger than key1, then
      % because they're in sorted order, we know we will never find key1 at any 
      % position further down. Therefore, abort
      if strmatch(key1(key1_down,:),key2(key2_down,:)),
         % if key1 and key2 are the same, then we lookup the description at key2_down
         % found it
         foundit=1;
         break;
      end;
      key2_down = key2_down+1; % otherwise, we go to the next key2_down position
      % because they are both sorted in order, key1 can skip elements, or have multiple
      % elements, key2 will dutifully move to the next position
   end;

   if foundit,
     imputestring = key2descript(key2_down,:);
   else
      imputestring = missingval;
      if key2_down>size(key2,1), 
         % we ran off the end of the list. Therefore, the lexicographic wasn't triggered
         % and we know that for all key1s (from now on, since they are in sorted order), 
         % are higher than the current key1, and are outside our range. Therefore, we can 
         % set key1 from now on to missing.
         fprintf('at row %d/%d of key1 we found there were no more key2 keys',key1_down,size(key1,1));
         fprintf('which were as big as that key1. Therefore early termination\n');
         fprintf('here is key1 for reference\n');
         key1(key1_down,:),
         key1descript(key1_down:size(key1,1),1:size(key2descript,2)) = ...
            ones(size(key1,1)-key1_down+1,1)*imputestring;
         key1_down = size(key1,1)+1; % exit the main loop
      end;
      
      % Second possibility: we didn't find key1 and ran over to a larger record
      % key2 ran forward into another key which was larger than the current key1
      % therefore, key1's descript is missing, but also, we know that key2 can be
      % reset to the position before it ran over the key1. 
      key2_down = key2_down-1;
      
      % this is the safest thing to do. If we get strange results, we can try this
      % key2_down = currentpos; 
      % reset it back to where it was before we started searching
      % note, this reset shouldn't go back this far. It should go back to the 
      no_missing = no_missing+1;
   end;
   
   % Set the description for this record
   % -----------------------------------
   
   key1descript(key1_down,1:size(key2descript,2)) = imputestring;
   
   
   % Fill duplicates (doubles as fastforward on missing)
   % ---------------------------------------------------
   
   old_key1_pos = key1_down; 
   % this is where key1 is now. Record this so we can compare key1 at that
   % location to the key at the new key1_down location
   key1_down = key1_down+1; 
   % this is where we're looking at
   % this also advances key1_down regardless of whether a duplicate is found below
   % so we are pointing at the new record   
   while key1_down<=size(key1,1),
      if ~isempty(strmatch(key1(old_key1_pos,:),key1(key1_down,:))),
         % this is a duplicate, we have to impute this
         key1descript(key1_down,1:size(key2descript,2)) = imputestring;
         fprintf('d');
      else
         % key1_down is not a duplicate of old_key1_pos, therefore, we'll break here
         % and continue as normal
         break;
      end;
      key1_down = key1_down+1; % look at the next record in key1 to see if its a duplicate
   end;
   
   % Duplicates filled. Record count and then continue
   % --------------------------------------------------
   
   if rem(key1_down,100)==0, fprintf('%d/%d ',key1_down,size(key1,1));end;   
   
end; % main key1_down < size(key1,1) loop
fprintf('\n');

if no_missing==0,
  fprintf('All entries in new were found in the lookup list\n');
else
  fprintf('%d entries in new could not be found in the key2 lookup list\n',no_missing);
end;

fprintf('about to reverse map\n');
key1         = key1(reverse_mapping,:);
key1descript = key1descript(reverse_mapping,:);

% We've now found a key2description for every key1. However, key1 is in sorted order.
% We want to put it back.

