function [mean_percent_continuous, ...
          percent_continuous] = measure_sparsity(series,fig,deciles);
%
% [m,series_cont] = measure_sparsity(series,fig,deciles);
%
% Given a timeseries with days across, and items down,
% returns a sparsity rating. This is 1- average number of continuous days
% in each series. 
%
% EXAMPLE
%
% measure_sparsity(item(1:100,2:size(item,2))); % fig 1
% measure_sparsity(item30(1:100,:)); % fig 3
%


% Parameters
% ----------

if nargin<2,         
  fig=1;      
end;
if isempty(fig),     
  fig=1;      
end;
if nargin<3,         
  deciles=20; 
end;
if isempty(deciles), 
  deciles=20; 
end;


% Calculate percent continuous
% ----------------------------
% the equation is 
% assuming days are going down, and different items across
% |sign(series(t+1..n))|' * |sign(series(t..n-1))| .* (1/(n-t-1) )
% (vertical vector of number of continuous days  mult by to get percent
% 11i * |sign(series(t=1..n))|' * |sign(series(t..n-1))| .* (1/(n-t-1)*i)

series = abs(sign(series)); 
% converts to 1 (present) and 0 (qty absent)

diff = series(:,2:size(series,2)) .* series(:,1:size(series,2)-1);
% following possibilities for day(t) day(t+1) and the score they get:
% 0 0 = 0, 0 1 = 0, 1 0 = 0, 1 1 = 1 (a continuous day)

total_continuous_days = sum(diff')'; 
% add up all the continuous days we found

percent_continuous = total_continuous_days ./ (size(series,2)-1);
% divide total continuous days in each series by total possible continuous
% days

mean_percent_continuous = mean(percent_continuous);
% the mean is the overall sparsity measure for the series we tested
% (its also the middle of the bell curve)

% Distribution of sparsities
% --------------------------
% We will go 100-percent_continuous to convert these to sparsity ratings

figure(fig);clf;
% [n,x] = hist(percent_continuous.*100,deciles);
inc = 1./deciles;
x = [inc:inc:1];
% divide 100 (0..100 are all possible sparse scores) into equal parts
%for i=1:deciles,
%  n(i) = size(find(percent_continuous>=x(i)-x(1)./2 & ...
%                   percent_continuous< x(i)+x(1)./2),1);
%end;

x=x.*100;
n = hist(percent_continuous.*100,x);

bar(100-x,round(100.*n./size(series,1)),'b');
xlabel('Sparsity rating (%)');% xlabel('Continuity rating (%)');
ylabel('% of items');
title('Distribution of sparsity scores');
% title('Distribution of continuity scores');
v = axis;
axis([0 100 v(1,3) v(1,4)]);

% show the mean
hold on;
xpos = 100-mean_percent_continuous.*100;
line([xpos xpos],[0 v(1,4)],'linestyle','--');
%text(full(xpos),v(1,4)./2,'average % continuous days');
% replace this with [0 max(n)] if we don't normalize n


% Cumulative sparsity
% -------------------

figure(fig+1);clf;
ncumm(1) = n(1);
for i=2:deciles,
  ncumm(1,i) = ncumm(1,i-1) + n(i);
end;

%plot(x,round(100.*ncumm./ncumm(deciles)));
%title('Cumulative continuity');
%ylabel('cummulative % of items');
%xlabel('Continuous rating (%)');
%grid on;
%figure(3);

plot(100-x,round(100.*ncumm./ncumm(deciles)));
title('Cumulative sparsity');
ylabel('cumm % of items');
xlabel('Sparsity rating (%)');
grid on;
v=axis;
axis([0 100 v(1,3) v(1,4)]);


% Return sparsity scores
% ----------------------

mean_percent_continuous = 1-mean_percent_continuous;
percent_continuous      = 1-percent_continuous;


% =================================================================
% NOTES
% ie. % of time that we have continuous days, out of all
% days the product was shipped on.
%
% INPUTS and OUTPUTS
% fig and deciles are optional
% m is average percent continuous over all series
% series_cont is a vertical vector with percent continuous for each series
%
% Sparse timeseries which have spikes separated by 0 days are hard to
% predict with ARMA.


