function [x,err] = newton_optimize( funfcn,x, lower,upper,...
                                    seconds, ...
                                    iterations, ...
                                    max_no_improvement, ...
                                    varargin);
% [x,f] = newton_optimize(funstr,x, ...
%                         seconds,iterations,max_no_improvement, ...
%                         otherparams_to_funstr);
%
% Form funfcn must be
% [f(x), df(x), ddf(x)] = funfcn(x,otherparams)
% 
% f(x) is a scalar result (can we change to multicriterion)
%
% x = [x1 x2 x3 ... xn]
% horizontal vector of the unknown parameters
%
% df(x) = [df/dx1 df/dx2 df/dx3 ... df/dxn]
% horizontal vector same size as x, but with the derivs of f wrt each
% x unknown
%
% ddf(x) = [ d (df/dx1) / x1      d (df/dx2) / x1   d (df/dx3) / x1 ...
%            d (df/dx1) / x2      d (df/dx2) / x2      ...
%            d (df/dx1) / x3      d (df/dx2) / x3      ...
%                 ...                ...               ...
% xdown xacross vector with the first derivatives across on the horizontal
% and the deriv of each of those wrt x1, x2, x3, down
%            
% EXAMPLE
% % See how far each can get (admittedly this is a specific function -
% % a convex quadratic with one optimum)
% % note that [f,df,ddf] = quadratic(x); is a special function
% % which must return the criterion and first derivative
%
% [x,f] = newton_optimize(funstr,x, ...
%                         seconds,iterations,max_no_improvement, ...
%                         otherparams_to_funstr);

warning off;
 
t0 = clock;
% Begin timer

funfcn = fcnchk(funfcn,length(varargin));
% Convert to inline function as needed.

% Input parameters
% ----------------

disp(' ');
disp('newton optimize');
disp([' terminate after ' num2str(seconds) ' seconds']);
disp([' terminate after ' num2str(iterations) ' iterations']);
disp([' terminate after ' num2str(max_no_improvement) ' consecutive no improvements']);
disp(' -----------------------------');

% Optimization loop
% -----------------

f         = feval(funfcn,x,varargin{:});
err       = [f+eps; f];
minerr    = f+eps;
starterr  = f+eps;
minx      = x;
iteration = 2;
no_improvement = 0;

while (iteration < iterations) & (etime(clock,t0)<seconds), 

   % Find deriv and second deriv at current point
   % --------------------------------------------

   [err(iteration,1),df,ddf] = feval(funfcn,x,varargin{:});

    % Inverse of ddf using svd method
    % --------------------------------

    % [u,s,v] = svds(ddf);
    % inv_ddf = v*diag(1./diag(s))*u';
    % the svd of a matrix is u*s*v'
    % The inverse of an svd is
    % u*s*v' * v*(1/s)*u' = I
    % note that some eigs should be zeroed out

   % Perform the Newton iteration
   % ----------------------------

   % Fit a line to the present 1st deriv surface, 
   % and find where this line intercepts 0
   % y    = m  .* x + c
   % y-mx = c
   %
   % y      = m      x + (y  - m    x) 
   % dy     = ddf .* x + (df - ddf.*x);
   % 
   % At 0, x = -c/m
   %       x = -(y-mx)/m
   %       x = -y/m + x
   %         = -df/ddf + x
   % 
   % If df = column vector, and so too ddf, then
   % 

   %   x = x - df ./ ddf;

        
    x = x - (inv(ddf)*df')';
    % Because the matrix is square, a Cholesky factorization
    % works. I beieve this is what inv is based on. So we can 
    % get the inverse for the square matrix


    % Look for upper / lower constraint violations 
    % ---------------------------------------------
    
    if ~isempty(upper),
      violation = find(x>upper);
      x(violation) = upper(violation);
    end;
   
    if ~isempty(lower),
      violation = find(x<lower);
      x(violation) = lower(violation);
    end;

    % check for min error so far
    % --------------------------
    
    if err(iteration)<minerr, 
      minerr = err;
      minx   = x;
    end;
       
    % check for no_improvement
    % ------------------------

   if err(iteration)>err(iteration-1),
     no_improvement = no_improvement+1;
     if no_improvement>max_no_improvement, 
       fprintf('\nno improvement after %d consecutive attempts\n', ...
               max_no_improvement);
       break;
     end;
   else
     no_improvement=0;
   end;
   fprintf('%f ',err(iteration));
   iteration=iteration+1;
end;

fprintf('\n');

if (iteration >= iterations),
  fprintf('max iterations reached (%d)\n',iterations);
end;
if etime(clock,t0)>=seconds, 
  fprintf('%d seconds have elapsed\n',seconds);
end;

x = minx;
warning on;
