function [y,pivotted_vals,pivotted_dims] = pivot(x,groupbydims,pivotdims,statisticdims,operation,missingval);
% [y,pivotted_vals,pivotted_dims] = pivot(x,groupbydims,pivotdims,statisticdims,operation,missingval);
% 
% Pivot - arbitrary data re-arrangement with some variable-values forming new variables, and 
% group-by operations aggregating other rows. See examples
% Code has been vectorized.
% 
% INPUTS
% 
% x                            == data
% pivotdims                    == dims that need to be pivoted - sent across the top of the page by value
% groupbydims / dimstokeep     == left-hand-side group-by columns
% statisticdims                == dims that need to be aggregated
% operation                    == operation to apply to each statisticdims column, eg. {'mean','sum','min'}
% missingval                   == value (eg. nan, -9999, or something similar) that will be put into cells 
%                                 if that variable-value-combination does not exist in the data
% 
% EXAMPLE
%
% %%%% Basic pivot
% pivot([1 2 3 4 1 0.5;1 2 3 4 1 0.6;1 2 3 4 2 0.7],[1:4],[5],[6],{'min'})
%   % Use the first 4 columns as the group-by. Take column 5 and break-it-out by value (has values 1 and 2). In each 
%   % cell of former-column-5, take the min of the many records in column 6.
% 
% %%%% Pivot one dimension
% [y,pivotted_vals]=pivot([1 2 3 4 1 0.5;1 2 3 4 1 0.6;1 2 3 4 2 0.7;4 4 4 4 1 0.1;4 4 4 4 1 0.2; 4 4 4 4 5 0.3],[1:4],[5],[6],{'min'})
%   % Use the first 4 columns as the group-by. Take column 5 and break-it-out by value (has values 1,2 5). 
%   % In each cell of former column 5, take the min of the records in column 6.
% 
% %%%% Pivot two dimensions
% [y,pivotted_vals,pivotted_dims]=pivot([1 2 3 4 1 0.5;1 2 3 4 1 0.6;1 2 3 4 2 0.7;4 4 4 4 1 0.1;4 4 4 4 1 0.2; 4 4 4 4 5 0.3],... 
%   [1:2 4],[3 5],[6],{'min'},nan)
%   % We can pivot more than one dimension. Pivot dimensions 3 and 5. Group by 1,2,4. Each new variable-value will 
%   % contain the min of column 6
% 
% %%%% Pivot and group_by_numerical 
% dat = [1 2 3 4 1 0.5;1 2 3 4 1 0.6;1 2 3 4 2 0.7;4 4 4 4 1 0.1;4 4 4 4 1 0.2; 4 4 4 4 5 0.3];
% [xout,yout,count] = group_by_numerical(dat(:,[1 2 4]),dat(:,6),{'sum'});[xout yout],
% [y,pivotted_vals,pivotted_dims]=pivot(dat,[1:2 4],[3],[6],{'sum'},nan),
%   % Check the sum of column 6 for each group 1,2,4 against the same sum as computed by group_by_numerical
%
% %%%% Do pivot and group_by_numerical agree? 
% r = [ones(1000,1) round(rand(1000,1).*5) rand(1000,1)]; % First column, all 1, second column, 0..5, third, random number
% [xout,yout,count]=group_by_numerical(r(:,1),r(:,3),{'sum'}); % Group by dim1 - so we'll have one group with the sum of the random numbers
% [xout yout], % yout is the sum of the random numbers
% y=pivot(r,[1],[2],[3],{'sum'}); 
% sum(y(2:length(y))),
%   % Again we'll just have one group, but we will break-out by dimension 2.
%   % We end up getting dim1=1, dim2..6 are the sums. If we sum up those break-out sums, we should get the sum of all 
%   % the random numbers.
% 
% 
% SEE ALSO
%
% group_by_numerical
% 


if nargin<6, missingval=-9999;end;
if length(statisticdims)>1, fprintf('For the time being (until I think about it some more) we can only accept one statisticdim\n');return;end;
if length(operation)>1,     fprintf('For the time being (until I think about it some more) we can only accept one operation and one statisticdim\n');return;end;

dimstokeep = groupbydims;
%dimstokeep = setdiff([1:size(x,2)],pivotdims);

[uniquerows,uniquerowsid] = unique(x(:,dimstokeep),'rows');
[sortedrows,sortedrowsid] = sortrows(x(:,dimstokeep));
d = abs(diff(sortedrows)); % Find out when are the transitions
d = sum(d')';
transitions = find(d>0); % Any non-zero row indicates that it is different from the previous row

nexttransition = [transitions;size(x,1)]; 
% transitions, because the end of a group is exactly at position transition-1
% Could have 10 10 at the end
% Last row will not ordinarily be recorded because of the diff. Therefore, stick it onto the end.

transitions = [1;transitions+1]; % First row is "different" to the last

x = x(sortedrowsid,:); 
% Re-order the original x data. We will 
% now traverse it in groups, and run operations on each group

pivotted_dims = [];
pivotted_vals = [];
for dim=1:length(pivotdims),
  vals = unique(x(:,pivotdims(dim)))';
  pivotted_vals = [pivotted_vals vals];
  pivotted_dims = [pivotted_dims ones(size(vals)) .* pivotdims(dim)];
end;

% pivotted_vals is now a set of pivot values
% pivotted_dims is now a set of pivot dims (which dimension to use out of pivotdims)

y = zeros(size(uniquerows,1),length(pivotted_vals)); % This is the output matrix

for t = 1:length(transitions),
  % Each transition corresponds to a group of data that we will need to execute 
  % a pivot operation on 
  currentdatid = transitions(t):nexttransition(t);
  for i=1:length(pivotted_dims),
    d = pivotted_dims(i); % The absolute dimension that is being pivotted across the top
    v = pivotted_vals(i); % The value of the dimension to look for
    ii = find(x(currentdatid,d)==v);
    if isempty(ii), 
      y(t,i) = missingval; 
    else
      y(t,i) = feval(operation{1},x(currentdatid(ii),statisticdims));
    end;
  end;
end;

y = [uniquerows y];

