function [price_prof,prof, price_profunc,profunc, ...
          price_rev, rev, price_revunc, revunc] = single_item_optimize(price, ...
           cost, wmat, cons, lower, upper);

% [price_prof,prof, price_profunc,profunc, ...
%  price_rev,rev, price_revunc, revunc] = single_item_optimize(price, ...
%                            cost,wmat,cons,lower,upper);
% 
% Derives the optimum price for each price model, by assuming that each
% model for item i is only based on its own price (price i)
%
% INPUTS
% price  normally a 1 row vector with prices for each
%        item across - although it could have several different cases
%        - days going down), and a 
% wmat   this is the model for each item, however, this model
%        must be expressed as a square matrix with items down, items
%        across column 1 across, all down are the weights on each
%        price in the model
% cons   a horizontal vector size equal to no items with the constants
%        which are added to each model
% 
% EXAMPLE
%
% % Get the optima, then remove prices with positive slope as these 
% % will not be real prices
% % Assign those back to what they were before, ie. don't touch them
% 
% [price_prof,prof, price_profunc,profunc, ...
%  price_rev,rev, price_revunc, revunc] = single_item_optimize(price, ...
%                            cost,wmat,cons,lower,upper);
%

% Zero out positive weights in wmat
% ---------------------------------

% these will result in optima (actually, minima) at negative prices.
% remove these negatives

diag_W = diag(wmat); % Currently diag_W is a single column vector diagonal
%i=find(diag_W>0);    % find positive weights
%diag_W(i,1) = zeros(size(i,1),1); % zero them out

% Form diagonal
% -------------

diag_W = diag(diag_W); 
% now diag_W is a full matrix with a diagonal filled

% Calculate optima
% ----------------

price_rev = (-1.*cons) * (0.5.*diag_W.^-1);
price_prof = (cost*diag_W-cons) * (0.5.*diag_W.^-1);

% Check for revenue price constraint violation
% --------------------------------------------

% If there is a constraint violation, on the upper or lower,
% reset this price to the upper or lower value

price_revunc  = price_rev;
%i=find(price_rev>upper);if ~isempty(i),price_rev(1,i)=upper(1,i);end;
%i=find(price_rev<lower); if ~isempty(i),price_rev(1,i)=lower(1,i);end;
i=find(price_rev>upper);if ~isempty(i),price_rev(1,i)=price(1,i);end;
i=find(price_rev<lower); if ~isempty(i),price_rev(1,i)=price(1,i);end;

% Check for profit price constraint violation
% -------------------------------------------

price_profunc = price_prof;
%i=find(price_prof>upper);if ~isempty(i),price_prof(1,i)=upper(1,i);end;
%i=find(price_prof<lower);if ~isempty(i), price_prof(1,i)=lower(1,i);end;
i=find(price_prof>upper);if ~isempty(i),price_prof(1,i)=price(1,i);end;
i=find(price_prof<lower);if ~isempty(i), price_prof(1,i)=price(1,i);end;

% Eval functions at each of these points
% --------------------------------------

prof    = [price_prof    1] * [wmat;cons];
profunc = [price_profunc 1] * [wmat;cons]; 
rev     = [price_rev     1] * [wmat;cons];
revunc  = [price_revunc  1] * [wmat;cons];

return


% ---------------------------------------------------------------
% Derivations
% revi          = (pricei*wi + c)*pricei
%               = pricei^2*wi + c*pricei
% drevi/dpricei = 2pricei*wi + c
%               = 2.*qty
% at 0, pricei  = 2pricei*wi + c
%       pricei = -c / 2wi
%              = (-cons) * 0.5*diag(W)^-1
%
% profi          = (pricei*wi + c)*(pricei-costi)
%                = wi*pricei^2 + c*pricei - pricei*wi*costi - c*costi
% dprofi/dpricei = 2wi*pricei + c - wi*costi
% at 0, pricei   = -(c - wi*cost)/(2*wi)
%                = (wi*cost-c)/(2*wi)
%                = (cost*diag(W)-cons) * 0.5 * diag(W)^-1
% ===============================================================
