function dependent = sumseries(series,windowsize,direction,usemean);
%
% series2 = sumseries(series,windowsize,direction,usemean);
%
% Given a normal series, this converts that series into a moving
% sum of the series. For example, if direction==-1 then
% the new series at i is the sum of i-windowsize elements from the 
% original series
%
% This can be used for determining arma accuracy for week or 
% month stock inventories (sum forecast and actual over 7 days), 
% or for converting a spikey series into a summed, smooth 
% timeseries which arma can handle (sum past 30..120 days).
% The tradeoff is that for a continuous but high-frequency varying
% timeseries, the variations will be lost and the series could collapse
% to the mean, if the windowsize is large enough.
%
% There is a relation between doing this and taking a low-pass
% of the timeseries. Compare these two. (eg. is the low-pass 
% hurt because the series is spikey, and so is this a better way
% of doing it?)
%
% This operation is also the same as changing resolution
% to windowsize, obtaining the sum, but then keeping the original
% resolution of days. Function superceeds create_dependent
%
% INPUTS
%
% series     timeseries to be converted
% windowsize size of the summing window
% direction  1 means to the right, -1 means to the left or 1 means each
% point in new series is a composite of the future, -1 means each new
% point is a sum of the past
% usemean    1==use mean, 0==use sum
%
% OUTPUTS
% series2 is a sparse matrix. Convert this to normal by typing 
% series2 = full(series2);
%
% EXAMPLES
%
% % create dependent for measuring weekly stock accuracy
% 
% % elminate spikes from spikey timeseries
% sumseries2 = sumseries(item(:,2:size(item,2)),60,-1);
% for i=1:42,figure(1);clf;plot(sumseries2(i,:));drawnow;pause;end;
% 
% % compare low-pass and sumseries
% 
% SEE ALSO
%
% series2 = sumseries(series,windowsize,direction,usemean);

% Note: looking at these convinces me that there is no autocorrelated
% structure in summed series, except perhaps on the 6month to year level.
% This makes sense, since this is like taking a low-pass, so the only
% autocorrelated structure is also low-frequency. 
%
% You know, a high frequency filter of the spikey series should reveal
% frequency components. But then how do you predict based on them.
% 
% Investigate the time_to_spike to series translation
%

if windowsize==1, dependent = series; return; end;

dependent = sparse(size(series,1),size(series,2)); % initialize zeros(size(series));


% Input Parameters
% -----------------

if nargin<3,
  direction=-1;
  usemean=0;
end;
if nargin<4,
  usemean=0;
end;

if ~(direction==1 | direction==-1),
  disp('direction should be a 1 or -1');
  disp('defaulting to past (-1)');
  direction=-1;
end;


% Perform the sum
% ---------------

if direction== -1,
  % Sum the past
  for i=1:size(series,2)-windowsize+1,
    if usemean,dependent(:,i) = mean( series(:,i:i+windowsize-1)' ,1 )';
    else dependent(:,i) = sum( series(:,i:i+windowsize-1)' ,1 )';end;
  end;
  dependent = dependent(:,1:1+size(series,2)-windowsize);
  % added +1
else
  % Sum the future
  ahead=windowsize;
  for i=ahead:size(series,2),
    if usemean,dependent(:,i) = mean( series(:,i-ahead+1:i)' ,1 )';
    else dependent(:,i) = sum( series(:,i-ahead+1:i)' ,1)';
    end;
  end;
  dependent = dependent(:,ahead:size(series,2));
  % removed ahead+1
end;

   

