
function dist = sequential_edit_distance(str1,str2,case_sensitive);
%
% dist = sequential_edit_distance(str1,str2,case_sensitive);
%
% Returns the edit distance between two string cells, where those strings are compared 
% in their original sequential orders.
%
% EXAMPLE
% 
% sequential_edit_distance({'hello','there'},{'there','hello'})
% % Will return a sequential edit distance of 0
%
% INPUTS
% 
% str1 and str2 are cell arrays (eg. {'hello','there'}
% Use str1 = tokenize(str1); to create a string of this kind 
% case_sensitive==1 means case sensitive
% case_sensitive may be ommitted, in which case it defaults to no case sensitivity
% 
% SEE ALSO
% 
% edit_distance
%

if nargin<3,
  case_sensitive = 0;
end;

numwords1 = length(str1);
numwords2 = length(str2);
min_numwords = min([numwords1 numwords2]);
max_numwords = max([numwords1 numwords2]);
total_match_score = 0; % we will add 1 for every full word that we match, and a fraction for every part word

for word=1:min_numwords,
  top1 = str1{word};
  top2 = str2{word};
  
  if case_sensitive,
    perfect_match = strcmp(top1,top2);
  else
    perfect_match = strcmpi(top1,top2);
  end;
  
  if perfect_match~=1,
    % We need to go character by character
    % Find the smaller
    len1 = length(top1);
    len2 = length(top2);
    maxlen = max([len1 len2]); 
    minlen = min([len1 len2]); % min because we only want to run through the min number of chars
    
    matches = 0;
    for char=1:minlen,
      if case_sensitive, 
        match = strcmp(top1(char),top2(char));
      else
        match = strcmpi(top1(char),top2(char));
      end;
      if match==0, 
        break;
      end;
      matches = matches + 1; % we got a match
    end;  
    
    word_match_score = matches ./ maxlen;  % Implicity penalize if there were extra characters on the end
  else % if not a perfect match
    word_match_score = 1;
  end;
    
  total_match_score = total_match_score + word_match_score;
end; % for each word

final_score = total_match_score ./ max_numwords; % again, implicitly penalize if one of the collocs had 3 words, and the other had 4

dist = final_score;