function [series2,dat] = change_resolution(series,binsize,operation);
%
% [series2] = change_resolution(series,no_days,operation);
%
% Changes the timeseries resolution, for example, from 
% daily series to weekly (7-day), monthly (30-day), or 
% longer.
% Note that this is similar to taking a lowpass reconstruction
% of the original function. However, the lowpass filter..
%
% INPUTS
%
% series     is a timeseries on its side (dates across, different series down)
% no_bins    is the number of elements in the new series.
% For example, original series is 400, no_bins is 10, then
% the new series will have 10 elements, with element 1
% being equal to the sum(series(1,1:40)), element 2 = 
% sum(series(1,41:80)), and so on
% operation = 'sum', 'mean', 'std', 'median', etc
% operation = 1 usemean, operation=0, sum
%
% EXAMPLE
%
% % (1) create a series
% s=sin([1:0.05:5]);
% %i=find(s<0.8);s(i) = zeros(1,size(i,1));
% figure(1);clf;plot(s,'b-');
% s2=change_resolution(s,7);
% figure(2);clf;plot(s2,'r-');
%
% % (2)
% d=round(rand(3,10).*4);
% change_resolution(d,4,0),
% change_resolution(d,5,0),
% change_resolution(d,6,0),
% d,
%
% SEE ALSO
% series2 = change_resolution(series,no_bins,use_mean);
%

if nargin<3,
   operation='sum';
end;
if operation(1)==0, operation='sum';end;
if operation(1)==1, operation='mean';end;

no_bins = floor(size(series,2)./binsize);
% this means there could be an incomplete bin at the end which
% isn't filled and is discarded. This is fine, since the other bins
% will all respect the 7 day (or whatever) sum.

series2 = zeros(size(series,1),no_bins);
if nargout>1,
   dat = zeros(binsize,no_bins);
   % Special purpose, collect the first row of data into 
   % Detailed data with all binsize data as a vertical vector, 
   % and no_bins across. This is the detailed data used to calculate
   % the mean and so on
end;

for bin=1:no_bins,
   
   % get start end end of range
   startrange = round(binsize.*(bin-1))+1;
   endrange   = round(binsize.*bin);
   
   % check out of bounds
   if endrange>size(series,2), 
      disp('warning: end of range reached');
      endrange = size(series,2);
   end;
   if startrange>size(series,2), 
      disp('warning: end of range reached');
      startrange = size(series,2);
   end;
   
   % perform the sum
   sumrange = startrange : endrange;
   eval(['series2(:,bin) = ' operation '(series(:,sumrange)'')'';']);
   %   series2(:,bin) = sum(series(:,sumrange)')';
   
   if nargout>1,
      dat(1:max(size(sumrange)),bin) = series(1,sumrange)';
   end;
end;
