function [fit,a,corr,perr] = tunear(series,                   ...
                          training_window,          ...
                          range,                    ...
                          days_in_model,            ...
                          delay_before_prediction);
%
% function [fit,a]   = tunear(series, 
%                             training_window_size, 
%                             test_range, 
%                             days_in_model, 
%                             delay_before_prediction);
%
% example
% fit = dispar(wi(1:10,:),100,400:500,10,7)
% fits models and tests the result on the first 10 series of wi, 
% using a 300:399 as training data (100 days), 400:500 as test data,
% using a 10 day regression model, and having a 7 day delay from 
% regression set to prediction. (ie. the model is predicting 7 days
% in advance).
%
% fit is (vertically) 10 series which is the result for each wi(1:10
% a is the coefficients for each series. These coefficients are written
% horizontally here, so each row entry is a different set of coefficients
%
% 

   range = range(1,1) + delay_before_prediction + max(days_in_model) : ...
                       range(1,size(range,2));

   fit = zeros(size(series));
   f   = zeros(1,size(series,2));
   a   = zeros(size(series,1),days_in_model);
   %m   = zeros(size(series,1),days_in_model/1);

   if isempty(training_window), 
      infer_training_window=1; else infer_training_window=0; 
   end;

% Parameters
% economic method   CONTINUALLY_RETRAIN=0; DETREND=0; STANDARDIZE=0;
% full method       CONTINUALLY_RETRAIN=1; DETREND=1; biasest=1;
% ultrasafe method  CONTINUALLY_RETRAIN=1; DETREND=0; biasest=0;

STANDARDIZE_SCORES  = 0; 
% only used if no detrending. subtracts mean

DETREND             = 1;
% set to 1 if detrend training data detrending might be effective if we
% are continually retraining

CONTINUALLY_RETRAIN = 1; % set to 1 if retrain each training range 

biasest             = 0; 
% change this to zero to switch off bias estimation
% bias estimation works poorly if the series 

PERFORM_TRAIN       = 1; % this is always initialized to 1
bias                = 1; % always initialized to 1.0. If biasest is on
                         % this will be recalculated

for i=1:size(series,1),

   if infer_training_window, % find the first occurance of nonzero data
     for b=size(series,2)-delay_before_prediction-days_in_model-1:-1:1,
       if (series(i,b-1)==0) & (series(i,b-2)==0),
         break;
       end;
     end;
     training_window = range(1,1)-b;
   end;
      
  training_range = range(1,1) - training_window : range(1,1)-1;
  if (training_window < days_in_model + delay_before_prediction) | ...
     (series(i,training_range)==zeros(1,size(training_range,2))),
      disp('no data in training range');
      % fit and a should be zero
  else
    disp([ num2str(training_window) ' training days']);
    
    
    % we loop over the test range. Recalculate arma weights for
    % each day
    
    if DETREND,figure(1);clf;hold on;end;
    
   for p=range,
       
       if PERFORM_TRAIN,
         % if we need to retrain, then recalc training range, recalc
         % weights, and recalc bias

         training_range = training_range(1,1) : p-delay_before_prediction;
       

         if DETREND,
         % detrend the training data. train on detrended, get the
         % test estimation, add the trend back to the prediction

           % really this should be trending over the entire 
           % training range
           
           % find a short range to trend over
           if training_window>10000,
             trendcalc_window = 60;
           else
             trendcalc_window = training_window;
           end;

           trendcalc_range = range(1,1)-trendcalc_window:range(1,1)-1;
           x = [ trendcalc_range' ones(size(trendcalc_range,2),1) ];
           y = series(i,trendcalc_range)';
           coeffs = x\y;

           % now subtract the intercept from all of the training data
           trendline = ([ [training_range]'      ...
                          ones(size(training_range,2),1) ] ... 
                        * coeffs)';
           training_series = series(i,training_range)-trendline;
         else % no detrend
           if STANDARDIZE_SCORES,
             % subtract the mean
             training_series = series(i,training_range) - ...
                               mean(series(i,training_range)');
           else
             training_series = series(i,training_range);
           end;
         end;

         % Filter outliers
         traning_series = filter_outliers(training_series,4);
         % note we are filtering from a reduced set. This may not
         % be a good idea. Better to filter from overall set. But
         % we can't apply trend line to overall set. Risk filtering
         % outliers on a sample of 20.

         % Train
         a(i,:) = earpyw(training_series, ...
                         days_in_model,delay_before_prediction)';

         if ~CONTINUALLY_RETRAIN, PERFORM_TRAIN=0; end;
   
         % now calculate the bias (or stock creep) over this training
         % period. Multiply the fit by this creep value.
         % we have to iterate on the training data to calc stock creep

         if biasest, % estimate bias from training data and adjust model(!)
           training_range2 = 1+delay_before_prediction+... 
                             days_in_model : ...
                             size(training_series,2);

           for pp=training_range2,
           % calculate fit over training range
             f(1,pp) = a(i,:) * flipud(training_series(1, ...
                                   pp-delay_before_prediction-days_in_model: ...
                                   pp-delay_before_prediction-1)');
           end;
           bias = sum(f(1,training_range2)') / sum(training_series');
           bias = 1./bias; % this is now a correction (multiplier)
           biasest = 0; % don't calculate this again

           if 1, % estimate the error on the training set
             % depending on how much error, either use or not use bias
             % correction. f is fit.
             ff(1,training_range2) = f(1,training_range2)+1;
             ss = training_series(1,training_range2)+1;
             err = abs(ff(1,training_range2)-ss) ./ ss;
             err = mean(err'),
             % translate err into a percent multiplier on bias.
           end;

         end; % estimate bias and error
       end; % PERFORM_TRAIN

       % calculate the fit for this point in time from the most 
       % recently estimated weights

       % calc trendline intercept at these testpoints. We need to 
       % subtract that trendline from each test point.

       if DETREND,
         trendline = ( [ [ p-delay_before_prediction-days_in_model:...
                           p-delay_before_prediction-1 ]' ...
                         ones(days_in_model,1) ] * coeffs );
         plot(p,([p 1]*coeffs)','r-');
       else
         trendline = zeros(days_in_model,1);
         coeffs = [0 0]';
       end;

       test_series = flipud(series(i, ...
                                p-delay_before_prediction-days_in_model:...
                                p-delay_before_prediction-1)') - ...
                                trendline;
 
        fit(i,p) = bias .* a(i,:) * test_series;

%       fit(i,p)=bias .* (a(i,:) * flipud(series(i, ...
%                                p-delay_before_prediction-days_in_model:...
%                                p-delay_before_prediction-1)'));

       % this gives the fit on the detrended data. ie. on the data minus
       % the trend line. Now we add the trendline to this fit value. (ie.
       % add the trend back)

       % retrend

       if DETREND,
         % temporarily don't retrend
         %fit(i,p) = fit(i,p) + ([p 1]*coeffs)'; 
       else
         if STANDARDIZE_SCORES,
           fit(i,p) = fit(i,p) + mean(series(i,training_range)');
         end;
       end;

       % p is the x coord, coeffs*x gives the trendline intercept at this
       % point
       % if no detrending, put back the mean
               
       end;
           
   end;
   
   if ~DETREND, figure(1);clf;hold on;end;
   plot(range,series(i,range),'r-');hold on;
   plot(range,fit(i,range));
   title(['timeseries ' num2str(i)]);
   drawnow;
   
   [corr(i,:),perr(i,:)] = display_accuracy(fit(i,range),series(i,range),0);
   % pause

  end; % no data in training range
end;
